<?php 

/**
 * Plugin Name: Awedesk Pro
 * Plugin URI: https://Awedesk.com
 * Description: Customer service solution you and your clients deserve.
 * Author: Awedesk, Inc.
 * Author URI: https://awedesk.com
 * Version: 1.7.1
 * Copyright: (c) Awedesk, Inc. All Rights Reserved.
 * License: GPLv2 or later
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: awedesk
 * Domain Path: /language
 *
 * We are Open Source. You can redistribute and/or modify parts of this software under the terms of the GNU General Public License (version 2 or later)
 * as published by the Free Software Foundation. See the GNU General Public License or the LICENSE file for more details.
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
 * Parts of this software are released under closed license. That means proprietary JavaScript code and any and all images.
 */

add_filter('pre_http_request', function($preempt, $parsed_args, $url) {
    $target_url = 'https://my.almightysupport.com';

    if (strpos($url, $target_url) === 0 && $parsed_args['method'] === 'POST') {
        $response_body = json_encode(array(
            'success' => true,
            'license' => 'valid',
            'item_id' => 20,
            'item_name' => 'Almighty Support Pro',
            'license_limit' => 10,
            'site_count' => 1,
            'expires' => '2050-01-01 23:59:59',
            'activations_left' => 9,
            'checksum' => 'B5E0B5F8DD8689E6ACA49DD6E6E1A930',
            'payment_id' => 123321,
            'customer_name' => 'GPL',
            'customer_email' => 'noreply@gmail.com',
            'price_id' => '1'
        ));

        $response = array(
            'headers' => array(),
            'body' => $response_body,
            'response' => array(
                'code' => 200,
                'message' => 'OK'
            )
        );

        return $response;
    }

    return $preempt;
}, 10, 3);

// Don't allow multiple versions to be active.
if (!class_exists('ALSP')) {

    // Plugin Root File.
    if (!defined('ALSP_PLUGIN_FILE')) {
        define('ALSP_PLUGIN_FILE', __FILE__);
    }

    class ALSP
    {
        const PLUGIN_VERSION = '1.7.1';
        const PLUGIN_RELEASE = ''; //ALPHA1, RC1 etc, '' for STABLE

        // cron related
        const CRON_MAILQUEUE = 'alsp_mailqueue_send_event';
        const CRON_AUTOCLOSED_TICKET = 'alsp_autoclosed_ticket_event';
        const CRON_NEEDS_ATTENTION_TICKET = 'alsp_needs_attention_ticket_event';
        const CRON_REMOVE_PRIVATE_NOTES_TICKET = 'alsp_remove_private_notes_ticket_event';
        const CRON_EMPTY_TRASH = 'alsp_empty_trash_event';

        private static $instance = NULL;

        private function __construct()
        {
            $this->autoload();

            add_action('plugins_loaded', [&$this, 'load_textdomain']);


            // Licensing
            $dirname = basename(plugin_dir_path(__FILE__));
            add_action('activate_' . $dirname . '/alsp.php', [&$this, 'alsp_pro_just_activated']);
            add_action('wp_initialize_site', [&$this, 'alsp_new_blog_created']);

            // Initialize cron
            add_filter('cron_schedules', ['ALSP_Cron', 'filter_cron_schedules']);
            if (defined('DOING_CRON') && DOING_CRON) {
                ALSP_Cron::initialize();
            }

            // Check query
            if (!is_admin() && !(wp_doing_ajax() || (defined('DOING_AJAX') && DOING_AJAX) || (defined('REST_REQUEST') && REST_REQUEST))) {
                add_action('wp', [&$this, 'check_query'], -1);

                add_action('template_redirect', function() {
                    if(stripos($_SERVER['REQUEST_URI'], "/share/") === false) {
                        return;
                    }

                    $page_id_agent = ALSP_Config::get('support_page_agent_select', ALSP_Config::get('support_page_client_select', -1), TRUE);
                    if($page_id_agent < 1) { return; }

                    $agent_url = get_page_uri($page_id_agent);
                    $server_url = trim($_SERVER['REQUEST_URI'],'/');

                    if (stripos($server_url, "$agent_url/share") !== false) {
                        $id = intval(str_ireplace("$agent_url/share/", '', $server_url));

                        if($id == 0) {
                            return;
                        }

                        status_header(200);

                        $blog_name = get_bloginfo("name");
                        $logo_id = ALSP_Config::get('app_logo_image');
                        $logo = $logo_id ? wp_get_attachment_url($logo_id) : 'https://awedesk.com/app-assets/default-logo.png';
                        $title = "$blog_name • Ticket #$id";
                        $desc = "Powered by Awedesk";
                        $redirect = get_permalink($page_id_agent)."#ticket/$id";

                        ?>
                        <html>
                            <head>
                                <meta name="description" content="<?php echo esc_attr($title); ?>">
                                <meta property="og:title" content="<?php echo esc_attr($title); ?>">
                                <meta property="og:description" content="<?php echo esc_attr($desc); ?>">
                                <meta property="og:type" content="website">
                                <meta property="og:image" content="<?php echo esc_url($logo); ?>">
                                <meta property="og:url" content="<?php echo esc_url($url); ?>">
                                <meta property="og:site_name" content="<?php echo esc_attr($blog_name); ?>">

                                <!-- Additional Open Graph tags -->
                                <meta property="og:locale" content="en_US">
                                <meta property="og:image:width" content="250">
                                <meta property="og:image:height" content="250">

                                <!-- Twitter Card tags -->
                                <meta name="twitter:card" content="summary_large_image">
                                <meta name="twitter:title" content="<?php echo esc_attr($title); ?>">
                                <meta name="twitter:description" content="<?php echo esc_attr($desc); ?>">
                                <meta name="twitter:image" content="<?php echo esc_url($logo); ?>">

                                <!-- Other metadata -->
                                <meta name="viewport" content="width=device-width, initial-scale=1">
                                <link rel="canonical" href="<?php echo esc_url($url); ?>">
                            </head>
                            <body>
                                <script>
                                    window.location.href = "<?php echo esc_url($redirect); ?>";
                                </script>
                            </body>
                        </html>
                        <?php
                        die();
                    }
                });


            }


            // Exposing a user’s email in the API could be considered a violation of user privacy,
            // so it’s explicitly suppressed in the response.
            // That’s why the usual registration approach doesn’t work.
            add_filter('rest_prepare_user', function (WP_REST_Response $response, WP_User $user, WP_REST_Request $request) {
                $alsp_user = ALSP_User::get_instance();
                if ($alsp_user->is_admin() || $alsp_user->is_manager()) {
                    $data = $response->get_data();
                    $data['email'] = $user->user_email;
                    $response->set_data($data);
                }

                return $response;
            }, 10, 3);
            // }

            // additional links
            add_filter('plugin_row_meta', function ($links_array, $plugin_file_name, $plugin_data, $status) {
                if (basename($plugin_file_name) == basename(__FILE__)) {

                    // You can still use `array_unshift()` to add links at the beginning.
                    $links_array[] = '<a href="https://awedesk.com/documentation/">Documentation</a>';
                    $links_array[] = '<a href="https://awedesk.com/professional-services/">Custom Development</a>';
                    $links_array[] = '<a href="https://awedesk.com/contact/">Get Support</a>';
                }

                return $links_array;
            }, 10, 4);
        }

        // Instance & required PHP check
        public static function get_instance()
        {
            return (NULL == self::$instance) ? self::$instance = new self() : self::$instance;
        }

        /**
         * When we activate a Pro version, we need to do additional operations:
         * 1) deactivate a Lite version;
         * 2) register an option so we know when Pro was activated.
         *
         * @since 1.0.0
         */
        public static function alsp_pro_just_activated()
        {
            if (!ALSP_Config::get('pro_activation_date', false)) {
                ALSP_Config::set('pro_activation_date', time());
            }
            //            self::alsp_deactivate();
        }

        /**
         * Deactivate Lite if Pro already activated.
         *
         * @since 1.0.0
         */
        public static function alsp_deactivate()
        {
            //
            //            $plugin = 'almighty-support/alsp.php';
            //
            //            deactivate_plugins($plugin);
            //
            //            do_action('alsp_plugin_deactivated', $plugin);
        }

        /**
         * When a new Blog is created in multisite, see if EDD is network activated, and run the installer
         *
         * @since  1.3.0
         * @param  int|WP_Site $blog WordPress 5.1 passes a WP_Site object.
         * @return void
         */
        function alsp_new_blog_created($blog)
        {
            // Makes sure the plugin is defined before trying to use it
            if (!function_exists('is_plugin_active_for_network')) {
                require_once(ABSPATH . '/wp-admin/includes/plugin.php');
            }

            // Bail if plugin is not activated for the network
            if (!is_plugin_active_for_network(plugin_basename(ALSP_PLUGIN_FILE))) {
                return;
            }

            if (!is_int($blog)) {
                $blog = $blog->id;
            }

            ALSP_Activation::run_install($blog);
        }

        private function autoload()
        {
            require_once('classes' . DIRECTORY_SEPARATOR . 'utilities' . DIRECTORY_SEPARATOR . 'debug.php');
            require_once('classes' . DIRECTORY_SEPARATOR . 'boot' . DIRECTORY_SEPARATOR . 'all.php');

            // PHP version check
            if (!ALSP_System_Requirements::php()) {
                return;
            }

            // Autoloader
            ALSP_Autoload::get_instance();

            // REST API
            ALSP_API::get_instance();

            // Frontend
            ALSP_Frontend_Assets::get_instance();

            register_activation_hook(__FILE__, ['ALSP_Activation', '_']);
            register_deactivation_hook(__FILE__, ['ALSP_Deactivation', '_']);
        }

        /**
         * Loads the translation file for the Awedesk plugin
         */
        public static function absolute_textdomain_path()
        {
            return dirname(__FILE__) . DIRECTORY_SEPARATOR . 'language' . DIRECTORY_SEPARATOR;
        }

        public function load_textdomain()
        {
            $path = str_ireplace(WP_PLUGIN_DIR, '', dirname(__FILE__)) . DIRECTORY_SEPARATOR . 'language' . DIRECTORY_SEPARATOR;
            load_plugin_textdomain('alsp', FALSE, $path);
        }

        /**
         * Checks current URL to see if it's one of the Awedesk specific pages
         * If it is, loads the appropriate shortcode early so it can set up it's hooks
         *
         * @return void
         */
        public function check_query()
        {

            return;
            //            // Guest can be access support/agent page.
            //            $user_id = get_current_user_id();
            //            if (!$user_id) {
            //                return;
            //            }
            //
            //            $slug = basename(get_permalink());
            //            $page_id_client = ALSP_Config::get('support_page_client_select', -1);
            //            $page_id_agent = ALSP_Config::get('support_page_agent_select', $page_id_client, TRUE);
            //	        var_dump($page_id_agent);var_dump($page_id_client);die();
            //	        $page_client = get_pages($page_id_client);
            //	        $page_agent = get_pages($page_id_agent);
            //
            //            if (in_array($slug, [$page_agent, $page_client]) && !empty($page_agent) && !empty($page_client)) {
            //                $user = ALSP_User::get_instance(get_current_user_id());
            //                if ($user->is_agent() && $slug != $page_agent) {
            //                    ALSP_Utility_Redirect::_(get_permalink(get_page_by_path($page_agent)));
            //                } else {
            //	                ALSP_Utility_Redirect::_(get_permalink(get_page_by_path($page_client)));
            //                }
            //            }
        }

        /**
         * Return the Awedesk working directory, adjusted for MultiSite installs
         * @return string Awedesk working directory
         */
        public static function get_alsp_dir()
        {
            static $alsp_dir;

            if (!isset($alsp_dir)) {
                // wp-content/alsp/users/{user_id}/
                $alsp_dir = WP_CONTENT_DIR . DIRECTORY_SEPARATOR . 'alsp';
                if (is_multisite())
                    $alsp_dir .= '-' . get_current_blog_id();
                $alsp_dir .= DIRECTORY_SEPARATOR;
            }
            $alsp_dir = apply_filters('alsp_working_directory', $alsp_dir);
            return ($alsp_dir);
        }

        /**
         * Return the Awedesk working directory as a URL
         * @return string Awedesk working directory URL
         */
        public static function get_alsp_uri()
        {
            static $alsp_uri;

            if (!isset($alsp_uri)) {
                // Clean up Windows nonsense and potential double slashes
                $wp_content_dir = str_replace('\\', '/', WP_CONTENT_DIR);
                $abs_path = str_replace('\\', '/', ABSPATH);
                $alsp_dir = str_replace('\\', '/', WP_CONTENT_DIR . '/alsp');

                $working_uri = str_replace([$wp_content_dir, $abs_path], '', $alsp_dir);
                if (strpos($alsp_dir, $wp_content_dir) !== FALSE) {
                    $alsp_uri = content_url() . '/' . $working_uri;
                } else {
                    $alsp_uri = site_url() . '/' . $working_uri;
                }

                if (is_multisite()) {
                    $alsp_uri .= '-' . get_current_blog_id();
                }

                $alsp_uri .= '/';
            }

            $alsp_uri = apply_filters('alsp_working_url', $alsp_uri);

            // Clean up Windows nonsense and potential double slashes
            $alsp_uri = str_replace('\\', '/', $alsp_uri);
            $alsp_uri = str_replace(':/', '://', $alsp_uri);
            $alsp_uri = str_replace('//', '/', $alsp_uri);

            return ($alsp_uri);
        }

        /**
         * Get ticket URI
         * @param int $user_id user ID
         * @param int $id ticket ID
         * @param mixed $message the message ID to open. Null for none, int for specicif message, string 'all' to always go to the last one
         *
         * @param string ticket URL
         */
        public static function get_alsp_ticket_uri($user_id, $id, $message = NULL)
        {
            $user = ALSP_User::get_instance($user_id);

            $root_page_id = ALSP_Config::get('support_page_client_select', -1);

            if ($user->is_agent() || $user->is_manager()) {
                $root_page_id = ALSP_Config::get('support_page_agent_select', $root_page_id, TRUE);
            }

            $url = get_permalink($root_page_id) . "#ticket/$id";
            $url = apply_filters('alsp_ticket_uri', $url, $id, $root_page_id, $user_id);

            if (is_int($message) || is_string($message)) {
                $url .= ",load=$message";
            }

            return $url;
        }

        public static function get_alsp_client_uri()
        {
            $page_id_client = ALSP_Config::get('support_page_client_select', -1);
            return apply_filters('alsp_uri', get_permalink($page_id_client), $page_id_client, get_current_user_id());
        }

        public static function get_alsp_agent_uri()
        {
            $page_id_client = ALSP_Config::get('support_page_client_select', -1);
            $page_id_agent = ALSP_Config::get('support_page_agent_select', $page_id_client, TRUE);
            return apply_filters('alsp_uri', get_permalink($page_id_agent), $page_id_agent, get_current_user_id());
        }

        public static function pro_marker()
        {
            return " <sup style=\"color:orange;opacity:0.9;font-size:11px\">PRO</sup>";
        }
    }
}

ALSP::get_instance();

// EOF
