alsp.class('ALSPAdminAbstract', function (name, alsp, $) {
	return class extends alsp.class('ALSPAbstract') {
		constructor(container) {
			super(container);

			this.$container.on('input', 'input[type=number]', e => this.onNumberInput(e));
			this.$container.on('focus', 'input[type=number]', e => this.onNumberFocus(e));
			this.$container.on('blur', 'input[type=number]', e => this.onNumberBlur(e));
			this.$container.on('focus', 'textarea', e => this.onTextareaFocus(e));
			this.$container.on('blur', 'textarea', e => this.onTextareaBlur(e));
			this.$container.on('alsp:canceled', 'textarea', e => this.onTextareaCanceled(e));
			this.$container.on('alsp:saved', 'textarea', e => this.onTextareaSaved(e));
		}

		progress(el, type, onComplete) {
			let $config = $(el).closest('[data-type]');
			let $inputs = $config.find(
				'[type=text], [type=number], [type=password], [type=checkbox], select, textarea'
			);
			let $buttons = $config.find('[data-btn=cancel], [data-btn=save]');
			let $loading = $config.find('[data-loading]');
			let $check = $config.find('[data-check]');

			if (type === 'loading') {
				$config.data('force-loading', 1).css('opacity', 0.5);
				$inputs.attr('disabled', 'disabled');
				$buttons.attr('disabled', 'disabled');
				$loading.stop().show();
				$check.stop().hide();

				// Force loading progress for current config to be shown for at least 1 second.
				setTimeout(() => {
					$config.removeData('force-loading');
					let queue = $config.data('queue');
					if (queue) {
						$config.removeData('queue');
						this.progress($config, queue.type, queue.onComplete);
					}
				}, 1000);
			} else if ($config.data('force-loading')) {
				$config.data('queue', { type, onComplete });
			} else if (type === 'done') {
				$config.css('opacity', '');
				$inputs.removeAttr('disabled');
				$buttons.removeAttr('disabled');
				$loading.stop().hide();
				$check.stop().show().delay(1000).fadeOut();
				if ('function' === typeof onComplete) {
					onComplete();
				}
			} else if (type === 'fail') {
				$config.css('opacity', '');
				$inputs.removeAttr('disabled');
				$buttons.removeAttr('disabled');
				$loading.stop().hide();
				$check.stop().hide();
				if ('function' === typeof onComplete) {
					onComplete();
				}
			}
		}

		toggleChildren(parent, opts = { disable: null, visible: null, animate: true }) {
			if (typeof parent === 'object') {
				opts = parent;
				parent = null;
			}

			if (parent && typeof parent === 'string') {
				let $parent = this.$container.find(`[id="${parent}"]`);

				let $children = this.$container.find(`[data-parent="${parent}"]`);
				if ($parent.is('select')) {
					$children = $children.add(this.$container.find(`[data-parent^="${parent}="]`));
				}

				let disableable = [
					'[type=text]',
					'[type=number]',
					'[type=password]',
					'[type=checkbox]',
					'select',
					'textarea',
					'[data-btn=cancel]',
					'[data-btn=save]'
				].join(',');

				if (opts.disable) {
					$children.css('opacity', 0.5);
					$children.find(disableable).attr('disabled', 'disable');
				} else {
					$children.css('opacity', '');
					$children.find(disableable).removeAttr('disabled');

					if ($parent.is('select')) {
						let sVisible;
						if ($parent.is(':visible')) {
							sVisible = `[data-parent^="${parent}=${opts.visible}"]`;
							if (isNaN(opts.visible) ? !!opts.visible : +opts.visible) {
								sVisible += `, [data-parent="${parent}"]`;
							}
						}

						let $visible = sVisible ? $children.filter(sVisible) : $();
						let $hidden = $children.not($visible);

						opts.animate ? $visible.slideDown('fast') : $visible.show();
						opts.animate ? $hidden.slideUp('fast') : $hidden.hide();
					} else if (opts.visible === true) {
						opts.animate ? $children.slideDown('fast') : $children.show();
					} else if (opts.visible === false) {
						opts.animate ? $children.slideUp('fast') : $children.hide();
					}
				}
			} else {
				let parentIds = this.$container
					.find(`[data-parent]`)
					.map((i, el) => el.getAttribute('data-parent').split('=')[0])
					.toArray()
					.filter(id => !!id);

				let uniqueParentIds = parentIds.filter((item, i, ar) => ar.indexOf(item) === i);

				uniqueParentIds.forEach(id => {
					let $parent = this.$container.find(`[id="${id}"]`);
					let visible = true;

					if ($parent.is('select')) {
						visible = $parent.val();
						visible &&= $parent.is(':visible');
					} else if ($parent.is(':checkbox')) {
						visible = $parent.is(':checked');
						visible &&= $parent.siblings('[data-alsp=checkbox]').is(':visible');
					} else {
						let value = $parent.val();
						visible = !!(/^\d+$/.test(value) ? +value : value);
						visible &&= $parent.is(':visible');
					}

					this.toggleChildren(id, { visible, animate: opts.animate });
				});
			}
		}

		onNumberInput(e) {
			let $input = $(e.currentTarget);
			let max = +$input.attr('max');
			let value = +$input.val();

			if (max && value && value > max) {
				$input.val(max);
			}
		}

		onNumberFocus(e) {
			let $input = $(e.currentTarget);
			let $desc = $input.closest('[data-type=int]').find('[data-alsp=number-desc]');

			$desc.show();
		}

		onNumberBlur(e) {
			let $input = $(e.currentTarget);
			let $desc = $input.closest('[data-type=int]').find('[data-alsp=number-desc]');
			let min = +$input.attr('min');
			let value = +$input.val();

			if (min && (!value || value < min)) {
				$input.val(min);
			}

			$desc.hide();
		}

		onTextareaFocus(e) {
			let $textarea = $(e.currentTarget);
			let initialHeight = $textarea.outerHeight();

			if ($textarea.get(0).style.height) return; // Skip if height is set.

			$textarea.css('height', initialHeight);
			$textarea.data('initial-height', initialHeight);
			$textarea.attr('rows', 10);
			$textarea.animate({ height: $textarea.prop('scrollHeight') }, 'fast', function () {
				$textarea.alsp_autosize().triggerHandler('input.alsp-autosize');
			});
		}

		onTextareaBlur(e) {
			let textarea = e.currentTarget;

			// Save last caret position.
			textarea.lastCaretPos = textarea.selectionStart;

			textarea.timer && clearTimeout(textarea.timer);
			textarea.timer = setTimeout(() => {
				delete textarea.timer;

				if (textarea === document.activeElement) return; // Skip if textarea still has focus.

				let $textarea = $(textarea);
				let modified = $textarea.val() !== $textarea.attr('data-value');
				if (modified) return; // Do NOT collapse the textarea if content is modified.

				// Collapse textarea to the default height.
				let initialHeight = $textarea.data('initial-height');
				$textarea.removeAttr('rows');
				$textarea.off('input.alsp-autosize');
				$textarea.stop().animate({ height: initialHeight }, 'fast', function () {
					$textarea.css('height', '');
				});
			}, 100);
		}

		onTextareaCanceled(e) {
			let textarea = e.currentTarget;

			if ('undefined' !== textarea.lastCaretPos) {
				delete textarea.lastCaretPos;
			}

			this.onTextareaBlur(e);
		}

		onTextareaSaved(e) {
			let textarea = e.currentTarget;

			if (document.body === document.activeElement) {
				textarea.focus();

				if ('undefined' !== textarea.lastCaretPos) {
					textarea.selectionEnd = textarea.lastCaretPos;
					delete textarea.lastCaretPos;
				}
			} else {
				this.onTextareaBlur(e);
			}
		}
	};
});
