alsp.class('ALSPAdminConfigEmailPiping', function (name, alsp, $) {
	const { ajax, template, data: config, dialog } = alsp;

	return class extends alsp.class('ALSPAdminAbstract') {
		constructor(container) {
			super(container);

			this.$enabled = this.$container.find('#ep_enabled');
			this.$connection = this.$container.find('#ep_connection_select');

			this.toggleConnectionType();
			this.toggleMultiInbox();
			this.$enabled.on('click', e => {
				this.toggleConnectionType(); 
				this.toggleMultiInbox();
			});
			this.$connection.on('change.ep', e => this.toggleConnectionType());
			this.$container.on('click', '[data-btn=check]', e => this.onBtnCheckClick(e));
			this.$container.on('click', '[data-id$=_ep_multi_inbox_delete_button]', e =>
				this.onInboxDeleteClick(e)
			);
			this.$container.on('click', '[data-id=ep_multi_inbox_button]', e =>
				this.onInboxAddMoreClick(e)
			);
		}

		toggleConnectionType() {
			let type = this.$enabled.is(':checked') ? this.$connection.val() : null;
			let types = [
				['imap', 'imap'],
				['gmail', 'gmail'],
				['microsoft-exchange', 'me']
			];

			types.forEach(conn => {
				let $panel = this.$container.find(`[data-id=email_piping_${conn[1]}_conn]`);
				conn[0] === type ? $panel.show() : $panel.hide();
			});
		}

		toggleMultiInbox() {
			let $panel = this.$container.find(`[data-id=email_piping_multi_inbox]`);
			this.$enabled.is(':checked') ? $panel.show() : $panel.hide();
		}

		onBtnCheckClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let $button = $(e.currentTarget);
			let $config = $button.closest('[data-type]');
			let type = $button.data('id');

			// Save unsaved inputs first.
			$.Deferred(defer => {
				let $group = $button.closest(`[data-id*=email_piping_]`);
				let $buttons = $group.find('[data-btn=save]:visible');
				if (!$buttons.length) return defer.resolve();

				$buttons.trigger('click');
				let maxLoops = 60;
				let timer = setInterval(function () {
					if (!$buttons.filter(':visible').length) {
						clearInterval(timer);
						defer.resolve();
					} else if (!maxLoops--) {
						clearInterval(timer);
						defer.reject();
					}
				}, 500);
			}).done(() => {
				$button.attr('disabled', 'disabled');
				this.progress($config, 'loading');
				ajax(`almightysupport/v1/admin_${type}`, {}, 'POST')
					.always(() => {
						this.progress($config, 'done', () => $button.removeAttr('disabled'));
					})
					.done(json => {
						if (json.redirectURL) {
							window.location.href = json.redirectURL;
						} else if (json.success) {
							dialog.alert('success');
						} else if (json.message) {
							dialog.alert(json.message);
						}
					});
			});
		}

		onInboxDeleteClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let $button = $(e.currentTarget);
			let $config = $button.closest('[data-type]');
			let id = $button.data('forwarder-id');

			$button.attr('disabled', 'disabled');
			this.progress($config, 'loading');
			ajax(`almightysupport/v1/admin_ep_forwarder`, { id }, 'DELETE')
				.always(() => {
					this.progress($config, 'done', () => $button.removeAttr('disabled'));
				})
				.done(json => {
					if (json.success) {
						let $toBeRemoved = $button
							.closest('[data-type]')
							.prevUntil('[data-type=separator]')
							.addBack()
							.prev('[data-type=separator]')
							.addBack();

						$toBeRemoved.remove();
					}
				});
		}

		onInboxAddMoreClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let $button = $(e.currentTarget);
			let $config = $button.closest('[data-type]');

			$button.attr('disabled', 'disabled');
			this.progress($config, 'loading');
			ajax(`almightysupport/v1/admin_ep_forwarder`, {}, 'POST')
				.always(() => {
					this.progress($config, 'done', () => $button.removeAttr('disabled'));
				})
				.done(json => {
					if (json.success && 'object' === typeof json.data) {
						$.each(json.data, (itemId, item) => {
							let tmpl = config(`templates.config_item_${item.type}`);

							if (tmpl) {
								item.label = item.title;
								let html = template(tmpl)(item);
								$config.prev('[data-type=separator]').before(html);
							} else {
								console.log('Unknown admin item ' + item.type);
							}
						});
					}
				});
		}
	};
});
