alsp.class('ALSPAdminConfigLicense', function (name, alsp, $) {
	const { ajax } = alsp;

	return class extends alsp.class('ALSPAdminAbstract') {
		constructor(container) {
			super(container);

			this.$activate = this.$container.find('[name=edd_license_activate]');
			this.$deactivate = this.$container.find('[name=edd_license_deactivate]');
			this.$installNotice = this.$container.find('[data-alsp=install-notice]').hide();

			// Disable default form submit behavior in favor of AJAX request.
			this.$container.find('form').on('submit', e => e.preventDefault());
		}

		onActivateClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let $input = this.$container.find('[name=alsp_license_key]');
			let $error = $input.siblings('[data-alsp=license-key-error]');
			let alsp_license_key = $input.val().trim();
			let params = { action: 'activate', alsp_license_key };

			// Reset error.
			$input.css({ borderColor: '', boxShadow: '' });
			$error.empty().hide();

			this.progress($input, 'loading');
			ajax('almightysupport/v1/admin_license', params, 'POST').done(json => {
				if (json.success) {
					this.$activate.hide();
					this.$deactivate.show();
					this.install(alsp_license_key)
						.fail(() => this.progress($input, 'fail'))
						.done(() => location.reload());
				} else {
					this.progress($input, 'fail');
					$input.css({ borderColor: '#f44336', boxShadow: '0 0 0 4px #ffcfcf' });
					json.message && $error.html(json.message).show();
				}
			});
		}

		onDeactivateClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let $input = this.$container.find('[name=alsp_license_key]');
			let $error = $input.siblings('[data-alsp=license-key-error]');
			let alsp_license_key = $input.val().trim();
			let params = { action: 'deactivate', alsp_license_key };

			// Reset error.
			$input.css({ borderColor: '', boxShadow: '' });
			$error.empty().hide();

			this.progress($input, 'loading');
			ajax('almightysupport/v1/admin_license', params, 'POST').done(json => {
				if (json.success) {
					this.progress($input, 'done', () => location.reload());
				} else {
					this.progress($input, 'fail');
					$input.css({ borderColor: '#f44336', boxShadow: '0 0 0 4px #ffcfcf' });
					json.message && $error.html(json.message).show();
				}
			});
		}

		install(alsp_license_key) {
			let params = { action: 'install', alsp_license_key };

			return $.Deferred(defer => {
				this.$installNotice.show();
				ajax('almightysupport/v1/admin_license', params, 'POST')
					.always(() => this.$installNotice.hide())
					.fail(() => defer.reject())
					.done(json => (json.success ? defer.resolve() : defer.reject()));
			});
		}
	};
});
