alsp.class('ALSPAdminConfiguration', function (name, alsp, $) {
	const { ajax, template, data: config, dialog } = alsp;

	return class extends alsp.class('ALSPAdminAbstract') {
		constructor(container) {
			super(container);

			ajax('almightysupport/v1/admin_config_menu').done(menus => {
				let hash = location.hash.replace(/^#/, '');
				let menu = menus[hash] ? hash : Object.keys(menus)[0];
				let tmpl = template(config('templates.config'));
				let html = tmpl({ menus, menu });

				this.menus = menus;
				this.$container.html(html);
				this.$left = this.$container.find('[data-alsp=config-left]');
				this.$right = this.$container.find('[data-alsp=config-right]');

				this.$container.on('click', '[data-alsp=config-menu] a', e => this.onMenuClick(e));
				this.$container.on('click', 'input[type=checkbox]', e => this.onCheckboxClick(e));
				this.$container.on(
					'input',
					'input[type=text], input[type=number], input[type=password], textarea',
					e => this.onTextInput(e)
				);
				this.$container.on('click', 'button[data-btn=cancel]', e => this.onCancelClick(e));
				this.$container.on('click', 'button[data-btn=save]', e => this.onSaveClick(e));
				this.$container.on('change', 'select', e => this.onSelectChange(e));

				this.selectMenu(menu);

				window.addEventListener(
					'hashchange',
					() => {
						requestAnimationFrame(() => {
							let menu = location.hash.replace(/^#/, '');
							if (menu !== this.menu) {
								this.selectMenu(menu);
							}
						});
					},
					false
				);
			});

			this.onBeforeUnload = this.onBeforeUnload.bind(this);
			window.removeEventListener('beforeunload', this.onBeforeUnload, { capture: true });
			window.addEventListener('beforeunload', this.onBeforeUnload, { capture: true });
		}

		selectMenu(menu) {
			// Check any unsaved config.
			let $buttons = this.$container.find('[data-type] [data-btn=save]');
			let $visible = $buttons.filter(':visible').not('[data-alsp]');
			if (!$visible.length) {
				this.selectMenuConfirm(menu);
				return;
			}

			dialog.confirm(
				config('lost_content_warning'),
				() => this.selectMenuConfirm(menu),
				() => (location.hash = this.menu)
			);
		}

		selectMenuConfirm(menu) {
			this.menu = menu;

			let xhrs = [];

			let $menu = this.$container.find(`[data-alsp=config-menu] a[href=#${menu}]`);
			if ($menu.length) {
				$menu.addClass('alsp-active').siblings().removeClass('alsp-active');
			}

			['left', 'right'].forEach(position => {
				let $position = this.$container.find(`[data-alsp=config-${position}]`);
				let xhr = ajax('almightysupport/v1/admin', { position: `${menu}-${position}` });

				xhr.done(json => this.render($position, json));
				xhrs.push(xhr);
			});

			// Highlight submenu.
			let $sidemenus = $('#adminmenu > li.toplevel_page_alsp > ul.wp-submenu a');
			$sidemenus = $sidemenus.filter((i, a) => a.href.match(/\?page=alsp#/));
			let $active = $sidemenus.filter((i, a) => a.href.indexOf(`?page=alsp#${menu}`) > -1);
			$active.closest('li').addClass('current');
			$sidemenus.not($active).closest('li').removeClass('current');

			$.when(...xhrs).done(() => {
				// Initialize config subclass if available.
				let subClass = alsp.class(`ALSPAdminConfig${capitalize(menu)}`);
				if ('function' === typeof subClass) {
					new subClass(this.$container);
				}

				// Initialize file selector.
				this.$container.find('[data-type=file]').each((i, el) => this.initFileSelector(el));

				// Initialize image selector.
				this.$container
					.find('[data-type=image]')
					.each((i, el) => this.initImageSelector(el));

				// Initialize colorpicker.
				this.$container
					.find('[data-type=color] input')
					.each((i, el) => this.initColorpicker(el));

				this.toggleChildren({ animate: false });
			});
		}

		render($position, json) {
			$position.empty();

			$.each(json, (groupId, group) => {
				// Render custom HTML directly.
				if (group.html) {
					$position.append(group.html);
					return;
				}

				// Skip config group with no items.
				if ('object' !== typeof group.items || !Object.keys(group.items).length) {
					return true;
				}

				let content = '';

				$.each(group.items, (itemId, item) => {
					let tmpl = config(`templates.config_item_${item.type}`);

					if (tmpl) {
						item.label = item.title;
						content += template(tmpl)(item);
					} else {
						console.log('Unknown admin item ' + item.type);
					}
				});

				let tmpl = template(config('templates.config_group'));
				let html = tmpl(Object.assign(group, { content }));

				$position.append(html);
			});
		}

		initFileSelector(el) {}

		initImageSelector(el) {
			let frame;
			let $config = $(el);
			let $input = $config.find('[type=hidden]');
			let $select = $config.find('[data-btn=select]');
			let $remove = $config.find('[data-btn=remove]');
			let $preview = $config.find('[data-alsp=preview]');

			$select.on('click', e => {
				e.preventDefault();

				if (!frame) {
					frame = wp.media({
						title: 'Select or Upload Image',
						button: { text: 'Use this image' },
						library: { type: ['image'] },
						multiple: false
					});

					frame.on('select', () => {
						let attachment = frame.state().get('selection').first().toJSON();
						let name = $input.attr('id');
						let value = attachment.id;

						ajax('almightysupport/v1/admin', { name, value }, 'POST').done(() => {
							$input.val(value).attr('data-value', value);
							this.progress($config, 'done', () => {
								let attachmentUrl = attachment.url.replace(/^https?:/i, '');

								$preview.children('img').attr('src', attachmentUrl);
								$preview.show();
								$remove.show();
							});
						});
					});
				}

				frame.open();
			});

			$remove.on('click', e => {
				e.preventDefault();

				let name = $input.attr('id');
				let value = '';

				ajax('almightysupport/v1/admin', { name, value }, 'POST').done(() => {
					$input.val(value).attr('data-value', value);
					this.progress($config, 'done', () => {
						$preview.children('img').removeAttr('src');
						$preview.hide();
						$remove.hide();
					});
				});
			});
		}

		initColorpicker(input) {
			let $input = $(input).hide();
			let $toggle = $input.next('button');
			let $clear = $input.closest('[data-type]').find('[data-btn=clear]');

			let pickr = Pickr.create({
				el: $toggle.get(0),
				theme: 'monolith',
				default: $input.val() || null,
				appClass: 'alsp-colorpicker',
				swatches: [
					'rgba(244, 67, 54, 1)',
					'rgba(233, 30, 99, 1)',
					'rgba(156, 39, 176, 1)',
					'rgba(103, 58, 183, 1)',
					'rgba(63, 81, 181, 1)',
					'rgba(33, 150, 243, 1)',
					'rgba(3, 169, 244, 1)',
					'rgba(0, 188, 212, 1)',
					'rgba(0, 150, 136, 1)',
					'rgba(76, 175, 80, 1)',
					'rgba(139, 195, 74, 1)',
					'rgba(205, 220, 57, 1)',
					'rgba(255, 235, 59, 1)',
					'rgba(255, 193, 7, 1)'
				],
				components: {
					preview: true,
					opacity: false,
					hue: true,
					interaction: {
						hex: false,
						rgba: false,
						hsva: false,
						input: true,
						save: true,
						cancel: false,
						clear: false
					}
				}
			});

			pickr.on('change', function (color, pickr) {
				// Pickr's `toRGB()` method result in an undesired long-precision floating values,
				// so we will just use `toHEXA()` method instead.
				color = color.toHEXA().toString();
				color = maybeHexaToRgba(color);

				// Automatically save color on `change` event.
				pickr.setColor(color, true);
			});

			pickr.on('save', (color, pickr) => {
				if (null !== color) {
					// Pickr's `toRGB()` method result in an undesired long-precision floating values,
					// so we will just use `toHEXA()` method instead.
					color = color.toHEXA().toString();
					color = maybeHexaToRgba(color);
				} else {
					color = '';
				}

				if ($input.val() !== color) {
					$input.val(color);
					this.saveColorInput(input);
				}

				pickr.hide();
			});

			$clear.on('click', () => pickr.setColor(null));
		}

		saveColorInput(input) {
			let $input = $(input).hide();
			let $config = $input.closest('[data-type]');
			let $clear = $input.closest('[data-type]').find('[data-btn=clear]');

			let name = $input.attr('id');
			let value = $input.val().trim();

			// Hide early on value.
			if (!value) $clear.hide();

			this.progress($config, 'loading');
			return ajax('almightysupport/v1/admin', { name, value }, 'POST')
				.done(() => {
					$input.attr('data-value', value);
					this.progress($config, 'done', () => {
						value && setTimeout(() => $clear.show(), 1250);
					});
				})
				.fail(xhr => {
					this.progress($config, 'fail');
					$input.attr('data-value') ? $clear.show() : $clear.hide();
				});
		}

		onMenuClick(e) {
			e.stopPropagation();

			requestAnimationFrame(() => {
				let menu = location.hash.replace(/^#/, '');
				if (menu !== this.menu) {
					this.selectMenu(menu);
				}
			});
		}

		onCheckboxClick(e) {
			e.stopPropagation();

			let $checkbox = $(e.currentTarget);
			let $config = $checkbox.closest('[data-type]');
			let name = $checkbox.attr('id');
			let value = $checkbox.is(':checked') ? 1 : 0;

			this.progress($config, 'loading');
			this.toggleChildren(name, { disable: true });
			ajax('almightysupport/v1/admin', { name, value }, 'POST')
				.done(() => {
					this.progress($config, 'done', () => {
						this.toggleChildren(name, { visible: !!value });
					});
				})
				.fail(xhr => this.progress($config, 'fail'));
		}

		onTextInput(e) {
			e.stopPropagation();

			let $input = $(e.currentTarget);
			let $config = $input.closest('[data-type]');
			let $error = $config.find('[data-error-msg]');
			let $cancel = $config.find('button[data-btn=cancel]');
			let $save = $config.find('button[data-btn=save]');
			let value = $input.val().trim();
			let previousValue = $input.attr('data-value');

			if (value === previousValue) {
				$error.hide();
				$cancel.hide();
				$save.hide();
			} else {
				$cancel.show();
				$save.show();
			}
		}

		onCancelClick(e) {
			e.stopPropagation();

			let $cancel = $(e.currentTarget);
			let $config = $cancel.closest('[data-type]');
			let $input = $config.find('[type=text], [type=number], [type=password], textarea');
			let $error = $config.find('[data-error-msg]');
			let $save = $config.find('button[data-btn=save]');

			$input.val($input.attr('data-value'));
			$error.hide();
			$cancel.hide();
			$save.hide();

			$input.trigger('alsp:canceled');
		}

		onSaveClick(e) {
			e.stopPropagation();

			let $save = $(e.currentTarget);
			let $config = $save.closest('[data-type]');
			let $input = $config.find('[type=text], [type=number], [type=password], textarea');
			let $error = $config.find('[data-error-msg]');
			let $cancel = $config.find('button[data-btn=cancel]');

			let name = $input.attr('id');
			let value = $input.val().trim();
			let alert = $error.length ? false : undefined;

			$error.hide();

			$input.trigger('alsp:saving');
			this.progress($config, 'loading');
			ajax('almightysupport/v1/admin', { name, value, alert }, 'POST')
				.done(() => {
					$input.attr('data-value', value);
					this.progress($config, 'done', () => {
						$cancel.hide();
						$save.hide();
					});
				})
				.fail(xhr => {
					this.progress($config, 'fail', () => {
						if ($error.length && xhr && xhr.responseJSON) {
							let json = xhr.responseJSON;
							let error =
								json.message || (typeof json.data === 'string' ? json.data : null);

							error && $error.html(error).show();
						}
					});
				})
				.always(() => $input.trigger('alsp:saved'));
		}

		onSelectChange(e) {
			e.stopPropagation();

			let $select = $(e.currentTarget);
			let $config = $select.closest('[data-type]');
			let name = $select.attr('id');
			let value = $select.val();

			this.progress($config, 'loading');
			this.toggleChildren(name, { disable: true });
			ajax('almightysupport/v1/admin', { name, value }, 'POST')
				.done(() => {
					$select.attr('data-value', value);
					this.progress($config, 'done', () => {
						let visible = !!(/^\d+$/.test(value) ? +value : value);
						this.toggleChildren(name, { visible });
					});
				})
				.fail(xhr => this.progress($config, 'fail'));
		}

		onBeforeUnload(e) {
			// Show confirmation if there is any unsaved config.
			let $buttons = this.$container.find('[data-type] [data-btn=save]');
			let $visible = $buttons.filter(':visible').not('[data-alsp]');
			if ($visible.length) {
				(e || window.event).returnValue = null;
				return null;
			}
		}
	};

	/**
	 * Convert HEXA color with aplha value to RGBA representation.
	 *
	 * @param {string} hexa
	 * @param {string}
	 */
	function maybeHexaToRgba(hexa) {
		return hexa.replace(/#(.{2})(.{2})(.{2})(.{2})/, function (hexa, r, g, b, a) {
			r = parseInt(r, 16);
			g = parseInt(g, 16);
			b = parseInt(b, 16);
			a = parseInt(a, 16) / 255;

			// Round opacity value.
			a = Math.floor(a * 10000) / 10000;

			return 'rgba(' + r + ',' + g + ',' + b + ',' + a + ')';
		});
	}

	function capitalize(str) {
		return str.replace(/(?:^|-)([a-z])/gi, function (match, letter) {
			return letter.toUpperCase();
		});
	}
});

jQuery(function ($) {
	let $container = $('[data-alsp=admin-page][data-id=configuration]');
	if ($container.length) {
		new (alsp.class('ALSPAdminConfiguration'))($container.get(0));
	}
});
