alsp.class('ALSPAdminDepartments', function (name, alsp, $) {
	const { ajax, hooks, datastore: ds, template, data: config, dialog } = alsp;

	return class extends alsp.class('ALSPAdminAbstract') {
		constructor(container) {
			super(container);

			this.name = name;

			this.agents = null;

			let tmpl = template(config('templates.admin_departments'));
			let html = tmpl();

			this.$container.html(html);
			this.$departments = this.$container.find('[data-alsp=departments]');
			this.$newDepartmentName = this.$container.find('[data-alsp=new-department-name]');
			this.$newDepartmentDescription = this.$container.find(
				'[data-alsp=new-department-description]'
			);
			this.$newDepartmentAssignee = this.$container.find(
				'[data-alsp=new-department-assignee]'
			);
			this.$newDepartmentAgentOnly = this.$container.find(
				'[data-alsp=new-department-agent-only]'
			);
			this.$newDepartmentPublished = this.$container.find(
				'[data-alsp=new-department-published]'
			);
			this.$newDepartmentSave = this.$container
				.find('[data-alsp=new-department-save]')
				.attr('disabled', 'disabled');

			this.loadDepartments().done(() => this.postinit());

			var bar = this.$container.find('.alsp-settings__bar');
			bar.addClass('alsp-settings__bar--static');
			$(window).scroll(function () {
				var bar = $('.alsp-settings__bar');
				var scrollVal = $(this).scrollTop();
				if (scrollVal > 50) {
					bar.removeClass('alsp-settings__bar--static');
				} else {
					bar.addClass('alsp-settings__bar--static');
				}
			});

			this.getAgents().done(agents => {
				if (agents instanceof Array) {
					agents.forEach(agent => {
						let html = `<option value="${agent.agent.id}">${
							agent.agent.display_name || agent.agent.user_login
						}</option>`;
						this.$newDepartmentAssignee.append(html);
					});
				}
			});
		}

		getAgents() {
			return ds.get('admin_agents', { role: 'admin,manager,agent' });
		}

		getDepartments() {
			return ajax('almightysupport/v1/admin_department');

			// return $.Deferred(defer => {
			// 	ajax('almightysupport/v1/admin_department')
			// 		.fail(defer.reject)
			// 		.done(departments => {
			// 			if (departments && departments.length) {
			// 				let xhrs = [];

			// 				for (let i = 0; i < departments.length; i++) {
			// 					let endpoint = 'almightysupport/v1/pro_admin_department_ep';
			// 					let params = { department_id: departments[i].id };
			// 					let xhr = ajax(endpoint, params);
			// 					xhr.done(opts => Object.assign(departments[i], opts || {}));
			// 					xhrs.push(xhr);
			// 				}

			// 				$.when(...xhrs).done(() => {
			// 					defer.resolve(departments);
			// 				});
			// 			}
			// 		});
			// });
		}

		removeDepartment(id) {
			return ajax(`almightysupport/v1/admin_department/${id}`, {}, 'DELETE');
		}

		resetForm() {
			this.$newDepartmentName.val('');
			this.$newDepartmentDescription.val('');
			this.$newDepartmentAssignee.get(0).selectedIndex = 0;
			this.$newDepartmentAgentOnly.get(0).checked = false;
			this.$newDepartmentAgentOnly.eq(0).trigger('change');
			this.$newDepartmentPublished.get(0).checked = true;
			this.$newDepartmentPublished.eq(0).trigger('change');
			this.$newDepartmentSave.attr('disabled', 'disabled');
		}

		updateBoxStatus(id, status, data = {}) {
			if ('all' === id) {
				id = this.$departments.find('[data-alsp=department-item]');
				id = $.makeArray(id).map(el => el.getAttribute('data-id'));
			} else if (!(id instanceof Array)) {
				id = [id];
			}

			data = Object.assign(data, { id, status, action: 'box_status' });

			return ajax('almightysupport/v1/admin_department', data, 'PATCH');
		}

		loadDepartments() {
			return $.Deferred(defer => {
				this.getAgents().done(agents => {
					this.getDepartments().done(departments => {
						this.$departments.empty();

						if (departments instanceof Array && departments.length) {
							let tmpl = template(config('templates.admin_departments_item'));
							departments.forEach(item => {
								item = Object.assign({}, item, { agents });
								this.$departments.append(tmpl(item));
							});
						} else {
							let tmpl = template(config('templates.admin_departments_item_empty'));
							this.$departments.append(tmpl());
						}

						// Make items sortable.
						this.$departments.sortable({
							handle: '.alsp-department__head',
							update: () => {
								let id = this.$departments
									.children('[data-alsp=department-item]')
									.map((i, el) => $(el).data('id'))
									.toArray();

								return ajax(
									'almightysupport/v1/admin_department',
									{ action: 'reorder', id: JSON.stringify(id) },
									'PATCH'
								);
							}
						});

						defer.resolve();

						hooks.doAction('admin_departments_render');
					});
				});
			});
		}

		onDepartmentExpandAllClick(e) {
			e.preventDefault();
			e.stopPropagation();

			this.updateBoxStatus('all', 1).done(() => this.loadDepartments());
		}

		onDepartmentCollapseAllClick(e) {
			e.preventDefault();
			e.stopPropagation();

			this.updateBoxStatus('all', 0).done(() => this.loadDepartments());
		}

		onDepartmentToggleClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let id = $(e.currentTarget).data('id');
			let open = $(e.currentTarget).data('open');
			let status = open == '1' ? 0 : 1;
			this.updateBoxStatus(id, status).done(() => {
				this.$department = this.$departments.find('[data-alsp=department-' + id + ']');
				if (open == '1') {
					this.$department.hide();
					$(e.currentTarget).data('open', 0);
					$(e.currentTarget).removeClass('alsp-i-compress').addClass('alsp-i-expand');
				} else {
					this.$department.show();
					$(e.currentTarget).data('open', 1);
					$(e.currentTarget).removeClass('alsp-i-expand').addClass('alsp-i-compress');
				}
			});
		}

		onDepartmentNameEditClick(e) {
			let $department = $(e.currentTarget).closest('[data-alsp=department-item]');
			let $viewer = $department.find('[data-alsp=department-name-viewer]');
			let $editor = $viewer.siblings('[data-alsp=department-name-editor]');
			let $name = $editor.find('[data-alsp=department-name]');
			let $save = $editor.find('[data-alsp=department-name-save]');

			$viewer.hide();
			$editor.show();
			$name.select();
			$name.data('value', $name.val());
			$name.val().trim() ? $save.removeAttr('disabled') : $save.attr('disabled', 'disabled');
		}

		onDepartmentNameInput(e) {
			let $editor = $(e.currentTarget).closest('[data-alsp=department-name-editor]');
			let $name = $editor.find('[data-alsp=department-name]');
			let $save = $editor.find('[data-alsp=department-name-save]');

			$name.val().trim() ? $save.removeAttr('disabled') : $save.attr('disabled', 'disabled');
		}

		onDepartmentNameCancelClick(e) {
			let $editor = $(e.currentTarget).closest('[data-alsp=department-name-editor]');
			let $viewer = $editor.siblings('[data-alsp=department-name-viewer]');
			let $input = $editor.find('[data-alsp=department-name]');

			$editor.hide();
			$viewer.show();
			$input.val($input.data('value'));
		}

		onDepartmentNameSaveClick(e) {
			let $editor = $(e.currentTarget).closest('[data-alsp=department-name-editor]');
			let $viewer = $editor.siblings('[data-alsp=department-name-viewer]');
			let $input = $editor.find('[data-alsp=department-name]');

			let id = $input.data('id');
			let value = $input.val().trim();
			let params = { id, action: 'update_prop', prop: 'name', value };

			this.progress($input, 'loading');
			ajax('almightysupport/v1/admin_department', params, 'PATCH').done(json => {
				if (json.success) {
					this.progress($input, 'done', () => {
						$editor.hide();
						$viewer.show();
						$viewer.find('[data-alsp=department-name-label]').text(value);
					});
				} else {
					this.progress($input, 'fail');
					json.message && dialog.alert(json.message);
				}
			});
		}

		onDepartmentDescriptionEditClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let $viewer = $(e.currentTarget).closest('[data-alsp=department-description-viewer]');
			let $editor = $viewer.siblings('[data-alsp=department-description-editor]');
			let $input = $editor.find('[data-alsp=department-description]');

			$viewer.hide();
			$editor.show();
			$input.select();
			$input.data('value', $input.val());
		}

		onDepartmentDescriptionCancelClick(e) {
			let $editor = $(e.currentTarget).closest('[data-alsp=department-description-editor]');
			let $viewer = $editor.siblings('[data-alsp=department-description-viewer]');
			let $input = $editor.find('[data-alsp=department-description]');

			$editor.hide();
			$viewer.show();
			$input.val($input.data('value'));
		}

		onDepartmentDescriptionSaveClick(e) {
			let $editor = $(e.currentTarget).closest('[data-alsp=department-description-editor]');
			let $viewer = $editor.siblings('[data-alsp=department-description-viewer]');
			let $input = $editor.find('[data-alsp=department-description]');

			let id = $input.data('id');
			let value = $input.val().trim();
			let params = { id, action: 'update_prop', prop: 'description', value };

			this.progress($input, 'loading');
			ajax('almightysupport/v1/admin_department', params, 'PATCH').done(json => {
				if (json.success) {
					this.progress($input, 'done', () => {
						$editor.hide();
						$viewer.show();
						$viewer.find('[data-alsp=department-description-label]').text(value);
					});
				} else {
					this.progress($input, 'fail');
					json.message && dialog.alert(json.message);
				}
			});
		}

		onDepartmentAssigneeChange(e) {
			let $select = $(e.currentTarget);
			let id = $select.data('id');
			let value = $select.val();
			let params = { id, action: 'update_prop', prop: 'default_assignee', value };

			this.progress($select, 'loading');
			ajax('almightysupport/v1/admin_department', params, 'PATCH').done(json => {
				if (json.success) {
					this.progress($select, 'done');
				} else {
					this.progress($select, 'fail');
					json.message && dialog.alert(json.message);
				}
			});
		}

		onDepartmentAgentOnlyClick(e) {
			let $checkbox = $(e.currentTarget);
			let id = $checkbox.data('id');
			let value = $checkbox.is(':checked') ? 1 : 0;
			let params = { id, action: 'update_prop', prop: 'agent_only', value };

			this.progress($checkbox, 'loading');
			ajax('almightysupport/v1/admin_department', params, 'PATCH').done(json => {
				if (json.success) {
					this.progress($checkbox, 'done');
				} else {
					this.progress($checkbox, 'fail');
					json.message && dialog.alert(json.message);
				}
			});
		}

		onDepartmentPublishedClick(e) {
			let $checkbox = $(e.currentTarget);
			let id = $checkbox.data('id');
			let value = $checkbox.is(':checked') ? 1 : 0;
			let params = { id, action: 'update_prop', prop: 'published', value };

			this.progress($checkbox, 'loading');
			ajax('almightysupport/v1/admin_department', params, 'PATCH').done(json => {
				if (json.success) {
					this.progress($checkbox, 'done');
				} else {
					this.progress($checkbox, 'fail');
					json.message && dialog.alert(json.message);
				}
			});
		}

		onDepartmentRemoveClick(e) {
			e.preventDefault();
			e.stopPropagation();

			dialog.confirm('Are you sure want to delete this Department?', () => {
				// $(this).html('<img src="images/wpspin_light.gif">');
				let id = $(e.currentTarget).data('id');
				this.removeDepartment(id).done(() => {
					this.loadDepartments();
				});
			});
		}

		onNewDepartmentNameInput(e) {
			if (e.target.value.trim()) {
				this.$newDepartmentSave.removeAttr('disabled');
			} else {
				this.$newDepartmentSave.attr('disabled', 'disabled');
			}
		}

		onNewDepartmentSaveClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let name = this.$newDepartmentName.val().trim();
			let description = this.$newDepartmentDescription.val().trim();
			let assignee = this.$newDepartmentAssignee.val();
			let published = this.$newDepartmentPublished.is(':checked') ? 1 : '';
			let agent_only = this.$newDepartmentAgentOnly.is(':checked') ? 1 : '';
			let params = { name, description, default_assignee: assignee, published, agent_only };

			ajax('almightysupport/v1/admin_department', params, 'POST').done(json => {
				if (json.error) {
					dialog.alert(json.message || 'Undefined error.');
				} else {
					this.resetForm();
					this.loadDepartments();
				}
			});
		}
	};
});

jQuery(function ($) {
	let $container = $('[data-alsp=admin-page][data-id=departments]');
	if ($container.length) {
		new (alsp.class('ALSPAdminDepartments'))($container.get(0));
	}
});
