alsp.class('ALSPAdminLicense', function (name, alsp, $) {
	const { ajax, template, data: config, dialog } = alsp;

	return class extends alsp.class('ALSPAdminAbstract') {
		constructor(container) {
			super(container);

			let tmpl = template(config('templates.admin_license'));
			let html = tmpl();

			this.$container.html(html);

			this.$activate = this.$container.find('[name=edd_license_activate]');
			this.$deactivate = this.$container.find('[name=edd_license_deactivate]');
			this.$installNotice = this.$container.find('[data-alsp=install-notice]').hide();

			// Disable default form submit behavior in favor of AJAX request.
			this.$container.find('form').on('submit', e => e.preventDefault());
		}

		onActivateClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let $input = this.$container.find('[name=alsp_license_key]');
			let alsp_license_key = $input.val().trim();
			let params = { action: 'activate', alsp_license_key };

			this.progress($input, 'loading');
			ajax('almightysupport/v1/admin_license', params, 'POST').done(json => {
				if (json.success) {
					this.$activate.hide();
					this.$deactivate.show();
					this.install(alsp_license_key)
						.fail(() => this.progress($input, 'fail'))
						.done(() => location.reload());
				} else {
					this.progress($input, 'fail');
					json.message && dialog.alert(json.message);
				}
			});
		}

		onDeactivateClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let $input = this.$container.find('[name=alsp_license_key]');
			let alsp_license_key = $input.val().trim();
			let params = { action: 'deactivate', alsp_license_key };

			this.progress($input, 'loading');
			ajax('almightysupport/v1/admin_license', params, 'POST').done(json => {
				if (json.success) {
					this.progress($input, 'done', () => location.reload());
				} else {
					this.progress($input, 'fail');
					json.message && dialog.alert(json.message);
				}
			});
		}

		install(alsp_license_key) {
			let params = { action: 'install', alsp_license_key };

			return $.Deferred(defer => {
				this.$installNotice.show();
				ajax('almightysupport/v1/admin_license', params, 'POST')
					.always(() => this.$installNotice.hide())
					.fail(() => defer.reject())
					.done(json => (json.success ? defer.resolve() : defer.reject()));
			});
		}
	};
});

jQuery(function ($) {
	let $container = $('[data-alsp=admin-page][data-id=license]');
	if ($container.length) {
		new (alsp.class('ALSPAdminLicense'))($container.get(0));
	}
});
