alsp.class('ALSPAdminMailqueue', function (name, alsp, $) {
	const { ajax, template, data: config, dialog } = alsp;

	return class extends alsp.class('ALSPAdminAbstract') {
		constructor(container) {
			super(container);

			let tmpl = template(config('templates.admin_mailqueue'));
			let html = tmpl();

			this.$container.html(html);
			this.$mailqueue = this.$container.find('[data-alsp=mailqueue]');
			this.$bulkSelect = this.$container.find('[data-alsp=bulk-select]');
			this.$bulkCheckAll = this.$container.find('[data-alsp=bulk-check-all]');

			this.loadMailqueue();
		}

		getMailqueue() {
			return ajax('almightysupport/v1/admin_mailqueue');
		}

		loadMailqueue() {
			return this.getMailqueue().done(mails => {
				this.$mailqueue.empty();
				this.$bulkSelect.prop('selectedIndex', 0);
				this.$bulkCheckAll.prop('checked', false);

				if (mails instanceof Array && mails.length) {
					let tmpl = template(config('templates.admin_mailqueue_item'));
					mails.forEach(mail => this.$mailqueue.append(tmpl(mail)));
				} else {
					let tmpl = template(config('templates.admin_mailqueue_item_empty'));
					this.$mailqueue.append(tmpl());
				}
			});
		}

		// Sync bulk select dropdown.
		onBulkSelectChange(e) {
			let select = e.currentTarget;
			let selectedIndex = select.selectedIndex;

			this.$bulkSelect.not(select).prop('selectedIndex', selectedIndex);
		}

		onBulkApplyClick(e) {
			let $mailIds = this.$mailqueue.find('[data-alsp=item-check]:checked');
			if (!$mailIds.length) return;

			let mail_ids = $.makeArray($mailIds).map(el => el.value);
			let action = this.$bulkSelect.val();

			this.progress(this.$bulkSelect, 'loading');
			ajax('almightysupport/v1/admin_mailqueue', { action, mail_ids }, 'POST')
				.fail(() => this.progress(this.$bulkSelect, 'fail'))
				.done(json => {
					if (json.success) {
						json.message && dialog.alert(json.message);
						this.progress(this.$bulkSelect, 'done', () => this.loadMailqueue());
					} else {
						this.progress(this.$bulkSelect, 'fail');
					}
				});
		}
	};
});

jQuery(function ($) {
	let $container = $('[data-alsp=admin-page][data-id=mailqueue]');
	if ($container.length) {
		new (alsp.class('ALSPAdminMailqueue'))($container.get(0));
	}
});
