alsp.class('ALSPAbstract', function (name, alsp, $) {
	const { uuid, hooks, template, data: config } = alsp;

	return class {
		constructor(container, html) {
			this.uuid = uuid();

			let $container = $(container);

			// Replace element if necessary.
			try {
				if (!html) {
					let tmpl = $container.data('tmpl');
					if (tmpl) tmpl = config(`templates.${tmpl}`);
					if (tmpl) html = template(tmpl)();
				}

				if (html) {
					let $replacement = $(html);
					$container.replaceWith($replacement);
					$container = $replacement;
				}
			} catch (error) {}

			this.$container = $container;
			this.$container.on('click change input', '[data-alsp]', e => {
				this.triggerEventListener(e);
			});

			hooks.addFilter('section_match', this.onSectionMatch.bind(this));
			hooks.addAction('toggle_section', this.onToggleSection.bind(this));
		}

		postinit() {
			hooks.doAction('init', this.name, this.$container.get(0), this.uuid);
		}

		progress(el, type, onComplete) {
			let $config = $(el).closest('[data-config]');
			let $inputs = $config.find(
				'[type=text], [type=number], [type=password], [type=checkbox], select, textarea'
			);
			let $buttons = $config.find('[data-btn=cancel], [data-btn=save]');
			let $loading = $config.find('[data-loading]');
			let $check = $config.find('[data-check]');

			if (type === 'loading') {
				$config.data('force-loading', 1).css('opacity', 0.5);
				$inputs.attr('disabled', 'disabled');
				$buttons.attr('disabled', 'disabled');
				$loading.stop().show();
				$check.stop().hide();

				// Force loading progress for current config to be shown for at least 1 second.
				setTimeout(() => {
					$config.removeData('force-loading');
					let queue = $config.data('queue');
					if (queue) {
						$config.removeData('queue');
						this.progress($config, queue.type, queue.onComplete);
					}
				}, 1000);
			} else if ($config.data('force-loading')) {
				$config.data('queue', { type, onComplete });
			} else if (type === 'done') {
				$config.css('opacity', '');
				$inputs.removeAttr('disabled');
				$buttons.removeAttr('disabled');
				$loading.stop().hide();
				$check.stop().show().delay(1000).fadeOut();
				if ('function' === typeof onComplete) {
					onComplete();
				}
			} else if (type === 'fail') {
				$config.css('opacity', '');
				$inputs.removeAttr('disabled');
				$buttons.removeAttr('disabled');
				$loading.stop().hide();
				$check.stop().hide();
				if ('function' === typeof onComplete) {
					onComplete();
				}
			}
		}

		triggerEventListener(e) {
			let elementId = e.currentTarget.getAttribute('data-alsp');
			let eventListener = `on${capitalize(elementId + '-' + e.type)}`;

			if ('function' === typeof this[eventListener]) {
				this[eventListener](e);
			}
		}

		getNestedTemplate(name) {
			this.__tmplNested = this.__tmplNested || {};

			if ('undefined' === typeof this.__tmplNested[name]) {
				let $template = this.$container.find(`[data-template="${name}"]`).eq(0);
				this.__tmplNested[name] = $template.text() || '';
				$template.remove();
			}

			return this.__tmplNested[name];
		}

		onSectionMatch(section) {
			return section;
		}

		onToggleSection() {}
	};

	function capitalize(str) {
		return str.replace(/(?:^|-)([a-z])/gi, function (match, letter) {
			return letter.toUpperCase();
		});
	}
});
