alsp.class('ALSPGuest', function (name, alsp, $) {
	const { ajax, hooks, template, data: config, dialog, recaptcha } = alsp;

	const USER_ID = +config('user_id');
	const RECAPTCHA_ENABLE = +config('recaptcha_enabled') && !USER_ID;

	return class extends alsp.class('ALSPAbstract') {
		constructor(container) {
			super(container);

			this.name = name;

			this.$container.append(template(config('templates.guest'))());
			this.$el = this.$container.find('[data-alsp=guest]');
			this.$toggle = this.$el.find('[data-alsp=guest-toggle]');
			this.$menu = this.$el.find('[data-alsp=guest-menu]');
			this.$tabs = this.$el.find('[data-alsp=guest-tab]');

			// Login
			this.$loginEmail = this.$container.find('[data-alsp=login-email]');
			this.$loginPasswd = this.$container.find('[data-alsp=login-passwd]');
			this.$loginTfa = this.$container.find('[data-alsp=login-tfa]');
			this.$loginRemember = this.$container.find('[data-alsp=login-rememberme]');
			this.$loginNonce = this.$container.find('[data-alsp=login-nonce]');
			this.$loginSubmit = this.$container.find('[data-alsp=login-submit]');
			this.$loginError = this.$container.find('[data-alsp=login-error]');
			this.$loginErrorMsg = this.$container.find('[data-alsp=login-error-msg]');

			// Register
			this.$registerFirstName = this.$container.find('[data-alsp=register-first-name]');
			this.$registerLastName = this.$container.find('[data-alsp=register-last-name]');
			this.$registerEmail = this.$container.find('[data-alsp=register-email]');
			this.$registerPasswd = this.$container.find('[data-alsp=register-passwd]');
			this.$registerPasswdVerify = this.$container.find('[data-alsp=register-passwd-verify]');
			this.$registerNonce = this.$container.find('[data-alsp=register-nonce]');
			this.$registerSubmit = this.$container.find('[data-alsp=register-submit]');
			this.$registerError = this.$container.find('[data-alsp=register-error]');
			this.$registerErrorMsg = this.$container.find('[data-alsp=register-error-msg]');

			// Guest Submission
			this.$guestEmail = this.$container.find('[data-alsp=guest-email]');
			this.$guestPasswd = this.$container.find('[data-alsp=guest-passwd]');
			this.$guestTfa = this.$container.find('[data-alsp=guest-tfa]');
			this.$guestRemember = this.$container.find('[data-alsp=guest-rememberme]');
			this.$guestNonce = this.$container.find('[data-alsp=guest-nonce]');
			this.$guestSubmit = this.$container.find('[data-alsp=guest-submit]');
			this.$guestError = this.$container.find('[data-alsp=guest-error]');
			this.$guestErrorMsg = this.$container.find('[data-alsp=guest-error-msg]');

			// Open first tab.
			this.toggleTab(this.$tabs.eq(0).data('tab'));

			this.postinit();
		}

		toggleTab(tab) {
			let $menu = this.$toggle.filter(`[data-tab="${tab}"]`);
			this.$toggle.not($menu).removeClass('active alsp-active');
			$menu.addClass('active alsp-active');

			let $tab = this.$tabs.filter(`[data-tab="${tab}"]`);
			this.$tabs.not($tab).hide();
			$tab.show();

			if ('login-as-guest' === tab) {
				this.updateGuestMode('guest');
				this.$guestEmail.off('blur').on('blur', e => this.onGuestEmailBlur(e));
			}
		}

		submitLogin(args = {}) {
			let email = this.$loginEmail.val().trim();
			let passwd = this.$loginPasswd.val().trim();
			let rememberme = this.$loginRemember.is(':checked') ? 1 : 0;
			let tfa = this.$loginTfa.length ? this.$loginTfa.val().trim() : undefined;
			let nonce = this.$loginNonce.val().trim();
			let params = { email, passwd, rememberme, two_factor_code: tfa, nonce };

			if (RECAPTCHA_ENABLE) {
				params.recaptcha = args.recaptcha;
			}

			this.$loginError.hide();

			loading(this.$loginSubmit, true);

			this.submitLoginTwoFactorAuth(email)
				.fail(() => loading(this.$loginSubmit, false))
				.done(() => {
					ajax('almightysupport/v1/login', params, 'POST')
						.always(() => loading(this.$loginSubmit, false))
						.done(json => {
							if (json.success) {
								window.location.reload();
							} else if (json.message) {
								this.$loginEmail.closest('[data-alsp=form-field]').show();
								this.$loginPasswd.val('').closest('[data-alsp=form-field]').show();
								this.$loginRemember.closest('[data-alsp=form-field]').show();
								this.$loginTfa.val('').closest('[data-alsp=form-field]').hide();
								this.$loginErrorMsg.html(json.message);
								this.$loginError.show();
							}
						});
				});
		}

		submitLoginTwoFactorAuth(email) {
			let params = { action: 'simbatfa-init-otp', user: email };

			return $.Deferred(defer => {
				if (!this.$loginTfa.length || this.$loginTfa.is(':visible')) {
					return defer.resolve();
				}

				ajax(config('ajax_url'), params, 'POST')
					.fail(defer.resolve)
					.done(json => {
						if (json && json.status === true) {
							this.$loginEmail.closest('[data-alsp=form-field]').hide();
							this.$loginPasswd.closest('[data-alsp=form-field]').hide();
							this.$loginRemember.closest('[data-alsp=form-field]').hide();
							this.$loginTfa.closest('[data-alsp=form-field]').show();
							this.$loginTfa.focus();
							this.$loginError.hide();
							defer.reject();
						} else {
							defer.resolve();
						}
					});
			});
		}

		submitRegister() {
			let firstName = this.$registerFirstName.val().trim();
			let lastName = this.$registerLastName.val().trim();
			let email = this.$registerEmail.val().trim();
			let passwd = this.$registerPasswd.val().trim();
			let passwdVerify = this.$registerPasswdVerify.val().trim();
			let nonce = this.$registerNonce.val().trim();

			let params = {
				email,
				passwd,
				passwd_verify: passwdVerify,
				first_name: firstName,
				last_name: lastName,
				nonce
			};

			this.$registerError.hide();

			loading(this.$registerSubmit, true);

			ajax('almightysupport/v1/register', params, 'POST')
				.always(() => loading(this.$registerSubmit, false))
				.done(json => {
					if (json.id && json.user_email) {
						this.$registerFirstName.val('');
						this.$registerLastName.val('');
						this.$registerEmail.val('');
						this.$registerPasswd.val('');
						this.$registerPasswdVerify.val('');
						this.$registerError.hide();

						let message = `User ${json.display_name || json.user_login} (${
							json.user_email
						}) has been created. Please login to continue.`;
						dialog.alert(message);

						this.toggleTab('login');
					} else if (json.message) {
						this.$registerPasswd.val('');
						this.$registerPasswdVerify.val('');
						this.$registerErrorMsg.html(json.message);
						this.$registerError.show();
					}
				});
		}

		updateGuestMode(mode = 'guest') {
			let $fieldPasswd = this.$guestPasswd.closest('[data-alsp=form-field]');
			let $fieldTfa = this.$guestTfa.closest('[data-alsp=form-field]');
			let $fieldRemember = this.$guestRemember.closest('[data-alsp=form-field]');

			if ('login' === mode) {
				$fieldPasswd.show();
				$fieldTfa.hide();
				$fieldRemember.show();
				this.$guestSubmit.html(this.$guestSubmit.data('text-login'));
			} else if ('login-tfa' === mode) {
				$fieldPasswd.hide();
				$fieldTfa.show();
				$fieldRemember.hide();
				this.$guestSubmit.html(this.$guestSubmit.data('text-login'));
			} else {
				$fieldPasswd.hide();
				$fieldTfa.hide();
				$fieldRemember.hide();
				this.$guestSubmit.html(this.$guestSubmit.data('text-continue'));
			}
		}

		checkGuestEmail() {
			return $.Deferred(defer => {
				let email = this.$guestEmail.val().trim();
				let checkedEmail = this.$guestEmail.data('checked-email');
				if (checkedEmail && checkedEmail === email) {
					defer.resolve(!!+this.$guestEmail.data('checked-email-registered'));
					return;
				}

				let $loading = this.$guestEmail.siblings('[data-loading]').show();

				this.$guestPasswd.attr('disabled', 'disabled');
				this.$guestTfa.attr('disabled', 'disabled');
				this.$guestRemember.attr('disabled', 'disabled');
				this.$guestError.hide();

				this.checkGuestEmailXhr && this.checkGuestEmailXhr.abort();
				this.checkGuestEmailXhr = ajax(`almightysupport/v1/client_check`, { email })
					.always(() => $loading.hide())
					.fail(defer.reject)
					.done(json => {
						if (json.success) {
							this.$guestEmail.data('checked-email', email);
							this.$guestEmail.data('checked-email-registered', +json.is_registered);

							if (+json.is_registered) {
								this.updateGuestMode('login');
								this.$guestPasswd.removeAttr('disabled').val('');
								this.$guestTfa.removeAttr('disabled').val('');
								this.$guestRemember.removeAttr('disabled');
								defer.resolve(true);
							} else {
								this.updateGuestMode('guest');
								defer.resolve(false);
							}
						} else {
							defer.reject(json);
						}
					});
			});
		}

		submitGuest(args = {}) {
			let email = this.$guestEmail.val().trim();
			let params = { email };

			if (RECAPTCHA_ENABLE) {
				params.recaptcha = args.recaptcha;
			}

			let prevMode =
				this.$guestPasswd.is(':visible') || this.$guestTfa.is(':visible')
					? 'login'
					: 'guest';

			this.$guestError.hide();

			loading(this.$guestSubmit, true);

			this.checkGuestEmail()
				.fail(json => {
					loading(this.$guestSubmit, false);

					if (json && json.message) {
						this.$guestErrorMsg.html(json.message);
						this.$guestError.show();
					}
				})
				.done(registered => {
					if (registered) {
						this.submitGuestLogin(prevMode, args);
					} else {
						ajax(`almightysupport/v1/client_check`, params, 'POST')
							.fail(() => loading(this.$guestSubmit, false))
							.done(json => {
								if (json.success) {
									let url = document.location.pathname + '#new';
									history.replaceState(null, null, url);
									setTimeout(() => window.location.reload(), 1);
								} else if (json.message) {
									loading(this.$guestSubmit, false);
									this.$guestErrorMsg.html(json.message);
									this.$guestError.show();
								}
							});
					}
				});
		}

		submitGuestLogin(prevMode, args = {}) {
			let email = this.$guestEmail.val().trim();
			let passwd = this.$guestPasswd.val().trim();
			let rememberme = this.$guestRemember.is(':checked') ? 1 : 0;
			let tfa = this.$guestTfa.length ? this.$guestTfa.val().trim() : undefined;
			let nonce = this.$guestNonce.val().trim();
			let params = { email, passwd, rememberme, two_factor_code: tfa, nonce };

			if (RECAPTCHA_ENABLE) {
				params.recaptcha = args.recaptcha;
			}

			this.$guestError.hide();

			loading(this.$guestSubmit, true);

			this.submitGuestLoginTwoFactorAuth(email, passwd)
				.fail(() => loading(this.$guestSubmit, false))
				.done(() => {
					ajax('almightysupport/v1/login', params, 'POST')
						.always(() => loading(this.$guestSubmit, false))
						.done(json => {
							if (json.success) {
								let url = document.location.pathname + '#new';
								history.replaceState(null, null, url);
								setTimeout(() => window.location.reload(), 1);
							} else if (json.message) {
								this.updateGuestMode('login');
								if (prevMode === 'login') {
									this.$guestErrorMsg.html(json.message);
									this.$guestError.show();
								}
							}
						});
				});
		}

		submitGuestLoginTwoFactorAuth(email, passwd) {
			let params = { action: 'simbatfa-init-otp', user: email };

			return $.Deferred(defer => {
				if (!(email && passwd)) return defer.resolve();
				if (!this.$guestTfa.length || this.$guestTfa.is(':visible')) return defer.resolve();

				ajax(config('ajax_url'), params, 'POST')
					.fail(defer.resolve)
					.done(json => {
						if (json && json.status === true) {
							this.updateGuestMode('login-tfa');
							this.$guestTfa.focus();
							this.$guestError.hide();
							defer.reject();
						} else {
							defer.resolve();
						}
					});
			});
		}

		onGuestToggleClick(e) {
			e.preventDefault();

			let tab = $(e.currentTarget).data('tab');

			this.toggleTab(tab);
		}

		onLoginSubmitClick(e) {
			e.preventDefault();
			e.stopPropagation();

			if (RECAPTCHA_ENABLE) {
				recaptcha.getToken(this.$el).then(recaptcha => {
					this.submitLogin({ recaptcha });
				});
			} else {
				this.submitLogin();
			}
		}

		onRegisterSubmitClick(e) {
			e.preventDefault();
			e.stopPropagation();

			this.submitRegister();
		}

		onGuestEmailBlur(e) {
			e.preventDefault();
			e.stopPropagation();

			this.checkGuestEmail();
		}

		onGuestSubmitClick(e) {
			e.preventDefault();
			e.stopPropagation();

			if (RECAPTCHA_ENABLE) {
				recaptcha.getToken(this.$el).then(recaptcha => {
					this.submitGuest({ recaptcha });
				});
			} else {
				this.submitGuest();
			}
		}

		onSectionMatch(section, parts) {
			if (parts[0] === 'form' && USER_ID) {
				history.replaceState(null, null, document.location.pathname + '#new');
				section = ['new-ticket'];
			}

			return section;
		}

		onToggleSection(section) {
			if ('guest' === section[0]) {
				this.$el.show();
				if (RECAPTCHA_ENABLE && !this.recaptchaInitialized) {
					this.recaptchaInitialized = true;
					recaptcha.init(this.$el);
				}
			} else {
				this.$el.hide();
			}
		}
	};

	function loading(btn, show) {
		btn = $(btn).get(0);
		show = 'undefined' === typeof show ? true : !!+show;

		if (show) {
			btn.disabled = true;
			btn.classList.add('alsp-btn--loading');

			// Loading indicator should be visible at least for 1 second.
			clearTimeout(btn.loadingTimer);
			btn.loadingTimer = setTimeout(() => {
				delete btn.loadingTimer;
				if (btn.loadingDone) {
					delete btn.loadingDone;
					loading(btn, false);
				}
			}, 500);
		} else if (btn.loadingTimer) {
			btn.loadingDone = true;
		} else {
			btn.disabled = false;
			btn.classList.remove('alsp-btn--loading');
		}
	}
});
