alsp.class('ALSPPreferences', function (name, alsp, $) {
	const { ajax, hooks, data: config, dialog } = alsp;

	const USER_ID = +config('user_id');

	// Do not initialize on guest access.
	if (!USER_ID) return;

	return class extends alsp.class('ALSPAbstract') {
		constructor(container) {
			container = container.appendChild(document.createElement('div'));
			super(container, config('templates.preferences'));

			ajax('almightysupport/v1/account').done(data => {
				this.initAccountName(data);
				this.initAccountEmail(data);
				this.initAccountPassword(data);
			});

			ajax('almightysupport/v1/preferences').done(data => {
				this.initPreferences(data);
			});
		}

		initAccountName(json) {
			let data = Object.assign({}, json || {});
			let $container = this.$container.find('[data-alsp=account-name]');
			let $view = $container.find('[data-alsp=view]');
			let $edit = $container.find('[data-alsp=edit]');
			let $displayName = $container.find('[data-alsp=display-name]').html(data.display_name);
			let $firstName = $container.find('[name=first_name]').val(data.first_name);
			let $lastName = $container.find('[name=last_name]').val(data.last_name);
			let $btnEdit = $container.find('[data-alsp=btn-edit]');
			let $btnCancel = $container.find('[data-alsp=btn-cancel]');
			let $btnSave = $container.find('[data-alsp=btn-save]');

			$btnEdit.on('click', () => {
				$btnSave.hide();
				$view.hide();
				$edit.show();
				$firstName.focus().select();
			});

			$firstName.add($lastName).on('input', e => {
				let firstName = $firstName.val().trim();
				let lastName = $lastName.val().trim();
				let unchanged = firstName === data.first_name && lastName === data.last_name;

				if (unchanged) {
					$btnSave.hide();
					$container.find('[data-alsp=notice]').hide();
				} else if (firstName.length > 1 && lastName.length > 1) {
					$btnSave.show();
					$container.find('[data-alsp=notice]').hide();
				} else {
					$btnSave.hide();

					if ($firstName.is(e.currentTarget)) {
						let $wrapper = $firstName.closest('[data-alsp=input-wrapper]');
						let $notice = $wrapper.next('[data-alsp=notice]');
						firstName.length < 2 ? $notice.show() : $notice.hide();
					} else if ($lastName.is(e.currentTarget)) {
						let $wrapper = $lastName.closest('[data-alsp=input-wrapper]');
						let $notice = $wrapper.next('[data-alsp=notice]');
						lastName.length < 2 ? $notice.show() : $notice.hide();
					}
				}
			});

			$btnCancel.on('click', () => {
				$firstName.val(data.first_name);
				$lastName.val(data.last_name);
				$container.find('[data-alsp=notice]').hide();
				$edit.hide();
				$view.show();
			});

			$btnSave.on('click', () => {
				let firstName = $firstName.val().trim();
				let lastName = $lastName.val().trim();
				let params = { action: 'change_name', first_name: firstName, last_name: lastName };

				this.progress($container, 'loading');
				ajax('almightysupport/v1/account', params, 'POST').done(json => {
					if (json.success) {
						this.progress($container, 'done', () => ($edit.hide(), $view.show()));
						data.display_name = `${firstName} ${lastName}`;
						data.first_name = firstName;
						data.last_name = lastName;
						$displayName.html(data.display_name);
					} else if (json.error) {
						this.progress($container, 'fail');
						dialog.alert(json.message || 'Undefined error.');
					}
				});
			});
		}

		initAccountEmail(json) {
			let data = Object.assign({}, json || {});
			let $container = this.$container.find('[data-alsp=account-email]');
			let $view = $container.find('[data-alsp=view]');
			let $edit = $container.find('[data-alsp=edit]');
			let $displayEmail = $container.find('[data-alsp=email]').html(data.email);
			let $email = $container.find('[name=email]').val(data.email);
			let $emailConfirm = $container.find('[name=email_confirm]').val('');
			let $oldPassword = $container.find('[name=old_password]').val('');
			let $btnEdit = $container.find('[data-alsp=btn-edit]');
			let $btnCancel = $container.find('[data-alsp=btn-cancel]');
			let $btnSave = $container.find('[data-alsp=btn-save]');

			$btnEdit.on('click', () => {
				$btnSave.hide();
				$view.hide();
				$edit.show();
				$oldPassword.focus().select();
			});

			$oldPassword
				.add($email)
				.add($emailConfirm)
				.on('input', e => {
					let password = $oldPassword.val();
					let email = $email.val().trim();
					let emailConfirm = $emailConfirm.val().trim();
					let reEmail = /^\S+@\S+\.\S+$/;

					if (password.length > 4 && email.match(reEmail) && email === emailConfirm) {
						$btnSave.show();
						$container.find('[data-alsp=notice]').hide();
					} else {
						$btnSave.hide();

						if ($oldPassword.is(e.currentTarget)) {
							let $wrapper = $oldPassword.closest('[data-alsp=input-wrapper]');
							let $notice = $wrapper.next('[data-alsp=notice]');
							password.length < 5 ? $notice.show() : $notice.hide();
						} else {
							let $wrapper = $email.closest('[data-alsp=input-wrapper]');
							let $notice = $wrapper.next('[data-alsp=notice]');
							!email.match(reEmail) ? $notice.show() : $notice.hide();
							$wrapper = $emailConfirm.closest('[data-alsp=input-wrapper]');
							$notice = $wrapper.next('[data-alsp=notice]');
							email !== emailConfirm ? $notice.show() : $notice.hide();
						}
					}
				});

			$btnCancel.on('click', () => {
				$email.val(data.email);
				$emailConfirm.add($oldPassword).val('');
				$container.find('[data-alsp=notice]').hide();
				$edit.hide();
				$view.show();
			});

			$btnSave.on('click', () => {
				let email = $email.val().trim();
				let emailConfirm = $emailConfirm.val().trim();
				let password = $oldPassword.val();
				let params = { action: 'change_email', email, old_password: password };

				if (!email.match(/^\S+@\S+\.\S+$/) || email !== emailConfirm || !password) {
					return;
				}

				this.progress($container, 'loading');
				ajax('almightysupport/v1/account', params, 'POST').done(json => {
					if (json.success) {
						this.progress($container, 'done', () => ($edit.hide(), $view.show()));
						data.email = email;
						$displayEmail.html(data.email);
					} else if (json.error) {
						this.progress($container, 'fail');
						dialog.alert(json.message || 'Undefined error.');
					}
				});
			});
		}

		initAccountPassword(json) {
			let data = Object.assign({}, json || {});
			let $container = this.$container.find('[data-alsp=account-password]');
			let $view = $container.find('[data-alsp=view]');
			let $edit = $container.find('[data-alsp=edit]');
			let $oldPassword = $container.find('[name=old_password]').val('');
			let $newPassword1 = $container.find('[name=new_password_1]').val('');
			let $newPassword2 = $container.find('[name=new_password_2]').val('');
			let $btnEdit = $container.find('[data-alsp=btn-edit]');
			let $btnCancel = $container.find('[data-alsp=btn-cancel]');
			let $btnSave = $container.find('[data-alsp=btn-save]');

			$btnEdit.on('click', () => {
				$btnSave.hide();
				$view.hide();
				$edit.show();
				$oldPassword.focus();
			});

			$oldPassword
				.add($newPassword1)
				.add($newPassword2)
				.on('input', e => {
					let old = $oldPassword.val();
					let new1 = $newPassword1.val();
					let new2 = $newPassword2.val();

					if (old.length > 4 && new1.length > 4 && new1 === new2) {
						$btnSave.show();
						$container.find('[data-alsp=notice]').hide();
					} else {
						$btnSave.hide();

						if ($oldPassword.is(e.currentTarget)) {
							let $wrapper = $oldPassword.closest('[data-alsp=input-wrapper]');
							let $notice = $wrapper.next('[data-alsp=notice]');
							old.length < 5 ? $notice.show() : $notice.hide();
						} else {
							let $wrapper = $newPassword1.closest('[data-alsp=input-wrapper]');
							let $notice = $wrapper.next('[data-alsp=notice]');
							new1.length < 5 ? $notice.show() : $notice.hide();
							$wrapper = $newPassword2.closest('[data-alsp=input-wrapper]');
							$notice = $wrapper.next('[data-alsp=notice]');
							new1 !== new2 ? $notice.show() : $notice.hide();
						}
					}
				});

			$btnCancel.on('click', () => {
				$oldPassword.add($newPassword1).add($newPassword2).val('');
				$container.find('[data-alsp=notice]').hide();
				$edit.hide();
				$view.show();
			});

			$btnSave.on('click', () => {
				let oldPassword = $oldPassword.val();
				let newPassword1 = $newPassword1.val();
				let newPassword2 = $newPassword2.val();
				let params = {
					action: 'change_password',
					old_password: oldPassword,
					new_password_1: newPassword1,
					new_password_2: newPassword2
				};

				if (!oldPassword || !newPassword1 || newPassword1 !== newPassword2) {
					return;
				}

				this.progress($container, 'loading');
				ajax('almightysupport/v1/account', params, 'POST').done(json => {
					if (json.success) {
						this.progress($container, 'done', () => location.reload());
					} else if (json.error) {
						this.progress($container, 'fail');
						dialog.alert(json.message || 'Undefined error.');
					}
				});
			});
		}

		initPreferences(json) {
			let data = Object.assign({}, json || {});
			let $preferences = this.$container.find('[data-alsp=preference-item]');

			$preferences.each((i, el) => {
				let $pref = $(el);
				let $input = $pref.find('[data-alsp|=pref]');
				let id = $input.attr('name');

				if ('undefined' !== typeof data[id]) {
					if ($input.is('textarea')) {
						data[id] = (data[id] || '').replace(/\r\n/g, '\n');
						$input.val(data[id]).data('value', data[id]).alsp_autosize();
						$pref.find('[data-alsp=preview]').html(data[id].replace(/\n/g, '<br>'));
					} else if ($input.is(':checkbox')) {
						$input[0].checked = !!+data[id];
					}
				}
			});

			$preferences.on('input', '[data-alsp=pref-textarea]', e => {
				let $textarea = $(e.currentTarget);
				let $pref = $textarea.closest('[data-config]');
				let $btns = $pref.find('[data-alsp=btn-save], [data-alsp=btn-cancel]');

				$textarea.val() !== $textarea.data('value') ? $btns.show() : $btns.hide();
			});

			$preferences.on('change', '[data-alsp=pref-checkbox]', e => {
				let $checkbox = $(e.currentTarget);
				let id = $checkbox.attr('name');
				let value = +$checkbox[0].checked;

				this.progress($checkbox, 'loading');
				ajax('almightysupport/v1/preferences', { id, value }, 'POST').done(json => {
					if (json.success) {
						this.progress($checkbox, 'done');
						hooks.doAction('preferences_updated', USER_ID, { [id]: value });
					} else if (json.error) {
						this.progress($checkbox, 'fail');
						dialog.alert(json.message || 'Undefined error.');
						$checkbox.get(0).checked = !value;
					}
				});
			});

			$preferences.on('click', '[data-alsp=btn-cancel]', e => {
				let $pref = $(e.currentTarget).closest('[data-config]');
				let $textarea = $pref.find('textarea');
				let $btns = $pref.find('[data-alsp=btn-save], [data-alsp=btn-cancel]');

				$textarea.val($textarea.data('value'));
				$btns.hide();
			});

			$preferences.on('click', '[data-alsp=btn-save]', e => {
				let $pref = $(e.currentTarget).closest('[data-config]');
				let $textarea = $pref.find('textarea');
				let id = $textarea.attr('name');
				let value = $textarea.val();

				this.progress($textarea, 'loading');
				ajax('almightysupport/v1/preferences', { id, value }, 'POST').done(json => {
					if (json.success) {
						this.progress($textarea, 'done', () => {
							$textarea.data('value', value).trigger('input');
						});
						hooks.doAction('preferences_updated', USER_ID, { [id]: value });
					} else if (json.error) {
						this.progress($textarea, 'fail');
						dialog.alert(json.message || 'Undefined error.');
					}
				});
			});
		}

		onSectionMatch(section, parts) {
			if (parts[0] === 'preferences') {
				section = ['preferences', parts[1]];
			}

			return section;
		}

		onToggleSection(section) {
			if ('preferences' !== section[0]) {
				this.$container.hide();
				return;
			}

			this.$container.show();
		}
	};
});
