alsp.class('ALSPTicketAttachment', function (name, alsp, $) {
	const { hooks, template, data: config } = alsp;

	const REST_URL = window.wpApiSettings.root;
	const REST_NONCE = window.wpApiSettings.nonce;

	const USER_ID = +config('user_id');

	// class initialization hook
	hooks.addAction('init', (name, container, uuid) => {
		if ('ALSPTicketReply' === name) {
			new (alsp.class('ALSPTicketAttachment'))(container, uuid, false);
		} else if ('ALSPNewTicket' === name) {
			new (alsp.class('ALSPTicketAttachment'))(container, uuid, true);
		} else if ('ALSPGuest' === name) {
			container = $(container).find('[data-alsp=guest-tab][data-tab=continue-as-guest]')[0];
			new (alsp.class('ALSPTicketAttachment'))(container, uuid, true);
		}
	});

	return class {
		constructor(container, uuid, newTicketForm) {
			this.uuid = uuid;
			this.newTicketForm = newTicketForm;

			this.$container = $(container);
			this.$button = this.$container.find('[data-alsp=attachment-button]');
			this.$file = this.$button.find('[type=file]');
			this.$attachments = this.$container.find('[data-alsp=attachments]');
			this.$list = this.$container.find('[data-alsp=attachments-list]');

			this.itemTemplate = this.$container.find('[data-tmpl=attachment-item]').text();
			this.itemTemplate = template(this.itemTemplate);

			this.comboButton = this.$list.find(this.$button).length > 0;

			this.initUpload(this.$file);

			this.$list.on('click', '[data-alsp=remove]', e => this.onRemoveClick(e));

			hooks.addAction('ticket_loaded', ticket => this.reset(ticket));
			hooks.addFilter('ticket_message_params', this.filterMessageParams.bind(this));
			hooks.addAction('toggle_section', this.onToggleSection.bind(this));
		}

		reset(ticket) {
			this.ticket = ticket;
			this.$attachments.hide();
			this.$list.children().not(this.$button).remove();

			let status = ticket.status && ticket.status.status;
			if ('closed' === status && !this.newTicketForm) {
				this.$button.hide();
			} else {
				this.$button.show();
				this.initUpload(this.$file);
			}

			let lockedBy = +this.ticket.lock ? this.ticket.lock_agent : null;
			if (lockedBy && +lockedBy.id === USER_ID) {
				lockedBy = null;
			}

			if (lockedBy && !this.newTicketForm) this.lock();
			else this.unlock();
		}

		lock() {
			this.locked = true;
			this.$file.add(this.$button).attr('disabled', 'disabled');
		}

		unlock() {
			this.locked = false;
			this.$file.add(this.$button).removeAttr('disabled');
		}

		initUpload($file) {
			$file.fileupload({
				url: `${REST_URL}almightysupport/v1/pro_ticket_attachments`,
				dataType: 'json',
				beforeSend: xhr => xhr.setRequestHeader('X-WP-Nonce', REST_NONCE),
				add: (e, data) => {
					let id = (data.id = generateRandomId());
					let file = data.files[0];
					let name = (data.name = file.name);
					let html = this.itemTemplate({ id, name, loading: 1 });

					this.comboButton ? this.$button.before(html) : this.$list.append(html);
					this.$attachments.show();
					data.submit();
				},
				done: (e, data) => {
					let json = (data && data.result) || {};
					let $item = this.$list.children(`[data-id=${data.id}]`);

					if (json.success) {
						let name = json.data;
						let html = this.itemTemplate({ id: data.id, name });
						$item.replaceWith(html);
					} else {
						let name = data.name;
						let error = json.message || true;
						let html = this.itemTemplate({ id: data.id, name, error });
						$item.replaceWith(html);
					}
				}
			});
		}

		filterMessageParams(params, uuid) {
			if (uuid === this.uuid) {
				let files = this.$list.children('[data-id]').not('.error').toArray();
				params.files = files.map(el => el.getAttribute('data-name'));
			}

			return params;
		}

		onToggleSection(section) {
			if ('new-ticket' === section[0]) {
				this.$attachments.hide();
				this.$list.children().not(this.$button).remove();
			}
		}

		onRemoveClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let itemSelector = '[data-alsp=attachment-item]';

			$(e.target).closest(itemSelector).remove();
			if (!this.$list.children(itemSelector).length) {
				this.$attachments.hide();
			}
		}
	};

	function generateRandomId() {
		let ts = new Date().getTime();
		let rd = 1000 + Math.floor(Math.random() * 1000);

		return `${ts}-${rd}`;
	}
});
