alsp.class('ALSPTicketSidebar', function (name, alsp, $) {
	const { ajax, hooks, template, data: config } = alsp;

	return class extends alsp.class('ALSPAbstract') {
		constructor(sidebar) {
			super(sidebar);

			this.$toggle = $('[data-alsp=ticket-sidebar-toggle]');
			this.$sidebar = $(sidebar);
			if (!this.$sidebar.length) {
				return;
			}
			this.name = name;

			this.ticket = null;
			this.client_id = null;
			this.ticket_id = null;
			this.mode = null;

			this.$toggle.on('click', e => this.onToggleButtonClick(e));

			hooks.addAction('ticket_sidebar_toggle', mode => this.onToggleSidebar(mode));
			hooks.addAction('ticket_loaded', ticket => {
				if (ticket.client && ticket.client.id) {
					this.update(ticket);
					this.client_id = ticket.client.id;
					this.ticket_id = ticket.id;
				}
			});

			this.postinit();
		}

		update(ticket) {
			this.ticket = ticket;

			this.$sidebar.css('opacity', 0.4);

			return ajax('almightysupport/v1/ticket_author', { id: ticket.client.id }).done(json => {
				if (json.success && json.data) {
					this.render(json.data);
					hooks.doAction('ticket_sidebar_loaded', ticket);
				}
			});
		}

		render(data) {
			let $sidebar = $(template(config('templates.ticket/sidebar'))(data));
			$sidebar.on('click', '[data-alsp=toggle]', e => this.toggle('mini'));
			this.$sidebar.replaceWith($sidebar);
			this.$sidebar = $sidebar;

			// By default is expanded on desktop and collapsed on mobile view.
			this.mode = window.innerWidth < 980 ? 'mini' : 'full';

			this.toggle(this.mode);

			this.$sidebar.on('click', '[data-alsp=copy-email]', e => this.onCopyEmail(e));
		}

		renderNotes(data) {}

		toggle(mode) {
			hooks.doAction('ticket_sidebar_toggle', mode);
		}

		onToggleSidebar(mode = 'full') {
			this.mode = mode;

			if ('mini' === mode) {
				this.$sidebar.hide();
				this.$toggle.removeClass('alsp-active');
			} else {
				this.$sidebar.show();
				this.$toggle.addClass('alsp-active');
			}
		}

		onToggleButtonClick(e) {
			e.preventDefault();
			e.stopPropagation();

			const mode = this.mode === 'full' ? 'mini' : 'full';
			this.toggle(mode);
		}

		onCopyEmail(e) {
			e.preventDefault();
			e.stopPropagation();

			let $el = $(e.currentTarget);
			alsp.copyToClipboard($el.data('email'));
			$el.addClass('alsp-tip--alt').on('mouseleave.alsp-copy', () => {
				$el.off('mouseleave.alsp-copy');
				setTimeout(() => $el.removeClass('alsp-tip--alt'), 500);
			});
		}
	};
});
