(function (alsp, $) {
	const { ajax } = alsp;
	const cache = {};

	// Endpoint shorthands.
	const endpoints = {
		tickets: 'almightysupport/v1/tickets',
		ticket_filters: 'almightysupport/v1/ticket_filter',
		ticket_filters_count: 'almightysupport/v1/tickets_count',
		statuses: 'almightysupport/v1/statuses',
		agents: 'almightysupport/v1/agents',
		departments: 'almightysupport/v1/departments',
		admin_agents: 'almightysupport/v1/admin_agent',
		admin_departments: 'almightysupport/v1/admin_department'
	};

	function get(name, params = {}, force) {
		let endpoint = endpoints[name] || name;
		let id = `${endpoint}_${JSON.stringify(params)}`;

		if (force) {
			clear(endpoint, params);
		}

		if (cache[id]) {
			let fakeXhr = $.Deferred(defer => defer.resolve(cache[id]));
			fakeXhr.abort = function () {};
			return fakeXhr;
		}

		let xhr = ajax(endpoint, params);
		let fakeXhr = $.Deferred(defer => {
			xhr.fail(defer.reject).done(result => {
				cache[id] = result;
				defer.resolve(cache[id]);
			});
		});

		fakeXhr.abort = xhr.abort;
		return fakeXhr;
	}

	function clear(name, params = {}) {
		let endpoint = endpoints[name] || name;
		let id = `${endpoint}_${JSON.stringify(params)}`;

		delete cache[id];
	}

	Object.assign(alsp, { datastore: { get, clear } });
})(window.alsp, window.jQuery);
