(function (alsp, $) {
	const { template, data: config } = alsp;

	let $dialog;
	let callback;
	let callbackCancel;
	let type;

	function _createDialog() {
		let $dialog = $(template(config('templates.dialog'))())
			.hide()
			.appendTo(document.body);

		$dialog.on('click', e => {
			e.preventDefault();
			e.stopPropagation();
			_hideDialog();
		});

		$dialog.on('click', '[data-alsp=box]', e => {
			e.preventDefault();
			e.stopPropagation();
		});

		$dialog.on('click', '[data-alsp=btn-ok]', e => {
			e.preventDefault();
			e.stopPropagation();

			if ('function' === typeof callback) {
				callback($dialog);
				callback = undefined;
			}

			_hideDialog();
		});

		$dialog.on('click', '[data-alsp=btn-cancel]', e => {
			e.preventDefault();
			e.stopPropagation();
			_hideDialog();
		});

		$dialog.on('click', '[data-alsp=btn-confirm]', e => {
			e.preventDefault();
			e.stopPropagation();

			if ('function' === typeof callback) {
				callback($dialog);
				callback = undefined;
			}

			_hideDialog();
		});

		return $dialog;
	}

	function _showAlertDialog(content, title, fn) {
		$dialog = $dialog || _createDialog();

		if ('function' === typeof title) {
			fn = title;
			title = '';
		}

		type = 'alert';
		callback = fn;
		callbackCancel = undefined;

		$dialog.find('[data-alsp=title]').html(title || 'Alert');
		$dialog.find('[data-alsp=content]').html(content);
		$dialog.find('[data-alsp=btn-ok]').show();
		$dialog.find('[data-alsp=btn-cancel]').hide();
		$dialog.find('[data-alsp=btn-confirm]').hide();
		$dialog.show();

		_attachOnEscape();
	}

	function _showConfirmDialog(content, title, fn, fnCancel) {
		$dialog = $dialog || _createDialog();

		if ('function' === typeof title) {
			fnCancel = fn;
			fn = title;
			title = '';
		}

		type = 'confirm';
		callback = fn;
		callbackCancel = fnCancel;

		$dialog.find('[data-alsp=title]').html(title || 'Confirmation');
		$dialog.find('[data-alsp=content]').html(content);
		$dialog.find('[data-alsp=btn-ok]').hide();
		$dialog.find('[data-alsp=btn-cancel]').show();
		$dialog.find('[data-alsp=btn-confirm]').show();
		$dialog.show();

		_attachOnEscape();
	}

	function _hideDialog() {
		if ('alert' === type) {
			'function' === typeof callback && callback($dialog);
		} else if ('confirm' === type) {
			'function' === typeof callbackCancel && callbackCancel($dialog);
		}

		type = undefined;
		callback = undefined;
		callbackCancel = undefined;

		$dialog && $dialog.hide();
		_detachOnEscape();
	}

	function _detachOnEscape() {
		$(document).off('keyup.alsp-dialog');
	}

	function _attachOnEscape() {
		_detachOnEscape();
		$(document).on('keyup.alsp-dialog', e => {
			if (e.key === 'Escape') {
				_hideDialog();
			}
		});
	}

	const dialog = {
		alert: _showAlertDialog,
		confirm: _showConfirmDialog,
		close: _hideDialog
	};

	Object.assign(alsp, { dialog });
})(window.alsp, window.jQuery);
