(function (alsp, alspData, $, wp = {}) {
	// Redirect to the correct ALSP page if necessary.
	function redirectALSP(data) {
		if (data[`is_wc_page`]) return;

		let isAlsp = document.querySelector('[data-alsp=root]');
		if (!isAlsp) return false; // Skip, we are not in an Awedesk page.

		let attachedPageId = (document.body.getAttribute('class') || '').match(/\bpage-id-(\d+)\b/);
		if (!attachedPageId) return false; // Skip, no attached page_id found.

		let page_id = attachedPageId[1];
		let { page_id_client, page_id_agent, is_admin, is_manager, is_agent, home_url } = data;
		let i_am_agent = +is_admin || +is_manager || +is_agent;

		// Fallback to client page if agent page is not set.
		page_id_agent = page_id_agent || page_id_client;

		if (i_am_agent) {
			if (data[`page_${page_id_agent}`]) {
				return `${data[`page_${page_id_agent}`]}${window.location.hash}`;
			} else if (+page_id_agent > 0 && +page_id_agent !== +page_id) {
				return `${home_url}/?page_id=${page_id_agent}${window.location.hash}`;
			}
		} else {
			if (data[`page_${page_id_client}`]) {
				return `${data[`page_${page_id_client}`]}${window.location.hash}`;
			} else if (+page_id_client > 0 && +page_id_client !== +page_id) {
				return `${home_url}/?page_id=${page_id_client}${window.location.hash}`;
			}
		}
	}

	function initALSP() {
		document.querySelectorAll('[data-alsp=root]').forEach(el => {
			if (!el.getAttribute('data-init')) {
				el.setAttribute('data-init', 1);
				new (alsp.class('ALSPRoot'))(el);
			}
		});
	}

	// Initialize newly-added block inside WordPress editor.
	alsp.hooks.addAction('block_added', initALSP);

	let redirectURL = redirectALSP(alspData || {});
	if (redirectURL) {
		let currentURL = window.location.href.replace(window.location.hash, '');
		if (redirectURL.replace(window.location.hash, '') !== currentURL) {
			console.log(`Redirecting Awedesk page to: ${redirectURL}`);
			window.location = redirectURL;
			return;
		}
	}

	$(function () {
		if (wp && wp.domReady) {
			wp.domReady(() => setTimeout(initALSP, 1000));
		} else {
			initALSP();
		}
	});
})(window.alsp, window.alspData, window.jQuery, window.wp);
