(function (alsp, $) {
	const { data: config } = alsp;

	const USE_GLOBAL_URL = +config('recaptcha_use_global_url');
	const RECAPTCHA_HOST = USE_GLOBAL_URL ? 'https://www.recaptcha.net' : 'https://www.google.com';
	const RECAPTCHA_URL = `${RECAPTCHA_HOST}/recaptcha/api.js`;
	const RECAPTCHA_KEY = config('recaptcha_key');

	let __getTokenResolve = {};

	function loadLibrary() {
		return new Promise(resolve => {
			let scripts = [...document.querySelectorAll('script')];
			let isLoaded = !!scripts.map(s => s.src).find(src => src.indexOf(RECAPTCHA_URL) >= 0);
			if (!isLoaded) {
				let script = document.createElement('script');
				script.type = 'text/javascript';
				script.src = `${RECAPTCHA_URL}?render=explicit`;
				document.body.appendChild(script);
			}

			// Check for library availability for a maximum of 2 minutes.
			(function check(loops = 1) {
				if (window.grecaptcha && grecaptcha.ready) grecaptcha.ready(resolve);
				else if (loops <= 240) setTimeout(() => check(++loops), 500);
			})();
		});
	}

	function init(btn) {
		loadLibrary().then(() => {
			let div = document.createElement('div');
			let $btn = $(btn).before(div);
			let id = grecaptcha.render(div, {
				sitekey: RECAPTCHA_KEY,
				size: 'invisible',
				callback(token) {
					if ('function' === typeof __getTokenResolve[id]) {
						__getTokenResolve[id](token);
						delete __getTokenResolve[id];
					}
				}
			});

			$btn.data('recaptcha-id', id);
		});
	}

	function getToken(btn) {
		return new Promise((resolve, reject) => {
			let id = $(btn).data('recaptcha-id');
			if ('number' !== typeof id) return reject();

			grecaptcha.reset(id);

			let token = grecaptcha.getResponse(id);
			if (token) return resolve(token);

			grecaptcha.execute(id);
			__getTokenResolve[id] = resolve;
		});
	}

	const recaptcha = { init, getToken };

	Object.assign(alsp, { recaptcha });
})(window.alsp, window.jQuery);
