alsp.class('ALSPAdminDepartmentEmail', function (name, alsp, $) {
	const { ajax, hooks, template, data: config } = alsp;

	// class initialization hook
	hooks.addAction('init', (name, container, uuid) => {
		if ('ALSPAdminDepartments' === name) {
			new (alsp.class('ALSPAdminDepartmentEmail'))(container, uuid);
		}
	});

	return class extends alsp.class('ALSPAdminAbstract') {
		constructor(container) {
			super(container);

			this.$container = $(container);
			this.$departments = this.$container.find('[data-alsp=departments]');

			let par = '[data-alsp=email-piping]';
			this.$departments
				.on('click', `${par} input[type=checkbox]`, e => this.onCheckboxClick(e))
				.on('input', `${par} input[type=text]`, e => this.onTextInput(e))
				.on('input', `${par} input[type=number]`, e => this.onTextInput(e))
				.on('input', `${par} input[type=password]`, e => this.onTextInput(e))
				.on('input', `${par} textarea`, e => this.onTextInput(e))
				.on('change', `${par} select`, e => this.onSelectChange(e))
				.on('click', `${par} button[data-btn=cancel]`, e => this.onCancelClick(e))
				.on('click', `${par} button[data-btn=save]`, e => this.onSaveClick(e))
				.on('click', `${par} button[data-btn=check]`, e => this.onCheckClick(e));

			hooks.addAction('admin_departments_render', () => this.init());
		}

		init() {
			let $departments = this.$departments.children('[data-alsp=department-item]');
			let xhrs = [];

			$departments.each((i, department) => {
				let $department = $(department);
				let $ep = $department.find('[data-alsp=email-piping]');
				let params = { department_id: $department.data('id') };
				let xhr = ajax('almightysupport/v1/pro_admin_department_ep', params).done(opts => {
					this.render($ep, opts);
				});

				xhrs.push(xhr);
			});

			$.when(...xhrs).done(() => this.toggleChildren({ animate: false }));
		}

		render($el, opts = {}) {
			let $fields = $();

			for (const key in opts) {
				let items = opts[key].items;
				if (items instanceof Object) {
					for (const id in items) {
						let tmpl = config(`templates.config_item_${items[id].type}`);

						if (tmpl) {
							items[id].label = items[id].title;

							let $field = $(template(tmpl)(items[id]));
							if (!id.match(/_ep_enabled$/)) $field.css('display', 'none');
							$fields = $fields.add($field);
						} else {
							console.log('Unknown admin item ' + items[id].type);
						}
					}
				}
			}

			$fields.appendTo($el);
		}

		onCheckboxClick(e) {
			e.stopPropagation();

			let $checkbox = $(e.currentTarget);
			let $config = $checkbox.closest('[data-type]');
			let name = $checkbox.attr('id');
			let value = $checkbox.is(':checked') ? 1 : 0;

			this.progress($config, 'loading');
			this.toggleChildren(name, { disable: true });
			ajax('almightysupport/v1/admin', { name, value }, 'POST')
				.done(() => {
					this.progress($config, 'done', () => {
						this.toggleChildren(name, { visible: !!value });
					});
				})
				.fail(xhr => this.progress($config, 'fail'));
		}

		onTextInput(e) {
			e.stopPropagation();

			let $input = $(e.currentTarget);
			let $config = $input.closest('[data-type]');
			let $cancel = $config.find('button[data-btn=cancel]');
			let $save = $config.find('button[data-btn=save]');
			let value = $input.val().trim();
			let previousValue = $input.attr('data-value');

			if (value === previousValue) {
				$cancel.hide();
				$save.hide();
			} else {
				$cancel.show();
				$save.show();
			}
		}

		onCancelClick(e) {
			e.stopPropagation();

			let $cancel = $(e.currentTarget);
			let $config = $cancel.closest('[data-type]');
			let $input = $config.find('[type=text], [type=number], [type=password], textarea');
			let $save = $config.find('button[data-btn=save]');

			$input.val($input.attr('data-value'));
			$cancel.hide();
			$save.hide();
		}

		onSaveClick(e) {
			e.stopPropagation();

			let $save = $(e.currentTarget);
			let $config = $save.closest('[data-type]');
			let $input = $config.find('[type=text], [type=number], [type=password], textarea');
			let $cancel = $config.find('button[data-btn=cancel]');

			let name = $input.attr('id');
			let value = $input.val().trim();

			$cancel.hide();
			$save.hide();

			this.progress($config, 'loading');
			ajax('almightysupport/v1/admin', { name, value }, 'POST')
				.done(() => {
					$input.attr('data-value', value);
					this.progress($config, 'done');
				})
				.fail(xhr => this.progress($config, 'fail'));
		}

		onCheckClick(e) {
			e.stopPropagation();

			let $check = $(e.currentTarget);
			let type = $check.data('id');
			let department_id = $check.closest('[data-alsp=department-item]').data('id');
			let endpoint = `almightysupport/v1/pro_admin_department_${type}`;
			let params = { department_id };

			ajax(endpoint, params, 'POST').done(json => {
				if (json.redirectURL) {
					window.location.href = json.redirectURL;
				} else if (json.success) {
					alert('success');
				} else if (json.error && json.message) {
					alert(json.message);
				}
			});
		}

		onSelectChange(e) {
			e.stopPropagation();

			let $select = $(e.currentTarget);
			let $config = $select.closest('[data-type]');
			let name = $select.attr('id');
			let value = $select.val();

			this.progress($config, 'loading');
			this.toggleChildren(name, { disable: true });
			ajax('almightysupport/v1/admin', { name, value }, 'POST')
				.done(() => {
					$select.attr('data-value', value);
					this.progress($config, 'done', () => {
						this.toggleChildren(name, { visible: value });
					});
				})
				.fail(xhr => this.progress($config, 'fail'));
		}
	};
});
