alsp.class('ALSPNewTicketSuggestions', function (name, alsp, $) {
	const { ajax, hooks, template, data: config, dialog } = alsp;

	// class initialization hook
	hooks.addAction('init', (initName, container) => {
		if ('ALSPNewTicket' === initName) {
			new (alsp.class(name))(container);
		}
	});

	return class extends alsp.class('ALSPAbstract') {
		constructor(container) {
			super(container);

			this.$subject = this.$container.find('[data-alsp=new-ticket-subject]');
			this.$suggestionsList = this.$container.find('[data-alsp=new-ticket-sugg-list]');
			this.$suggestionsLoading = this.$container.find('[data-alsp=new-ticket-sugg-loading]');
		}

		getSuggestions() {
			let $list = this.$suggestionsList.empty().hide();
			let $loading = this.$suggestionsLoading.show();
			let s = this.$subject.val().trim();

			if (!$list.length) return;

			this.suggXhr && this.suggXhr.abort();
			if (s.length < 5) {
				$loading.hide();
				delete this.suggToken;
				return;
			}

			// Fetch suggestion.
			let suggToken = (this.suggToken = new Date().getTime());
			setTimeout(() => {
				if (suggToken !== this.suggToken) return;
				this.suggXhr = ajax('almightysupport/v1/pro_suggestions', { s })
					.always(() => {
						$loading.hide();
						delete this.suggToken;
					})
					.done(json => {
						let cpts = (json && json.meta && json.meta.cpts) || {};
						let data = (json && json.data) || {};

						let tmplData = [];
						Object.keys(cpts).forEach(cptKey => {
							let items = data[cptKey];
							if (items instanceof Array && items.length) {
								tmplData.push({ label: cpts[cptKey].label, items });
							}
						});

						if (tmplData.length) {
							let tmpl = template(config('templates.new_ticket/suggestions'));
							let html = tmpl(tmplData);
							$list.html(html).show();
						}
					});
			}, 1000);
		}

		onNewTicketSubjectInput() {
			this.getSuggestions();
		}

		onToggleSection(section) {
			if ('new-ticket' === section[0]) {
				this.$suggestionsList.hide();
			}
		}
	};
});
