alsp.class('ALSPTicketPrivateNotes', function (name, alsp, $) {
	const { ajax, hooks, template, data: config, loading, dialog } = alsp;

	// class initialization hook
	hooks.addAction('init', (initName, container) => {
		if ('ALSPTicket' === initName) {
			container = container.querySelector('[data-alsp=ticket-secret]');
			new (alsp.class(name))(container);
		}
	});

	return class extends alsp.class('ALSPAbstract') {
		constructor(container) {
			super(container, template(config('templates.ticket/secret'))());

			this.id = null;

			this.$btnToggle = this.$container.find('[data-alsp=btn-toggle]');
			this.$notice = this.$container.find('[data-alsp=content-notice]');
			this.$viewer = this.$container.find('[data-alsp=content-viewer]');
			this.$editor = this.$container.find('[data-alsp=content-editor]');
			this.$content = this.$viewer.find('[data-alsp=content]');
			this.$btnClear = this.$viewer.find('[data-alsp=btn-clear]');
			this.$btnEdit = this.$viewer.find('[data-alsp=btn-edit]');
			this.$textarea = this.$editor.find('[data-alsp=textarea]').alsp_autosize();
			this.$btnCancel = this.$editor.find('[data-alsp=btn-cancel]');
			this.$btnSave = this.$editor.find('[data-alsp=btn-save]');
			this.$titleEmpty = this.$container.find('[data-alsp=title-empty]');

			hooks.addAction('ticket_loaded', ticket => this.reset(ticket));
		}

		toggle(show) {
			let $btn = this.$btnToggle;
			let $icon = $btn.children('i');

			if (show) {
				$btn.addClass('alsp-tip--alt');
				$icon.attr('class', $icon.data('class-visible'));
				this.$notice.add(this.$viewer).show();
			} else {
				$btn.removeClass('alsp-tip--alt');
				$icon.attr('class', $icon.data('class-hidden'));
				this.$notice.add(this.$viewer).hide();
			}
		}

		reset(ticket) {
			let sameTicket = this.id == ticket.id;

			this.id = ticket.id;
			this.update(ticket.secret);
			// this.$viewer.show();
			sameTicket || this.toggle(false);
			this.$editor.hide();
		}

		update(secret = '') {
			secret = secret.trim();
			let escapedSecred = escapeMessage(secret);

			this.$textarea.val(secret);
			this.$content.html(escapedSecred || this.$content.data('html-empty'));
			this.$btnClear[escapedSecred ? 'show' : 'hide']();
			this.$titleEmpty[escapedSecred ? 'hide' : 'show']();
		}

		onTitleClick(e) {
			e.stopPropagation();

			this.toggle(true);
		}

		onBtnToggleClick(e) {
			e.stopPropagation();

			let $btn = $(e.currentTarget);
			let visible = $btn.hasClass('alsp-tip--alt');
			this.toggle(!visible);
		}

		onBtnClearClick(e) {
			e.stopPropagation();

			dialog.confirm('Are you sure want to clear delete this private note?', () => {
				this.$btnClear.attr('disabled', 'disabled');
				loading(this.$btnEdit);

				let ticket = this.id;
				let secret = '';

				ajax('almightysupport/v1/pro_ticket_private_notes', { ticket, secret }, 'PATCH')
					.always(() => {
						this.$btnClear.removeAttr('disabled');
						loading(this.$btnEdit, false);
					})
					.done(json => {
						this.update(secret);
						hooks.doAction('ticket_update', this.id);
					});
			});
		}

		onBtnEditClick(e) {
			e.stopPropagation();

			this.$textarea.data('value', this.$textarea.val());
			this.$btnSave.attr('disabled', 'disabled');

			this.$viewer.hide();
			this.$editor.show();
			this.$btnToggle.hide();

			this.$textarea.trigger('input').focus();
		}

		onTextareaInput(e) {
			if (this.$textarea.val() !== this.$textarea.data('value')) {
				this.$btnSave.removeAttr('disabled');
			} else {
				this.$btnSave.attr('disabled', 'disabled');
			}
		}

		onBtnCancelClick(e) {
			e.stopPropagation();

			this.$textarea.val(this.$textarea.data('value'));

			this.$editor.hide();
			this.$viewer.show();
			this.$btnToggle.show();
		}

		onBtnSaveClick(e) {
			e.stopPropagation();

			loading(this.$btnSave);

			let ticket = this.id;
			let secret = this.$textarea.val().trim();

			ajax('almightysupport/v1/pro_ticket_private_notes', { ticket, secret }, 'PATCH')
				.always(() => loading(this.$btnSave, false))
				.done(json => {
					this.update(secret);
					this.$editor.hide();
					this.$viewer.show();
					this.$btnToggle.show();
					hooks.doAction('ticket_update', this.id);
				});
		}
	};

	function escapeMessage(message) {
		// Escape HTML.
		message = message
			.replace(/&/g, '&amp;')
			.replace(/</g, '&lt;')
			.replace(/>/g, '&gt;')
			.replace(/"/g, '&quot;')
			.replace(/'/g, '&#039;');

		// Convert line-breaks.
		message = message.replace(/\r?\n/g, '<br>');

		return message;
	}
});
