alsp.class('ALSPTicketSidebarEdd', function (name, alsp, $) {
	const { ajax, hooks, template, data: config } = alsp;

	// class initialization hook
	hooks.addAction('init', (name, container, uuid) => {
		if ('ALSPTicket' === name) {
			container = container.querySelector('[data-alsp=ticket]');
			new (alsp.class('ALSPTicketSidebarEdd'))(container);
		}
	});

	return class {
		constructor(container) {
			this.$ticket = $(container);

			this.$ticket.on(
				'click',
				'[data-alsp=sidebar-edd-detail] [data-alsp=toggle-detail]',
				e => this.toggleItem(e)
			);

			hooks.addAction('ticket_sidebar_loaded', ticket => this.update(ticket));
		}

		update(ticket) {
			// Missing element means that the feature is disabled.
			let $summary = this.$ticket.find('[data-alsp=sidebar-edd-summary]');
			if (!$summary.length) {
				return;
			}

			return ajax('almightysupport/v1/pro_edd', { id: ticket.client.id }).done(json => {
				if (json.payments) {
					this.render(this.mapData(json));
				}
			});
		}

		render(data) {
			let summary = template(config('templates.pro/ticket/sidebar_edd_summary'))(data);
			let $summary = this.$ticket.find('[data-alsp=sidebar-edd-summary]');
			$summary.hide().eq(0).before(summary);
			$summary.remove();

			let detail = template(config('templates.pro/ticket/sidebar_edd_detail'))(data);
			this.$ticket.find('[data-alsp=sidebar-edd-detail]').replaceWith(detail);
		}

		toggleItem(e) {
			e.preventDefault();
			e.stopPropagation();

			let $btn = $(e.currentTarget);
			let $item = $btn.closest('[data-alsp=edd-item]');
			let $detail = $item.find('[data-alsp=detail]');

			if ($detail.is(':visible')) {
				$detail.hide();
				$btn.removeClass('alsp-i-angle-small-up').addClass('alsp-i-angle-small-down');
			} else {
				$detail.show();
				$btn.removeClass('alsp-i-angle-small-down').addClass('alsp-i-angle-small-up');
			}
		}

		mapData(data) {
			if (data.payments && data.payments.length) {
				data.payments = data.payments.map(payment => {
					payment.name = payment.cart_details.map(item => item.name).join(', ');
					return payment;
				});
			}

			return data;
		}
	};
});
