alsp.class('ALSPTicketSidebarWoo', function (className, alsp, $) {
	const { ajax, hooks, template, data: config } = alsp;

	// class initialization hook
	hooks.addAction('init', (name, container, uuid) => {
		if ('ALSPTicket' === name) {
			container = container.querySelector('[data-alsp=ticket]');
			new (alsp.class(className))(container);
		}
	});

	return class {
		constructor(container) {
			this.$ticket = $(container);

			this.$ticket.on(
				'click',
				'[data-alsp=sidebar-woo-detail] [data-alsp=toggle-detail]',
				e => this.toggleItem(e)
			);

			hooks.addAction('ticket_sidebar_loaded', ticket => this.update(ticket));
		}

		update(ticket) {
			// Missing element means that the feature is disabled.
			let $summary = this.$ticket.find('[data-alsp=sidebar-woo-summary]');
			if (!$summary.length) {
				return;
			}

			let customer = ticket.client.id;

			ajax(`wc/v3/customers/${customer}`, { alert: false }).done((customerData = {}) => {
				if (!customerData.is_paying_customer) return;

				ajax('wc/v3/orders', { customer, alert: false }).done(orders => {
					this.render({ customer: customerData, orders: this.mapData(orders) });
				});
			});
		}

		render(data) {
			let tmpl = template(config('templates.pro/ticket/sidebar_woo_summary'));
			let summary = tmpl(data.customer);
			let $summary = this.$ticket.find('[data-alsp=sidebar-woo-summary]');
			$summary.hide().eq(0).before(summary);
			$summary.remove();

			let detail = template(config('templates.pro/ticket/sidebar_woo_detail'))(data.orders);
			this.$ticket.find('[data-alsp=sidebar-woo-detail]').replaceWith(detail);
		}

		toggleItem(e) {
			e.preventDefault();
			e.stopPropagation();

			let $btn = $(e.currentTarget);
			let $item = $btn.closest('[data-alsp=woo-item]');
			let $detail = $item.find('[data-alsp=detail]');

			if ($detail.is(':visible')) {
				$detail.hide();
				$btn.removeClass('alsp-i-angle-small-up').addClass('alsp-i-angle-small-down');
			} else {
				$detail.show();
				$btn.removeClass('alsp-i-angle-small-down').addClass('alsp-i-angle-small-up');
			}
		}

		mapData(data) {
			if (data instanceof Array) {
				data = data.map(order => {
					order.name = (order.line_items || []).map(item => item.name).join(', ');
					return order;
				});
			}

			return data;
		}
	};
});
