alsp.class('ALSPTicketStaffNotesClient', function (name, alsp, $) {
	const { ajax, hooks, template, data: config, dialog } = alsp;

	// class initialization hook
	hooks.addAction('init', (initName, container) => {
		if ('ALSPTicket' === initName) {
			new (alsp.class(name))(container);
		}
	});

	return class extends alsp.class('ALSPAbstract') {
		constructor(container) {
			super(container);

			this.ticket = null;
			this.$el = null;
			this.$textarea = null;
			this.$btnCancel = null;
			this.$btnCancel = null;
			this.$btnCancel = null;

			hooks.addAction('ticket_sidebar_loaded', ticket => {
				this.ticket = ticket;
				this.init();

				let delay = +config('awedesk.is_client') ? 5000 : 0;
				setTimeout(() => {
					this.fetch(ticket).done(json => this.render((json && json.client) || []));
				}, delay);
			});
		}

		init() {
			let $container = this.$container.find('[data-alsp=ticket-staff-notes-client]');
			let tmpl = template(config('templates.ticket/staff_notes_client'));

			this.$el = $(tmpl());
			$container.replaceWith(this.$el);

			this.$textarea = this.$el.find('textarea');
			this.$btnCancel = this.$el.find('[data-alsp=btn-cancel]');
			this.$btnSubmit = this.$el.find('[data-alsp=btn-submit]');
			this.$btnAddnew = this.$el.find('[data-alsp=btn-addnew]');

			this.$el
				.on('input', 'textarea', e => this.onTextareaInput(e))
				.on('click', '[data-alsp=btn-cancel]', e => this.onCancelClick(e))
				.on('click', '[data-alsp=btn-submit]', e => this.onSubmitClick(e))
				.on('click', '[data-alsp=btn-addnew]', e => this.onAddnewClick(e))
				.on('click', '[data-alsp=btn-delete]', e => this.onDeleteClick(e));
		}

		fetch(ticket) {
			let params = { client_id: ticket.client.id, ticket_id: ticket.id };
			return ajax('almightysupport/v1/pro_staff_notes', params);
		}

		render(notes = []) {
			let $list = this.$el.find('[data-alsp=list]').empty();

			if (notes instanceof Array && notes.length) {
				let tmpl = template(config('templates.ticket/staff_notes_client_item'));
				notes.forEach(note => $list.append(tmpl(note)));
			} else {
				let tmpl = template(config('templates.ticket/staff_notes_client_item_empty'));
				$list.html(tmpl());
			}
		}

		onTextareaInput(e) {
			if (this.$textarea.val()) {
				this.$btnSubmit.removeAttr('disabled');
			} else {
				this.$btnSubmit.attr('disabled', 'disabled');
			}
		}

		onCancelClick(e) {
			this.$textarea.hide();
			this.$btnCancel.hide();
			this.$btnSubmit.hide();
			this.$btnAddnew.show();
		}

		onSubmitClick(e) {
			this.$textarea.attr('disabled', 'disabled');
			this.$btnCancel.attr('disabled', 'disabled');
			this.$btnSubmit.attr('disabled', 'disabled');

			let params = {
				client_id: this.ticket.client.id,
				ticket_id: this.ticket.id,
				type: 'client',
				message: this.$textarea.val()
			};

			return ajax('almightysupport/v1/pro_staff_notes', params, 'POST')
				.always(() => {
					this.$textarea.removeAttr('disabled');
					this.$btnCancel.removeAttr('disabled');
					this.$btnSubmit.removeAttr('disabled');
				})
				.done(json => {
					if (json.error) {
						dialog.alert(json.message || json.error);
					} else {
						this.$textarea.hide();
						this.$btnCancel.hide();
						this.$btnSubmit.hide();
						this.$btnAddnew.show();
						this.render(json.client || []);
					}
				});
		}

		onAddnewClick(e) {
			this.$textarea.val('').show().focus();
			this.$btnCancel.show();
			this.$btnSubmit.attr('disabled', 'disabled').show();
			this.$btnAddnew.hide();
		}

		onDeleteClick(e) {
			let $btn = $(e.target).closest('[data-alsp=btn-delete]').attr('disabled', 'disabled');

			let params = {
				client_id: this.ticket.client.id,
				ticket_id: this.ticket.id,
				type: 'client',
				id: $btn.data('id')
			};

			return ajax('almightysupport/v1/pro_staff_notes', params, 'DELETE')
				.always(() => $btn.removeAttr('disabled'))
				.done(json => {
					if (json.error) {
						dialog.alert(json.message || json.error);
					} else {
						let $item = $btn.closest('[data-alsp=item]');
						if ($item.siblings().length) {
							$item.remove();
						} else {
							this.render([]);
						}
					}
				});
		}
	};
});
