<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OXF5MExmM21OTmxsWVoxUTNzSEo1YVVxMXVKOUdOVFIvUHhPeHRVYlRwZk9OL29hL1dXSThUWUhxaERCMVBvQ05FR3N2UDk5L3pIdmhtaXFkdzRJN0FZSEFialArTkNKK21xRG1WR0JLS1ZnUDU5SE1Gc3JiNmtJUFgzb0J2MXU0PQ==*/

abstract class ALSP_REST_V1_Endpoint
{

    protected static $instance;

    protected $input;
    protected $wpdb;
    protected $wpuser;
    protected $alsp_user;
    protected $request;
    protected $model;
    protected $items;
    protected $links;

    public static $cred = [
        'create' => WP_REST_Server::CREATABLE,
        'read' => WP_REST_Server::READABLE,
        'edit' => WP_REST_Server::EDITABLE,
        'delete' => WP_REST_Server::DELETABLE,
    ];

    public function __construct()
    {
        $this->alsp_user = ALSP_User::get_instance();
        $this->input = new ALSP_Input();
        $this->wpdb = $GLOBALS['wpdb'];
        $this->wpuser = wp_get_current_user();
    }

    final public static function get_instance()
    {

        static $instances = [];

        $called_class = get_called_class();

        if (!isset($instances[$called_class])) {
            $instances[$called_class] = new $called_class();
        }

        return $instances[$called_class];
    }

    // Keep this method final and do not override it, it breaks mobile app auth
    public final function can($method)
    {
        if ($this->alsp_user->is_admin()) {
            return TRUE;
        }

        if (in_array($method, ['create', 'read', 'edit', 'delete'])) {
            $method = "can_$method";
            return $this->$method();
        }
    }

    protected function rest_route_for_class($endpoint, $id = NULL)
    {
        $url = ALSP_API::REST_NAMESPACE . '/' . ALSP_API::REST_V . '/' . $endpoint;
        if ($id) {
            $url .= '/' . $id;
        }
        return rest_url($url);
    }

    protected function items_to_response()
    {

        if (!is_array($this->items) || !count($this->items)) {
            return rest_ensure_response([]);
        }

        // Single item
        $id = isset($this->request['id']) ? $this->request['id'] : FALSE;
        if ($id) {
            $this->items = [$this->items];
        }

        foreach ($this->items as $item) {

            $response = rest_ensure_response($item);

            if (is_array($this->links) && count($this->links)) {

                $links = [];
                foreach ($this->links as $key => $config) {
                    $links[$key] = [
                        'href' => $this->rest_route_for_class($config['endpoint'], $item[$config['object_key']]),
                        'embeddable' => $config['embeddable'],
                    ];
                }

                $response->add_links($links);
            }

            $server = new WP_REST_Server();
            $embed = isset($this->request['_embed']) ? $this->request['_embed'] : FALSE;
            $response = $server->response_to_data($response, $embed);

            $result[] = $response;
        }

        // Single item
        if ($id) {
            $result = $response;
        }

        return rest_ensure_response($result);
    }

    public function init(WP_REST_Request $request)
    {
        $this->request = $request;
    }
}

// EOF
