<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OVJHWGpYOW1jVENNLzZoQ3ZMWlRWUVU0OEcxYkxXYVZwcU5jano2VjJtdTJGK2IvOU54TEo0bkxiUVBqdWp3YWZUMVFsTWk1SVBybjV3NWxodEdUTDh4b2hYeTMzWlNEVlM3YU1NWVFhZXNETjNQRFRYQ0dTYlFjcFFSMzdaQTljPQ==*/

class ALSP_REST_V1_Endpoint_Account extends ALSP_REST_V1_Endpoint {

    public function read(WP_REST_Request $request) {
        $user = wp_get_current_user();
        $user_id = $user->ID;
        $user_data = get_userdata($user_id);

        $data = [
            'id' => $user_id,
            'first_name' => $user_data->first_name,
            'last_name' => $user_data->last_name,
            'display_name' => $user_data->display_name,
            'email' => $user_data->user_email,
        ];

        return $data;
    }

    public function create(WP_REST_Request $request) {

        $data = $request->get_params();
        $user = wp_get_current_user();
        $user_id = $user->ID;
        $user_data = get_userdata($user_id);

        $actions = [
            'change_name',
            'change_email',
            'change_password',
        ];

        $action = $data['action'] ?? NULL;

        // Action must be defined and existing
        if(!in_array($action, $actions)) {
            return [
                'success' => FALSE,
                'error' => 'invalid_action',
                'message' => 'Invalid Action',
            ];
        }

        // Changing name requires first_name and last_name to exist and be at least 1 character long
        if($action == 'change_name') {

            // Sanitize
            $data['first_name'] = $data['first_name'] ?? '';
            $data['last_name'] = $data['last_name'] ?? '';
            $data['first_name'] = sanitize_user_field('first_name', $data['first_name'], $user_id, 'edit');
            $data['last_name'] = sanitize_user_field('last_name', $data['last_name'], $user_id, 'edit');
            $data['display_name'] = $data['first_name'] . ' ' . $data['last_name'];
            // Validate
            if(strlen($data['first_name']) < 1 || strlen($data['last_name']) < 1) {
                return [
                    'success' => FALSE,
                    'error' => 'invalid_name',
                    'message' => 'Invalid name',
                ];
            }

            // Save
            $user_data->first_name = $data['first_name'];
            $user_data->last_name = $data['last_name'];
            $user_data->display_name = $data['display_name'];

            wp_update_user($user_data);

            return ['success' => TRUE];
        }

        // Changing email or password requires old_password verification
        if(in_array($action, ['change_email','change_password'])) {

            $data['old_password'] = $data['old_password'] ?? '';
            $data['old_password'] = sanitize_text_field($data['old_password']);

            if(!wp_check_password($data['old_password'], $user_data->user_pass, $user_id)) {
                return [
                    'success' => FALSE,
                    'error' => 'old_password_mismatch',
                    'message' => __('Incorrect password', 'awedesk'),
                ];
            }

            if($action == 'change_email') {

                $data['email'] = $data['email'] ?? '';
                $data['email'] = sanitize_email($data['email']);
                if(!is_email($data['email'])) {
                    return [
                        'success' => FALSE,
                        'error' => 'invalid_email',
                        'message' => __('Invalid email', 'awedesk'),
                    ];
                }

                $user_data->user_email = $data['email'];
                wp_update_user($user_data);

                return ['success' => TRUE];
            }

            if($action == 'change_password') {

                $data['new_password_1'] = $data['new_password_1'] ?? '';
                $data['new_password_1'] = sanitize_text_field($data['new_password_1']);

                $data['new_password_2'] = $data['new_password_2'] ?? '';
                $data['new_password_2'] = sanitize_text_field($data['new_password_2']);

                if($data['new_password_1'] != $data['new_password_2']) {
                    return [
                        'success' => FALSE,
                        'error' => 'new_password_mismatch',
                        'message' => __('Passwords do not match', 'awedesk'),
                    ];
                }

                // @TODO in the future use something like zxcvbn to check password strength
                if(strlen($data['new_password_1']) < 5) {
                    return [
                        'success' => FALSE,
                        'error' => 'password_too_short',
                        'message' => __('Password is too weak', 'awedesk'),
                    ];
                }

                wp_set_password($data['new_password_1'], $user_id);

                return ['success' => TRUE];
            }
        }
    }

    public function can_create() {
        return get_current_user_id() > 0;
    }

    public function can_read() {
        return get_current_user_id() > 0;
    }
}

// EOF
