<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2L0t5TEtPWjlMS25nZlRxVDlWSE9NV0xyenVPSW5pY0tXRjVieEtQYkZBRUVSTnBScW5ScFA4bHo4dDdEbzAzYTA3VEcwVW4raGRGQzdGd2NIbWM2aFZLc0EwWGpVTlFuOHp1R3B6SDh0WUhHOFNIYnNUMDVoMk53T05ZalNub01RPQ==*/

class ALSP_REST_V1_Endpoint_Admin_Department extends ALSP_REST_V1_Endpoint
{
    public function __construct()
    {
        parent::__construct();
    }

    public function read(WP_REST_Request $request)
    {
        $this->model = new ALSP_Model_Departments($request);
        $this->init($request);

        $this->items = $this->model->read('all');

        return $this->items_to_response();
    }

    public function create(WP_REST_Request $request)
    {
        $name = $request['name'];
        $description = $request['description'];
        $default_assignee = $request['default_assignee'];
        $published = $request['published'];
        $agent_only = $request['agent_only'];
        $order = $request['order'];

        // sanitize name and description
        $name = sanitize_text_field(strip_tags($name));
        $description = sanitize_text_field(strip_tags($description));

        if (empty($name)) {
            return [
                'success' => FALSE,
                'error' => 'missing_mandatory_fields',
                'message' => 'Something went wrong'
            ];
        }

        if ($default_assignee) {
            $assignee = ALSP_User::get_instance($default_assignee);
            if (!$assignee->wp_user->ID) {
                return [
                    'success' => FALSE,
                    'error' => 'agent_not_found',
                    'message' => 'Something went wrong'
                ];
            }
        }

        $department = new ALSP_Model_Department();
        $result = $department->create_item($name, $description, $default_assignee, $published, $agent_only, $order);
        if (!$result) {
            return [
                'success' => FALSE,
                'error' => 'create_department_failed',
                'reason' => 'ALSP_Model_Department->create_item()',
                'message' => 'Something went wrong'
            ];
        }

        // Assign department to an agent
        $agent_department = new ALSP_Model_Agent_Department(NULL);
        $list_department = $agent_department->get_by_agent($default_assignee);
        $is_exists = false;
        foreach ($list_department as $key => $adept) {
            if(isset($adept['department']['id']) && $adept['department']['id'] == $result) {
                $is_exists = true;
                break;
            }
        }

        if (!$is_exists) {
            $agent_department->create_item($default_assignee, $result);
        }

        return ['success' => TRUE, 'class' => __CLASS__, 'method' => __METHOD__];
    }

    public function edit(WP_REST_Request $request)
    {
        $action = $request['action'];
        $id = $request['id'];

        if ($action == 'box_status') {
            $status = $request['status'];
            foreach ($id as $dept_id) {
                update_user_meta(get_current_user_id(), 'alsp_admin_departments_open_' . (int) $dept_id, $status);
            }
        } elseif ($action == 'reorder') {
            // SQL safe, expected JSON
            if( $id = json_decode($id) ) {
                $i = 1;
                foreach( $id as $dept_id ) {
                    $department = new ALSP_Model_Department($dept_id);
                    try {
                        $department->set('order', $i++);
                    } catch (Exception $e) {
                        return [
                            'success' => FALSE,
                            'error' => 'update_department_property_failed',
                            'reason' => 'ALSP_Model_Department->set()',
                            'message' => $e->getMessage(),
                        ];
                    }
                }
            }
        } elseif ($action == 'update_prop') {
            $prop = $request['prop'];
            $value = $request['value'];

            // sanitize first name and last name
            $prop = sanitize_text_field(strip_tags($prop));
            $value = sanitize_text_field(strip_tags($value));

            $department = new ALSP_Model_Department($id);
            try {
                $department->set($prop, $value);
            } catch (Exception $e) {
                return [
                    'success' => FALSE,
                    'error' => 'update_department_property_failed',
                    'reason' => 'ALSP_Model_Department->set()',
                    'message' => $e->getMessage(),
                ];
            }
        } else {
            $name = $request['name'];
            $description = $request['description'];

            // sanitize first name and last name
            $name = sanitize_text_field(strip_tags($name));
            $description = sanitize_text_field(strip_tags($description));

            if (empty($name) || empty($description)) {
                return [
                    'success' => FALSE,
                    'error' => 'missing_mandatory_fields',
                    'message' => 'Something went wrong'
                ];
            }

            $department = new ALSP_Model_Department($id);
            $result = $department->update_item($id, $name, $description);
            if (!$result) {
                return [
                    'success' => FALSE,
                    'error' => 'update_department_failed',
                    'reason' => 'ALSP_Model_Department->update_item()',
                    'message' => 'Something went wrong'
                ];
            }
        }

        return ['success' => TRUE, 'class' => __CLASS__, 'method' => __METHOD__];
    }

    public function delete(WP_REST_Request $request)
    {
        if ($request['id'] && $this->model = new ALSP_Model_Department($request['id'])) {
            $this->model->delete($request);
        }

        return ['success' => TRUE, 'class' => __CLASS__, 'method' => __METHOD__];
    }

    public function can_read()
    {
        return $this->wpuser->has_cap('alsp_ticket_list_departments');
    }

    public function can_create()
    {
        return $this->wpuser->has_cap('alsp_ticket_manage_departments');
    }

    public function can_edit()
    {
        return $this->wpuser->has_cap('alsp_ticket_edit_departments');
    }

    public function can_delete()
    {
        return $this->wpuser->has_cap('alsp_ticket_delete_departments');
    }
}

// EOF
