<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OENSYmE2V1BHcG0xZjBFSHdzWVYvamlrSGI3ZXBPbnlTQ08wNmhnajgvMEZ1MzhyWUQwYlpWZm11NFJGTHpWV1hzNEdrL1phL0FyR0xsQU12YkhPZjdzSGNTUlAxM2djK29Od204Vm9VVmpOcW9xSWxyUk8xSmJ2RnNFMlFPd2M4PQ==*/

class ALSP_REST_V1_Endpoint_Admin_License extends ALSP_REST_V1_Endpoint
{
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Sanitize license
     *
     * @param string $cache_key
     * @return string
     */
    private function edd_sanitize_license($new)
    {
        $old = ALSP_Config::get('license_key');
        if ($old && $old !== $new) {
            ALSP_Config::del('license_status'); // new license has been entered, so must reactivate
        }

        return sanitize_text_field($new);
    }

    /**
     * Activate license using EDD.
     *
     * @param string $license license key
     * @return bool|array true, if successfully activate the license. Otherwise, it will return array.
     */
    public function edd_activate_license($license)
    {
        ALSP_Mayfly::del('license_valid');

        if (!$license) {
            return [
                'success' => FALSE,
                'error' => 'missing_mandatory_fields',
                'message' => 'Invalid license',
            ];
        }

        // data to send in our API request
        $api_params = [
            'edd_action' => 'activate_license',
            'license' => $license,
            'item_id' => Awedesk_License_Helper::get_item_id(),
            'url' => home_url(),
            'environment' => function_exists('wp_get_environment_type') ? wp_get_environment_type() : 'production',
        ];

        // Call the custom API.
        $response = wp_remote_post(
            Awedesk_License_Helper::get_url(),
            [
                'timeout' => 15,
                'sslverify' => false,
                'body' => $api_params,
            ]
        );
        //new ALSP_Debug('calling Awedesk site ' . __METHOD__);

        // make sure the response came back okay
        if (is_wp_error($response) || 200 !== wp_remote_retrieve_response_code($response)) {
            // Error in communication, assume good intentions
            ALSP_Mayfly::set('myalmightysupportcom_maybe_offline', 1, 3600);
            ALSP_Mayfly::set('license_valid', 1, 3600);
            ALSP_Mayfly::set('license_valid_override', 1, 3600);

            if (is_wp_error($response)) {
                $message = $response->get_error_message();
            } else {
                $message = 'An error occurred, please try again';
            }
        } else {
            ALSP_Mayfly::del('myalmightysupportcom_maybe_offline');

            $license_data = json_decode(wp_remote_retrieve_body($response));

            if (false === $license_data->success) {

                $date_format = ALSP_Utility_WP::get_option('date_format');
                switch ($license_data->error) {

                    case 'expired':
                        $message = sprintf(
                            'Your license key expired on %s',
                            date_i18n($date_format, strtotime($license_data->expires, current_time('timestamp')))
                        );
                        break;

                    case 'disabled':
                    case 'revoked':
                        $message = 'Your license key has been disabled';
                        break;

                    case 'missing':
                        $message = 'Invalid license';
                        break;

                    case 'invalid':
                    case 'site_inactive':
                        $message = 'Your license is not active for this URL';
                        break;

                    case 'item_name_mismatch':
                        /* translators: the plugin name */
                        $message = sprintf('This appears to be an invalid license key for %s', 'Awedesk Pro');
                        break;

                    case 'no_activations_left':
                        $message = 'Your license key has reached its activation limit';
                        break;

                    default:
                        $message = 'An error occurred, please try again';
                        break;
                }
            }
        }

        // Check if anything passed on a message constituting a failure
        if (!empty($message)) {
            return [
                'success' => FALSE,
                'error' => 'failed_activate_license',
                'message' => $message,
            ];
        }

        // $license_data->license will be either "valid" or "invalid"
        if ('valid' === $license_data->license) {
            ALSP_Config::set('license_key', $license);
        }

        ALSP_Config::set('license_status', $license_data->license);
        return TRUE;
    }

    /**
     * Deactivate license using EDD.
     *
     * @param string $license license key
     * @return bool|array true, if successfully activate the license. Otherwise, it will return array.
     */
    private function edd_deactivate_license($license)
    {
        ALSP_Mayfly::set('license_valid', 0);

        // retrieve the license from the database
        $license = trim(ALSP_Config::get('license_key'));
        ALSP_Config::del('license_key');
        ALSP_Mayfly::del('license_valid');
        ALSP_Mayfly::del('license_valid_override');
        ALSP_Config::del('license_status');

        // data to send in our API request
        $api_params = [
            'edd_action' => 'deactivate_license',
            'license' => $license,
            'item_id' => Awedesk_License_Helper::get_item_id(),
            //			'item_name'   => rawurlencode( 'Awedesk Pro' ), // the name of our product in EDD
            'url' => home_url(),
            'environment' => function_exists('wp_get_environment_type') ? wp_get_environment_type() : 'production',
        ];

        // Call the custom API.
        $response = wp_remote_post(
            Awedesk_License_Helper::get_url(),
            [
                'timeout' => 15,
                'sslverify' => false,
                'body' => $api_params,
            ]
        );

        return TRUE;
    }

    /**
     * Install and activate Awedesk (Pro) version.
     *
     * @return bool|array true, if successfully activate the license. Otherwise, it will return array.
     */
    private function edd_install_package()
    {

//        $status = ALSP_Config::get('license_status');
//        if ($status !== 'valid') {
//            return [
//                'success' => FALSE,
//                'error' => 'invalid_license',
//                'message' => 'Invalid license',
//            ];
//        }
//
//        // skip process when Awedesk (Pro) already installed.
//        if (is_plugin_active('almighty-support-pro/alsp.php')) {
//            return TRUE;
//        }
//
//        require_once ABSPATH . 'wp-admin/includes/plugin.php';
//        require_once ABSPATH . 'wp-includes/pluggable.php';
//        require_once ABSPATH . 'wp-admin/includes/file.php';
//        require_once ABSPATH . 'wp-admin/includes/misc.php';
//        require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
//
//        // retrieve the license from the database
//        $license = trim(ALSP_Config::get('license_key'));
//        if ($license) {
//            $api_params = [
//                'edd_action' => 'get_version',
//                'license' => $license,
//                'item_id' => Awedesk_License_Helper::get_item_id(),
//                'url' => home_url(),
//            ];
//
//            $request = wp_remote_post(
//                Awedesk_License_Helper::get_url(),
//                [
//                    'timeout' => 15,
//                    'sslverify' => TRUE,
//                    'body' => $api_params
//                ]
//            );
//            if (!is_wp_error($request)) {
//                $request = json_decode(wp_remote_retrieve_body($request));
//                if (isset($request->download_link)) {
//                    $upgrader = new Plugin_Upgrader(new WP_Ajax_Upgrader_Skin(['title' => $request->name]));
//                    $result = $upgrader->install($request->download_link, [
//                        'overwrite_package' => TRUE
//                    ]);
//
//                    if (is_wp_error($result)) {
//                        return [
//                            'success' => FALSE,
//                            'error' => 'download_package_failed',
//                            'message' => $result->get_error_message(),
//                        ];
//                    }
//
//                    // Activate the plugins
//                    $activate = activate_plugin('almighty-support-pro/alsp.php');
//                    if (is_wp_error($activate)) {
//                        return [
//                            'success' => FALSE,
//                            'error' => 'activate_plugins_failed',
//                            'message' => $activate->get_error_message(),
//                        ];
//                    }
//                } else if (isset($request->msg)) {
//                    return [
//                        'success' => FALSE,
//                        'error' => 'package_not_available',
//                        'message' => $request->msg,
//                    ];
//                }
//            }
//        } else {
//            return [
//                'success' => FALSE,
//                'error' => 'invalid_license',
//                'message' => 'Invalid license',
//            ];
//        }
//
        return TRUE;
    }

    public function create(WP_REST_Request $request)
    {
        $action = $request['action'];
        $license = $request['alsp_license_key'];

        switch ($action) {
            case 'activate':
                $result = $this->edd_activate_license($license);
                break;

            case 'deactivate':
                $result = $this->edd_deactivate_license($license);
                break;


            case 'install':
                $result = $this->edd_install_package();
                break;
        }

        // If we got an error from the private function,
        // just return the response.
        if ($result !== TRUE) {
            return $result;
        }

        return ['success' => TRUE, 'class' => __CLASS__, 'method' => __METHOD__];
    }

    public function can_create()
    {
        return current_user_can('manage_options');
    }
}

// EOF
