<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OCt1aUpDYU5IbnVpcEZRSnFWUHBNTWVCNGp5ei9zN2Nlb0tYeXo4TWxuUG9QUEtTSE9heFdJbElJZ3V4NlhaVFV6WmlNT3JTcmQ2SjA2RFduWE9QYWRKeXJoU3VSdmU1TlVVTG5vRHQ1a1B1SDVvY2NJUXQ4ZURyZVV2SjBpS3dvPQ==*/

class ALSP_REST_V1_Endpoint_Admin_Mailqueue extends ALSP_REST_V1_Endpoint
{
    public function __construct()
    {
        parent::__construct();
    }

    public function read(WP_REST_Request $request)
    {
        $listMailqueue = ALSP_Email_Queue::fetch_all();
        $response = [];
        foreach ($listMailqueue as $key => $mail) {
            $mail['message'] = substr(wp_strip_all_tags($mail['message']), 0, 49);
            $mail['status_icon'] = $this->icon_mail_status($mail);
            $mail['status'] = $this->column_mail_status($mail);
            $mail['status_color'] = $this->color_mail_status($mail);

            $response[$key] = $mail;
        }
        return $response;
    }

    public function create(WP_REST_Request $request)
    {
        $action = $request['action'];
        $mailIds = $request['mail_ids'];
        $count = count($mailIds);

        if (!is_array($mailIds) || $count == 0) {
            return [
                'success' => FALSE,
                'error' => 'missing_mandatory_fields',
                'message' => 'Invalid input',
            ];
        }

        if ('bulk-delete' === $action) {
            foreach ($mailIds as $mailId) {
                ALSP_Email_Queue::delete_by_id($mailId);
            }

            $message = 'deleted';
        } else if (in_array($action, ['bulk-waiting', 'bulk-delay'])) {
            if ('bulk-waiting' === $action) {
                $status = ALSP_Email_Queue::STATUS_PENDING;
            } else {
                $status = ALSP_Email_Queue::STATUS_DELAY;
            }

            foreach ($mailIds as $mailId) {
                ALSP_Email_Queue::update_status_by_id($mailId, $status);
            }

            $message = 'updated';
        }

        return [
            'success' => TRUE,
            'message' => sprintf(
                '%1$d %2$s %3$s',
                $count,
                ($count == 1) ? 'item' : 'items',
                $message
            ),
            'class' => __CLASS__,
            'method' => __METHOD__
        ];
    }

    private function color_mail_status($item)
    {
        switch ($item['status']) {
            case ALSP_Email_Queue::STATUS_FAILED:
                $color = 'red';
                break;
            case ALSP_Email_Queue::STATUS_DELAY:
            case ALSP_Email_Queue::STATUS_RETRY:
                $color = 'orange';
                break;
            default:
                $color = "black";
                break;
        }

        return $color;
    }

    private function column_mail_status($item)
    {
        $error = '<br/> <textarea rows="5" wrap="soft" style="white-space: nowrap;  overflow: auto;width:100%">' . $item['error_log'] . '</textarea>';
        switch ($item['status']) {
            case ALSP_Email_Queue::STATUS_PENDING:
                $ret = 'Waiting';
                break;
            case ALSP_Email_Queue::STATUS_PROCESSING:
                $ret = 'Processing';
                break;
            case ALSP_Email_Queue::STATUS_DELAY:
                $ret = 'Delayed';
                break;
            case ALSP_Email_Queue::STATUS_FAILED:
                $ret = 'Failed';
                $ret .= $error;
                break;
            case ALSP_Email_Queue::STATUS_RETRY:
                $ret = 'Retrying';
                $ret .= $error;
                break;
            default:
                $ret = 'Unknown';
                break;
        }

        return '<p style="color:' . $this->color_mail_status($item) . '">' . $ret . '</p>';
    }

    private function icon_mail_status($item)
    {
        switch ($item['status']) {
            case ALSP_Email_Queue::STATUS_PENDING:
            case ALSP_Email_Queue::STATUS_DELAY:
                $fa = 'clock';
                break;
            case ALSP_Email_Queue::STATUS_PROCESSING:
                $fa = 'hourglass-half';
                break;
            case ALSP_Email_Queue::STATUS_FAILED:
                $fa = 'warning';
                break;
            case ALSP_Email_Queue::STATUS_RETRY:
                $fa = 'refresh';
                break;
            default:
                $fa = 'question';
                break;
        }

        return $fa;
    }

    public function can_read()
    {
        return current_user_can('manage_options');
    }

    public function can_create()
    {
        return current_user_can('manage_options');
    }
}

// EOF
