<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OGo4SWpuWWdPVlBuSVYxQ01XOTJjaHB2SEIyZDhKNGhNOVVLeC81RkdaVUlGdCs5Q3RNTncvZEtwdXBmZUtQcWFlK3Q3TnpjdHc3eVByemVHcXNDR3Y5NUtTdk9iMUNxZUZUZk85OVBpb2N3PT0=*/

class ALSP_REST_V1_Endpoint_Login extends ALSP_REST_V1_Endpoint
{

    public function __construct()
    {
        parent::__construct();
    }

    public function create(WP_REST_Request $request)
    {
        $email = isset($request['email']) ? $request['email'] : '';
        $passwd = isset($request['passwd']) ? $request['passwd'] : '';
        $rememberme = isset($request['rememberme']) ? $request['rememberme'] : FALSE;

        if (ALSP_Config::get('recaptcha_enabled', 0)) {
            $valid = ALSP_ReCaptcha::validate($request);
            if (!$valid) {
                return [
                    'success' => FALSE,
                    'error' => 'recaptcha_security_check_failed',
                    'message' => __('ReCaptcha security check failed.', 'awedesk'),
                ];
            }
        }

        if (!$email) {
            return [
                'success' => FALSE,
                'error' => 'invalid_email_address',
                'message' => __('Invalid email', 'awedesk'),
            ];
        }

        $user = FALSE;
        $using_username = ALSP_Config::get('login_by_username_enabled', FALSE);
        if ($using_username) {
            $user = get_user_by('login', $email);
        }

        if (FALSE === $user) {
            $user = get_user_by('email', $email);
            if (FALSE === $user) {
                return [
                    'success' => FALSE,
                    'error' => 'email_not_registered',
                    'message' => $using_username ? __('Username or Email not found', 'awedesk') : __('Email not found', 'awedesk'),
                ];
            }
        }

        if (!$passwd) {
            return [
                'success' => FALSE,
                'error' => 'invalid_password',
                'message' => __('Invalid password', 'awedesk'),
            ];
        }

        $valid_pass = wp_authenticate_email_password(null, $email, $passwd);
        if (is_wp_error($valid_pass)) {
            return [
                'success' => FALSE,
                'error' => 'invalid_password',
                'message' => __('Invalid password', 'awedesk'),
            ];
        } else {
            $info = [];
            $info['user_login'] = $email;
            $info['user_password'] = $passwd;
            $info['remember'] = $rememberme;

            $secure_cookie = NULL;
            if (force_ssl_admin()) {
                $secure_cookie = TRUE;
            }
            $user_signon = wp_signon($info, $secure_cookie);
            if (is_wp_error($user_signon)) {
                return [
                    'success' => FALSE,
                    'error' => 'invalid_password',
                    'message' => $user_signon->get_error_message(),
                ];
            } else {
                return [
                    'success' => TRUE,
                    'class' => __CLASS__,
                    'method' => __METHOD__,
                ];
            }
        }
    }

    public function can_create()
    {
        return !get_current_user_id();
    }
}

// EOF
