<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2L1ZZbzhZQTl3MGpCampSZG42ZW1lZUl0UnFnd0luNjdYU0xJeUxOVDVyd2Z3VTJza2RPcVdWRWRxK3NvTWs5V1BXMWtLVng3dS9SakRqd1ZxQkdEQlo3WUNMVkxtQ2IzQmErMjE0UUo0cExzNlFQZ2JVbmQzSUh4UEt1MFhCdnlZPQ==*/

class ALSP_REST_V1_Endpoint_Pro_EDD extends ALSP_REST_V1_Endpoint
{

	public function __construct()
	{
		parent::__construct();
	}

	public function read(WP_REST_Request $request)
	{
		$email = $_REQUEST['email'] ?? FALSE;
		$id = $_REQUEST['id'] ?? FALSE;

		$response = [];
		if (!class_exists('EDD_Customer')) {
			$response['error'] = 'EDD_Customer class not found';
			return $response;
		}

		$customer = new EDD_Customer($id, TRUE);
		if (!$customer->id) {
			$customer = new EDD_Customer($email);
			if (!$customer->id) {
				$response['error'] = "EDD_Customer $email not found";
				return $response;
			}
		}

		// Customer

		$response_customer = [];
		$keys = ['id', 'user_id', 'name', 'purchase_value', 'purchase_count'];

		foreach ($keys as $key) {
			$response_customer[$key] = $customer->$key;
		}

		$response_customer['currency_symbol'] = edd_currency_symbol();
		ksort($response_customer);

		$response['customer'] = $response_customer;

		// Payments

		$payments = $customer->get_payments();

		if (is_array($payments) && count($payments)) {
			$date_format = ALSP_Utility_WP::get_option('date_format');

			$keys = ['ID', 'subtotal', 'total', 'status', 'date', 'completed_date', 'cart_details', 'currency'];
			foreach ($payments as $payment) {
				$payment = $payment->array_convert();
				$response_payment = [];
				foreach ($keys as $key) {
					$response_payment[$key] = $payment[$key];
					if (in_array($key, ['date', 'completed_date'])) {
						$response_payment["{$key}_formatted"] = wp_date($date_format, strtotime($payment[$key]));
					}
				}

				// Status name
				$response_payment['status_name'] = edd_get_payment_status_label($response_payment['status']);

				// Cart details
				$response_cart_details = [];
				if (is_array($response_payment['cart_details']) && count($response_payment['cart_details'])) {

					foreach ($response_payment['cart_details'] as $detail) {
						$response_cart_details[] = [
							'name' => $detail['name'],
							'id' => $detail['id'],
							'price' => $detail['price'],
						];
					}
				}

				$response_payment['cart_details'] = $response_cart_details;
				$response_payment['currency_symbol'] = edd_currency_symbol($response_payment['currency']);

				ksort($response_payment);
				$response['payments'][] = $response_payment;
			}
		}


		// Licenses
		if (function_exists('edd_software_licensing')) {

			$licenses = edd_software_licensing()->licenses_db->get_licenses(
				[
					'number' => -1,
					'customer_id' => $customer->id,
					'orderby' => 'id',
					'order' => 'DESC'
				]
			);

			if (is_array($licenses) && count($licenses)) {
				$date_format = ALSP_Utility_WP::get_option('date_format');
				foreach ($licenses as $license) {
					$response_license = [];
					$keys = ['id', 'license_key', 'status', 'is_lifetime', 'download_id'];

					foreach ($keys as $key) {
						$response_license[$key] = $license->$key;
					}

					$response_license['expiration'] = date_i18n($date_format, $license->expiration);
					$response_license['download_name'] = $license->get_download()->get_name();

					if ($license->price_id) {
						$response_license['download_name'] .= ' - ' . edd_get_price_option_name($license->download_id, $license->price_id);
					}

					$current_license = edd_software_licensing()->get_license($license->id);
					$sites = $current_license->get_activations();

					$response_license['licensed_url'] = [];
					if (!empty($sites)) {
						foreach ($sites as $site) {
							array_push($response_license['licensed_url'], $site->site_name);
						}
					}

					ksort($response_license);

					$response['licenses'][] = $response_license;
				}

			}
		}

		return rest_ensure_response(apply_filters('almighty_support_tbd', $response));
	}

	public function can_read()
	{
		// The endpoint does its own granular checks as well
        $this->wpuser = wp_get_current_user();
		return $this->wpuser->has_cap('alsp_view_edd');
	}
}

// EOF
