<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OWIvOExwSnJkOW9FczUxLzEvM21EYWlQcVRXRFpuYktQVWVLOUZxTTVDN3Vsb21qM3haYmg5UXp6RGRkalRSK2ZrQW5nN0lBTmhQWE5EM0lHQWllQ2k0Qi9yaE9BYWtXSzB6QUtEOHhYNGZyUHJRaXcyOVg5NDVUNTJJRzI0a0xnPQ==*/

class ALSP_REST_V1_Endpoint_Pro_Ticket_Attachments extends ALSP_REST_V1_Endpoint
{
    public function __construct()
    {
        parent::__construct();
    }

    public function read(WP_REST_Request $request)
    {
        $this->model = new ALSP_Model_Ticket_Attachments($request);
        $this->init($request);

        // Message ID is mandatory to prevent load another attachment
        if (intval($request['message_id']) == 0) {
            return [
                'success' => FALSE,
                'error' => 'missing_mandatory_fields',
                'message' => __('Missing mandatory fields', 'awedesk'),
            ];
        }

        $this->items = $this->model->read();

        return $this->items_to_response();
    }

    public function create(WP_REST_Request $request)
    {
        $file_parameters = $request->get_file_params();
        if (!isset($file_parameters['file'])) {
            return [
                'success' => FALSE,
                'error' => 'missing_mandatory_fields',
                'message' => __('Missing parameter(s): file', 'awedesk'),
            ];
        }

        $file = $file_parameters['file'];
        $filename = sanitize_file_name($file['name']);
        $filename = preg_replace('/(\.[a-z\d]+)$/i', '_' . time() . '$1', $filename);
        $extension = strtolower(pathinfo($filename, PATHINFO_EXTENSION));

        // Allowed file extension.
        $allowed_file_extensions = ALSP_Config::get('attachment_allowed_file_types_textarea');
        $allowed_file_extensions = explode(',', $allowed_file_extensions);
        $allowed_file_extensions = array_map('trim', $allowed_file_extensions);
        $allowed_file_extensions = array_map('strtolower', $allowed_file_extensions);
        if (!(in_array($extension, $allowed_file_extensions))) {
            return [
                'success' => FALSE,
                'error' => 'not_allowed_file_extensions',
                'message' => __('File extension not allowed', 'awedesk'),
            ];
        }

        $current_user_id = get_current_user_id();
        $is_agent = FALSE;
        if ($current_user_id) {
            $alsp_user = ALSP_User::get_instance($current_user_id);
            $is_agent = ($alsp_user->is_admin() || $alsp_user->is_manager() || $alsp_user->is_agent());

            // Max Quota
            if(!$is_agent) {
                $max_quota = ALSP_Config::get('attachment_max_user_quota_number', 100);
                $files = ALSP_Model_Ticket_Attachments::get_ticket_attachments_by_user(get_current_user_id());
                if (count($files) > $max_quota) {
                    return [
                        'success' => FALSE,
                        'error' => 'not_allowed_max_quota',
                        'message' => __('You cannot add more attachments', 'awedesk'),
                    ];
                }
            }
        }

        // Max file size.
        if(!$is_agent) {
            $max_upload_size = ALSP_Config::get('attachment_max_filesize_number');

            // use WP max upload size if it is smaller than PeepSo max upload size
            $wp_max_size = max(wp_max_upload_size(), 0);
            $wp_max_size /= pow(1024, 2);
            if ($wp_max_size < $max_upload_size) {
                $max_upload_size = $wp_max_size;
            }

            $allowed_file_size = intval($max_upload_size) * 1048576;
            if (!(isset($file['size']) && $file['size'] <= $allowed_file_size)) {
                return [
                    'success' => FALSE,
                    'error' => 'file_size_exceeds',
                    'message' => __('The attachment is too big', 'awedesk'),
                ];
            }
        }

        // File path.
        $file_path = ALSP_Attachments::get_tmp_dir();
        if (!file_exists($file_path)) {
            mkdir($file_path, 0755, true);
        }
        $file_path .= '/' . $filename;

        // Create file.
        if (move_uploaded_file($file['tmp_name'], $file_path)) {
            return [
                'success' => TRUE,
                'data' => $filename,
            ];
        }

        return [
            'success' => FALSE,
            'error' => 'failed_move_uploaded_file',
            'message' => 'Something went wrong',
        ];
    }

    public function can_read()
    {
        $can_access = FALSE;

        // Allow ticket attachment for non-loggedin.
        $this->wpuser = wp_get_current_user();
        if (!$this->wpuser->ID) {
            $can_access = TRUE;
        } else {
            $can_access = $this->wpuser->has_cap('alsp_attach_files');
        }

        return $can_access;
    }

    public function can_create()
    {
        return $this->can_read();
    }

}

// EOF
