<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2L2Qxaml3cHJOYnZHL1lLOXZaMk9BaG9WcWlkeGJ5bWlLa1Bld295OTVrbW5DNkxpUUJHT3BqM3VERGcrSFNkTk9leFBqQVJRWXd3VG81Q2t4TW1OYk5yRDRYYXZsRG9ONXVzS3Z3L1c0UWVEL0FBelg1bEhPVncwaVFtcmxmRzlpNVlhbG5GZFZGWmdWT1RSbU5jUUZM*/

class ALSP_REST_V1_Endpoint_Pro_Ticket_Private_Notes extends ALSP_REST_V1_Endpoint
{
    public function __construct()
    {
        parent::__construct();
    }

    public function edit(WP_REST_Request $request)
    {
        $secret = strlen($request['secret']) > 0 ? $request['secret'] : '';
        $ticket = intval($request['ticket']) > 0 ? intval($request['ticket']) : NULL;
        $author = isset($request['author']) ? $request['author'] : get_current_user_id();

        $ticket_model = new ALSP_Model_Ticket($ticket);
        if (!$ticket_model->id) {
            return [
                'success' => FALSE,
                'error' => 'ticket_not_found',
                'message' => 'Something went wrong'
            ];
        }

        $author_model = ALSP_User::get_instance($author);
        if (!$author_model->wp_user->ID) {
            return [
                'success' => FALSE,
                'error' => 'author_not_found',
                'message' => 'Something went wrong'
            ];
        }

        if ($author_model->is_client() && intval($ticket_model->client) !== intval($author)) {
            return [
                'success' => FALSE,
                'error' => 'forbidden_access',
                'message' => 'Something went wrong'
            ];
        }

        $ciphertext = ALSP_Utility_Crypto::encrypt($secret);
        if (FALSE === $ticket_model->update_private_notes($ticket, $ciphertext)) {
            return [
                'success' => FALSE,
                'error' => 'failed_to_update_ticket_private_notes',
                'message' => 'Something went wrong'
            ];
        }

        $ticket_message = new ALSP_Model_Ticket_Message();
        $type = 'ticket_updates';
        $message = __('updated the private note', 'awedesk');

        $result = $ticket_message->create_item($ticket, $author, $type, $message);
        if (!$result) {
            return [
                'success' => FALSE,
                'error' => 'create_ticket_message_failed',
                'reason' => 'ALSP_Model_Ticket_Message->create_item()',
                'message' => 'Something went wrong'
            ];
        }

        // Check for an agent
        $agent = $ticket_model->agent;
        if ($agent != NULL && intval($agent) != intval($author)) {
            $agent_model = ALSP_User::get_instance($agent);
            if (!$agent_model->wp_user->ID) {
                return [
                    'success' => FALSE,
                    'error' => 'agent_not_found',
                    'message' => 'Something went wrong'
                ];
            }

            // Set ticket update message
            $message = sprintf('%s '. __('updated the private note', 'awedesk'), $author_model->wp_user->display_name);

            /**
             * Send email notification for an agent
             */
            $data = [];
            $data['ticket_number'] = $ticket;
            $ticket_data = $ticket_model->get_item();
            $data['ticket_title'] = $ticket_data['subject'];
            $data['ticket_url'] = ALSP::get_alsp_ticket_uri($agent, $ticket);
            $data['agent_name'] = $agent_model->wp_user->display_name;
            $data['recipient_name'] = $agent_model->wp_user->display_name;
            $data['recipient_email'] = $agent_model->wp_user->user_email;
            $data['message'] = $ticket_message->get_first_message($ticket);
            ALSP_Email_Queue::add_message($agent_model->wp_user->ID, $data, __('[Ticket #{ticket_number}] private note updated', 'awedesk'), 'ticket_private_notes_template');
        }

        ALSP_Push::send_to_agents($ticket, $author, 'private_note');

        $req = new WP_REST_Request;
        $req['id'] = $result;

        $this->model = new ALSP_Model_Ticket_Messages($req);
        $this->init($req);

        $this->items = $this->model->read();

        return $this->items_to_response();
    }

    public function can_edit()
    {
        $this->wpuser = wp_get_current_user();
        return $this->wpuser->has_cap('alsp_edit_private_notes');
    }

    public static function description($multiline=FALSE) {
        $interval = ALSP_Config::get('secret_delete_interval_number', 7);
        ob_start();
        echo __('Used to provide sensitive information such as passwords.','awedesk');
        echo $multiline ? "<br>" : " ";
        echo sprintf(_n('Private notes are encrypted and deleted %d day after the ticket is closed.','Private notes are encrypted and deleted %d days after the ticket is closed.',  $interval,'awedesk'), $interval);
        return ob_get_clean();
    }
}
