<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2LytMUXFLQ3RSZ3l0NW9GVDNISGxWVjNBaEJsdGFUclBOZnBJcTI2QmExVk1FQ2ZiY1N2WEllTnJkSGowVmE3TjEvTDVzc1dNN2RpTENCRFNHd3lsK0J5eHhRZ3dxTG0rK01aZU1zMmZkL1BlUTVvZTJpWWplSXBmWmdaL01kUkhNPQ==*/

class ALSP_REST_V1_Endpoint_Ticket_Assign extends ALSP_REST_V1_Endpoint
{
    public function __construct()
    {
        parent::__construct();
    }

    public function create(WP_REST_Request $request)
    {
        $agent = intval($request['agent']) > 0 ? intval($request['agent']) : NULL;
        $ticket = intval($request['ticket']) > 0 ? intval($request['ticket']) : NULL;
        $author = isset($request['author']) ? $request['author'] : get_current_user_id();

        $ticket_model = new ALSP_Model_Ticket($ticket);
        if (!$ticket_model->id) {
            return [
                'success' => FALSE,
                'error' => 'ticket_not_found',
                'message' => 'Something went wrong'
            ];
        }

		$old_agent = $ticket_model->get_item();
		$old_agent = $old_agent['agent'];
		$old_agent = strlen($old_agent['user_nicename']) ? $old_agent['user_nicename'] : $old_agent['user_login'];
		if(!strlen($old_agent)) $old_agent = null;


        if (FALSE === $ticket_model->assign_ticket($author, $ticket, $agent)) {
            return [
                'success' => FALSE,
                'error' => 'failed_to_assign_ticket',
                'message' => 'Something went wrong'
            ];
        }

        do_action('alsp_ticket_assignee_changed', $ticket, $agent);

        $ticket_message = new ALSP_Model_Ticket_Message();
        $type = 'ticket_updates';
        $message = $old_agent ? sprintf(__('unassigned %s', 'awedesk'), $old_agent) : __('unassigned', 'awedesk');

	    ALSP_Push::send_to_agents($ticket, $author, 'agent', ['action_id' => $agent]);

        // Check for an agent
        if ($agent != NULL) {
            $agent_model = ALSP_User::get_instance($agent);
            if (!$agent_model->wp_user->ID) {
                return [
                    'success' => FALSE,
                    'error' => 'agent_not_found',
                    'message' => 'Something went wrong'
                ];
            }

            // Set ticket update message
            $message = sprintf(__('assigned to %s', 'awedesk'), $agent_model->wp_user->display_name);

            /**
             * Send email notification for an agent
             */
            $data = [];
            $data['ticket_number'] = $ticket;
            $ticket_data = $ticket_model->get_item();
            $data['ticket_title'] = $ticket_data['subject'];
            $data['ticket_url'] = ALSP::get_alsp_ticket_uri($agent, $ticket);
            $data['agent_name'] = $agent_model->wp_user->display_name;
            $data['recipient_name'] = $agent_model->wp_user->display_name;
            $data['recipient_email'] = $agent_model->wp_user->user_email;
            $data['message'] = $ticket_message->get_first_message($ticket);

            ALSP_Email_Queue::add_message($agent_model->wp_user->ID, $data, __('[Ticket #{ticket_number}] assigned', 'awedesk'), 'ticket_assigned_template');


        }

        $result = $ticket_message->create_item($ticket, $author, $type, $message);
        if (!$result) {
            return [
                'success' => FALSE,
                'error' => 'create_ticket_message_failed',
                'reason' => 'ALSP_Model_Ticket_Message->create_item()',
                'message' => 'Something went wrong'
            ];
        }


        $req = new WP_REST_Request;
        $req['id'] = $result;

        $this->model = new ALSP_Model_Ticket_Messages($req);
        $this->init($req);

        $this->items = $this->model->read();

        return $this->items_to_response();
    }

    public function can_create()
    {
        $this->wpuser = wp_get_current_user();
        return $this->wpuser->has_cap('alsp_assign_ticket');
    }

}

// EOF
