<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2L1dNNGg4TXA3WG9ONElCVnY0SWdaaWcyTzVyRmx2QnExS05NMTYwOHorK1N0eEw1M1M0NHRwaUhxZFZrSXBCRTFFSTFSbUk5ZHpPZUZkK29wRUc5aGpqR2ZmcVN6eDRhbmNhaEZqSnduRlRUOUZRcjYwVFN2a0p3NWFHR1lTUkNjPQ==*/

class ALSP_REST_V1_Endpoint_Ticket_Closed extends ALSP_REST_V1_Endpoint
{
    public function __construct()
    {
        parent::__construct();
    }

    public function create(WP_REST_Request $request)
    {
        $ticket = $request['ticket'];
        $author = isset($request['author']) ? $request['author'] : get_current_user_id();

        $ticket_model = new ALSP_Model_Ticket($ticket);
        if (!$ticket_model->id) {
            return [
                'success' => FALSE,
                'error' => 'ticket_not_found',
                'message' => 'Something went wrong'
            ];
        }

        $author_model = ALSP_User::get_instance($author);
        if (!$author_model->wp_user->ID) {
            return [
                'success' => FALSE,
                'error' => 'author_not_found',
                'message' => 'Something went wrong'
            ];
        }

        if (FALSE === $ticket_model->close_ticket($ticket)) {
            return [
                'success' => FALSE,
                'error' => 'failed_to_close_ticket',
                'message' => 'Something went wrong'
            ];
        }

        $ticket_data = $ticket_model->get_item();
        $ticket_message = new ALSP_Model_Ticket_Message();
        $type = 'ticket_updates';
        $message = __('closed the ticket', 'awedesk');
        $result = $ticket_message->create_item($ticket, $author, $type, $message);
        if (!$result) {
            return [
                'success' => FALSE,
                'error' => 'create_ticket_message_failed',
                'reason' => 'ALSP_Model_Ticket_Message->create_item()',
                'message' => 'Something went wrong'
            ];
        }

        do_action('alsp_ticket_closed', $ticket);

        /**
         * Send email notification for an agent
         */
        $data = [];
        $data['ticket_number'] = $ticket;
        $data['ticket_title'] = $ticket_model->subject;

        if ($author_model->is_client() && $ticket_data['agent'] !== NULL) {
            $template = 'ticket_closed_client_template';
            $data['recipient_name'] = $ticket_data['agent']['display_name'];
            $data['recipient_email'] = $ticket_data['agent']['user_email'];
            $data['agent_name'] = $ticket_data['agent']['display_name'];
            $data['ticket_url'] = ALSP::get_alsp_ticket_uri($ticket_data['agent']['id'], $ticket_data['id']);

            ALSP_Email_Queue::add_message($author_model->wp_user->ID, $data, __('[Ticket #{ticket_number}] was closed', 'awedesk'), $template);

        } elseif (!$author_model->is_client()) {
            $template = 'ticket_closed_agent_template';
            $data['recipient_name'] = $ticket_data['client']['display_name'];
            $data['recipient_email'] = $ticket_data['client']['user_email'];
            $data['agent_name'] = $author_model->wp_user->display_name;
            $data['client_name'] = $ticket_data['client']['display_name'];
            $data['ticket_url'] = ALSP::get_alsp_ticket_uri($ticket_data['client']['id'], $ticket_data['id']);

            ALSP_Email_Queue::add_message($author_model->wp_user->ID, $data, __('[Ticket #{ticket_number}] was closed', 'awedesk'), $template);

        }

	    ALSP_Push::send_to_agents($ticket, $author, 'closed');

        $req = new WP_REST_Request;
        $req['id'] = $result;

        $this->model = new ALSP_Model_Ticket_Messages($req);
        $this->init($req);

        $this->items = $this->model->read();

        return $this->items_to_response();
    }

    public function can_create()
    {
        $this->wpuser = wp_get_current_user();
        return $this->wpuser->has_cap('alsp_close_ticket');
    }

}

// EOF
