<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2ODYra2swNTYyTG9SZ1hiWlROWmR2MFE1eXhKbEJvaVJ2YVlsakIzMlJPR01iaDhtRWtLeWZ2cXVHSk9PVzhYQ2dKZkFsZXhYRk1zTjBHUEIxdkh5bVRzL3g3RW5IVWozT3VDWm5KR1V3SjgxMGp0d1lGaEU5U2k0M1pndjdEOC8wPQ==*/

class ALSP_REST_V1_Endpoint_Ticket_Messages extends ALSP_REST_V1_Endpoint
{

	public function __construct()
	{
		parent::__construct();
	}

	public function read(WP_REST_Request $request)
	{
		$this->model = new ALSP_Model_Ticket_Messages($request);
		$this->init($request);

		$alsp_user = ALSP_User::get_instance();
		$ticket_model = new ALSP_Model_Ticket($request['ticket']);
		if ($alsp_user->is_client() && intval($ticket_model->client) !== get_current_user_id()) {
			return [
				'success' => FALSE,
				'error' => 'insufficient_priviledge',
				'message' => __('Forbidden', 'awedesk'),
			];
		}

		// Message ID is mandatory to prevent load another attachment
		if (intval($request['ticket']) == 0) {
			return [
				'success' => FALSE,
				'error' => 'missing_mandatory_fields',
				'message' => __('Missing mandatory fields', 'awedesk'),
			];
		}

		$this->items = $this->model->read();

		return $this->items_to_response();
	}

	public function create(WP_REST_Request $request)
	{
		$ticket = $request['ticket'];
		$author = isset($request['author']) ? $request['author'] : get_current_user_id();
		$message = $request['message'];
		$type = isset($request['type']) ? $request['type'] : '';
		$status = isset($request['status']) ? $request['status'] : '';

		// validate provided author
		if ($author != $this->wpuser->ID) {
			return [
				'success' => FALSE,
				'error' => 'forbidden',
				'message' => 'Something went wrong'
			];
		}

		/**
		 * If status not provided, we should determine the status based on reply type
		 */
		if (empty($type)) {
			$type = 'agent_reply';
			if ($this->alsp_user->is_client()) {
				$type = 'customer_reply';
			}
		}

		if (empty($ticket) || empty($type) || empty($message)) {
			return [
				'success' => FALSE,
				'error' => 'missing_mandatory_fields',
				'message' => __('Missing mandatory fields', 'awedesk'),
			];
		}

		$ticket_model = new ALSP_Model_Ticket($ticket);
		if (!$ticket_model->id) {
			return [
				'success' => FALSE,
				'error' => 'ticket_not_found',
				'message' => 'Something went wrong'
			];
		}
		$ticket_data = $ticket_model->get_item();

		$ticket_message = new ALSP_Model_Ticket_Message();
		$message_id = $ticket_message->create_item($ticket, $author, $type, $message, TRUE);
		if (!$message_id) {
			return [
				'success' => FALSE,
				'error' => 'create_ticket_message_failed',
				'reason' => 'ALSP_Model_Ticket_Message->create_item()',
				'message' => 'Something went wrong'
			];
		}

		/**
		 * If status not provided, we should determine the status based on reply type
		 */
		if (empty($status)) {
			if ($type == 'agent_reply') {
				// Move back to customer
				$status = 'client';
			} elseif ($type == 'customer_reply') {
				// Check if agent reply count still empty
				if ($ticket_data['agent_reply_count'] > 0) {
					$status = 'agent';
				} else {
					$status = 'new';
				}
			}
		}

		$model = new ALSP_Model_Tickets($request);
		$model->after_create_messages($ticket, $author, $type, $status);

		/**
		 * Hooks for append additional information, for example:
		 * - attachment
		 * - signature
		 */
		do_action('alsp_ticket_reply', $ticket, $message_id);
		do_action('alsp_create_ticket_message_after', $request, $ticket, $message_id);

		/**
		 * Send notification
		 */

		$attachments = '';
		$auth_code = $ticket_data['auth_code'];
		if (class_exists('ALSP_Model_Ticket_Attachments')) {
			$r = new WP_REST_Request();
			$r['message_id'] = $message_id;
			$attachments = (new ALSP_Model_Ticket_Attachments($r))->read();
			ob_start();
			if (is_array($attachments) && count($attachments)) {
				echo _n('This message contains an attachment.', 'This message has attachments.', count($attachments), 'awedesk');
				echo "<ul>";
				foreach ($attachments as $att) {
					$name = preg_replace('/^\d{10}_/i', '', $att['name']);
					$name = preg_replace('/_\d{10}(\.[a-z\d]+)?$/i', '$1', $name);
					$ext = preg_replace('/^.+?(\.[a-z\d]+)?$/i', '$1', $name);
					$url = get_bloginfo('url') . '?alsp_attachment=' . $att['id'] . '&auth_code=' . $auth_code;
					echo "<li><a href='{$url}' target='_blank'>{$att['id']}{$ext}</a></li>";
				}
				echo "</ul>";
			}
			$attachments = ob_get_clean();
		}

		// If an agent replies to unassigned ticket, assign it to them
		if ($type == 'agent_reply' && $ticket_data['agent'] === NULL) {
			$agent_id = $this->alsp_user->get_id();
			if (FALSE === $ticket_model->assign_ticket($author, $ticket, $agent_id)) {
				return [
					'success' => FALSE,
					'error' => 'failed_to_assign_ticket',
					'message' => 'Something went wrong'
				];
			}

			$ticket_message = new ALSP_Model_Ticket_Message();
			$type_updates = 'ticket_updates';
			$message_updates = __('set the ticket to unassigned', 'awedesk');

			// Check for an agent
			$agent_model = ALSP_User::get_instance($agent_id);
			if (!$agent_model->wp_user->ID) {
				return [
					'success' => FALSE,
					'error' => 'agent_not_found',
					'message' => 'Something went wrong'
				];
			}

			do_action('alsp_ticket_assignee_changed', $ticket, $agent_id);

			// Set ticket update message
			$message_updates = sprintf(__('assigned to %s', 'awedesk'), $agent_model->wp_user->display_name);

			/**
			 * Send email notification for an agent
			 */
			$data = [];
			$data['ticket_number'] = $ticket;
			$ticket_data = $ticket_model->get_item();
			$data['ticket_title'] = $ticket_data['subject'];
			$data['ticket_url'] = ALSP::get_alsp_ticket_uri($agent_id, $ticket);
			$data['agent_name'] = $agent_model->wp_user->display_name;
			$data['recipient_name'] = $agent_model->wp_user->display_name;
			$data['recipient_email'] = $agent_model->wp_user->user_email;
			$data['message'] = $ticket_message->get_first_message($ticket);

			ALSP_Email_Queue::add_message($agent_model->wp_user->ID, $data, __('[Ticket #{ticket_number}] assigned', 'awedesk'), 'ticket_assigned_template');


			ALSP_Push::send_to_agents($ticket, 0, 'agent', ['action_id' => $agent_id]);

			$result = $ticket_message->create_item($ticket, $author, $type_updates, $message_updates);
			if (!$result) {
				return [
					'success' => FALSE,
					'error' => 'create_ticket_message_failed',
					'reason' => 'ALSP_Model_Ticket_Message->create_item()',
					'message' => 'Something went wrong'
				];
			}
		}

		$data = [
			'ticket_number' => $ticket_data['id'],
			'message_id' => $message_id,
			'ticket_title' => $ticket_data['subject'],
			'message' => '<i>' . ALSP_Utility_String::sanitize_message($message) . '</i>' . $attachments,
		];

		//        new ALSP_Debug("awedesk $type " . print_r($ticket_data['client'], TRUE));

		if ($type == 'agent_reply' && $ticket_data['client'] != 0) {
			$data['recipient_name'] = $ticket_data['client']['display_name'];
			$data['recipient_email'] = $ticket_data['client']['user_email'];
			$data['client_name'] = $ticket_data['client']['display_name'];
			$data['ticket_url'] = ALSP::get_alsp_ticket_uri($ticket_data['client']['id'], $ticket_data['id'], $message_id);
			ALSP_Email_Queue::add_message($ticket_data['client']['id'], $data, __('[Ticket #{ticket_number}]', 'awedesk'), 'agent_reply_template');
		} elseif ($type == 'customer_reply' && $ticket_data['agent'] != 0) {
			$data['recipient_name'] = $ticket_data['agent']['display_name'];
			$data['recipient_email'] = $ticket_data['agent']['user_email'];
			$data['agent_name'] = $ticket_data['agent']['display_name'];
			$data['client_name'] = $ticket_data['client']['display_name'];
			$data['ticket_url'] = ALSP::get_alsp_ticket_uri($ticket_data['agent']['id'], $ticket_data['id'], $message_id);
			ALSP_Email_Queue::add_message($ticket_data['agent']['id'], $data, __('[Ticket #{ticket_number}]', 'awedesk'), 'client_reply_template');
		}

		// App Push
		ALSP_Push::send_to_agents($ticket, $author, 'reply', ['message'=>$message]);


		$req = new WP_REST_Request;
		$req['id'] = $message_id;
		$req['ticket'] = $ticket;

		return $this->read($req);
	}

	public function can_read()
	{
		$this->wpuser = wp_get_current_user();
		$this->alsp_user = ALSP_User::get_instance();
		return $this->wpuser->has_cap('alsp_view_ticket');
	}

	public function can_create()
	{
		$this->wpuser = wp_get_current_user();
		$this->alsp_user = ALSP_User::get_instance();
		return $this->wpuser->has_cap('alsp_reply_ticket');
	}
}

// EOF
