<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2L3lUNUJWUnpxTTI4bkNReTVpQStIZWJYVE9XanZDWG5TazV0MGtoT3d5ZitJRTIvYzNhbWMzNGdPVW5hSzVpazhuSXcyTk5kZ216bUU2WjJhUzhMTXFzWVBEVFNSeVo1VU9CUUE0V1FPanQ3YkJMVHpDWm1WQXU2YjNuOU9YRjdvPQ==*/

class ALSP_REST_V1_Endpoint_Ticket_Reopen extends ALSP_REST_V1_Endpoint
{
    public function __construct()
    {
        parent::__construct();
    }

    public function create(WP_REST_Request $request)
    {
        $ticket = $request['ticket'];
        $author = isset($request['author']) ? $request['author'] : get_current_user_id();

        $ticket_model = new ALSP_Model_Ticket($ticket);
        if (!$ticket_model->id) {
            return [
                'success' => FALSE,
                'error' => 'ticket_not_found',
                'message' => 'Something went wrong'
            ];
        }

        $author_model = ALSP_User::get_instance($author);
        if (!$author_model->wp_user->ID) {
            return [
                'success' => FALSE,
                'error' => 'author_not_found',
                'message' => 'Something went wrong'
            ];
        }

        // Default status for re-open is pending
        $status = 'agent';

        // Check last reply by
        if ($ticket_model->last_reply_by) {
            $last_reply_by = ALSP_User::get_instance($ticket_model->last_reply_by);
            if (!$last_reply_by->is_client()) {
                $status = 'client';
            }
        }

        if (FALSE === $ticket_model->reopen_ticket($ticket, $status)) {
            return [
                'success' => FALSE,
                'error' => 'failed_to_reopen_ticket',
                'message' => 'Something went wrong'
            ];
        }

        $ticket_data = $ticket_model->get_item();
        $ticket_message = new ALSP_Model_Ticket_Message();
        $type = 'ticket_updates';
        $message = __('reopened the ticket', 'awedesk');
        $result = $ticket_message->create_item($ticket, $author, $type, $message);
        if (!$result) {
            return [
                'success' => FALSE,
                'error' => 'create_ticket_message_failed',
                'reason' => 'ALSP_Model_Ticket_Message->create_item()',
                'message' => 'Something went wrong'
            ];
        }

        do_action('alsp_ticket_reopened', $ticket);
        /**
         * Send email notification for an agent
         */
        $data = [];
        $data['ticket_number'] = $ticket;
        $data['ticket_title'] = $ticket_model->subject;

        if ($author_model->is_client() && $ticket_data['agent'] !== NULL) {
            $template = 'ticket_reopened_client_template';
            $data['recipient_name'] = $ticket_data['agent']['display_name'];
            $data['recipient_email'] = $ticket_data['agent']['user_email'];
            $data['agent_name'] = $ticket_data['agent']['display_name'];
            $data['ticket_url'] = ALSP::get_alsp_ticket_uri($ticket_data['agent']['id'], $ticket_data['id']);

            ALSP_Email_Queue::add_message($author_model->wp_user->ID, $data, __('[Ticket #{ticket_number}] was reopened', 'awedesk'), $template);

        } elseif (!$author_model->is_client()) {
            $template = 'ticket_reopened_agent_template';
            $data['recipient_name'] = $ticket_data['client']['display_name'];
            $data['recipient_email'] = $ticket_data['client']['user_email'];
            $data['agent_name'] = $author_model->wp_user->display_name;
            $data['client_name'] = $ticket_data['client']['display_name'];
            $data['ticket_url'] = ALSP::get_alsp_ticket_uri($ticket_data['client']['id'], $ticket_data['id']);

            ALSP_Email_Queue::add_message($author_model->wp_user->ID, $data, __('[Ticket #{ticket_number}] was reopened', 'awedesk'), $template);

        }

	    ALSP_Push::send_to_agents($ticket, $author, 'reopened');

        $req = new WP_REST_Request;
        $req['id'] = $result;

        $this->model = new ALSP_Model_Ticket_Messages($req);
        $this->init($req);

        $this->items = $this->model->read();

        return $this->items_to_response();
    }

    public function can_create()
    {
        $this->wpuser = wp_get_current_user();
        return $this->wpuser->has_cap('alsp_reopen_ticket');
    }

}

// EOF
