<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OC9oZEF4ak03ZUR6T1dpQWVzelZlMWo3aHpydE5DK2ZRY0I2cFpWM2VnbUc0bmdOdVNFK091ZkFyUDR5SnZ4TVN1dUF0c1k1MWFqYnoxNHJiM1NJeXRhZVczVVo0eGk1V2I2K0piQ00yT0o3RnR5T2NLb3RMckprakQrNzN3cFo4PQ==*/
class ALSP_REST_V1_Endpoint_Tickets extends ALSP_REST_V1_Endpoint
{
	public function __construct()
	{
		parent::__construct();
	}

	public function read(WP_REST_Request $request)
	{
		$this->model = new ALSP_Model_Tickets($request);
		$this->init($request);

		if ($request['id']) {
			(ALSP_Recent_Tickets::get_instance())->insert_or_update($request['id']);
		}

		$this->items = $this->model->read();

		return $this->items_to_response();
	}

	public function create(WP_REST_Request $request)
	{
		$client = $request['client'];
		$author = $request['author'];
		$agent = isset($request['agent']) ? $request['agent'] : NULL; // it can be auto assign to an agent
		$department = isset($request['department']) ? $request['department'] : NULL; // it can be not provided
		$subject = $request['subject'];
		$secret = $request['secret'] ?? "";
		$message = $request['message'];
		$status = isset($request['status']) ? $request['status'] : 'new';

		// sanitize subject and message
		$subject = sanitize_text_field(strip_tags($subject));

		// respect maximum characters length configuration
		$subject = substr($subject, 0, ALSP_Config::get('chars_limit_subject_number', 150));

		$on_behalf = FALSE;
		$authorized = TRUE;


		if($author!=get_current_user_id()) {
			// Author must be the current user
			$authorized = FALSE;
		} else if($client!=$author) {
			// If author is not the same as client, the author must be a staff member
			if(!$this->alsp_user->can_open_on_behalf()) {
				$authorized = FALSE;
			}

			$on_behalf = TRUE;
		}

		if(!$authorized) {
			return [
				'success' => FALSE,
				'error' => 'unauthorized',
				'message' => 'Unauthorized'
			];
		}

		if ( empty($message) || empty($subject) ) {
			return [
				'success' => FALSE,
				'error' => 'missing_mandatory_fields',
				'message' => 'Missing mandatory fields',
			];
		}


		// If it's a guest, then create a wp user with role subscriber based on provided email
		if (!$this->wpuser->ID || $client==0) {
			// Email validation
			$email = isset($request['email']) ? $request['email'] : '';
			if (!$email || !is_email($email)) {
				return [
					'success' => FALSE,
					'error' => 'invalid_email_address',
					'message' => __('Invalid email', 'awedesk'),
				];
			}

			// Check if email is already registered
			$current_user = get_user_by('email', $email);
			$alsp_user_id = FALSE;
			if (!$current_user) {

				if(ALSP_Config::get('login_guest_enabled')) {
					// Create user based on email if it's not registered
					$alsp_user_id = ALSP_User::create_user_from_email($email);
				}

				if (!$alsp_user_id) {
					return [
						'success' => FALSE,
						'error' => 'failed_register_new_user',
						'reason' => 'create_user_from_email()',
						'message' => 'Something went wrong'
					];
				}

				// Logging in the user
				if(!$this->wpuser->ID) {
					wp_clear_auth_cookie();
					wp_set_current_user($alsp_user_id); // Set the current user detail
					wp_set_auth_cookie($alsp_user_id); // Set auth details in cookie

					$author = $alsp_user_id;
					$client = $alsp_user_id;
				} else if ($client==0) {
					$client = $alsp_user_id;
				}
			} else {
				return [
					'success' => FALSE,
					'error' => 'email_already_registered',
					'message' => __('You already have an account. Please log in to continue.', 'awedesk'),
				];
			}

		}


		// Set default assignee
		if ($department !== NULL) {
			$obj_department = new ALSP_Model_Department($department);
			$department_details = $obj_department->get_item();
			if (isset($department_details['id']) && $agent === NULL) {
				$agent = $department_details['default_assignee']['id'] ?? NULL;
			}
		}


		// If ticket is being opened by staff member on behalf of the client
		// and has no department assignee
		// assign it to person opening the ticket
		if(NULL === $agent && $on_behalf) {
			$agent = $author;
		}

		if(NULL !== $agent ) {
			$agent_model = ALSP_User::get_instance($agent);
			if (!$agent_model->wp_user->ID) {
				return [
					'success' => FALSE,
					'error' => 'agent_not_found',
					'message' => 'Something went wrong'
				];
			}
		}

		$ticket = new ALSP_Model_Ticket($request);

		$ticket_id = $ticket->create_item($client, $author, $agent, $department, $subject, $status, $secret, 0, 1);
		if (!$ticket_id) {
			return [
				'success' => FALSE,
				'error' => 'create_ticket_failed',
				'reason' => 'ALSP_Model_Ticket->create_item()',
				'message' => 'Something went wrong'
			];
		}

		$ticket_message = new ALSP_Model_Ticket_Message($request);
		$message_id = $ticket_message->create_item($ticket_id, $author, 'customer_reply', $message);
		if (!$message_id) {
			return [
				'success' => FALSE,
				'error' => 'create_ticket_message_failed',
				'reason' => 'ALSP_Model_Ticket_Message->create_item()',
				'message' => 'Something went wrong'
			];
		}

		/**
		 * Hooks for append additional information, for example:
		 * - attachment
		 * - signature
		 */
		do_action('alsp_ticket_new', $ticket_id, $message_id);
		do_action('alsp_create_ticket_message_after', $request, $ticket_id, $message_id);

		/**
		 * Send notification new ticket submission to a manager
		 */
		$data = [
			'ticket_number' => $ticket_id,
			'ticket_title' => $subject,
			'message' => ALSP_Utility_String::sanitize_message($message),
		];
		$list_manager = ALSP_User::get_users_by_role(ALSP_User::ROLE_MANAGER);
		if (count($list_manager) > 0) {
			foreach ($list_manager as $manager_id) {
				$manager = ALSP_User::get_instance($manager_id);

				$data['recipient_name'] = $manager->wp_user->display_name;
				$data['recipient_email'] = $manager->wp_user->user_email;
				$data['manager_name'] = $manager->wp_user->display_name;
				$data['ticket_url'] = ALSP::get_alsp_ticket_uri($manager->wp_user->ID, $ticket_id);
				$data['ticket_subject'] = $data['ticket_title'];
				ALSP_Email_Queue::add_message($manager_id, $data, sprintf(__('[Ticket #%d] was submitted', 'awedesk'), $ticket_id), 'new_ticket_template');
			}
		}

		$client_model = ALSP_User::get_instance($client);
		$data['recipient_name'] = $client_model->wp_user->display_name;
		$data['recipient_email'] = $client_model->wp_user->user_email;
		$data['client_name'] = $client_model->wp_user->display_name;
		$data['ticket_url'] = ALSP::get_alsp_ticket_uri($client_model->wp_user->ID, $ticket_id);
		ALSP_Email_Queue::add_message($client_model->wp_user->ID, $data, sprintf(__('[Ticket #%d] was submitted', 'awedesk'), $ticket_id), 'submission_confirmation_template');

		// Send email notification to an agent
		if ($agent !== NULL) {
			$data = [];
			$data['ticket_number'] = $ticket_id;
			$data['ticket_title'] = $subject;
			$data['ticket_url'] = ALSP::get_alsp_ticket_uri($agent_model->wp_user->ID, $ticket_id);
			$data['recipient_name'] = $agent_model->wp_user->display_name;
			$data['recipient_email'] = $agent_model->wp_user->user_email;
			$data['agent_name'] = $agent_model->wp_user->display_name;
			$data['message'] = ALSP_Utility_String::sanitize_message($ticket_message->get_first_message($ticket_id));

			ALSP_Email_Queue::add_message($agent_model->wp_user->ID, $data, __('[Ticket #{ticket_number}] assigned', 'awedesk'), 'ticket_assigned_template');

			$ticket_data = [ 'id' => $ticket_id, 'subject' => $subject];
			ALSP_Push::send_to_agents($ticket_id, 0, 'agent', ['action_id'=>$agent_model->wp_user->ID]);
		}


		$req = new WP_REST_Request;
		$req['id'] = $ticket_id;

		return $this->read($req);
	}

	public function edit(WP_REST_Request $request)
	{
		$action = $request['action'];
		$id = $request['id'];
		$author = isset($request['author']) ? $request['author'] : get_current_user_id();

		$author_model = ALSP_User::get_instance($author);
		if (!$author_model->wp_user->ID) {
			return [
				'success' => FALSE,
				'error' => 'author_not_found',
				'message' => 'Something went wrong'
			];
		}

		if ($action == 'update_prop') {
			$prop = $request['prop'];
			$value = $request['value'];



			// sanitize first name and last name
			$prop = sanitize_text_field(strip_tags($prop));
			$value = sanitize_text_field(strip_tags($value));

			$ticket_model = new ALSP_Model_Ticket($id);
			if (!$ticket_model->id) {
				return [
					'success' => FALSE,
					'error' => 'ticket_not_found',
					'message' => 'Something went wrong'
				];
			}

			if($prop == 'status') {
				$old_status = $ticket_model->status;
			}

			if (FALSE === $ticket_model->set($prop, $value)) {
				return [
					'success' => FALSE,
					'error' => 'failed_to_update_ticket_property',
					'reason' => 'ALSP_Model_Ticket_Message->create_item()',
					'message' => 'Something went wrong'
				];
			}

			if ($prop == 'department') {

				do_action('alsp_ticket_department_changed', $id, $value);
				$ticket_data = $ticket_model->get_item();

				// Check if current agent have access to new department
				// Then, we should unassign the ticket.
				if ($ticket_data['agent']!=0) {
					error_log(json_encode($ticket_data['agent']));
					$agent_department_model = new ALSP_Model_Agent_Department(NULL);
					$agent = ALSP_User::get_instance($ticket_data['agent']['id']);

					if (!in_array($agent->get('role'), [ALSP_User::ROLE_ADMIN, ALSP_User::ROLE_MANAGER]) && $agent_department_model->is_agent_registered($ticket_data['agent']['id'], $value) === FALSE) {
						$ticket_data['agent'] = NULL;
						if (FALSE === $ticket_model->assign_ticket($author, $id, NULL)) {
							return [
								'success' => FALSE,
								'error' => 'failed_to_unassign_ticket',
								'message' => 'Something went wrong'
							];
						}

						do_action('alsp_ticket_assignee_changed', $id, $author);
					}
				}

				if ($ticket_data['agent'] == 0) {
					$obj_department = new ALSP_Model_Department($value);
					$department_details = $obj_department->get_item();
					$agent = $department_details['default_assignee']['id'] ?? NULL;

					if ($agent !== NULL) {
						$agent_model = ALSP_User::get_instance($agent);
						if (!$agent_model->wp_user->ID) {
							return [
								'success' => FALSE,
								'error' => 'agent_not_found',
								'message' => 'Something went wrong'
							];
						}

						if (FALSE === $ticket_model->assign_ticket($author, $id, $agent)) {
							return [
								'success' => FALSE,
								'error' => 'failed_to_assign_ticket',
								'message' => 'Something went wrong'
							];
						}

						do_action('alsp_ticket_assignee_changed', $id, NULL);

						$ticket_message = new ALSP_Model_Ticket_Message();
						$type = 'ticket_updates';
						$message = sprintf(__('assigned to %s', 'awedesk'), $agent_model->wp_user->display_name);
						$result = $ticket_message->create_item($id, $author, $type, $message);
						if (!$result) {
							return [
								'success' => FALSE,
								'error' => 'create_ticket_message_failed',
								'reason' => 'ALSP_Model_Ticket_Message->create_item()',
								'message' => 'Something went wrong'
							];
						}

						/**
						 * Send email notification for an agent
						 */
						$data = [];
						$data['ticket_number'] = $id;
						$data['ticket_url'] = ALSP::get_alsp_ticket_uri($agent_model->wp_user->ID, $id);
						$data['recipient_name'] = $agent_model->wp_user->display_name;
						$data['recipient_email'] = $agent_model->wp_user->user_email;
						$data['agent_name'] = $agent_model->wp_user->display_name;
						$data['message'] = ALSP_Utility_String::sanitize_message($ticket_message->get_first_message($id));
						ALSP_Email_Queue::add_message($agent_model->wp_user->ID, $data, __('[Ticket #{ticket_number}] assigned', 'awedesk'), 'ticket_assigned_template');

						ALSP_Push::send_to_agents($id, 0, 'agent', ['action_id'=>$agent_model->wp_user->ID]);
					}
				}
			}

			// change ticket updates message
			$ticket_message = new ALSP_Model_Ticket_Message();
			$type = 'ticket_updates';
			$message = sprintf(__('updated the ticket %s', 'awedesk'), $prop);

			if ($prop == 'prevent_autoclose') {
				$message = (intval($value) === 1) ? __('disabled auto-close', 'awedesk') : __('enabled auto-close', 'awedesk');
			}

			if ($prop == 'prevent_clientclose') {
				$message = (intval($value) === 1) ? __('disabled client close', 'awedesk') : __('enabled client close', 'awedesk');
			}

			if($prop == 'prevent_reopen') {
				$message = (intval($value) === 1) ? __('disabled client reopening', 'awedesk') : __('enabled client reopening', 'awedesk');
			}

			if($prop == 'department') {
				$department = new ALSP_Model_Department($value);
				$department = $department->get_item();
				$department = $department['name'];
				$message = sprintf(__('moved to %s', 'awedesk'), $department);
				ALSP_Push::send_to_agents($id, $author, 'department', ['action_id'=>$value, 'action_label'=>$department]);
			}

			if($prop == 'status') {
				// Trigger notifications if something actually changed
				if($old_status != $value) {
					if($value =='closed') {
						// Ticket was closed
						$message = __('closed the ticket', 'awedesk');
						ALSP_Push::send_to_agents($id, $author, 'closed');

					} elseif($old_status =='closed') {
						// Ticket was reopened
						$message = __('reopened the ticket', 'awedesk');
						ALSP_Push::send_to_agents($id, $author, 'reopened');
					} else {
						// Other status change
						$statuses = ALSP_Model_Ticket_Status::list();
						$filtered = array_filter($statuses, fn($status) => $status['status'] === $value);
						$status =  !empty($filtered) ? reset($filtered)['label'] : $value;
						$message = sprintf(__('changed status to %s', 'awedesk'), $status);
						ALSP_Push::send_to_agents($id, $author, 'status', ['action_id' => $value, 'action_label'=>$status]);
					}
				}
			}


			$result = $ticket_message->create_item($id, $author, $type, $message);
			if (!$result) {
				return [
					'success' => FALSE,
					'error' => 'create_ticket_message_failed',
					'reason' => 'ALSP_Model_Ticket_Message->create_item()',
					'message' => 'Something went wrong'
				];
			}

			//@TODO: should we send an email notifications.
		} else {
			return [
				'success' => FALSE,
				'error' => 'unknown_action',
				'reason' => 'ALSP_REST_V1_Endpoint_Tickets->edit()',
				'message' => 'Something went wrong'
			];
		}

		$req = new WP_REST_Request;
		$req['id'] = $id;

		$this->model = new ALSP_Model_Ticket_Messages($req);
		$this->init($req);

		$this->items = $this->model->read();

		return $this->items_to_response();
	}

	public function delete(WP_REST_Request $request)
	{
		if($this->alsp_user->is_staff()) {
			ALSP_Tickets::process_empty_trash(TRUE);
		}

		return [
			'success' => TRUE,
			'class' => __CLASS__,
			'method' => __METHOD__,
			'message' => __('Done!', 'awedesk'),
		];
	}

	public function can_read()
	{
		$can_access = FALSE;
		//@TODO: somehow like $this->request not always return a valid value,
		// since the WP REST API expected to run twice [https://github.com/WP-API/WP-API/issues/2400].
		// We need to get directly from the $_GET param.
		$id = isset($_GET['id']) ? $_GET['id'] : FALSE;
		$this->wpuser = wp_get_current_user();
		if (!$id) {
			$can_access = $this->wpuser->has_cap('alsp_list_ticket');
		} else {
			$can_access = $this->wpuser->has_cap('alsp_view_ticket');
		}

		if ($this->alsp_user->is_client() && $id) {
			$ticket_model = new ALSP_Model_Ticket($id);
			if (!$ticket_model->id) {
				$can_access = FALSE;
			} else {
				$ticket_data = $ticket_model->get_item();
				if (intval($ticket_data['client']['id']) !== $this->wpuser->ID) {
					$can_access = FALSE;
				}
			}
		}

		return $can_access;
	}

	public function can_create()
	{
		$can_access = FALSE;
		// Allow ticket submission for non-loggedin.
		$this->wpuser = wp_get_current_user();
		if (!$this->wpuser->ID) {
			$can_access = TRUE;
		} else {
			$can_access = $this->wpuser->has_cap('alsp_create_ticket');
		}

		return $can_access;
	}

	public function can_edit()
	{
		$can_access = FALSE;
		$this->wpuser = wp_get_current_user();
		$this->alsp_user = ALSP_User::get_instance();

		// Allow ticket submission for non-loggedin.
		if (!$this->wpuser->ID) {
			$can_access = TRUE;
		} else {
			$can_access = $this->wpuser->has_cap('alsp_assign_ticket');
		}

		return $can_access;
	}

	public function can_delete() {
		$this->alsp_user = ALSP_User::get_instance();
		return $this->alsp_user->is_staff();
	}
}

// EOF
