<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2K3UwOHFORGlNOC9xTm40SjFLd0l4YjlpR1cyNW1KNWdjL0R3SVVqMENuL3ZOUWhHN21vc3RKbTRpQXo4VWpFSGc5YVdCeFZxOXNPU3YxRXRjVnQwS2pYWlR5UzBnOGUzRnhZUzhTemh4NXl3WktEZ003QTZNZlFVRDBQeDk2MWdRPQ==*/

class Awecloud_Client
{

    private static $instance = NULL;
    private $superadmin = FALSE;
    private $menu = [];
    private $allowlist = [];

    public static function get_instance()
    {
        return (NULL == self::$instance) ? self::$instance = new self() : self::$instance;
    }

    private function __construct()
    {
        if (defined('DOING_AJAX') && DOING_AJAX) { return; }

        if (1 == get_current_user_id()) { $this->superadmin = TRUE; }

        $this->menu = [

            'Configuration' => [
                'icon'=>'gears',
                'url'=> admin_url('/'),
                'active' => [
                    'wp-admin',
                ],
                'page' => '',
            ],
            'Departments' =>
                [
                    'icon'=>'folder-open',
                    'url' => admin_url('admin.php?page=alsp-department'),
                    'active' => [
                    ],
                    'page' => 'alsp-department',
                ],
            'Agents' =>
                [
                    'icon'=>'user-check',
                    'url' => admin_url('admin.php?page=alsp-agent'),
                    'active' => [
                    ],
                    'page' => 'alsp-agent',
                ],
            'Users' =>
                [
                    'icon'=>'users',
                    'url' => admin_url('users.php'),
                    'active' => [
                        'users.php',
                        'profile.php',
                        'user-new.php',
                        'user-edit.php',
                    ],
                    'page' => '',
                ]
        ];

        foreach($this->menu as $args) {
            foreach($args['active'] as $url) {
                $url_parts = parse_url($url);
                $this->allowlist[] = basename($url_parts['path']);
            }
        }
        // CSS, JS and small modifications
        $this->enqueue_styles_and_mods();


        // If user_id is 1, do not modify wp-admin
        if ($this->superadmin) {
            return;
        }

        // Admin mods
        $this->mod_admin_hide();
        $this->mod_admin_redirect();

        // Remove fields from WP native user pages
        $this->mod_admin_profile();
        $this->mod_admin_users_list();

        $this->mod_example_client();
    }

    public function mod_example_client() {
        add_filter('get_avatar_url', function ($url, $id_or_email, $args) {
            if(!is_numeric($id_or_email) || $id_or_email < 10) {
                return $url;
            }
            $user = get_user_by('id', $id_or_email);
            if($user->user_email == 'example-client@awedesk.com') {
                $url = 'https://awedesk.com/app-assets/example-user.png';
            }
            return $url;
        },10,3);
    }

    public function enqueue_styles_and_mods()
    {
        add_action('admin_enqueue_scripts', function () {
            if ($this->superadmin)
                return;
            wp_enqueue_style('awedesk-admin', ALSP_Frontend_Assets::get_instance()->get_assets_url('css/admin-awedesk.css'), [], '1.0');
        });

        add_action('wp_enqueue_scripts', function () {
            wp_enqueue_style('awedesk-frontend', ALSP_Frontend_Assets::get_instance()->get_assets_url('css/admin-awedesk.css'), [], '1.0');
        });

        add_filter('admin_body_class', function ($c) {
            $c .= ' awedesk-active';
            return $c;
        });

        add_filter('body_class', function ($c) {
            $c[] = ' awedesk-active';
            return $c;
        });

        // Modify the HTML layout of default WP admin pages to enable custom styles
        add_action('admin_head', function () { ?>
            <script>
                jQuery(document).ready(function ($) {

                    $('#add-new-user').addClass('wp-heading-inline');

                    var headingElement = $('.wp-heading-inline');
                    var originalText = headingElement.text().trim();

                    headingElement.html('<i class="alsp-i-users"></i><span>' + originalText + '</span>');

                    var h1Content = $('.wrap h1').clone();
                    var wrapContent = $('.wrap').contents().not('h1');
                    $('.wrap').empty().append(h1Content);
                    $('<div id="awedesk-wrap" class="awe-wrapper"></div>').appendTo('.wrap').append(wrapContent);

                    $('.wrap').css('opacity', 1);
                });
            </script>
            <?php
        });

        // Sync WP options with ALSP config fields
        add_action('alsp_config_set', function ($name, $value) {
            if ('awecloud_blogname_label' == $name) {
                ALSP_Utility_WP::update_option('blogname', $value);
            }

            if ('awecloud_blogtimezone_select' == $name) {

                $gmt_offset = $tz_string = '';

                if ( preg_match( '/^UTC[+-]/', $value ) ) {
                    $gmt_offset = $value;
                    $gmt_offset = preg_replace( '/UTC\+?/', '', $gmt_offset);
                } else {
                    $tz_string = $value;
                }

                ALSP_Utility_WP::update_option('timezone_string', $tz_string);
                ALSP_Utility_WP::update_option('gmt_offset', $gmt_offset);
            }
        }, 10, 2);

        // ALSP frontend data
        add_filter('alsp_frontend_data', function ($data) {
            if (is_array($data)) {
                $data['awedesk'] = [
                    'is_client' => apply_filters('awecloud_is_client', FALSE)
                ];
            }

            return $data;
        }, 10, 1);

        // Rename Subscriber to Client
        if (TRUE) {
            global $wp_roles;

            if (!isset($wp_roles)) {
                $wp_roles = new WP_Roles();
            }

            $wp_roles->roles['subscriber']['name'] = 'Client';
            $wp_roles->role_names['subscriber'] = 'Client';
        }
    }

    public function mod_admin_redirect()
    {
        // Trick redirect
        add_filter('gettext', function ($t, $u, $d) {
            if ("Sorry, you are not allowed to access this page." == $u) {
                wp_redirect(home_url());
                exit();
            }

            return $t;
        }, -1, 3);

        // Modify login page URL
        function custom_login_url($login_url, $redirect, $force_reauth) {
            // Change 'custom-login' to your desired login page slug

        }

        add_filter('login_url', function($login_ur, $redirect, $force_reauth){
            return "/";

            $custom_login_url = home_url('/');
            return esc_url(add_query_arg('redirect_to', urlencode($redirect), $custom_login_url));
        }, PHP_INT_MAX, 3);

        // Regular redirects
        add_action('admin_init', function () {

            // Non-admin
            if (!current_user_can('administrator')) {
                wp_redirect(home_url());
                exit();
            }


            // Admin
            if (current_user_can('administrator')) {
//                echo$this->admin_url_file()."<hr>";
//                var_dump($this->allowlist);
                if (in_array($this->admin_url_file(), $this->allowlist)) {
                    return TRUE;
                }

                // Check if the current page is not the allowed page
                if (!isset($_GET['page']) || substr($_GET['page'], 0, 4) !== 'alsp') {
                    // Redirect to the allowed page
                    wp_redirect(admin_url('/'));
                    exit;
                }
            }
        }, -1);
    }

    public function mod_admin_profile()
    {
        // Disable other fields
        add_action('admin_head', function () {

            add_filter('wp_is_application_passwords_available', '__return_false');
            // Color picker
            remove_action('admin_color_scheme_picker', 'admin_color_scheme_picker');

            // Everything else
            $tr = [
                "tr.user-language-wrap",
                "tr.user-rich-editing-wrap",
                "tr.user-comment-shortcuts-wrap",
                "tr.user-url-wrap",
                "tr.user-syntax-highlighting-wrap",
                "tr.user-admin-bar-front-wrap",
                "tr.user-profile-picture",
                "tr.user-user-login-wrap",
                "tr.user-description-wrap",
                "tr.user-generate-reset-link-wrap",
                "tr.user-nickname-wrap",
                "tr.user-display-name-wrap",
                "tr.user-sessions-wrap",
                "tr.user-role-wrap",
                "h2",
            ];

            $selectors = implode(", ", $tr);
            ?>
            <style>
                <?php echo $selectors; ?>
                {
                    display: none;
                }
            </style>
            <script type="text/javascript">
                jQuery(document).ready(function ($) {
                    // Remove selected <tr>'s
                    $('<?= $selectors; ?>').remove();
                    // Remove any empty table that may have been left over
                    $(".form-table:not(:has(tr))").remove();
                });
            </script>
            <?php
        });

        // Customize "new user" form
        add_action('user_new_form', function () {
            ?>
            <script type="text/javascript">
                jQuery(document).ready(function ($) {
                    $('#url').closest('.form-field').remove();

                    // Replace role dropdown with a hidden field
                    var roleSelect = $('#role');
                    var hiddenRole = $('<input type="hidden" name="role" value="subscriber">');
                    roleSelect.closest('tr').replaceWith(hiddenRole);
                });
            </script>
            <?php
        });
    }

    public function mod_admin_users_list()
    {
        // users.php 100 users page
        add_filter('users_per_page', function ($i) {
            return 100;
        });

        // Remove unnecessary columns, add custom columns, and remove specific actions
        add_filter('manage_users_columns', function ($columns) {
            foreach (['cb', 'posts', 'posts_num', 'role'] as $column_name) {
                if (isset($columns[$column_name])) {
                    unset($columns[$column_name]);
                }
            }

            $columns['registration_date'] = 'Registration Date';
            $columns['source'] = 'Source';

            return $columns;
        });

        // Custom columns: content
        add_filter('manage_users_custom_column', function ($value, $column_name, $user_id) {
            if ($column_name == 'registration_date') {
                $user = get_userdata($user_id);
                $registration_date = $user->user_registered;

                // Customize the date format if needed
                $date_format = ALSP_Utility_WP::get_option('date_format');
                $formatted_date = date_i18n($date_format, strtotime($registration_date));

                return $formatted_date;
            } elseif ($column_name == 'source') {
                $source = get_user_meta($user_id, 'alsp_source', true);

                // Default to "registration" if the source meta is empty
                $source = !empty($source) ? $source : 'registration';

                return ucfirst(str_replace('_', ' ', $source));
            }

            return $value;
        }, 10, 3);

        // Custom columns: sortable
        add_filter('manage_users_sortable_columns', function ($columns) {
            $columns['registration_date'] = 'user_registered';
            $columns['source'] = 'alsp_source';
            return $columns;
        });

        // Remove actions from user row
        add_filter('user_row_actions', function ($actions, $user_object) {
            foreach (['edit', 'view', 'delete', 'resetpassword'] as $action_name) {
                if (isset($actions[$action_name])) {
                    unset($actions[$action_name]);
                }
            }

            return $actions;
        }, PHP_INT_MAX, 2);

        // Set alsp_source to "added_by_admin" when admin adds a user manually
        add_action('user_register', function ($user_id) {
            if (current_user_can('administrator')) {
                update_user_meta($user_id, 'alsp_source', 'added_by_admin');
            }
        });
    }

    public function mod_admin_hide()
    {
        // Hide admin bar and admin notices
        remove_all_actions('admin_notices');
        add_action('admin_notices', function () {
            ob_start();
            ?>
            <div id="awedesk-user-header" class="alsp-wrapper awe-header__wrapper">
                <div class="alsp-header awe-header awe-header--<?php echo $atts['context']; ?>">
                    <div class="alsp-header__inner awe-header__inner">
                        <h1>
                            <a href="<?php echo esc_url(home_url()); ?>">
                                <?php if ($custom_logo_url = wp_get_attachment_url(ALSP_Config::get('awecloud_bloglogo_image', 0))) { ?>
                                    <img src="<?php echo esc_url($custom_logo_url); ?>"
                                         alt="<?php echo esc_html(get_bloginfo('name')); ?>">
                                <?php }

                                echo esc_html(get_bloginfo('name')); ?>
                            </a>
                        </h1>

                        <div class="alsp-header__menu awe-header__menu">
                            <?php


                            foreach($this->menu as $label=>$args) {
                                $active = in_array($this->admin_url_file(), $args['active']) ? TRUE : FALSE;
                                if(!$active && isset($_GET['page']) && $_GET['page']==$args['page']) {
                                    $active = TRUE;
                                }
                                if($active) {
                                    $label ="$label";
                                    $active = 'active';
                                }
                                echo " <a class=\"$active\"  href=\"{$args['url']}\"><i class=\"alsp-i-{$args['icon']}\"></i> $label</a> ";
                            }
                            ?>
                            <?php if (get_current_user_id()) { ?>
                                <a class="logout" href="/"><i class="alsp-i-sign-out-alt"></i> Tickets</a>
                            <?php } ?>
                        </div>
                    </div>
                </div>
            </div>
            <?php
            // prevent wpautop
            echo str_replace("\n", "", ob_get_clean());
        });

        add_filter('show_admin_bar', '__return_false', PHP_INT_MAX);

        // Modify footer
        add_filter('admin_footer_text', function ($text) {
            return '';
        }, PHP_INT_MAX, 1);

        add_filter('update_footer', function ($footer) {
            return "awedesk " . ALSP::PLUGIN_VERSION;
        }, PHP_INT_MAX, 1);

        // Hide all menu items
        add_action('admin_menu', function () {
            global $menu;

            // List of menu slugs to keep visible
            $allowed_menu_slugs = ['alsp'];

            // Loop through the menu and unset items that are not in the allowed list
            foreach ($menu as $index => $menu_item) {
                $menu_slug = $menu_item[2];
                if (!in_array($menu_slug, $allowed_menu_slugs)) {
                    // Remove menu items not in the allowed list
                    remove_menu_page($menu_slug);
                }
            }
        }, PHP_INT_MAX);

    }

    // Utilities: Parse URL
    public function admin_url_file()
    {
        $current_url = $_SERVER['REQUEST_URI'];
        $url_parts = parse_url($current_url);
        $current_page = basename($url_parts['path']);

        return $current_page;
    }

    public static function get($key, $default = NULL, $check_length = TRUE)
    {
        $value = ALSP_Utility_WP::get_option("awedesk_" . $key, FALSE);

        if (FALSE == $value) {
            return $default;
        }

        if (!strlen($value) && $check_length) {
            return $default;
        }

        return $value;
    }

    public static function set($k, $v)
    {
        ALSP_Utility_WP::update_option("awedesk_" . $k, $v);
    }
}

if (defined('AWECLOUD_CLIENT')) {

    add_filter('awecloud_is_client', function () {
        return (1 != get_current_user_id());
    });

    add_action('init', function () {
        $client = Awecloud_Client::get_instance();
    }, 10);


}
