<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2LzVseVk4L0VaSHRNL2xjMEJ0ZHNITzRCalNNMlMwdUZTa0s4OExoQnNwbGNnOWpUZ25YanFFcVRaSElEZXNJMGJmMUlIMnRHcVdDVTR6dld6ZFB5MDN5enVqS2ZNZVlQdm5oRjlkb1pVb2x4U28veXRVRWF6MTVJc005dmtFMWNZPQ==*/

class AweDesk_DevOps {

    private static $instance = NULL;

    public static function get_instance()
    {
        return (NULL == self::$instance) ? self::$instance = new self() : self::$instance;
    }

    private function __construct()
    {
        if (defined('DOING_AJAX') && DOING_AJAX) { return; }

        add_action('init', function() {
           $this->check_config();
           $this->check_suspension();
        });
    }

    public function check_config() {

        // Config importer
        $config_path = ABSPATH . 'config.json';

        if (file_exists($config_path)) {

            $config = json_decode(file_get_contents($config_path), true);

            if ($config !== null) {

                // Almighty config
                if (isset($config['alsp_config']) && is_array($config['alsp_config'])) {
                    // Iterate through key-value pairs inside alps_config
                    foreach ($config['alsp_config'] as $k => $v) {
                        // Do something with each key-value pair
                        ALSP_Config::set($k, $v);
                        if ('awecloud_blogname_label' == $k) {
                            ALSP_Utility_WP::update_option('blogname', $v);
                        }
                    }
                }

                // DevOps
                if (isset($config['devops']) && is_array($config['devops'])) {

                    foreach ($config['devops'] as $k => $v) {

                        // Suspend
                        if ('suspend' == $k) {
                            new ALSP_Debug("awedesk: suspend");
                            ALSP_Utility_WP::update_option('awedesk_suspend', $v);
                        }

                        // Unsuspend
                        if ('unsuspend' == $k) {
                            new ALSP_Debug("awedesk: unsuspend");
                            ALSP_Utility_WP::delete_option('awedesk_suspend');
                        }

                        // Create new admin & delete all older users
                        if ('new_admin' == $k && is_array($v)) {

                            require_once(ABSPATH . 'wp-admin/includes/user.php');

                            $new_admin_id = wp_create_user($v['username'], $v['password'], $v['email']);

                            if (is_int($new_admin_id)) {

                                new ALSP_Debug("awedesk: new admin $new_admin_id");

                                // Set first name
                                update_user_meta($new_admin_id, 'first_name', $v['first_name']);
                                update_user_meta($new_admin_id, 'last_name', $v['last_name']);
                                wp_update_user(['ID' => $new_admin_id, 'display_name' => $v['display_name']]);

                                $departments = [1, 2, 3]; // @TODO get it from the model/REST
                                // Make them a WP admin
                                $user = new WP_User($new_admin_id);
                                $user->set_role('administrator');

                                // Make them an ALSP admin
                                $alsp_user = ALSP_User::get_instance($new_admin_id);
                                $alsp_user->set_user_role(ALSP_User::ROLE_ADMIN);

                                // Add them as agent
                                $this->ALSP_REST_V1_Endpoint_Admin_Agent($new_admin_id, 'admin', $departments);

                                // Make them the default assignee in all departments
                                foreach ($departments as $department) {
                                    #id: 1
                                    #action: update_prop
                                    #prop: default_assignee
                                    #value: 1
                                    $request = new WP_REST_Request();
                                    $request['id'] = $department;
                                    $request['action'] = 'update_prop';
                                    $request['prop'] = 'default_assignee';
                                    $request['value'] = $new_admin_id;
                                    $alsp_department = new ALSP_REST_V1_Endpoint_Admin_Department();
                                    $alsp_department->edit($request);
                                }

                                // Update ALSP demo content
                                global $wpdb;

                                $now = current_time('mysql', true);

                                $table=$wpdb->prefix."alsp_tickets";
                                $wpdb->query("UPDATE $table SET `agent`=$new_admin_id,  `created`='$now', `updated` ='$now', `last_reply_time_client`='$now', `last_reply_time`='$now'");

                                $table=$wpdb->prefix."alsp_ticket_messages";
                                $wpdb->query("DELETE FROM $table where `type` NOT IN ('customer_reply')");
                                $wpdb->query("UPDATE $table SET `created`='$now' ");

                                // Delete old admins
                                for ($i = 1; $i <= 2; $i++) {
                                    // ALSP downgrade to client and remove access from  departments
                                    $this->ALSP_REST_V1_Endpoint_Admin_Agent($i);

                                    # WP delete user
                                    wp_delete_user($i, $new_admin_id);
                                }
                            } else {
                                new ALSP_Debug("awedesk: adding new admin failed " . print_r($new_admin_id, TRUE));
                            }
                        }

                        // Perform license activation
                        if('activate_license' == $k && strlen($v)) {//test
                            new ALSP_Debug("awedesk: activate license $v");
                            $ALSP_REST_V1_Endpoint_Admin_License = new ALSP_REST_V1_Endpoint_Admin_License();
                            new ALSP_Debug(print_r($ALSP_REST_V1_Endpoint_Admin_License->edd_activate_license($v), true));
                        }

                        if ('truncate_tables' == $k && is_array($v)) {
                            global $wpdb;
                            foreach ($v as $table)
                                ;
                            $wpdb->query("TRUNCATE TABLE {$wpdb->prefix}$table");
                        }

                    }
                }

                // WP options
                if (isset($config['wp_options']) && is_array($config['wp_options'])) {
                    foreach ($config['wp_options'] as $k => $v) {
                        $file = md5(microtime() . $_SERVER['HTTP_HOST']);
                        $alsp_debug_file = ALSP_Utility_WP::get_option('alsp_debug_file');
                        if ('alsp_debug_file' == $k) {
                            @unlink(WP_CONTENT_DIR . DIRECTORY_SEPARATOR . 'alsp' . DIRECTORY_SEPARATOR . $alsp_debug_file . '.log');
                            $v = md5(microtime() . $_SERVER['HTTP_HOST']);
                        }

                        ALSP_Utility_WP::update_option($k, $v);
                    }
                }

                // awedesk Config
                if (isset($config['awedesk_config']) && is_array($config['awedesk_config'])) {
                    foreach ($config['awedesk_config'] as $k => $v) {
                        Awecloud_Client::set($k, $v);
                    }
                }
            }

            unlink($config_path);
        }
    }

    public function check_suspension() {
        // Suspended instance
        $suspended = ALSP_Utility_WP::get_option('awedesk_suspend');

        if (strlen($suspended)) {
            die($suspended);
        }
    }

    // Utilities: ALSP REST API wrappers
    private function ALSP_REST_V1_Endpoint_Admin_Agent($user, $role = 'client', $department = [])
    {
        $request = new WP_REST_Request();
        $request['id'] = $user;
        $request['role'] = $role;
        $request['department'] = $department;
        $alsp_agents = new ALSP_REST_V1_Endpoint_Admin_Agent();
        $alsp_agents->edit($request);
    }
}

if(defined('AWECLOUD_CLIENT')) {
    AweDesk_DevOps::get_instance();
}
