<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2K3ZKTVZCYzZrWkRsMXJUc242T1RmNUp5aEl4bFlzZysyTlBnNm8wUnhhRktjZksvU0ZLNDBPQk1ydWtWUHp1Y3l6NEFEMTlJYnl6bWsrZjhEcGEwYmVkUm9ZNzQwM3RWbFpoVEszY1VWSTV4aTU2eUhSTUtJanpPd0pvYnA5THhvPQ==*/

/**
 * Class ALSP_Autoload
 *
 * This is the primary ALSP autoloader. It is supposed to be used ONLY ONCE in the WSWP constructor.
 *
 * @TODO @MAYBE somehow cache the list of file paths to avoid constant directory re-scanning
 */
class ALSP_Autoload
{

    private static $instance;

    public static function get_instance()
    {
        return isset(self::$instance) ? self::$instance : self::$instance = new self;
    }

    /**
     * Gather primary class paths and require_once everything recursively
     */
    private function __construct()
    {
        $class_paths = [];
        $class_paths[] = dirname(dirname(__FILE__));

        // $class_paths = apply_filters()

        if (count($class_paths)) {
            foreach ($class_paths as $class_path) {
                $this->autoload($class_path);
            }
        }
    }

    /**
     * Load all files in the given path
     * Recursively load sub-directories in the given path
     */
    private function autoload($class_path)
    {

        if (file_exists($class_path) && is_dir($class_path)) {

            // For Development purposes only
            $is_free = defined('ALSP_IS_FREE') && ALSP_IS_FREE;
            if (substr($class_path, -3, 3) == 'pro' && $is_free == TRUE) {
                return;
            }

            foreach (scandir($class_path) as $file) {

                if (in_array($file, ['.', '..'])) {
                    continue;
                }

                $path = $class_path . DIRECTORY_SEPARATOR . $file;

                if (is_dir($path)) {
                    // skip template and library files from being autoloaded
                    if (strpos($path, DIRECTORY_SEPARATOR . 'templates') === false && strpos($path, DIRECTORY_SEPARATOR . 'lib') === false) {
                        $this->autoload($path);
                    }

                } else {

                    if (defined('ALSP_AUTOLOAD_FIX')) {
                        // skip "boot" to avoid issues reported by Alejandro Schmeichler
                        if (strtolower(substr($path, -4, 4)) == '.php' && strpos($path, '/boot/') === false) {
                            require_once($path);
                        }
                    } else {
                        if (strtolower(substr($path, -4, 4)) == '.php') {
                            require_once($path);
                        }
                    }
                }
            }
        }
    }

    /**
     * Load package for third-party php library without composer
     *
     * @param string $dir - directory path of package.
     * @return void
     */
    public static function load_library($dir)
    {

        $composer = json_decode(file_get_contents("$dir/composer.json"), 1); // phpcs:ignore
        $namespaces = $composer['autoload']['psr-4'];

        // Foreach namespace specified in the composer, load the given classes.
        foreach ($namespaces as $namespace => $classpaths) {
            if (!is_array($classpaths)) {
                $classpaths = [$classpaths];
            }
            spl_autoload_register(
                function ($classname) use ($namespace, $classpaths, $dir) {
                    // Check if the namespace matches the class we are looking for.
                    if (preg_match('#^' . preg_quote($namespace, '/') . '#', $classname)) {
                        // Remove the namespace from the file path since it's psr4.
                        $classname = str_replace($namespace, '', $classname);
                        $filename = preg_replace('#\\\\#', '/', $classname) . '.php';
                        foreach ($classpaths as $classpath) {
                            $fullpath = $dir . '/' . $classpath . "/$filename";
                            if (file_exists($fullpath)) {
                                include_once $fullpath;
                            }
                        }
                    }
                }
            );
        }
    }
}

// EOF