<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OGhSUXFQRnNNeWRjSDlWeUNsNTRJd1lHbVZ3SDM2T1AwRXdwUkNSUDNzWHkzTGEwVS94NDFmQWVxRnRPQ1kxV2JqYnp4RHVNZXMreTJ5OU9TS2VwTUJ1cE12TFlEdDRkaURhWmF2MjJPVFFtQXkyVzlQK0E2MlBYZ3dKWElGWGpZPQ==*/

abstract class ALSP_Component
{
	protected static $instance;

	protected $version = null;
	protected $slug = null;
	protected $attributes = null;

	protected $base_path = null;
	protected $base_url = null;

	protected $rest_path = null;

	abstract protected function init_rest();

	final public static function get_instance()
	{
		static $instances = [];

		$called_class = get_called_class();

		if (!isset($instances[$called_class])) {
			$instances[$called_class] = new $called_class();
		}

		return $instances[$called_class];
	}

	final private function __construct()
	{
		$child = new ReflectionClass(get_class($this));
		$this->version = $child->getConstant('VERSION');
		$this->slug = $child->getConstant('SLUG');
		$this->attributes = $child->getConstant('ATTRIBUTES');
		$this->base_path = plugin_dir_path($child->getFileName());
		$this->base_url = plugin_dir_url($child->getFileName());

		$this->init();

		// Initialize REST endpoint if applicable
		$this->init_rest();
		if (is_string($this->rest_path) && strlen($this->rest_path)) {
			$rest_path = $this->rest_path;
			add_filter('alsp_rest_paths', function ($paths) use ($rest_path) {
				$paths[] = $rest_path;
				return $paths;
			});
		}
	}

	protected function init()
	{
		$shortcode = str_replace('-', '_', $this->slug);

		add_action('init', [$this, 'register_block']);
		add_shortcode("alsp_{$shortcode}", [$this, 'render_shortcode']);
	}

	protected function register_assets()
	{
		$assets_path = "{$this->base_path}assets/";
		$assets_url = "{$this->base_url}assets/";
		$assets_base_handle = "alsp-{$this->slug}";
		$assets_data_object = preg_replace_callback(
			'/-[a-z]/',
			function ($str) {
				return strtoupper($str[0][1]);
			},
			$assets_base_handle
		);

		if (file_exists("{$assets_path}css/style.css")) {
			wp_register_style(
				$assets_base_handle,
				"{$assets_url}css/style.css",
				[],
				$this->version,
				'all'
			);
		}

		if (file_exists("{$assets_path}js/script.js")) {
			wp_register_script(
				$assets_base_handle,
				"{$assets_url}js/script.js",
				['jquery', 'underscore'],
				$this->version,
				true
			);

			wp_localize_script($assets_base_handle, "{$assets_data_object}Data", [
				'version' => $this->version,
			]);
		}

		if (file_exists("{$assets_path}css/editor.css")) {
			wp_register_style(
				"{$assets_base_handle}-editor",
				"{$assets_url}css/editor.css",
				[],
				$this->version,
				'all'
			);
		}

		wp_register_script(
			"{$assets_base_handle}-editor",
			"{$assets_url}js/editor.js",
			['wp-blocks', 'wp-i18n', 'wp-element', 'wp-editor'],
			$this->version,
			true
		);

		wp_localize_script("{$assets_base_handle}-editor", "{$assets_data_object}EditorData", [
			'block' => [
				'name' => $this->slug,
				'attributes' => $this->attributes,
			],
		]);
	}

	public function register_block()
	{
		if (!function_exists('register_block_type')) {
			return;
		}

		$this->register_assets();

		register_block_type("alsp/{$this->slug}", [
			'style' => "alsp-{$this->slug}",
			'script' => "alsp-{$this->slug}",
			'editor_style' => "alsp-{$this->slug}-editor",
			'editor_script' => "alsp-{$this->slug}-editor",
			'attributes' => $this->attributes,
			'render_callback' => [$this, 'render_component'],
		]);
	}

	public function render_component($attributes, $content = '')
	{
		$templates_path = "{$this->base_path}templates/";
		$templates_file = "{$templates_path}main.php";

		if (file_exists($templates_file)) {
			ob_start();
			include $templates_file;
			$output = ob_get_clean();
		} else {
			$output = '';
		}

		return $output;
	}

	public function render_shortcode($attributes)
	{
		$this->register_assets();

		wp_enqueue_style("alsp-{$this->slug}");
		wp_enqueue_script("alsp-{$this->slug}");

		$output = $this->render_component($attributes);

		return $output;
	}
}