<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2K21VK0xYdlc3UkdqTzF0QVQzdSt6S0tPUm1OQmFRNjlDcUNGMW54NFprNlVaQkVOTGIzdTMzTGVQMlNkRTJJU3FZMFhDbjJJalJ3MFlsV3ZFcmZIcFdKL2VCWmtDRDJaWGVVSVFIaDg4bEhnPT0=*/

class ALSP_Config
{
    public static function get($name, $default = FALSE, $check_length = FALSE)
    {
        $name = 'alsp_' . $name;

        if (is_multisite()) {
            $id = get_current_blog_id();

            switch_to_blog( $id );
            $value = get_option( $name );
            restore_current_blog();
        } else {
            $value = get_option($name);
        }


        // Unset / empty settings return the default
        if (FALSE === $value) {
	        return apply_filters('awedesk_config_get', $default, $name);
        }

        if (is_array($value) && empty($value)) {
	        return apply_filters('awedesk_config_get', $default, $name);
        }

        // Some empty strings (for example profile slugs) should return the default
        if (!is_array($value) && !strlen($value) && $check_length) {
	        return apply_filters('awedesk_config_get', $default, $name);
        }

        return apply_filters('awedesk_config_get', $value, $name);
    }

    public static function set($name, $value)
    {
        do_action('alsp_config_set', $name, $value);
        $name = 'alsp_' . $name;
        if (is_multisite()) {
            $id = get_current_blog_id();

            switch_to_blog( $id );
            $return = update_option( $name, $value );
            restore_current_blog();
        } else {
            $return = update_option($name, $value);
        }

        return $return;
    }

    public static function add($name, $value)
    {
        do_action('alsp_config_add', $name, $value);
        $name = 'alsp_' . $name;
        if (is_multisite()) {
            $id = get_current_blog_id();

            switch_to_blog( $id );
            $return = add_option( $name, $value );
            restore_current_blog();
        } else {
            $return = add_option($name, $value);
        }

        return $return;
    }

    public static function del($name)
    {
        do_action('alsp_config_del', $name);
        $name = 'alsp_' . $name;
        if (is_multisite()) {
            $id = get_current_blog_id();

            switch_to_blog( $id );
            $return = delete_option( $name );
            restore_current_blog();
        } else {
            $return = delete_option($name);
        }

        return $return;

    }

    public static function admin_config($position = 'left')
    {
        $statuses = ALSP_Model_Ticket_Status::list();
        $statuses_items = [];
        foreach ($statuses as $key => $status) {
            $statuses_items = array_merge($statuses_items, [
                'status_' . $status['status'] . '_separator' => ALSP_Config_Panel::admin_item('status_' . $status['status'] . '_separator', FALSE, __($status['label'], 'awedesk')),
                'status_' . $status['status'] . '_color' => ALSP_Config_Panel::admin_item('status_' . $status['status'] . '_color', FALSE, __('Text color', 'awedesk')),
                'status_' . $status['status'] . '_background_color' => ALSP_Config_Panel::admin_item('status_' . $status['status'] . '_background_color', FALSE, __('Background color', 'awedesk')),
                // 'status_' . $status['status'] . '_message' => ALSP_Config_Panel::admin_item('status_' . $status['status'] . '_message', FALSE, FALSE),
            ]);
        }

        $config = [];
        $config['appearance'] = [
            'id' => 'appearance',
            'title' => __('Appearance', 'awedesk'),
            'icon' => 'alsp-i-brush',
            'position' => 'general-left',
            'items' => [


//                 'style_separator' => ALSP_Config_Panel::admin_item('style_separator', FALSE, __('Style', 'awedesk')),
                 'default_theme_select' => ALSP_Config_Panel::admin_item('default_theme_select', FALSE, __('Default color scheme', 'awedesk'), ['options' => apply_filters('alsp_theme_select', ['light' => __('Light', 'awedesk'), 'dark' => __('Dark', 'awedesk')])]),
                 'light_accent_color' => ALSP_Config_Panel::admin_item('light_accent_color', FALSE, __('Accent color (light)', 'awedesk')),
                 'dark_accent_color' => ALSP_Config_Panel::admin_item('dark_accent_color', FALSE, __('Accent color (dark)', 'awedesk')),

                 'auto_focus_select' => ALSP_Config_Panel::admin_item('auto_focus_select', FALSE, __('Automatic on Focus Mode', 'awedesk'), ['options' => ['never' => __('Never', 'awedesk'), 'mobile' => __('Mobile','awedesk'), 'peepsoapp' => __('PeepSo App', 'awedesk'), 'always' => __('Always', 'awedesk')]]),

                'powered_by_enabled' => ALSP_Config_Panel::admin_item('powered_by_enabled', FALSE,  __('Powered by Awedesk', 'awedesk')),
            ],
        ];

        $pages = ALSP_Config::pages();

        if (count($pages)) {
            $options_client_pages = [];
            foreach ($pages as $page) {
                $options_client_pages[$page['id']] = (isset($page['title']) && strlen($page['title'])) ? $page : $page['label'];
            }

            $options_agent_pages = $options_client_pages;
            $options_agent_pages[''] = __('Clients', 'awedesk');
        } else {
            $options_agent_pages = $options_client_pages = ['-1' => __('No pages found', 'awedesk')];
        }


        if(!apply_filters('awecloud_is_client', FALSE)) {
            $config['navigation'] = [
                'id' => 'navigation',
                'title' => __('Navigation', 'awedesk'),
                'icon' => 'alsp-i-folder-tree',
                'position' => 'general-left',
                'items' => [
                    // separate_support_page_enabled support_page_client_select
                    'support_page_client_select' => ALSP_Config_Panel::admin_item('support_page_client_select', FALSE, __('Clients', 'awedesk'), ['options' => $options_client_pages]),
                    'support_page_agent_select' => ALSP_Config_Panel::admin_item('support_page_agent_select', FALSE, __('Agents', 'awedesk'), ['options' => $options_agent_pages]),
                ],
            ];
        }

//        if (!apply_filters('awecloud_is_client', FALSE)) {
//            $config['lists'] = [
//                'id' => 'lists',
//                'title' => __('Filter Labels', 'awedesk'),
//                'icon' => 'alsp-i-filter',
//                'position' => 'general-left',
//                'items' => [
//                    'tickets_open_separator' => ALSP_Config_Panel::admin_item('tickets_open_separator', FALSE, ALSP_Model_Ticket_Status::default_name('open')),
//                    'tickets_open_label' => ALSP_Config_Panel::admin_item('tickets_open_label', FALSE),
//                    'tickets_mine_separator' => ALSP_Config_Panel::admin_item('tickets_mine_separator', FALSE, ALSP_Model_Ticket_Status::default_name('mine')),
//                    'tickets_mine_label' => ALSP_Config_Panel::admin_item('tickets_mine_label', FALSE),
//                    'tickets_watched_separator' => ALSP_Config_Panel::admin_item('tickets_watched_separator', FALSE, ALSP_Model_Ticket_Status::default_name('watched')),
//                    'tickets_watched_label' => ALSP_Config_Panel::admin_item('tickets_watched_label', FALSE),
//                    'tickets_new_separator' => ALSP_Config_Panel::admin_item('tickets_new_separator', FALSE, ALSP_Model_Ticket_Status::default_name('new')),
//                    'tickets_new_label' => ALSP_Config_Panel::admin_item('tickets_new_label', FALSE),
//                    'tickets_pending_separator' => ALSP_Config_Panel::admin_item('tickets_pending_separator', FALSE, ALSP_Model_Ticket_Status::default_name('pending')),
//                    'tickets_pending_label' => ALSP_Config_Panel::admin_item('tickets_pending_label', FALSE),
//                    'tickets_answered_separator' => ALSP_Config_Panel::admin_item('tickets_answered_separator', FALSE, ALSP_Model_Ticket_Status::default_name('answered')),
//                    'tickets_answered_label' => ALSP_Config_Panel::admin_item('tickets_answered_label', FALSE),
//                    'tickets_closed_separator' => ALSP_Config_Panel::admin_item('tickets_closed_separator', FALSE, ALSP_Model_Ticket_Status::default_name('closed')),
//                    'tickets_closed_label' => ALSP_Config_Panel::admin_item('tickets_closed_label', FALSE),
//                    'tickets_deleted_separator' => ALSP_Config_Panel::admin_item('tickets_deleted_separator', FALSE, ALSP_Model_Ticket_Status::default_name('deleted')),
//                    'tickets_deleted_label' => ALSP_Config_Panel::admin_item('tickets_deleted_label', FALSE),
//                ],
//            ];
//
//            if (!apply_filters('alsp_is_pro', FALSE)) {
//                unset($config['lists']['items']['tickets_watched_separator']);
//                unset($config['lists']['items']['tickets_watched_label']);
//            }
//        }


        if (defined('AWECLOUD_CLIENT')) {

            add_filter('alsp_config_description', function ($arr) {
                $arr['awecloud_blogname_label'] = 'Used as site title in the browser and in the emails.<br>Changes take full effect after page reload.';
                $arr['awecloud_bloglogo_image'] = 'Recommended size: 64x64 square. Recommended format: PNG.';
                return $arr;
            });

            $tz_options = ALSP_Config_Panel::timezone_options();

            $config['awedesk'] = [
                'id' => 'awedesk',
                'title' => 'Instance Settings',
                'icon' => 'alsp-i-gears',
                'position' => 'general-right',
                'items' => [
                    'awecloud_blogname_label' => ALSP_Config_Panel::admin_item('awecloud_blogname_label', FALSE, 'Title'),
                    'awecloud_bloglogo_image' => ALSP_Config_Panel::admin_item('awecloud_bloglogo_image', FALSE, 'Logo'),
                    'awecloud_color' => ALSP_Config_Panel::admin_item('awecloud_color', FALSE, 'Color'),
                    'awecloud_blogtimezone_select' => ALSP_Config_Panel::admin_item('awecloud_blogtimezone_select', FALSE, 'Timezone',['options'=>$tz_options]),
                ],
            ];
        }

        add_filter('alsp_config_description', function ($arr) {
            $arr['recaptcha_secretkey_password'] = sprintf(__( 'Get Google ReCaptcha keys %s','awedesk'), sprintf('<a href="https://www.google.com/recaptcha/admin" target="_blank">%s</a>', __('here','awedesk')))
            . '<br/><br/>'
            . sprintf(__('Please make sure to use %s','awedesk'),'<strong>ReCaptcha : invisible ReCaptcha badge</strong>')
            . '<br/>'
            . sprintf(__('Having issues or questions? Please refer to the %s or contact %s', 'awedesk' ), sprintf('<a href="https://awedesk.com/documentation/" target="_blank">%s</a>',__('documentation','awedesk')), sprintf('<a href="https://awedesk.com/support/" target="_blank">%s</a>', __('Awedesk Support','awedesk')));
            $arr['recaptcha_use_globally_enabled'] = __( 'ON: will use "www.recaptcha.net" in circumstances when "www.google.com" is not accessible.', 'awedesk' );
            return $arr;
        });

        if(!apply_filters('awecloud_is_client', FALSE)) {
            $config['security'] = [
                'id' => 'security',
                'title' => __('Accounts & Security', 'awedesk'),
                'icon' => 'alsp-i-key',
                'position' => 'general-right',
                'items' => [
                    'recaptcha_separator' => ALSP_Config_Panel::admin_item('recaptcha_separator', FALSE, __('ReCaptcha', 'awedesk')),
                    'recaptcha_enabled' => ALSP_Config_Panel::admin_item('recaptcha_enabled', FALSE, __('ReCaptcha enabled', 'awedesk')),
                    'recaptcha_sitekey_label' => ALSP_Config_Panel::admin_item('recaptcha_sitekey_label', 'recaptcha_enabled', __('Site Key', 'awedesk')),
                    'recaptcha_secretkey_password' => ALSP_Config_Panel::admin_item('recaptcha_secretkey_password', 'recaptcha_enabled', __('Secret Key', 'awedesk')),
                    'recaptcha_use_globally_enabled' => ALSP_Config_Panel::admin_item('recaptcha_use_globally_enabled', 'recaptcha_enabled', __('Use ReCaptcha Globally', 'awedesk')),

                    'login_by_username_separator' => ALSP_Config_Panel::admin_item('login_by_username_separator', FALSE, __('Login with username', 'awedesk')),
                    'login_by_username_enabled' => ALSP_Config_Panel::admin_item('login_by_username_enabled'),

                    'login_guest_separator' => ALSP_Config_Panel::admin_item('login_guest_separator', FALSE, __('Continue as guest', 'awedesk')),
                    'login_guest_enabled' => ALSP_Config_Panel::admin_item('login_guest_enabled'),

                    'login_forgot_separator' => ALSP_Config_Panel::admin_item('login_forgot_separator', FALSE, sprintf(__('"%s" link', 'awedesk'), __('Forgot password', 'awedesk'))),
                    'login_forgot_enabled' => ALSP_Config_Panel::admin_item('login_forgot_enabled'),

                    'login_register_separator' => ALSP_Config_Panel::admin_item('login_register_separator', FALSE, sprintf(__('"%s" link', 'awedesk'), __('Register', 'awedesk'))),
                    'login_register_enabled' => ALSP_Config_Panel::admin_item('login_register_enabled'),
                ],
            ];
        }

        $config = apply_filters('alsp_config', $config, $position);
        if ($position) {
            foreach ($config as $group_key => $group) {
                $group_position = $group['position'] ?? 'left';
                if ($group_position != $position) {
                    unset($config[$group_key]);
                }
            }
        }

        return $config;
    }

    /**
     * Get list admin config menu
     * @return array
     */
    public static function admin_config_menu()
    {
        $config_menu = [
            'general' => [
                'id' => 'general',
                'label' => __('General', 'awedesk'),
                'icon' => 'alsp-i-settings-sliders',
            ],
        ];

        $config_menu = apply_filters('alsp_config_menu', $config_menu);

        return $config_menu;
    }

    /**
     * List all publiched Awedesk pages
     *
     * @param string|bool $sc - shortcode to find, FALSE to skip shortcode search
     * @param bool $block - whether to look for the block
     *
     * @return array|int[][]|string[][]|void[][]
     */
    public static function pages($sc = 'awedesk', $block = TRUE)
    {
        $switched = false;
        if (is_multisite()) {
            $id = get_current_blog_id();

            switch_to_blog( $id );
            $switched = true;
        }

        $result = [];
        global $wpdb;

        $sql = "SELECT id as ID, post_content, post_title FROM $wpdb->posts WHERE post_type='page' AND post_status='publish' AND (post_content LIKE '%[almighty_support]%' OR post_content LIKE '%[awedesk]%' OR post_content LIKE '%wp:alsp/block%')";
        $rows = $wpdb->get_results($sql, OBJECT_K);

        if (count($rows)) {
            foreach ($rows as $page) {

                $site_url = preg_replace("(^https?://)", "", get_home_url());
                $page_url = preg_replace("(^https?://)", "", get_permalink($page->ID));
                $alsp_url = str_replace($site_url, '', $page_url);
                $alsp_url = trim(rtrim($alsp_url, '/'), '/');

                if (!strlen($alsp_url)) {
                    $title = '/';
                    $result[$page->ID] = ['id' => $page->ID, 'label' => __('Front Page'), 'title' => $title];
                } else {
                    $label = $page->post_title;
                    $title = '/' . $alsp_url . '/';
                    $result[$page->ID] = ['id' => $page->ID, 'label' => $label, 'title' => $title];
                }
            }
        }

        if ($switched) {
            restore_current_blog();
        }

        return apply_filters('alsp_config_pages', $result);
    }
}

// EOF
