<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OFVoK3BMZUlYcVRKMjRSTTFXK3V2MElRTE1Tc3NENncxS3R6d2hkd0ZtbFQzSGhIQkNZcDVrWjlvQ0xURVBCaVRKY3V6NVpKQmorRW0waTRxTDhndFNmcXgrYmxIb0llMUVnN1psaThLeDUxNXhNMzRoc2Zxdm1ORWJpQTJJU2FHdHYzRFVuWDVWRGM5N0M4MklDYmhl*/

class ALSP_Config_EP_Config_Connection_Checker
{

    /**
     * Generate oAuth URL
     * @param $client_id client ID
     * @param $tenant_id tenant ID
     * 
     * @return string
     */
    public static function generate_me_auth_url($client_id, $tenant_id, $department_id = NULL)
    {
        $state_department = '';
        if ($department_id) {
            $state_department = ',' . $department_id;
        }

        $url = 'https://login.microsoftonline.com/' . $tenant_id . '/oauth2/v2.0/authorize';
        $url .= '?client_id=' . $client_id;
        $url .= '&response_type=code';
        $url .= '&redirect_uri=' . esc_url_raw(admin_url('admin.php'));
        $url .= '&response_mode=query';
        $url .= '&scope=offline_access%20user.read%20mail.read%20mail.readwrite';
        $url .= '&state=alsp_ep_me,' . wp_create_nonce('alsp-ep-me') . $state_department;

        return $url;
    }



    /**
     * Process gmail connection response
     *
     * @return void
     */
    public static function process_response_microsoft_exchange()
    {
        $state = isset($_REQUEST['state']) ? array_filter(array_map('sanitize_text_field', explode(',', sanitize_text_field(wp_unslash($_REQUEST['state']))))) : [];
        if (!(isset($state[0]) && $state[0] == 'alsp_ep_me')) {
            return;
        }

        // verify nonce.
        $nonce = isset($state[1]) ? $state[1] : '';
        if (wp_verify_nonce($nonce, 'alsp-ep-me') != 1) {
            wp_die('Unauthorized request!');
        }

        // check whether code received or not.
        $code = isset($_REQUEST['code']) ? sanitize_text_field(wp_unslash($_REQUEST['code'])) : '';
        if (!$code) {
            wp_die('Bad request!');
        }

        $prefix = '';
        $page = 'alsp#email-piping';

        // department
        $department_id = isset($state[2]) ? $state[2] : FALSE;
        if ($department_id) {
            $prefix = 'dept' . $department_id . '_';
            $page = 'alsp-department';
        }

        $client_id = ALSP_Config::get($prefix . 'ep_me_client_id_label');
        $client_secret = ALSP_Config::get($prefix . 'ep_me_client_secret_label');
        $tenant_id = ALSP_Config::get($prefix . 'ep_me_tenant_id_label');

        $is_valid = true;

        // Get Refresh Token.
        $response = wp_remote_post(
            'https://login.microsoftonline.com/' . $tenant_id . '/oauth2/v2.0/token?scope=offline_access%20user.read%20mail.readbasic%20mail.read%20mail.readwrite',
            [
                'body' => [
                    'client_id' => $client_id,
                    'client_secret' => $client_secret,
                    'grant_type' => 'authorization_code',
                    'code' => $code,
                    'redirect_uri' => esc_url_raw(admin_url('admin.php')),
                ],
            ]
        );

        if (is_wp_error($response)) {
            $is_valid = false;
            $last_error = $response->get_error_message();
        }

        if ($is_valid && $response['response']['code'] !== 200) {

            $is_valid = false;
            $last_error = json_decode($response['body']);
            $last_error = $last_error->message;

        } else {

            $response = json_decode($response['body']);
            ALSP_Config::set($prefix . 'ep_me_refresh_token', $response->refresh_token);
            set_transient($prefix . 'alsp_ep_me_access_token', $response->access_token, $response->expires_in - 5);

            // get piping email address.
            $response = wp_remote_get(
                'https://graph.microsoft.com/v1.0/me',
                [
                    'headers' => [
                        'Authorization' => 'Bearer ' . $response->access_token,
                    ],
                ]
            );

            if (is_wp_error($response)) {
                $is_valid = false;
                $last_error = $response->get_error_message();
            }

            if ($is_valid && $response['response']['code'] !== 200) {
                $is_valid = false;
                $last_error = json_decode($response['body']);
                $last_error = $last_error->message;
            }

            if ($is_valid) {
                $response = json_decode($response['body']);
                $piping_address = $response->mail ?? $response->userPrincipalName;
                ALSP_Config::set($prefix . 'ep_me_piping_address_label', $piping_address);
            }
        }

        if ($is_valid) {
            ALSP_Config::set($prefix . 'ep_me_is_active', 1);
            ALSP_Config::set($prefix . 'ep_me_last_error', '');
            ALSP_Config::set($prefix . 'ep_me_delta_url', '');
            ALSP_Config::set($prefix . 'ep_me_is_init', 0);
            ALSP_Config::set($prefix . 'ep_me_init_current_page', 0);
            ALSP_Config::set($prefix . 'ep_me_init_total_pages', 0);
        } else {
            ALSP_Config::set($prefix . 'ep_me_is_active', 0);
            ALSP_Config::set($prefix . 'ep_me_last_error', $last_error);
        }

        delete_transient($prefix . 'alsp_ep_connection_init');

        ?>
        <script>
            window.location.href = "<?php echo esc_url(admin_url('admin.php')); ?>?page=<?php echo $page; ?>";
        </script>
        <?php
    }

    /**
     * Generate oAuth URL
     * @param $client_id client ID
     * @param $department_id tenant ID
     * 
     * @return string
     */
    public static function generate_gmail_auth_url($client_id, $department_id = NULL)
    {
        $state_department = '';
        if ($department_id) {
            $state_department = ',' . $department_id;
        }

        $url = 'https://accounts.google.com/o/oauth2/v2/auth';
        $url .= '?scope=' . esc_url_raw('https://www.googleapis.com/auth/gmail.readonly');
        $url .= '&access_type=offline';
        $url .= '&redirect_uri=' . esc_url_raw(admin_url('admin.php'));
        $url .= '&response_type=code';
        $url .= '&state=alsp_ep,' . wp_create_nonce('alsp-ep-gmail') . $state_department;
        $url .= '&client_id=' . $client_id;

        return $url;
    }

    /**
     * Process gmail connection response
     *
     * @return void
     */
    public static function process_response_gmail()
    {

        $state = isset($_REQUEST['state']) ? array_filter(array_map('sanitize_text_field', explode(',', sanitize_text_field(wp_unslash($_REQUEST['state']))))) : [];
        if (!(isset($state[0]) && $state[0] == 'alsp_ep')) {
            return;
        }

        // verify nonce.
        $nonce = isset($state[1]) ? $state[1] : '';
        if (wp_verify_nonce($nonce, 'alsp-ep-gmail') != 1) {
            wp_die('Unauthorized request!');
        }

        // check whether code received or not.
        $code = isset($_REQUEST['code']) ? sanitize_text_field(wp_unslash($_REQUEST['code'])) : '';
        if (!$code) {
            wp_die('Bad request!');
        }

        $prefix = '';
        $page = 'alsp#email-piping';

        // department
        $department_id = isset($state[2]) ? $state[2] : FALSE;
        if ($department_id) {
            $prefix = 'dept' . $department_id . '_';
            $page = 'alsp-department';
        }

        $email_address = ALSP_Config::get($prefix . 'ep_gmail_email_label');
        $client_id = ALSP_Config::get($prefix . 'ep_gmail_client_id_label');
        $client_secret = ALSP_Config::get($prefix . 'ep_gmail_client_secret_label');
        $is_valid = true;
        $last_error = FALSE;
        $history_id = '';

        // Get Refresh and Access Tokens.
        $url = 'https://www.googleapis.com/oauth2/v4/token';
        $response = wp_remote_post(
            $url,
            [
                'method' => 'POST',
                'timeout' => 45,
                'redirection' => 5,
                'httpversion' => '1.0',
                'blocking' => true,
                'headers' => [],
                'body' => [
                    'client_id' => $client_id,
                    'client_secret' => $client_secret,
                    'redirect_uri' => admin_url('admin.php'),
                    'code' => $code,
                    'grant_type' => 'authorization_code',
                ],
                'cookies' => [],
            ]
        );

        if (is_wp_error($response)) {
            $is_valid = false;
            $last_error = $response->get_error_message();
        }

        // refresh token.
        if ($is_valid) {

            $access = json_decode($response['body'], true);
            if (isset($access['refresh_token'])) {

                ALSP_Config::set($prefix . 'ep_gmail_refresh_token', $access['refresh_token']);

            } else {

                $is_valid = false;
                $last_error = 'Refresh token not found!';
            }
        }

        // get last history id.
        if ($is_valid) {

            $access = json_decode($response['body'], true);
            $response = wp_remote_post(
                'https://www.googleapis.com/gmail/v1/users/' . $email_address . '/profile',
                [
                    'method' => 'GET',
                    'timeout' => 45,
                    'redirection' => 5,
                    'httpversion' => '1.0',
                    'blocking' => true,
                    'headers' => [],
                    'body' => [
                        'access_token' => $access['access_token'],
                    ],
                    'cookies' => [],
                ]
            );

            if (is_wp_error($response)) {
                $is_valid = false;
                $last_error = $response->get_error_message();
            }

            if ($is_valid) {
                $profile = json_decode($response['body'], true);
                if (isset($profile['historyId'])) {
                    $history_id = $profile['historyId'];
                } else {
                    $is_valid = false;
                    $last_error = 'History ID not found!';
                }
            }
        }

        if ($is_valid) {
            ALSP_Config::set($prefix . 'ep_gmail_history_id', $history_id);
            ALSP_Config::set($prefix . 'ep_gmail_is_active', 1);
        } else {
            ALSP_Config::set($prefix . 'ep_gmail_last_error', $last_error);
        }

        delete_transient($prefix . 'alsp_ep_connection_init');
        echo 'Please wait!';
        ?>
        <script>
            window.location.href = "<?php echo esc_url(admin_url('admin.php')); ?>?page=<?php echo $page; ?>";
        </script>
        <?php
    }
}

// EOF