<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2K1Y3RlE0Zmt3RDFabzQ2ODJTbXN6amVkYk5wQkQzVWNTMVlwaWErUzZyVXVqNVluZnBnU3pmWnZkcjMzWENwL1RqeVU5QSt1OS9qRzV4SlEyVEtBb2JSeWpXbDFmZjIwQWFCN21lc3VmbzdvVWpMaTBJeUZUWUZ2L0RhTWl4cjJuOTJkYk9Yck9Lbi9yYmN2YnNXVDBz*/

class ALSP_Config_EP_Microsoft_Exchange
{
    private static $instance = NULL;

    public static function get_instance()
    {
        return (NULL == self::$instance) ? self::$instance = new self() : self::$instance;
    }

    private function __construct()
    {
        add_filter('alsp_config_description', function ($desc) {
            $desc['ep_me_tenant_id_label'] = FALSE;
            $desc['ep_me_client_id_label'] = FALSE;
            $desc['ep_me_client_secret_label'] = FALSE;
            $desc['ep_me_is_active_label'] = FALSE;
            $desc['ep_me_redirect_uri_label'] = FALSE;
            $desc['ep_me_piping_address_label'] = FALSE;
            $desc['ep_me_last_error_label'] = FALSE;
            $desc['ep_me_message'] = '<div class="alsp-opt__form-actions">
            <button class="alsp-opt__action alsp-btn alsp-btn--action" data-id="ep_me_check" data-btn="check" data-tooltip="Check" style="">
                <i class="alsp-i-plug"></i> Check
                <i class="alsp-i-spinner alsp-i-spin" data-loading style="display:none"></i>
            </button>
        </div>';

            return $desc;
        });

        add_filter('alsp_config', function ($config, $position) {
            if ($position !== 'email-piping-right') {
                return $config;
            }

            // Disable Microsoft Exchange.
            return $config;

            $is_active = ALSP_Config::get('ep_me_is_active', 0);
            $status = $is_active ? __('Connected', 'awedesk') : __('Not Connected', 'awedesk');
            $last_error = ALSP_Config::get('ep_me_last_error', '');

            $config['email_piping_me_conn'] = [
                'id' => 'email_piping_me_conn',
                'title' => 'Microsoft Exchange',
                'icon' => 'alsp-i-clip',
                'description' => sprintf(__('Configure connection: %s', 'awedesk'), 'Microsoft Exchange'),
                'position' => 'email-piping-right',
                'items' => [
                    'ep_me_tenant_id_label' => ALSP_Config_Panel::admin_item('ep_me_tenant_id_label', FALSE, __('Tenant ID', 'awedesk')),
                    'ep_me_client_id_label' => ALSP_Config_Panel::admin_item('ep_me_client_id_label', FALSE, __('Client ID', 'awedesk')),
                    'ep_me_client_secret_label' => ALSP_Config_Panel::admin_item('ep_me_client_secret_label', FALSE, __('Client Secret', 'awedesk')),
                    'ep_me_is_active_label' => ALSP_Config_Panel::admin_item('ep_me_is_active_label', FALSE, __('Status', 'awedesk'), ['default_value' => $status, 'disabled' => TRUE]),
                    'ep_me_redirect_uri_label' => ALSP_Config_Panel::admin_item('ep_me_redirect_uri_label', FALSE, __('Redirect URI', 'awedesk'), ['default_value' => esc_url(admin_url('admin.php')), 'disabled' => TRUE]),
                    'ep_me_piping_address_label' => ALSP_Config_Panel::admin_item('ep_me_piping_address_label', FALSE, __('Piping Address', 'awedesk'), ['default_value' => '', 'disabled' => TRUE]),
                    'ep_me_last_error_label' => ALSP_Config_Panel::admin_item('ep_me_last_error_label', FALSE, __('Last Error', 'awedesk'), ['default_value' => $last_error, 'disabled' => TRUE]),
                    'ep_me_message' => ALSP_Config_Panel::admin_item('ep_me_message', FALSE, FALSE),
                ],
            ];

            return $config;
        }, 33, 2);

        // microsoft exchange process connection response.
        add_action('admin_init', ['ALSP_Config_EP_Config_Connection_Checker', 'process_response_microsoft_exchange']);
    }

    /**
     * Get new access token using refresh token
     *
     * @return mixed
     */
    public static function get_access_token()
    {
        $access_token = get_transient('alsp_ep_me_access_token');
        if ($access_token !== false) {
            return $access_token;
        }

        $refresh_token = ALSP_Config::get('ep_me_refresh_token', '');
        if ($refresh_token === '') {
            return false;
        }

        $client_id = ALSP_Config::get('ep_me_client_id_label');
        $client_secret = ALSP_Config::get('ep_me_client_secret_label');
        $tenant_id = ALSP_Config::get('ep_me_tenant_id_label');

        // get access token using refresh token.
        $response = wp_remote_post(
            'https://login.microsoftonline.com/' . $tenant_id . '/oauth2/v2.0/token',
            [
                'body' => [
                    'client_id' => $client_id,
                    'client_secret' => $client_secret,
                    'grant_type' => 'refresh_token',
                    'refresh_token' => $refresh_token,
                ],
            ]
        );

        if (!self::is_valid_response($response)) {
            return false;
        }

        $response = json_decode($response['body']);
        set_transient('alsp_ep_me_access_token', $response->access_token, $response->expires_in - 5);
        return $response->access_token;
    }

    /**
     * Check whether API response is valid
     *
     * @param array $response - REST API response array.
     * @return boolean
     */
    public static function is_valid_response($response)
    {
        if (is_wp_error($response)) {
            $last_error = $response->get_error_message();
            ALSP_Config::set('ep_me_last_error', $last_error);
            return FALSE;
        }

        if ($response['response']['code'] !== 200) {
            $last_error = json_decode($response['body']);
            $last_error = $last_error->error->message;
            ALSP_Config::set('ep_me_last_error', $last_error);
            return FALSE;
        }

        ALSP_Config::set('ep_me_last_error', '');
        return TRUE;
    }
}

ALSP_Config_EP_Microsoft_Exchange::get_instance();

// EOF
