<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2L0RkQyszMW54K2FSa3VKYlZFa3p1M3pjenlvMFdoTE0wbXdsajlxQWhkM0pNaWNMc1RhaEsvOVNHLzYvQ3lQTWVhT1BwN2dudm5XVGFrYzhkbnBCSzdXbnhveDdrdjNzVUlYbUZqZCtNdHVNcUo3Y1R5Y1ZyOTd1VEkyVXV4RFRZPQ==*/

class ALSP_Config_Department_EP_Multi_Inbox
{
    private static $instance = NULL;

    public static function get_instance()
    {
        return (NULL == self::$instance) ? self::$instance = new self() : self::$instance;
    }

    private function __construct()
    {
        add_filter('alsp_config_description', function ($desc) {

            $request = new WP_REST_Request;
            $forwarders = new ALSP_Model_Email_Piping_Forwarders($request);

            foreach ($forwarders->read() as $key => $forwarder) {
                if (!isset($forwarder['id']) && !$forwarder['id']) {
                    continue;
                }

                $prefix = 'inboxes' . $forwarder['id'] . '_';
                $desc[$prefix . 'ep_multi_inbox_receiver_label'] = FALSE;
            }

            $desc['ep_multi_message'] = 'Set up Awedesk to assign tickets to a department if they were forwarded from a specific email address. For example, if email billing@example.com forwards to your primary support inbox, Awedesk can assign tickets coming from that address to the Billing department.';
            return $desc;
        });

        add_filter('alsp_validate_field', function ($valid, $name, $value) {
            if (!stristr($name, 'ep_multi_inbox_receiver_label')) {
                return $valid;
            }

            // Sanitize and check email
            $email = sanitize_email($value);
            if (!is_email($email)) {
                return new WP_Error('alsp-config', __('Invalid email', 'awedesk'));
            }

            return $valid;
        }, 10, 3);

        add_action('alsp_config_set', function ($name, $value) {
            if (stristr($name, 'ep_multi_inbox_receiver_label')) {
                // Remove the known prefix
                $prefix = 'inboxes';
                $text = str_replace($prefix, '', $name);

                // Find the position of the first numeric character
                $start_pos = strpos($text, '_');

                $id = substr($text, 0, $start_pos);
                $forwarder = new ALSP_Model_Email_Piping_Forwarder($id);
                $forwarder->update_item($id, $value, $forwarder->department);
            } else if (stristr($name, 'ep_multi_inbox_default_department_select')) {
                // Remove the known prefix
                $prefix = 'inboxes';
                $text = str_replace($prefix, '', $name);

                // Find the position of the first numeric character
                $start_pos = strpos($text, '_');

                $id = substr($text, 0, $start_pos);
                $forwarder = new ALSP_Model_Email_Piping_Forwarder($id);
                $forwarder->update_item($id, $forwarder->receiver, $value);
            }
        }, 10, 2);

        add_filter('alsp_config', function ($config, $position) {
            if ($position !== 'email-piping-left' && $position !== 'email-piping-right') {
                return $config;
            }

            $options_departments = [
                ['value' => 0, 'title' => __('None', 'awedesk'), 'label' => __('None', 'awedesk')]
            ];

            $depts_model = new ALSP_Model_Departments($r = new WP_REST_Request());
            $departments = $depts_model->read(1);

            if (count($departments)) {
                foreach ($departments as $department) {
                    $options_departments[] = [
                        'value' => $department['id'],
                        'title' => $department['name'],
                        'label' => $department['name']
                    ];
                }
            }

            $items = [];
            $items['ep_multi_message']  = ALSP_Config_Panel::admin_item('ep_multi_message');
            $request = new WP_REST_Request;
            $forwarders = new ALSP_Model_Email_Piping_Forwarders($request);
            foreach ($forwarders->read() as $key => $forwarder) {
                if (!isset($forwarder['id']) && !$forwarder['id']) {
                    continue;
                }

                $prefix = 'inboxes' . $forwarder['id'] . '_';
                $items[$prefix . 'ep_multi_inbox_default_department_separator'] = ALSP_Config_Panel::admin_item('ep_multi_inbox_default_department_separator', FALSE, FALSE);
                $items[$prefix . 'ep_multi_inbox_receiver_label'] = ALSP_Config_Panel::admin_item($prefix . 'ep_multi_inbox_receiver_label', FALSE, __('Email', 'awedesk'));
                $items[$prefix . 'ep_multi_inbox_default_department_select'] = ALSP_Config_Panel::admin_item($prefix . 'ep_multi_inbox_default_department_select', FALSE, __('Department', 'awedesk'), ['options' => $options_departments]);

                $btn_args = ['icon' => 'alsp-i-trash', 'btn_style' => 'delete', 'loading' => TRUE, 'data' => ['forwarder-id' => $forwarder['id']]];
                $items[$prefix . 'ep_multi_inbox_delete_button'] = ALSP_Config_Panel::admin_item($prefix . 'ep_multi_inbox_delete_button', FALSE, FALSE, $btn_args);
            }

            $items['ep_multi_separator'] = ALSP_Config_Panel::admin_item('ep_multi_separator', FALSE, FALSE);
            $items['ep_multi_inbox_button'] = ALSP_Config_Panel::admin_item('ep_multi_inbox_button', FALSE, __('Add New', 'awedesk'), ['loading' => TRUE]);

            $config['email_piping_multi_inbox'] = [
                'id' => 'email_piping_multi_inbox',
                'title' => __('Forward To Department', 'awedesk'),
                'icon' => 'alsp-i-envelope-open-text',
                'position' => 'email-piping-left',
                'items' => $items,
            ];

            return $config;
        }, 30, 2);
    }
}

ALSP_Config_Department_EP_Multi_Inbox::get_instance();

// EOF
