<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OFJURTdMa3V4M1VHS0JZZFVxeGd3T2Urb2pEUGpkM1hPaytGQ3ZnTXExZUVqZnp0cFZxYTdSYUQ1KzJGei9EekZQNHpWWWR2QXhNYWRBQ04wbWVsTnpHYTdJb0xjaDZEaEMwNkxpZE1Bb0dYZVVPVDhLUzRqdE85NElxMUhSWmtNPQ==*/
if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly!
}

if (!class_exists('ALSP_EP_Email')):

    final class ALSP_EP_Email
    {

        /**
         * Ticket ID
         *
         * @var integer
         */
        public $ticket_id = 0;

        /**
         * Department ID
         *
         * @var string
         */
        public $department_id = NULL;

        /**
         * Config Prefix
         *
         * @var string
         */
        public $config_prefix = '';

        /**
         * Email sent to addresses. Used to check forwarding emails
         *
         * @var array
         */
        public $to_addresses = [];

        /**
         * CC addresses. Used to insert as additional recipients if allowed in setting
         *
         * @var array
         */
        public $cc_addresses = [];

        /**
         * Email customer name
         *
         * @var string
         */
        public $from_name;

        /**
         * Email customer email address
         *
         * @var string
         */
        public $from_email;

        /**
         * Reply to email address
         *
         * @var string
         */
        public $reply_to;

        /**
         * Email subject line
         *
         * @var string
         */
        public $subject;

        /**
         * HTML body of email
         *
         * @var string
         */
        public $html_body = '';

        /**
         * Plain text body of email
         *
         * @var string
         */
        public $text_body = '';

        /**
         * Email attachments
         *
         * @var array
         */
        public $attachments = [];

        /**
         * Ticket source (imap, gmail, me, etc.)
         *
         * @var string
         */
        public $source;

        /**
         * Set whether incoming email is forwarded from email address listed in general settings
         *
         * @var boolean
         */
        public $is_forwarded = false;

        /**
         * Set forwarding address if it is forwarded from email address listed in general settings
         *
         * @var string
         */
        public $forwarded_from = '';

        /**
         * Set current ticket information
         *
         * @var object
         */
        public $alsp_ticket;

        /**
         * Set current user information
         *
         * @var object
         */
        public $alsp_user;

        /**
         * Set current wp user
         */
        public $wpuser;

        /**
         * Pipe current email
         *
         * @return void
         */
        public function pipe($department_id = NULL)
        {
            // set forwarding email if it is forwarded.
            $this->set_forwarding_address();

            // set department id
            $this->set_department_id($department_id);
            $this->set_config_prefix();
            $this->set_ticket_id();
            $this->set_wpuser();

            // do not pipe if email is not valid as per settings.
            if (!$this->is_valid()) {
                return;
            }

            // Set Eswp user
            $this->set_alsp_user();

            // decide whether it is ticket reply or new email.
            if ($this->ticket_id) {
                $this->pipe_reply();
            } else {
                $this->pipe_new_ticket();
            }
            // ALSP_Email_Notifications::send_background_emails();
        }

        /**
         * Set department id
         * @param $department_id department ID
         *
         * @return void
         */
        private function set_department_id($department_id = NULL)
        {
            if ($department_id !== NULL) {
                $this->department_id = $department_id;
            } else {
                $this->department_id = ALSP_Email_Piping::default_department($this->to_addresses);
            }
        }

        /**
         * Set config prefix
         * @param $department_id department ID
         *
         * @return void
         */
        private function set_config_prefix()
        {
            if ($this->department_id) {
                $this->config_prefix = $this->department_id . '_';
            }
        }

        /**
         * Set ticket id
         *
         * @return void
         */
        private function set_ticket_id()
        {
            $ticket_alice = 'Ticket #';
            if (preg_match('/\[' . $ticket_alice . '(\d*)\]/', $this->subject, $matches)) {
                $ticket_id = intval($matches[1]);
                $ticket_model = new ALSP_Model_Ticket($ticket_id);
                if ($ticket_model->id) {
                    $this->ticket_id = $ticket_id;
                    $this->alsp_ticket = $ticket_model;
                }
            }
        }

        private function set_wpuser()
        {
            $this->wpuser = get_user_by('email', $this->from_email);
        }

        private function set_alsp_user()
        {
            error_log("wp_user:" . json_encode($this->wpuser));
            if (!$this->wpuser) {
                // Create the user if not exists.
	            error_log("creating user");
                $user = ALSP_User::create_user_from_email($this->from_email, $this->from_name);
                if ($user) {
	                error_log("creating user OK");
                    $this->wpuser = get_user_by('email', $this->from_email);
                    $this->alsp_user = ALSP_User::get_instance($user);
                } else {
					ob_start();
					var_dump($user);
					error_log("creating user FAIL - " . ob_get_clean());
                }
            } else {

                $this->alsp_user = ALSP_User::get_instance($this->wpuser->ID);
	            ob_start();
	            var_dump($this->alsp_user);
	            error_log("getting user: " . ob_get_clean());
            }
        }

        /**
         * Set forwarding address if it is forwarded from listed forwarding addresses in general settings
         *
         * @return void
         */
        private function set_forwarding_address()
        {
            $piping_address = ALSP_Config_EP_General::get_piping_email_address($this->department_id);

            // if email piped through main piping addredss consider this as not forwarded.
            if (
                in_array($piping_address, $this->to_addresses) ||
                in_array($piping_address, $this->cc_addresses)
            ) {
                return;
            }

            // check every forwarding email and return matched.
            $forwarding_address = ALSP_Config::get('ep_forwarding_emails_textarea', []);
            $forwarding_address = str_replace("\r", '', $forwarding_address);
            $forwarding_address = explode("\n", $forwarding_address);
            foreach ($forwarding_address as $email) {
                if (
                    in_array($email, $this->to_addresses) ||
                    in_array($email, $this->cc_addresses)
                ) {
                    $this->is_forwarded = true;
                    $this->forwarded_from = $email;
                    return;
                }
            }
        }

        /**
         * Check whether it is valid to import as per settings
         *
         * @return boolean
         */
        private function is_valid()
        {
            // return if from address and piping address are same.
            if ($this->from_email == ALSP_Config_EP_General::get_piping_email_address($this->department_id)) {
                return false;
            }

            // block by email address.
            $block_emails = ALSP_Config::get('ep_block_emails_textarea');
            $block_emails = str_replace("\r", '', $block_emails);
            $block_emails = explode("\n", $block_emails);
            foreach ($block_emails as $pattern) {

                if (preg_match('/^\/[\s\S]+\/i$/', $pattern)) { // regex.

                    if (preg_match($pattern, $this->from_email)) {
                        return false;
                    }
                } elseif (preg_match('/^\*[\s\S]+\*$/', $pattern)) { // e.g. "*reply*".

                    $pattern = str_replace('*', '', $pattern);
                    if (preg_match('/' . $pattern . '/', $this->from_email)) {
                        return false;
                    }
                } elseif (preg_match('/^\*[\s\S]+$/', $pattern)) { // e.g. "*@test.com".

                    $pattern = str_replace('*', '[\s\S]+', $pattern);
                    if (preg_match('/' . $pattern . '$/', $this->from_email)) {
                        return false;
                    }
                } elseif (preg_match('/[\s\S]+\*$/', $pattern)) { // e.g. "noreply@*".

                    $pattern = str_replace('*', '[\s\S]+', $pattern);
                    if (preg_match('/^' . $pattern . '/', $this->from_email)) {
                        return false;
                    }
                } else { // e.g. "test@test.com".

                    if ($pattern == $this->from_email) {
                        return false;
                    }
                }
            }

            // block by subject.
            $block_subject = ALSP_Config::get('ep_block_subject_textarea');
            $block_subject = str_replace("\r", '', $block_subject);
            $block_subject = explode("\n", $block_subject);
            foreach ($block_subject as $pattern) {
                if (preg_match('/^\/[\s\S]+\/i$/', $pattern)) { // regex.

                    if (preg_match($pattern, $this->subject)) {
                        return false;
                    }
                } elseif (preg_match('/^\*[\s\S]+\*$/', $pattern)) { // e.g. "*reply*".

                    $pattern = str_replace('*', '', $pattern);
                    if (preg_match('/' . $pattern . '/', $this->subject)) {
                        return false;
                    }
                } elseif (preg_match('/^\*[\s\S]+$/', $pattern)) { // e.g. "*@test.com".

                    $pattern = str_replace('*', '[\s\S]+', $pattern);
                    if (preg_match('/' . $pattern . '$/', $this->subject)) {
                        return false;
                    }
                } elseif (preg_match('/[\s\S]+\*$/', $pattern)) { // e.g. "noreply@*".

                    $pattern = str_replace('*', '[\s\S]+', $pattern);
                    if (preg_match('/^' . $pattern . '/', $this->subject)) {
                        return false;
                    }
                } else {

                    $pattern = '/' . $pattern . '/i';
                    if (preg_match($pattern, $this->subject)) {
                        return false;
                    }
                }
            }

            // delivery failure email.
            if (
                preg_match('/postmaster@/i', $this->from_email) &&
                (
                    preg_match('/Delivery has failed to these recipients/i', $this->text_body) ||
                    preg_match('/Delivery has failed to these recipients/i', $this->html_body)
                )
            ) {
                return false;
            }

            $param = get_object_vars($this);
            // not allowed reply emails.
            $allowed_emails = ALSP_Config::get('ep_allowed_email_select');
            if ($this->ticket_id && $allowed_emails == 'new') {
                do_action('alsp_ep_reject_reply_emails', $param);
                return false;
            }

            // not allowed new emails.
            if (!$this->ticket_id && $allowed_emails == 'reply') {
                do_action('alsp_ep_reject_new_emails', $param);
                return false;
            }

            // allowed users.
            $allowed_users = ALSP_Config::get('ep_allowed_users_select');
            if (!$this->wpuser && $allowed_users == 'registered') {
                // User reject pipe email template and send email on below action.
                do_action('alsp_ep_reject_user_pipe', $param);
                return false;
            }

            return apply_filters('alsp_ep_email_is_valid', true, $param);
        }

        /**
         * Pipe reply to existing ticket
         *
         * @return void
         */
        private function pipe_reply()
        {
            $ticket = new ALSP_Model_Ticket($this->ticket_id);
            if (!$ticket->id) {
                return;
            }
            $ticket_data = $ticket->get_item();
            $reply_text_above = ALSP_Config::get('ep_reply_above_label', '-- Reply above --');

            // Description
            $description = $this->get_body();

            if ($reply_text_above) {
                // reply above.
                $pos = strpos($description, $reply_text_above);
                if (is_numeric($pos)) {
                    $description = substr($description, 0, $pos);
                    $description = preg_replace('/\nOn(.*?)wrote:(.*?)$/si', '', $description);
                }
            }

            // Check if agent and add his signature.
            $type = 'customer_reply';
            if ($this->alsp_user->is_agent() && $ticket_data['agent']['user_email'] == $this->from_email) {
                // $description .= $current_user->agent->get_signature();
                $type = 'agent_reply';
            }

            // submit reply.
            $ticket_message = new ALSP_Model_Ticket_Message();
            $message_id = $ticket_message->create_item($ticket_data['id'], $this->alsp_user->get('user_id'), $type, $description);
            if (!$message_id) {
                return;
            }

            do_action('alsp_ticket_reply', $ticket_data['id'], $message_id);

            // attachments.
            if (is_array($this->attachments) && count($this->attachments) > 0) {
                $failedAttachment = [];
                foreach ($this->attachments as $file) {
                    if ($file['status'] !== TRUE) {
                        array_push($failedAttachment, $file);
                        continue;
                    }

                    // Move file from tmp folder
                    $source = ALSP_Attachments::get_tmp_dir() . $file['file_name'];
                    if (!file_exists($source)) {
                        continue;
                    }

                    $destination = ALSP_Attachments::get_dir() . $file['file_name'];
                    rename($source, $destination);

                    // Store file path without wp-content.
                    $file_path = str_replace(WP_CONTENT_DIR, '', $destination);

                    $ticket_attachment = new ALSP_Model_Ticket_Attachment();
                    $result = $ticket_attachment->create_item($ticket_data['id'], $this->alsp_user->get('user_id'), $file['file_name'], $file_path, $message_id, 'email_piping');
                    if (!$result) {
                        continue;
                    }
                }

                if (count($failedAttachment) > 0) {
                    ALSP_Config::set('failed_attachments_' . $message_id, maybe_serialize($failedAttachment));
                }
            }

            /**
             * If status not provided, we should determine the status based on reply type
             */
            if ($type == 'agent_reply') {
                // Move back to customer
                $status = 'client';
            } elseif ($type == 'customer_reply') {
                // Move back to agent
                $status = 'agent';
            }

            $req = new WP_REST_Request;
            $model = new ALSP_Model_Tickets($req);
            $model->after_create_messages($ticket_data['id'], $this->alsp_user->get('user_id'), $type, $status);

            /**
             * Send notification
             */
            $data = [
                'ticket_number' => $ticket_data['id'],
                'ticket_title' => $ticket_data['subject'],
                'message' => $description,
            ];
            if ($type == 'agent_reply') {
                $data['recipient_name'] = $ticket_data['client']['display_name'];
                $data['recipient_email'] = $ticket_data['client']['user_email'];
                $data['client_name'] = $ticket_data['client']['display_name'];
                $data['ticket_url'] = ALSP::get_alsp_ticket_uri($ticket_data['client']['id'], $ticket_data['id']);
                ALSP_Email_Queue::add_message($ticket_data['client']['id'], $data, __('[Ticket #{ticket_number}]', 'awedesk'), 'agent_reply_template');
            } elseif ($type == 'customer_reply') {
                if ($ticket_data['agent'] !== NULL) {
                    $data['recipient_name'] = $ticket_data['agent']['display_name'];
                    $data['recipient_email'] = $ticket_data['agent']['user_email'];
                    $data['agent_name'] = $ticket_data['agent']['display_name'];
                    $data['client_name'] = $ticket_data['client']['display_name'];
                    $data['ticket_url'] = ALSP::get_alsp_ticket_uri($ticket_data['agent']['id'], $ticket_data['id']);
                    ALSP_Email_Queue::add_message($ticket_data['agent']['id'], $data, __('[Ticket #{ticket_number}]', 'awedesk'), 'client_reply_template');
                }
            }

	        // App Push
	        $actor = $this->alsp_user->get('user_id');
	        ALSP_Push::send_to_agents($ticket_data['id'], $actor, 'reply', ['message'=>$description]);


            $ticket_model = new ALSP_Model_Ticket_Message($message_id);
            do_action('alsp_post_reply', $ticket_model->get_item());
        }

        /**
         * Pipe new ticket
         *
         * @return void
         */
        private function pipe_new_ticket()
        {
            // start with subject.
            if (!$this->subject) {
                return;
            }

            // Description
            $description = $this->get_body();

            $ticket = new ALSP_Model_Ticket();
	        if(!method_exists($this->alsp_user,'get')) {
		        new ALSP_Debug(__FILE__." SKIPPING USER");
		        return;
	        }
            $user_id = $this->alsp_user->get('user_id');
            $author = $user_id;
            $client = $user_id;

            // if ticket was created by agent
            $status = 'new';
            $type = 'customer_reply';
            $agent = NULL;
            $agent_reply_count = 0;
            $client_reply_count = 0;
            if (in_array('alsp_manager', (array) $this->wpuser->roles) || in_array('alsp_agent', (array) $this->wpuser->roles)) {
                $agent_reply_count = 1;
                $type = 'agent_reply';
                $agent = $user_id;
            }
            // if it's from client
            else {
                $client_reply_count = 1;

                // Set default assignee
                if ($this->department_id !== 0) {
                    $obj_department = new ALSP_Model_Department($this->department_id);
                    $department_details = $obj_department->get_item();
                    if (isset($department_details['id']) && $agent === NULL) {
                        $agent = $department_details['default_assignee']['id'] ?? NULL;
                    }
                }
            }

            $ticket_id = $ticket->create_item($author, $client, $agent, $this->department_id, $this->subject, $status, NULL, $agent_reply_count, $client_reply_count);
            if (!$ticket_id) {
                return;
            }

            $ticket_message = new ALSP_Model_Ticket_Message();
            $message_id = $ticket_message->create_item($ticket_id, $author, $type, $description);
            if (!$message_id) {
                return;
            }

            do_action('alsp_ticket_new', $ticket_id, $message_id);

            // attachments.
            if (1 == ALSP_Config::get('attachment_enabled', 1) && is_array($this->attachments) && count($this->attachments) > 0) {
                $failedAttachment = [];
                foreach ($this->attachments as $file) {
                    if ($file['status'] !== TRUE) {
                        array_push($failedAttachment, $file);
                        continue;
                    }

                    // Move file from tmp folder
                    $source = ALSP_Attachments::get_tmp_dir() . $file['file_name'];
                    if (!file_exists($source)) {
                        continue;
                    }

                    $destination = ALSP_Attachments::get_dir() . $file['file_name'];
                    rename($source, $destination);

                    // Store file path without wp-content.
                    $file_path = str_replace(WP_CONTENT_DIR, '', $destination);

                    $ticket_attachment = new ALSP_Model_Ticket_Attachment();
                    $result = $ticket_attachment->create_item($ticket_id, $author, $file['file_name'], $file_path, $message_id, 'email_piping');
                    if (!$result) {
                        continue;
                    }
                }

                if (count($failedAttachment) > 0) {
                    ALSP_Config::set('failed_attachments_' . $message_id, maybe_serialize($failedAttachment));
                }
            }

            /**
             * Send notification new ticket submission to a manager
             */
            $data = [
                'ticket_number' => $ticket_id,
                'ticket_title' => $this->subject,
                'message' => $description,
            ];
            $list_manager = ALSP_User::get_users_by_role(ALSP_User::ROLE_MANAGER);
            if (count($list_manager) > 0) {
                foreach ($list_manager as $manager_id) {
                    $manager = ALSP_User::get_instance($manager_id);

                    $data['recipient_name'] = $manager->wp_user->display_name;
                    $data['recipient_email'] = $manager->wp_user->user_email;
                    $data['manager_name'] = $manager->wp_user->display_name;
                    $data['ticket_url'] = ALSP::get_alsp_ticket_uri($manager->wp_user->ID, $ticket_id);
					$data['ticket_subject'] = $data['ticket_title'];
                    ALSP_Email_Queue::add_message($manager_id, $data, sprintf(__('[Ticket #%d] was submitted', 'awedesk'), $ticket_id), 'new_ticket_template');
                }
            }

            // Send email notification to an agent
            if ($agent !== NULL) {
                $agent_model = ALSP_User::get_instance($agent);
                if ($agent_model->wp_user->ID) {
                    $data = [];
                    $data['ticket_number'] = $ticket_id;
                    $data['ticket_title'] = $this->subject;
                    $data['ticket_url'] = ALSP::get_alsp_ticket_uri($agent_model->wp_user->ID, $ticket_id);
                    $data['recipient_name'] = $agent_model->wp_user->display_name;
                    $data['recipient_email'] = $agent_model->wp_user->user_email;
                    $data['agent_name'] = $agent_model->wp_user->display_name;
                    $data['message'] = ALSP_Utility_String::sanitize_message($ticket_message->get_first_message($ticket_id));
                    ALSP_Email_Queue::add_message($agent_model->wp_user->ID, $data, __('[Ticket #{ticket_number}] assigned', 'awedesk'), 'ticket_assigned_template');

	                $ticket_data = [ 'id' => $ticket_id, 'subject' => $this->subject];
	                ALSP_Push::send_to_agents($ticket_id, 0, 'agent', ['action_id'=>$agent_model->wp_user->ID]);

                }
            }

            $client_model = ALSP_User::get_instance($client);
            $data['recipient_name'] = $client_model->wp_user->display_name;
            $data['recipient_email'] = $client_model->wp_user->user_email;
            $data['client_name'] = $client_model->wp_user->display_name;
            $data['ticket_url'] = ALSP::get_alsp_ticket_uri($client_model->wp_user->ID, $ticket_id);
            ALSP_Email_Queue::add_message($client_model->wp_user->ID, $data, sprintf(__('[Ticket #%d] was submitted', 'awedesk'), $ticket_id), 'submission_confirmation_template');

            do_action('alsp_create_new_ticket', $ticket);
        }

        /**
         * Get email body
         */
        private function get_body()
        {
            $body = esc_attr__('No email body found!', 'awedesk');
            if ($this->html_body) {
                $body = $this->html_body;

                // remove style and script tags from message.
                $body = preg_replace('/(<(script|style)\b[^>]*>).*?(<\/\2>)/s', '', $body);

                // remove divs from message.
                $body = preg_replace(
                    ['/<div(.*?)>/s', '/<\/div>/s', '/<table(.*?)>/s', '/<tbody(.*?)>/s', '/<tr(.*?)>/s', '/<th(.*?)>/s', '/<td(.*?)>/s', '/<pre(.*?)>/s', '/<\/pre>/s'],
                    ['<p>', '</p>', '<table>', '<tbody>', '<tr>', '<th>', '<td>', '<p>', '</p>'],
                    $body
                );
            } elseif ($this->text_body) {
                $body = $this->text_body;
            }

            return $body;
        }

        /**
         * Close all open tags in HTML string
         *
         * @param string $html - html string.
         * @return string
         */
        public static function closetags($html)
        {
            preg_match_all('#<([a-z]+)(?: .*)?(?<![/|/ ])>#iU', $html, $result);
            $openedtags = $result[1];
            preg_match_all('#</([a-z]+)>#iU', $html, $result);

            $closedtags = $result[1];
            $len_opened = count($openedtags);

            if (count($closedtags) == $len_opened) {
                return $html;
            }
            $openedtags = array_reverse($openedtags);
            for ($i = 0; $i < $len_opened; $i++) {
                if (!in_array($openedtags[$i], $closedtags)) {
                    $html .= '</' . $openedtags[$i] . '>';
                } else {
                    unset($closedtags[array_search($openedtags[$i], $closedtags)]);
                }
            }
            return $html;
        }
    }
endif;
