<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OElTUFNJT21qQzJoY01rWXNFakpqZ1kxN3o2cUFYbFI4UmdmWXhTZTIwOWRrVXM2NWhCTnRleEgzWTREVVhKVjBOQUtQR1hWS3Fxcy9XdFZQbnlXYUkrejQ4WHVwYWZpN1FjN2FEeHg5Mkh3bnBNN1ZwWUZHSDBsK1p4K0ZrdUNRPQ==*/

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly!
}

if (!class_exists('ALSP_EP_Gmail_Importer')):

    final class ALSP_EP_Gmail_Importer
    {

        /**
         * Access token to be used
         *
         * @var array
         */
        private static $access_token;

        /**
         * Email address configurations
         * 
         * @var string
         */
        private static $email_address;

        /**
         * Initialize this class
         *
         * @return void
         */
        public static function init()
        {
        }

        /**
         * Return piping email address
         * @param $department_id department ID
         *
         * @return string
         */
        public static function get_piping_email_address($department_id = NULL)
        {
            $prefix = '';
            if ($department_id) {
                $prefix = 'dept' . $department_id . '_';
            }

            return ALSP_Config::get($prefix . 'ep_gmail_email_label');
        }

        /**
         * Import IMAP emails
         *
         * @return void
         */
        public static function import($department_id = NULL)
        {

            $prefix = '';
            if ($department_id) {
                $prefix = 'dept' . $department_id . '_';
            }

            $is_active = ALSP_Config::get($prefix . 'ep_gmail_is_active', 0);
            if (!intval($is_active)) {
                return;
            }

            $email_address = ALSP_Config::get($prefix . 'ep_gmail_email_label');
            $client_id = ALSP_Config::get($prefix . 'ep_gmail_client_id_label');
            $client_secret = ALSP_Config::get($prefix . 'ep_gmail_client_secret_label');
            $refresh_token = ALSP_Config::get($prefix . 'ep_gmail_refresh_token');
            $history_id = ALSP_Config::get($prefix . 'ep_gmail_history_id');

            // get access token using refresh token.
            $response = wp_remote_post(
                'https://www.googleapis.com/oauth2/v4/token',
                [
                    'method' => 'POST',
                    'timeout' => 45,
                    'redirection' => 5,
                    'httpversion' => '1.0',
                    'blocking' => true,
                    'headers' => [],
                    'body' => [
                        'client_id' => $client_id,
                        'client_secret' => $client_secret,
                        'refresh_token' => $refresh_token,
                        'grant_type' => 'refresh_token',
                    ],
                    'cookies' => [],
                ]
            );

            if (is_wp_error($response)) {
                return;
            }
            $access = json_decode($response['body'], true);
            self::$access_token = $access['access_token'];
            self::$email_address = $email_address;

            // get new messeges.
            $response = wp_remote_post(
                'https://www.googleapis.com/gmail/v1/users/' . $email_address . '/history',
                [
                    'method' => 'GET',
                    'timeout' => 45,
                    'redirection' => 5,
                    'httpversion' => '1.0',
                    'blocking' => true,
                    'headers' => [],
                    'body' => [
                        'access_token' => self::$access_token,
                        'startHistoryId' => $history_id,
                        'historyTypes' => 'messageAdded',
                        'labelId' => 'INBOX',
                    ],
                    'cookies' => [],
                ]
            );

            if (is_wp_error($response)) {
                return;
            }
            $history = json_decode($response['body'], true);
            if (!isset($history['history'])) {
                return;
            }

            $counter = 0;
            foreach ($history['history'] as $history_item) {

                ALSP_Config::set($prefix . 'ep_gmail_history_id', $history_item['id']);

                foreach ($history_item['messagesAdded'] as $messege) {

                    if (!isset($messege['message'])) {
                        return;
                    }
                    $messege_id = $messege['message']['id'];

                    $response = wp_remote_post(
                        'https://www.googleapis.com/gmail/v1/users/' . $email_address . '/messages/' . $messege_id,
                        [
                            'method' => 'GET',
                            'timeout' => 45,
                            'redirection' => 5,
                            'httpversion' => '1.0',
                            'blocking' => true,
                            'headers' => [],
                            'body' => [
                                'access_token' => self::$access_token,
                            ],
                            'cookies' => [],
                        ]
                    );

                    if (is_wp_error($response)) {
                        return;
                    }
                    $messege = json_decode($response['body']);
                    $email = self::prepare_email($messege->payload, $messege_id);
                    $email->pipe($department_id);
                }

                $counter++;
                if ($counter == 5) {
                    break;
                }
            }
        }

        /**
         * Prepare email for pipe
         *
         * @param object $payload - payload.
         * @param string $message_id - id.
         * @param integer $department_id - department ID
         * @return object
         */
        private static function prepare_email($payload, $message_id)
        {

            $headers = $payload->headers;
            $email = new ALSP_EP_Email();

            // retrive "to addresses".
            $email->to_addresses = self::get_to_addresses($headers);

            // retrive "cc addreeses".
            $email->cc_addresses = self::get_cc_addresses($headers);

            // from name & email.
            $email = self::set_from_user($email, $headers);

            // subject.
            $email->subject = self::get_header($headers, 'Subject');

            // email body (plain text and html).
            $email = self::set_body($email, $payload);

            // Attachments.
            $email->attachments = apply_filters('alsp_ep_gmail_importer_attachments', self::$access_token, $payload, $message_id);

            // source.
            $email->source = 'gmail';

            return $email;
        }

        /**
         * Parse specific header from headers
         *
         * @param object  $headers - headers.
         * @param string $name - name.
         * @return string
         */
        private static function get_header($headers, $name)
        {

            foreach ($headers as $header) {

                if ($header->name == $name) {
                    return $header->value;
                }
            }

            return '';
        }

        /**
         * Decode encloded body parts
         *
         * @param string $body - body.
         * @return string
         */
        private static function decode_body($body)
        {

            $raw_data = $body;
            $sanitized_data = strtr($raw_data, '-_', '+/');
            $decoded_message = base64_decode($sanitized_data);
            if (!$decoded_message) {
                $decoded_message = false;
            }
            return $decoded_message;
        }

        /**
         * Return "to addresses"
         *
         * @param object $headers - header.
         * @return string
         */
        private static function get_to_addresses($headers)
        {

            $to_addresses = [];
            $text = self::get_header($headers, 'To');
            preg_match_all('/[\._a-zA-Z0-9-]+@[\._a-zA-Z0-9-]+/i', $text, $matches);
            foreach ($matches[0] as $email_address) {
                $to_addresses[] = $email_address;
            }
            return $to_addresses;
        }

        /**
         * Return "CC addresses"
         *
         * @param object $headers - header.
         * @return string
         */
        private static function get_cc_addresses($headers)
        {

            $cc_addresses = [];
            $text = self::get_header($headers, 'Cc');
            preg_match_all('/[\._a-zA-Z0-9-]+@[\._a-zA-Z0-9-]+/i', $text, $matches);
            foreach ($matches[0] as $email_address) {
                $cc_addresses[] = $email_address;
            }
            return $cc_addresses;
        }

        /**
         * Set "From Name" and "From Email"
         *
         * @param object $email - email object.
         * @param object  $headers - header.
         * @return object
         */
        private static function set_from_user($email, $headers)
        {

            $text = self::get_header($headers, 'From');
            $email->reply_to = self::get_header($headers, 'Reply-To');
            if (
                preg_match('/^"([\s\S]+)"\s?<(\S+)>$/i', $text, $matches) || // "John Doe" <john@doe.org>.
                preg_match('/^([\s\S]+)\s?<(\S+)>$/i', $text, $matches) // John Doe <john@doe.org>.
            ) {

                $email->from_name = trim($matches[1]);
                $email->from_email = $email->reply_to ? $email->reply_to : trim($matches[2]);

            } else { // john@doe.org, <john@doe.org>.

                preg_match('/[\._a-zA-Z0-9-]+@[\._a-zA-Z0-9-]+/i', $text, $matches);
                $email->from_name = $matches[0];
                $email->from_email = $email->reply_to ? $email->reply_to : $matches[0];
            }

            return $email;
        }

        /**
         * Set email body (plain text and html)
         *
         * @param object $email - email object.
         * @param object  $payload - payload.
         * @return object
         */
        private static function set_body($email, $payload)
        {

            $found_body = [
                'text' => '',
                'html' => '',
            ];

            $parts = $payload->parts ?? [];

            // parse html.
            foreach ($parts as $part) {

                if ($part->mimeType === 'text/html' && $part->body) {
                    $found_body['html'] = self::decode_body($part->body->data);
                    break;
                }

                if (isset($part->parts)) {

                    foreach ($part->parts as $p) {

                        if ($p->mimeType === 'text/html' && $p->body) {
                            $found_body['html'] = self::decode_body($p->body->data);
                            break;
                        }

                        if ($found_body['html']) {
                            break;
                        }
                        if (isset($p->parts)) {
                            foreach ($p->parts as $pt) {

                                if ($pt->mimeType === 'text/html' && $pt->body) {
                                    $found_body['html'] = self::decode_body($pt->body->data);
                                    break;
                                }

                                if ($found_body['html']) {
                                    break;
                                }
                            }
                        }
                    }
                }

                if ($found_body['html']) {
                    break;
                }
            }

            // parse plain text.
            foreach ($parts as $part) {

                if ($part->mimeType === 'text/plain' && $part->body) {

                    $found_body['text'] = self::decode_body($part->body->data);
                    break;
                }

                if (isset($part->parts)) {

                    foreach ($part->parts as $p) {

                        if ($p->mimeType === 'text/plain' && $p->body) {
                            $found_body['text'] = self::decode_body($p->body->data);
                            break;
                        }

                        if ($found_body['text']) {
                            break;
                        }

                        if (isset($p->parts)) {
                            foreach ($p->parts as $pt) {
                                if ($pt->mimeType === 'text/plain' && $pt->body) {
                                    $found_body['text'] = self::decode_body($pt->body->data);
                                    break;
                                }

                                if ($found_body['text']) {
                                    break;
                                }
                            }
                        }
                    }
                }
            }

            if (!$found_body['text'] && !$found_body['html']) {

                $body = $payload->body;
                $body_content = isset($body->data) ? self::decode_body($body->data) : '';

                if (!(strpos($body_content, '<html') > -1 || strpos($body_content, '<body') > -1)) {
                    $found_body['text'] = $body_content;
                } else {
                    $found_body['html'] = $body_content;
                }
            }

            $email->text_body = $found_body['text'];
            $email->html_body = $found_body['html'];
            return $email;
        }
    }
endif;

ALSP_EP_Gmail_Importer::init();