<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2L2hqSE9ncDBnUHFYZ3BvNG5nTitiWnVVNHBwUjhGcDhGVEZBRUlSdjkvL0RiQlcvMXRhUGFWclRQVVRlM2ZXbGQ4Vzh2UncxVGRaTGVaVnVaRERJSytkaXQ3aTBJUEpqeGd4MWJVWEZhKzVhYXV3TGYyckJwejV0azZPZCtuWjJRPQ==*/
if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly!
}

if (!class_exists('ALSP_EP_IMAP_Importer')):

    final class ALSP_EP_IMAP_Importer
    {
        /**
         * Initialize this class
         *
         * @return void
         */
        public static function init()
        {
        }

        /**
         * Return piping email address
         * @param $department_id department ID
         *
         * @return string
         */
        public static function get_piping_email_address($department_id = NULL)
        {
            $prefix = '';
            if ($department_id) {
                $prefix = 'dept' . $department_id . '_';
            }
            return ALSP_Config::get($prefix . 'ep_imap_email_label');
        }

        /**
         * Import IMAP emails
         *
         * @return void
         */
        public static function import($department_id = NULL)
        {

            $prefix = '';
            if ($department_id) {
                $prefix = 'dept' . $department_id . '_';
            }

            $is_active = ALSP_Config::get($prefix . 'ep_imap_is_active', 0);
            if (!$is_active) {
	            //new ALSP_Debug('!ep_imap_is_active');
                return;
            }

            $encryption = ALSP_Config::get($prefix . 'ep_imap_encryption_select', 'ssl');
	        if(!strlen($encryption)) $encryption = 'ssl';
            $encryption_text = '';
            if ($encryption == 'none') {
                $encryption_text = '/novalidate-cert';
            } elseif ($encryption == 'ssl') {
                $encryption_text = '/imap/ssl/novalidate-cert';
            } elseif ($encryption == 'tls') {
                $encryption_text = '/imap/tls/novalidate-cert';
            }

            // attachment directory.
            $filepath = ALSP_Attachments::get_dir() . 'imap';
            if (!file_exists($filepath)) {
	            //new ALSP_Debug('mkdir');
                mkdir($filepath, 0755, true);
            }

            $incoming_mail_server = ALSP_Config::get($prefix . 'ep_imap_incoming_mail_server_label', 'ssl');
            $port = ALSP_Config::get($prefix . 'ep_imap_port_number');
            $email_address = ALSP_Config::get($prefix . 'ep_imap_email_label');
            $password = ALSP_Config::get($prefix . 'ep_imap_email_password');
            $mailbox = new PhpImap\Mailbox(
                '{' . $incoming_mail_server . ':' . $port . $encryption_text . '}INBOX',
                // IMAP server and mailbox folder.
                $email_address,
                // Username for the before configured mailbox.
                $password,
                // Password for the before configured username.
                $filepath, // Directory, where attachments will be saved (optional).
            );

            $last_uid = ALSP_Config::get($prefix . 'ep_imap_uid', 0);
            $next_uid = intval($last_uid) + 1;

            try {
                $history = imap_fetch_overview($mailbox->getImapStream(), $next_uid . ':*', FT_UID);
                $uids = [];
                $counter = 0;
                foreach ($history as $overview) {
                    if ($overview->uid == $last_uid) {
                        continue;
                    }
                    $uids[] = $overview->uid;
                    $counter++;
                    if ($counter == 5) {
                        break;
                    }
                }

                foreach ($uids as $uid) {
                    ALSP_Config::set($prefix . 'ep_imap_uid', $uid);
                    self::empty_attachments($filepath);
                    $mail = $mailbox->getMail($uid);
                    $email = self::prepare_email($mail, $mailbox);
                    $email->pipe($department_id);
                }

                $mailbox->disconnect();

            } catch (Exception $ex) {
				error_log($ex->getMessage());
                if (preg_match('/AUTHENTICATIONFAILED/', $ex->getMessage())) {
                    // set not active flag in imap settings and show admin notice!
                    ALSP_Config::set($prefix . 'ep_imap_is_active', 0);
                    ALSP_Config::set($prefix . 'ep_imap_connection_notice', 1);
                }
            }
        }

        /**
         * Parse and pipe email
         *
         * @param object $mail - mail id.
         * @param object $mailbox - mailbox.
         *
         * @return ALSP_EP_Email
         */
        public static function prepare_email($mail, $mailbox)
        {

            $email = new ALSP_EP_Email();

            // retrive "to addresses".
            $email->to_addresses = array_keys($mail->to);

            // retrive "cc addreeses".
            $email->cc_addresses = array_keys($mail->cc);

            // from name.
            $email->from_name = $mail->fromName ? $mail->fromName : $mail->fromAddress; //phpcs:ignore

            // Reply to email.
            $email->reply_to = array_keys($mail->replyTo)[0]; //phpcs:ignore

            // from email.
            $email->from_email = $email->reply_to ? $email->reply_to : $mail->fromAddress; //phpcs:ignore

            // subject.
            $email->subject = $mail->subject;

            // HTML body.
            $email->html_body = $mail->textHtml; //phpcs:ignore

            // Text body.
            $email->text_body = $mail->textPlain; //phpcs:ignore

            // attachments.
            $email->attachments = apply_filters('alsp_ep_imap_importer_attachments', $mail);

            // source.
            $email->source = 'imap';

            return $email;
        }

        /**
         * Remove all attachments from temporary folder before importing next email
         *
         * @param string $filepath - filepath.
         * @return void
         */
        private static function empty_attachments($filepath)
        {

            $files = glob($filepath . '/*');
            foreach ($files as $file) { // iterate files.
                unlink($file); // delete file.
            }
        }

        /**
         * Parse attachments
         *
         * @param object        $mail - email object.
         * @return void|array
         */
        private static function parse_attachments($mail)
        {
            $result = [];
            if (!$mail->hasAttachments()) {
                return;
            }

            $attachments = $mail->getAttachments();
            foreach ($attachments as $attachment) {
                // @TODO: embed attachment
                // if ( $general['body-reference'] == 'html' && $attachment->contentId && preg_match( '/cid:' . $attachment->contentId . '/', $email->html_body ) ) { //phpcs:ignore
                // 	self::upload_embeded_img( $attachment, $mail, $email );
                // } else {
                $result[] = self::upload_thread_attachment($attachment);
                // }
            }

            return $result;
        }

        /**
         * Upload embeded image in the description
         *
         * @param object        $attachment - attachment array received from IMAP.
         * @param ALSP_EP_Email $email - email object.
         * @return void
         */
        private static function upload_embeded_img($attachment, $email)
        {

            // Database insert array init.
            $data = ['name' => $attachment->name];

            // saperate file name and extension.
            $filename = explode('.', $attachment->name);
            $extension = strtolower($filename[count($filename) - 1]);
            unset($filename[count($filename) - 1]);
            $filename = implode('.', $filename);
            $filename = str_replace(' ', '_', $filename);
            $filename = str_replace(',', '_', $filename);
            $filename = uniqid() . '_' . preg_replace('/[^A-Za-z0-9\-]/', '', $filename);

            // allowed file extenstions.
            $img_extensions = ['png', 'jpeg', 'jpg', 'gif'];
            if (!(in_array($extension, $img_extensions))) {
                self::upload_thread_attachment($attachment);
            }

            $data['is_image'] = 1;
            $data['source'] = 'img_editor';

            // Create file path.
            $today = new DateTime();
            $upload_dir = wp_upload_dir();
            $filepath = $upload_dir['basedir'] . '/wpsc/' . $today->format('Y');
            if (!file_exists($filepath)) {
                mkdir($filepath, 0755, true);
            }
            $filepath .= '/' . $today->format('m');
            if (!file_exists($filepath)) {
                mkdir($filepath, 0755, true);
            }
            $filepath .= '/' . $filename . '.' . $extension;

            $filepath_short = '/wpsc/' . $today->format('Y') . '/' . $today->format('m') . '/' . $filename . '.' . $extension;
            $data['file_path'] = $filepath_short;

            // Create time.
            $data['date_created'] = $today->format('Y-m-d H:i:s');

            // write file.
            file_put_contents($filepath, $attachment->getContents()); //phpcs:ignore

            // insert db record.
            $attach = WPSC_Attachment::insert($data);

            if ($attach->id) {
                $img_url = home_url('/') . '?wpsc_attachment=' . $attach->id;
                $email->html_body = str_replace('cid:' . $attachment->contentId, $img_url, $email->html_body); //phpcs:ignore
            }
        }

        /**
         * Upload regular thread attachments
         *
         * @param object         $attachment - attachment array received from microsoft exchange server.
         * @return void|string
         */
        private static function upload_thread_attachment($attachment)
        {
            // saperate file name and extension.
            $filename = explode('.', $attachment->name);
            $extension = strtolower($filename[count($filename) - 1]);
            unset($filename[count($filename) - 1]);
            $filename = implode('.', $filename);
            $filename = str_replace(' ', '_', $filename);
            $filename = str_replace(',', '_', $filename);
            $filename = uniqid() . '_' . preg_replace('/[^A-Za-z0-9\-]/', '', $filename);

            // allowed file extenstions.
            $allowed_file_extensions = ALSP_Config::get('attachment_allowed_file_types_textarea');
            $allowed_file_extensions = explode(',', $allowed_file_extensions);
            $allowed_file_extensions = array_map('trim', $allowed_file_extensions);
            $allowed_file_extensions = array_map('strtolower', $allowed_file_extensions);
            if (!(in_array($extension, $allowed_file_extensions))) {
                return;
            }

            // check for allowed file size.
            $max_upload_size = ALSP_Config::get('attachment_max_filesize_number');

            // use WP max upload size if it is smaller than PeepSo max upload size
            $wp_max_size = max(wp_max_upload_size(), 0);
            $wp_max_size /= pow(1024, 2);
            if ($wp_max_size < $max_upload_size) {
                $max_upload_size = $wp_max_size;
            }

            $allowed_file_size = intval($max_upload_size) * 1048576;
            if ($attachment->sizeInBytes >= $allowed_file_size) { //phpcs:ignore
                return;
            }

            // Check for image type.
            $img_extensions = ['png', 'jpeg', 'jpg', 'bmp', 'pdf', 'gif'];
            if (!in_array($extension, $img_extensions)) {
                $extension .= '.txt';
            } else {
                $data['is_image'] = 1;
            }

            // Write to temp folder
            $destination = ALSP_Attachments::get_tmp_dir() . $filename . '.' . $extension;

            // write file.
            file_put_contents($destination, $attachment->getContents()); //phpcs:ignore

            return $filename . '.' . $extension;
        }
    }
endif;

ALSP_EP_IMAP_Importer::init();
