<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OGVvdnI0OGZpREZaVFY2bVVtdE1BYXNraHBFN3pqVzlKVzFpUkhHTUkzcktCTkM4aWRwakVqU3B0MDFNM2k3L2xTWHpSc0FtTnYrOHRYNkJQYkNVLzcyblhvRFUyTGdrMU5hRzhQaEFNbnBoL3pJL2pJUmVIaVpGTlV5QmJ1M1VBPQ==*/
if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly!
}

if (!class_exists('ALSP_EP_ME_Importer')):

    final class ALSP_EP_ME_Importer
    {

        /**
         * Initialize this class
         *
         * @return void
         */
        public static function init()
        {
        }

        /**
         * Return piping email address
         * @param $department_id department ID
         *
         * @return string
         */
        public static function get_piping_email_address($department_id = NULL)
        {
            $prefix = '';
            if ($department_id) {
                $prefix = 'dept' . $department_id . '_';
            }

            return ALSP_Config::get($prefix . 'ep_me_email_label');
        }

        /**
         * Import IMAP emails
         *
         * @return void
         */
        public static function import($department_id = NULL)
        {
            $prefix = '';
            if ($department_id) {
                $prefix = 'dept' . $department_id . '_';
            }

            $is_active = ALSP_Config::get($prefix . 'ep_me_is_active', 0);
            if (!intval($is_active)) {
                return;
            }

            $access_token = ALSP_Config_EP_Microsoft_Exchange::get_access_token();
            if (false === $access_token) {
                return;
            }

            // get delta messages.
            $delta_url = ALSP_Config::get($prefix . 'ep_me_delta_url', 'https://graph.microsoft.com/v1.0/me/mailFolders/inbox/messages/delta?changeType=created', TRUE);
            $response = wp_remote_get(
                $delta_url,
                [
                    'headers' => [
                        'Authorization' => 'Bearer ' . $access_token,
                        'Prefer' => 'odata.maxpagesize=10',
                        'Content-Type' => 'application/json',
                    ],
                ]
            );

            if (!ALSP_Config_EP_Microsoft_Exchange::is_valid_response($response)) {
                return;
            }

            $response = json_decode($response['body'], true);
            $delta_url = isset($response['@odata.deltaLink']) ? $response['@odata.deltaLink'] : $response['@odata.nextLink'];
            ALSP_Config::set($prefix . 'ep_me_delta_url', $delta_url);

            if ($response['value']) {

                foreach ($response['value'] as $message) {

                    $email = new ALSP_EP_Email();

                    // retrive "to addresses".
                    $email->to_addresses = array_map(
                        fn($to) => $to['emailAddress']['address'],
                        $message['toRecipients']
                    );

                    // retrive "cc addreeses".
                    $email->cc_addresses = array_map(
                        fn($cc) => $cc['emailAddress']['address'],
                        $message['ccRecipients']
                    );

                    // from name.
                    $email->from_name = $message['from']['emailAddress']['name'];

                    // Reply to email.
                    $email->reply_to = $message['replyTo'] ? $message['replyTo'][0]['emailAddress']['address'] : '';

                    // from email.
                    $email->from_email = $email->reply_to ? $email->reply_to : $message['from']['emailAddress']['address'];

                    // subject.
                    $email->subject = $message['subject'];

                    // HTML body.
                    $email->html_body = $message['body']['contentType'] == 'html' ? $message['body']['content'] : '';

                    // Text body.
                    $email->text_body = $message['body']['contentType'] == 'text' ? $message['body']['content'] : '';

                    // Attachments.
                    $email->attachments = apply_filters('alsp_ep_microsoft_exchange_importer_attachments', $message);

                    // source.
                    $email->source = 'me';

                    $email->pipe();
                }
            }
        }
    }
endif;

ALSP_EP_ME_Importer::init();