<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OFNnMlBsOUhuMUYvL2lveVpZS3lUWmF6cUczL2drdVk3QkZHNll5eFVOR2dvUGdtQk1sSnpxRHRKMkZMTk5uVFJ5b2xKQzRZZFZHY00zYWF0Vks0b3dCMGdHRzhhUnFSY2F3N25YTWRRVkxQaEFKR0ZpcU5PUklqSVBKc2lESFZJPQ==*/

class ALSP_Email_Piping
{

    const COMPONENT_NAME = 'email-piping';
    const CRON_EMAIL_PIPING = 'alsp_ep_import_emails_event';

    private static $instance = NULL;

    /**
     * Connections
     *
     * @var array
     */
    public static $piping_connections = [];

    public static function get_instance()
    {
        return (NULL == self::$instance) ? self::$instance = new self() : self::$instance;
    }

    private function __construct()
    {
        if (is_admin()) {

            add_action('admin_init', function () {
                if (0 == ALSP_Config::get('cron_email_piping_enabled')) {
                    if (!stristr(json_encode(_get_cron_array()), ALSP_Email_Piping::CRON_EMAIL_PIPING)) {
                        add_action('admin_notices', [$this, 'cron_email_piping_notice']);
                    }
                }
            });

            add_action('admin_enqueue_scripts', function () {
                $assets = ALSP_Template::get_instance(ALSP_Email_Piping::COMPONENT_NAME);

                if ($assets->is_dev()) {
                    $order = json_decode(file_get_contents("{$assets->assets_dir}js/admin/index.json"));
                    $handle = null;

                    foreach ($order as $file) {
                        $dependencies = array_merge(['jquery', 'underscore', 'wp-api-request', 'alsp-admin'], $handle ? [$handle] : []);
                        $handle = 'alsp-admin' . ('init' === $file ? '-email-piping' : "-{$file}");
                        // Fix relative path issue in handle.
                        while (strpos($handle, '../') !== false) {
                            $handle = preg_replace('/\.\.\/[^\/]+\//', '', $handle);
                        }
                        wp_register_script($handle, $assets->get_assets_url("js/admin/{$file}.js"), $dependencies, ALSP::PLUGIN_VERSION, true);
                    }
                } else {
                    wp_register_script('alsp-admin-email-pipping', $assets->get_assets_url('js/admin.js'), ['jquery', 'underscore', 'wp-api-request', 'alsp-admin'], ALSP::PLUGIN_VERSION, true);
                }

                //                wp_localize_script('alsp-admin-class-alsp-admin-email-piping', 'alspAdminEmailPipingData', [
                //                    'templates' => [
                //                        'admin_ep_general' => $assets->get_template('admin/general'),
                //                    ]
                //                ]);
            }, 20);
        }

        add_action('wp', function () {
            if (isset($_GET['alsp_process_import_emails'])) {
                ALSP_Email_Piping::process_import_emails();
                die();
            }
        }, -1);

        add_filter('alsp_rest_paths', function ($rest_paths) {
            $rest_path = dirname(__FILE__) . DIRECTORY_SEPARATOR . 'classes' . DIRECTORY_SEPARATOR . 'api' . DIRECTORY_SEPARATOR . 'rest' . DIRECTORY_SEPARATOR . ALSP_API::REST_V;
            array_push($rest_paths, $rest_path);

            return $rest_paths;
        });

        // Load php-imap library.
        if (!class_exists('PhpImap\Mailbox')) {
            ALSP_Autoload::load_library(dirname(__FILE__) . DIRECTORY_SEPARATOR . 'lib' . DIRECTORY_SEPARATOR . 'php-imap');
        }

        // connections.
        add_action('init', [__CLASS__, 'set_piping_connections']);
    }

    public static function cron_email_piping_notice()
    {
        $result = FALSE;
        if (!wp_next_scheduled(ALSP_Email_Piping::CRON_EMAIL_PIPING)) {
            $result = wp_schedule_event(current_time('timestamp', TRUE), 'five_minutes', ALSP_Email_Piping::CRON_EMAIL_PIPING);
        }

        if (!$result || $result instanceof WP_Error) {
            echo '<div class="error alsp">' .
                'Awedesk: "Email Piping" cron encountered some problems and was re-initialized.<br><small>If you see this message repeatedly, there might be something wrong with your WordPress Cron. Consider deactivating and re-activating Awedesk or contacting Support.</small>'
                . '</strong></div>';
        }
    }

    /**
     * Set piping connections
     *
     * @return void
     */
    public static function set_piping_connections()
    {
        self::$piping_connections = apply_filters(
            'alsp_ep_piping_connections',
            [
                'imap' => [
                    'label' => 'IMAP',
                    'class' => 'ALSP_EP_IMAP_Importer',
                ],
                'gmail' => [
                    'label' => 'Gmail',
                    'class' => 'ALSP_EP_Gmail_Importer',
                ],
                // 'microsoft-exchange' => [
                //     'label' => 'Microsoft Exchange',
                //     'class' => 'ALSP_EP_ME_Importer',
                // ],
            ]
        );
    }

    public static function process_import_emails()
    {
        // Do not import while initialization process.
        if ('yes' === get_transient('alsp_ep_connection_init')) {
            //new ALSP_Debug('alsp_ep_connection_init');
            return;
        }

        $ep_enabled = ALSP_Config::get('ep_enabled', 0);
        if (!$ep_enabled) {
            return;
        }

        $ep_connection = ALSP_Config::get('ep_connection_select', 'imap');
        $connection_class = isset(self::$piping_connections[$ep_connection]) ? self::$piping_connections[$ep_connection]['class'] : '';
        if ($connection_class && class_exists($connection_class)) {
            $connection_class::import();
        }
    }

    public static function default_department($to_addresses = [])
    {

        $default = ALSP_Config::get('default_department_select');
        if (count($to_addresses) > 0) {
            $forwarders = new ALSP_Model_Email_Piping_Forwarders(new WP_REST_Request());
            foreach ($forwarders->read() as $key => $forwarder) {
                if (!isset($forwarder['id']) && !$forwarder['id']) {
                    continue;
                }

                if (in_array($forwarder['receiver'], $to_addresses)) {
                    $default = $forwarder['department']['id'];
                }
            }
        }

        if ($default > 0) {

            //new ALSP_Debug("Default department $default, checking");

            $r = new WP_REST_Request();
            $r['id'] = $default;
            $department = (new ALSP_Model_Departments($r))->read(1);

            if (array_key_exists('id', $department)) {
                return $department['id'];
            } elseif (array_key_exists(0, $department) && array_key_exists('id', $department[0])) {
                return $department[0]['id'];
            }

            // Department not found/unpublished
            ALSP_Config::set('default_department_select', 0);
        }

        return 0;
    }
}

ALSP_Email_Piping::get_instance();

// EOF
