<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OEN5WXVTOHY1UGY5dTdMY1EwWnhZRzBpUmlFWFhqYWhvRzQ2MGkyTjBTcE1hVjVTc1p6Myt4U2FhSkxIWFVmaUdqeVorZEw4d0t2aHNlVTRFcXIvR1oza2hxbmdFcUdyVjFQSktPOWFuc0RrOC9rUlI3anFJOEt5UEJhMmw0UVNZPQ==*/

class ALSP_Email_Queue
{
	const TABLE = 'alsp_mail_queue';

	// values for the `status` column
	const STATUS_PENDING = 0;
	const STATUS_PROCESSING = 1;
	const STATUS_DELAY = 2;
	const STATUS_FAILED = 3;
	//	const STATUS_SENT = 4;					// TODO: not used yet
	const STATUS_NOW = 5;
	const STATUS_RETRY = 6;

	private static $_max_exec_time = 30;

	/**
	 * Returns the privately defined $_max_exec_time variable value.
	 * @return int
	 */
	private static function get_max_exec_time()
	{
		$timeout = self::$_max_exec_time;

		if (isset($_GET['timeout'])) {
			$timeout = $_GET['timeout'];
		}

		return ($timeout);
	}

	/*
	 * Adds an item to the mail queue. NOTE: This should only be called from add_message()
	 * @param int $user_id The user id of the recipient
	 * @param string $email The email address of the recipient
	 * @param string $subject The email subject line
	 * @param string $content The contents of the email to send to the recipent
	 * @param int $now If set to 1 email message will be sent immediately
	 */
	private static function add($user_id, $email, $subject, $content, $now = 0)
	{
//		new ALSP_Debug("awedesk $user_id $email $now $subject $content");
		$now = current_time('mysql', TRUE);
		$aCols = [
			'user_id' => intval($user_id),
			'recipient' => $email,
			'created_at' => $now,
			'subject' => wp_specialchars_decode($subject, ENT_QUOTES),
			'message' => wp_specialchars_decode($content, ENT_QUOTES),
		];

		if (1 === $now) {
			$aCols['status'] = self::STATUS_NOW;
		}

		global $wpdb;
		$wpdb->insert($wpdb->prefix . self::TABLE, $aCols);
		return TRUE;
	}

	/*
	 * add item to the mail queue
	 * @param int $user_id The user id of the recipient
	 * @param array $data The tokens for the email template
	 * @param string $subject The email subject line
	 * @param string $template The template name to send to the recipent
	 * @param int $now If set to 1 the email will be sent immediately
	 * @param int $send If set to 1 the email will be enqueued. If 0, only the related action will be fired.
	 * @param int $run_action If se to 1 the related action will be fired
	 */
	public static function add_message($user_id, $data, $subject, $template, $now = 0, $send = 1, $run_action = 1)
	{
		if ($user_id !== 0) {
			if (!ALSP_User_Preferences::get_instance($user_id)->get('receive_emails')) {
				return FALSE;
			}
		}

		if (1 == $run_action) {
			do_action('alsp_mailqueue_add', $user_id, $data, $subject, $template, $now);
		}

		if (0 == $send) {
			return FALSE;
		}

		$data = apply_filters('alsp_mailqueue_add_data', $data, $user_id, $subject, $template);

		$assets = ALSP_Frontend_Assets::get_instance();
		$override = ALSP_Config::get('emails_override_entire_html_template', '');
		if (strlen($override)) {
			$contents = stripslashes($override);
		} else {
			// load the general email template
			$contents = $assets->get_template('general/email', ['user_id' => $user_id]);
		}

		// load the translated contents of the email if possible
		$locale = ALSP_MultiLang_User_Locale($user_id);
		$message = ALSP_Config::get('email_' . $template . '_' . strtolower($locale));
		if (!strlen($message)) {
			// Default language fallback
			$message = ALSP_Config::get('email_' . $template);
		}

		if (!isset($data['recipient_email'])) {
			$data['recipient_email'] = get_userdata($user_id)->user_email;
		}

		// setup the template replacement data
		$em = new ALSP_Email_Template();
		$em->set_tokens($data);

		// Override link if exists
		if (!isset($data['link']) && (isset($data['ticket_title']) && isset($data['ticket_number']) && isset($data['ticket_url']))) {
			$link_template = ALSP_Config::get('email_link_template');
			$data['link'] = $em->replace_content_tokens($link_template);
		}

		if (!isset($data['signature'])) {
			$signature_template = ALSP_Config::get('email_signature_template');
			$data['signature'] = $em->replace_content_tokens($signature_template);
		}

		// Repopulate tokens
		$em->set_tokens($data);

		$content_footer = $assets->get_template('general/email-footer', ['user_id' => $user_id]);
		$msg = $em->replace_tokens($contents, $message, $content_footer);
		$subject = $em->replace_content_tokens($subject);

		$ret = self::add($user_id, $data['recipient_email'], $subject, $msg, $now);

		if (1 === $now) {
			self::process_mailqueue(1);
		}

		return $ret;
	}

	/*
	 * Add item to the mail queue directly to an email address. This method is used when the recipient may not be a member.
	 * @param mixed $email The email address of the recipient
	 * @param array $data The tokens for the email template
	 * @param string $subject The email subject line
	 * @param string $message The message to send to the recipent
	 * @param string $type The type of notification settings to be checked if email is allowed or not
	 * @param int $module The module id originating the message
	 * @return int $sent The number of sent messages
	 */
	public static function send_message($emails, $data, $subject, $message, $type, $module = 0)
	{
		$assets = ALSP_Frontend_Assets::get_instance();
		if (!is_array($emails)) {
			$emails = [$emails];
		}

		// load the general email template
		$contents = file_get_contents($assets->get_template('general/email'));

		// setup the template replacement data
		$em = new ALSP_Email_Template();

		$sent = 0;
		foreach ($emails as $email) {
			if (!is_email($email))
				continue;

			$data['recipient_name'] = $email;
			$data['recipient_email'] = $email;

			$em->set_tokens($data);

			$msg = $em->replace_tokens($contents, $message);
			$subject = $em->replace_content_tokens($subject);

			self::add(0, $email, $subject, $msg);
			++$sent;
		}

		return ($sent);
	}

	/*
	 * Processes the items in the mail queue.
	 */
	public static function process_mailqueue($now = 0)
	{
		// Allow other plugins to run their code when Mail Queue is processed
		do_action('alsp_action_process_mailqueue');

		// MailQ
		global $wpdb;

		$sStartTime = microtime(true);

		$em = new ALSP_Email_Template();
		$from = ALSP_Config::get('site_emails_sender_label');
		$from = $em->replace_content_tokens($from);

		$headers[] = 'From: ' . $from . ' <' . ALSP_Config::get('site_emails_admin_email_label') . '>';
		$headers[] = 'Content-Type: text/html; charset=UTF-8';

		add_filter('wp_mail_content_type',[__CLASS__, 'html_content_type']);

		if (1 === $now) {
			$status = self::STATUS_NOW;
		} else {
			$status = self::STATUS_PENDING;
		}

		// Get 10 items to retry
		$queue_retry = self::get_by_status(self::STATUS_RETRY, 10);

		$limit = 50;
		if (isset($_GET['limit'])) {
			$limit = $_GET['limit'];
		}

		if (class_exists('Awedesk_Client')) {
			$limit = Awecloud_Client::get('max_emails_per_minute', 10);
		}

		// get regular items minus the amount of delayed we found
		$queue = self::get_by_status($status, $limit - count($queue_retry));

		$queue = array_merge($queue, $queue_retry);

		$iProcessed = 0;
		$sCurrentRunTime = 0;
		foreach ($queue as $mail) {

			// same WHERE clause is used multiple times
			$where = ['id' => $mail->id];

			$mail->attempts += 1;

			$wpdb->update(self::get_table_name(), ['status' => self::STATUS_PROCESSING], $where);
			$wpdb->update(self::get_table_name(), ['attempts' => $mail->attempts], $where);

			$table_name = self::get_table_name();

			add_action('wp_mail_failed', function ($wp_error) use ($wpdb, $where, $table_name, &$mail) {
				$mail->error_log .= "\n" . json_encode($wp_error);
				$wpdb->update($table_name, ['error_log' => $mail->error_log], $where);
			}, 10, 1);

			$mail = apply_filters('alsp_mailqueue_before_sending', $mail);
			$success = wp_mail($mail->recipient, wp_specialchars_decode($mail->subject), $mail->message, $headers);

			// update `status` on failure
			if (FALSE === $success) {

				// set retry by default
				$status = self::STATUS_RETRY;

				// if 5 attempts have been made, quit
				if ($mail->attempts > 5) {
					$status = self::STATUS_FAILED;
				}

				$wpdb->update(self::get_table_name(), ['status' => $status], $where);
			} else {
				do_action('alsp_mailqueue_after', $mail);
				$wpdb->delete(self::get_table_name(), $where);
			}

			++$iProcessed;

			$sCurrentRunTime = microtime(true) - $sStartTime;
			if ($sCurrentRunTime > self::get_max_exec_time())
				break;
		}

		remove_filter('wp_mail_content_type', [__CLASS__, 'html_content_type']);

		$aBatchHistory = ['elapsed' => $sCurrentRunTime, 'processed' => $iProcessed];

		$aALSPMailqueueHistory = ALSP_Config::get('mailqueue_history');

		if (!$aALSPMailqueueHistory)
			$aALSPMailqueueHistory = [];

		if (count($aALSPMailqueueHistory) >= 25)
			array_shift($aALSPMailqueueHistory);

		$aALSPMailqueueHistory[] = $aBatchHistory;

		ALSP_Config::set('mailqueue_history', $aALSPMailqueueHistory);
	}


	/*
	 * Sets the content type for email messages
	 * @return string The SMTP content type for the email
	 */
	public static function html_content_type($content_type)
	{
		//return ('text/html; charset=UTF-8');
		// #4279 @ 2.7.12 - the default encoding for wp_mail is UTF-8 anyway, and the fancy string seems to break MailPoet
		return ('text/html');
	}


	/**
	 * Fetches all mail queued up on the database.
	 * @param  int $limit  How many records to fetch.
	 * @param  int $offset Fetch records beginning from this index.
	 * @param  string  $order  Order by column.
	 * @param  string  $dir    The sort direction, defaults to 'asc'
	 * @return array Array of the result set.
	 */
	public static function fetch_all($limit = NULL, $offset = 0, $order = NULL, $dir = 'asc')
	{
		global $wpdb;

		$query = 'SELECT *				
			FROM `' . self::get_table_name() . '` ';

		if (isset($order))
			$query .= ' ORDER BY `' . $order . '` ' . $dir;

		if (isset($limit))
			$query .= ' LIMIT ' . $offset . ', ' . $limit;

		return ($wpdb->get_results($query, ARRAY_A));
	}

	/*
	 * Return list of MailQueue items based on the status
	 * @param string $status The status to filter with.
	 * @return array Returns array of MailQueue items in chronologic order
	 */
	public static function get_by_status($status, $limit = 25)
	{
		global $wpdb;

		$sql = 'SELECT * 
				FROM `' . self::get_table_name() . '`
				WHERE `status` = %d
				ORDER BY `created_at` ASC
				LIMIT %d ';
		$res = $wpdb->get_results($wpdb->prepare($sql, $status, $limit));
		return ($res);
	}

	/*
	 * Get a count of the pending items in the mail queue
	 * @return int A count of the items
	 */
	public static function get_pending_item_count()
	{
		global $wpdb;

		$sql = 'SELECT COUNT(*) AS `val`
				FROM `' . self::get_table_name() . '`
				WHERE `status`=%d ';
		$msg_count = $wpdb->get_var($wpdb->prepare($sql, self::STATUS_PENDING));

		return ($msg_count);
	}

	/*
	 * Get number of seconds to complete mail queue
	 * @return int
	 */
	public static function get_completion_estimate()
	{
		$queue_history = ALSP_Config::get('mailqueue_history');

		if (!$queue_history)
			return (0);

		$history_count = count($queue_history);
		$pending_item_count = self::get_pending_item_count();
		$run_time = 0;
		$processed = 0;

		// Get average running time and emails sent per process
		foreach ($queue_history as $history) {
			$run_time += $history['elapsed'];
			$processed += $history['processed'];
		}

		$average_run_time = $run_time / $history_count;
		$average_processed = $processed / $history_count;

		$runs_left = $pending_item_count / max($average_processed, 1);
		// Multiply	runs left to time interval and average run time
		$schedule = wp_get_schedule(ALSP::CRON_MAILQUEUE);
		$schedules = wp_get_schedules();
		$interval = $schedules[$schedule]['interval'] ?? 300;

		$estimate_time = ($interval * $runs_left) + ($average_run_time * $runs_left);

		return ($estimate_time);
	}

	/**
	 * Convenience function to return the mailqueue table name as a string.
	 * @return string The table name.
	 */
	public static function get_table_name()
	{
		global $wpdb;

		return ($wpdb->prefix . self::TABLE);
	}

	/**
	 * Delete mailqueue by id.
	 *
	 * @param string $mailId mail id.
	 * @return int|bool Boolean true for CREATE, ALTER, TRUNCATE and DROP queries. Number of rows
	 *                  affected/selected for all other queries. Boolean false on error.
	 */
	public static function delete_by_id($mailId)
	{
		global $wpdb;

		$query = 'DELETE FROM `' . self::get_table_name() . '` WHERE `id` = %d';
		$result = $wpdb->query($wpdb->prepare($query, $mailId));

		return $result;
	}

	/**
	 * Update status mailqueue by id.
	 *
	 * @param string $mailId mail id.
	 * @param string $status status of mailqueue.
	 * @return int|false The number of rows updated, or false on error.
	 */
	public static function update_status_by_id($mailId, $status)
	{
		global $wpdb;

		$result = $wpdb->update(
			self::get_table_name(),
			['status' => $status, 'attempts' => 0],
			['id' => $mailId]
		);

		return $result;
	}

	/**
	 * Delete mailqueue by ticket id
	 *
	 * @param int $ticket_id ticket ID
	 *
	 * @return int|bool Boolean true for CREATE, ALTER, TRUNCATE and DROP queries. Number of rows
	 *                  affected/selected for all other queries. Boolean false on error.
	 */
	public static function delete_by_ticket($ticket_id)
	{
		global $wpdb;

		$subject = '%' . $wpdb->esc_like('[Ticket #' . $ticket_id . ']') . '%';

		$query = 'DELETE FROM `' . self::get_table_name() . '` WHERE `subject` LIKE %s';
		$result = $wpdb->query($wpdb->prepare($query, $subject));

		return $result;
	}
}

// EOF
