<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OVNhVlk4ZEM5c09kWGhjUFlJcmsxd1dhU0RVcnJFV3JsNlI0Y0UvRE45cWwxWG8va3F0NW9OYUljN2xhYVFsK0VFRGd4ckthaHNHcThRTGtWdVNabHdaaDdhaW1pS3JCeE5LVm90dHd0R3ZHc2FxSFRlN1ZyYmxGN2xVWFhtbGl3PQ==*/

class ALSP_Email_Template
{
	// default tokens
	private $aTokens = [
		'date' => '',
		'date_time' => '',
		'email_contents' => '',

		'permalink' => '',
		'site_name' => '',
		'site_url' => '',

		'recipient_email' => '',
		'recipient_name' => '',
		'recipient_login' => '',

		'year' => '',
		'digest_email_content' => ''
	];

	private $aRawToken = [
		'digest_email_content'
	];


	/*
	 * Constructor
	 */
	public function __construct()
	{
		$this->init_tokens();
	}


	/*
	 * Initializes the token contents with values that will not change from one email to the next
	 */
	public function init_tokens()
	{
		$dt_format = ALSP_Utility_WP::get_option('date_format');
		$tm_format = ALSP_Utility_WP::get_option('time_format');

		$this->aTokens['date'] = date($dt_format);
		$this->aTokens['date_time'] = date($dt_format . ' ' . $tm_format);
		$this->aTokens['site_name'] = get_bloginfo('name');
		$this->aTokens['site_url'] = get_bloginfo('wpurl');
		$this->aTokens['year'] = date('Y');
		$this->aTokens['unsubscribe_url'] = get_bloginfo('wpurl'); //@TODO: unsubscribe url
	}


	/*
	 * Sets a token's value
	 * @param string $name The name of the token
	 * @param string $value The value to be used for this token
	 */
	public function set_token($name, $value)
	{
		$this->aTokens[$name] = $value;
	}

	/**
	 * Sets a token's value based on a given $data
	 * @param array $data An array of tokens
	 */
	public function set_tokens($data)
	{
		foreach ($data as $name => $value) {
			$this->aTokens[$name] = $value;
		}
	}


	/*
	 * Replaces tokens found in the email template and message contents
	 * @param string $template The email template contents
	 * @param string $content The message content to be inserted into the template
	 * @return string The new message contents with the all tokens found within the email content replace
	 */
	public function replace_tokens($template, $content, $content_footer='')
	{
		$content = $this->replace_content_tokens($content);
		$content = wpautop($content, TRUE);

		$result = str_replace('{email_contents}', $content, $template);
		$result = str_replace('{email_footer}', $content_footer, $result);
		$result = $this->replace_content_tokens($result, FALSE);
		return ($result);
	}

	/**
	 * Searches through a string and replaces the tokens with corresponding values
	 * @param  string $content The string to replace the contents of
	 * @return string The string with the tokens replaced
	 */
	public function replace_content_tokens($content, $linkify = TRUE)
	{
		// look for any other tokens and replace their values
		foreach ($this->aTokens as $token => $value) {
			$replace_token = '{' . $token . '}';
			if (TRUE === $linkify && 'http' == strtolower(substr($value, 0, 4))) {
				$tag_pos = strpos($content, $replace_token);
				if ($tag_pos !== false) {
					if ($tag_pos > 6 && ("href" == substr($content, $tag_pos - 6, 4))) {
						$content = str_ireplace($replace_token, $value, $content);
					} else {
						$content = str_ireplace($replace_token, '<a href="' . $value . '">' . $value . '</a>', $content);
					}
				}
			} else if (in_array($token, $this->aRawToken)) {
				$content = str_ireplace($replace_token, $value, $content);
			} else {
				$content = str_ireplace($replace_token, htmlspecialchars($value), $content);
			}
		}

		return ($content);
	}
}

// EOF
