<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2K3NiWGY0NlZGdGVXMVI0VHo3MEVqcXlKRDhhMTgrdGkrMWRYcFZveDlkSHZhYm80eW52dmE3V3YvVCtpb0hYbXhSUmc4VDg2U0NFOTB6T0U0VmNiTHJrVWdhakp4by9NNWZiQkpxa2F5UUh3PT0=*/

class ALSP_Frontend_Assets
{
    private static $instance;

    public $assets_url;
    public $assets_dir;
    public $templates_dir;
    public static function get_instance()
    {
        return isset(self::$instance) ? self::$instance : self::$instance = new self;
    }

    private function __construct()
    {
        $dir = dirname(dirname(__FILE__));

        $this->assets_url = plugin_dir_url($dir) . 'assets/';
        $this->assets_dir = plugin_dir_path($dir) . 'assets/';
        $this->templates_dir = plugin_dir_path($dir) . 'templates/';

        add_action('init', [$this, 'register_block_type']);
        add_action('init', [$this, 'register_shortcode']);
        add_action('wp_enqueue_scripts', [$this, 'register_frontend_assets']);
    }

    public function is_dev()
    {
        return defined('ALSP_DEV') && ALSP_DEV;
    }

    public function register_block_type()
    {
        if (!function_exists('register_block_type')) {
            return;
        }

        $this->register_frontend_assets();

        wp_register_script('alsp-block-editor', "{$this->assets_url}js/block-editor.js", ['wp-blocks', 'wp-i18n', 'wp-element'], ALSP::PLUGIN_VERSION, true);
        wp_localize_script('alsp-block-editor', 'alspBlockEditorData', []);

        register_block_type('alsp/block', [
            'style' => 'alsp',
            'script' => 'alsp',
            'editor_script' => 'alsp-block-editor',
            'render_callback' => [$this, 'render_component'],
        ]);
    }

    public function register_shortcode()
    {
        $func = function () {
            ob_start();
            echo $this->render_component();
            return ob_get_clean();
        };
        add_shortcode('almighty_support', $func);
        add_shortcode('awedesk', $func);
    }

    public function render_component()
    {
        wp_enqueue_style('alsp');
        wp_enqueue_script('alsp');
        if (apply_filters('alsp_is_pro', FALSE)) {
            wp_enqueue_script('alsp-pro');
        }


        $color_theme = ALSP_Config::get('default_theme_select', 'light', TRUE);
        $color_theme_class = $color_theme ? esc_attr($color_theme) : 'light';

        $style = '';
        $html = '<div class="alsp alsp--' . $color_theme_class . '" data-alsp="root">'
            . $this->get_template('root_loading'). '</div>';

        // Custom color for ticket statuses.
        foreach (ALSP_Model_Ticket_Status::list() as $key => $status) {
            $id = $status['status'];
            $color = ALSP_Config::get("status_{$id}_color");
            $bgcolor = ALSP_Config::get("status_{$id}_background_color");
            if ($color || $bgcolor) {
                $style .= ".alsp-status--{$id} {";
                $style .= $color ? " --c: {$color};" : '';
                $style .= $bgcolor ? " --bg: {$bgcolor};" : '';
                $style .= ' } ';
            }
        }

        return '<style type="text/css">' . $style . '</style>' . $html;
    }

    public function register_frontend_assets()
    {
        // Do not re-register if scripts are already registered.
        if (wp_script_is('alsp', 'registered')) {
            return;
        }

        wp_register_style('alsp-font', 'https://fonts.googleapis.com/css2?family=Inter:wght@300;400;500&display=swap', [], ALSP::PLUGIN_VERSION, 'all');
        wp_register_style('alsp', $this->get_assets_url('css/frontend.css'), ['alsp-font'], ALSP::PLUGIN_VERSION, 'all');

        wp_register_script(
            'alsp-fileupload',
            'https://cdnjs.cloudflare.com/ajax/libs/blueimp-file-upload/10.32.0/js/jquery.fileupload.min.js',
            ['jquery', 'jquery-ui-core'],
            ALSP::PLUGIN_VERSION,
            true
        );

        if ($this->is_dev()) {
            $order = json_decode(file_get_contents("{$this->assets_dir}js/frontend/index.json"));
            $handle = null;

            foreach ($order as $file) {
                $dependencies = array_merge(['jquery', 'underscore', 'wp-api-request', 'alsp-fileupload'], $handle ? [$handle] : []);
                $handle = 'alsp' . ('init' === $file ? '' : "-{$file}");
                wp_register_script($handle, $this->get_assets_url("js/frontend/{$file}.js"), $dependencies, ALSP::PLUGIN_VERSION, true);
            }

            if (apply_filters('alsp_is_pro', FALSE)) {
                $order = json_decode(file_get_contents("{$this->assets_dir}pro/js/frontend/index.json"));
                $handle = null;
                foreach ($order as $file) {
                    $dependencies = array_merge(['alsp'], $handle ? [$handle] : []);
                    $handle = 'alsp-pro' . ('init' === $file ? '' : "-{$file}");
                    wp_register_script($handle, $this->get_assets_url("pro/js/frontend/{$file}.js"), $dependencies, ALSP::PLUGIN_VERSION, true);
                }
            }
        } else {
            wp_register_script('alsp', $this->get_assets_url('js/frontend.js'), ['jquery', 'underscore', 'wp-api-request', 'alsp-fileupload'], ALSP::PLUGIN_VERSION, true);
            if (apply_filters('alsp_is_pro', FALSE)) {
                wp_register_script('alsp-pro', "{$this->assets_url}pro/js/frontend.js", ['alsp'], ALSP::PLUGIN_VERSION, true);
            }
        }

        $current_user_id = ALSP_User::get_id();
        $current_user = ALSP_User::get_instance();

        $templates = [];
        foreach ([
            'root', 'guest',
            'tickets', 'tickets/menu', 'tickets/filters', 'tickets/item', 'tickets/item_empty', 'tickets/item_loading',
            'ticket', 'ticket/status', 'ticket/status_options', 'ticket/agent', 'ticket/agent_options',
            'ticket/department', 'ticket/department_options', 'ticket/messages', 'ticket/message', 'ticket/message_empty',
            'ticket/reply', 'ticket/secret', 'ticket/sidebar', 'ticket/minibar',
            'ticket/staff_notes_client', 'ticket/staff_notes_client_item', 'ticket/staff_notes_client_item_empty',
            'ticket/staff_notes_ticket', 'ticket/staff_notes_ticket_item', 'ticket/staff_notes_ticket_item_empty',
            'ticket/tabs_bar_ticket', 'ticket/tabs_bar_ticket_item',
            'new_ticket', 'new_ticket/suggestions',
            'recently_viewed', 'recently_viewed/list', 'recently_viewed/item',
            'preferences', 'unsubscribe',
            'dialog',
        ] as $name) {
            $templates[$name] = $this->get_template($name);
        }

        $autoFocusSelect = ALSP_Config::get('auto_focus_select', 'never');
        if ('peepsoapp' === $autoFocusSelect) {
            $autoFocusSelect = class_exists('PeepSoNewAppHelper') && PeepSoNewAppHelper::is_app() ? 'always' : 'never';
        }

        $alspData = [
            'page_id_client' => -1,
            'page_id_agent' => -1,
            'user_id' => $current_user_id,
            'is_admin' => current_user_can('manage_options'),
            'is_manager' => $current_user->is_manager(),
            'is_agent' => $current_user->is_agent(),
            'is_client' => $current_user->is_client(),
            'can_open_on_behalf' => $current_user->can_open_on_behalf(),
            'can_register_on_behalf' => $current_user->can_register_on_behalf(),
            'home_url' => get_home_url(),
            'ajax_url' => admin_url('admin-ajax.php'),
            'loadmore_enabled' => ALSP_Config::get('loadmore_select'),
            'loadmore_repeat' => ALSP_Config::get('loadmore_repeat_select'),
            'login_guest_enabled' => ALSP_Config::get('login_guest_enabled'),
            'recaptcha_enabled' => ALSP_Config::get('recaptcha_enabled', FALSE),
            'recaptcha_key' => ALSP_Config::get('recaptcha_sitekey_label', ''),
            'recaptcha_use_global_url' => ALSP_Config::get('recaptcha_use_globally_enabled', FALSE),
            'theme_default' => ALSP_Config::get('default_theme_select', 'light', TRUE),
            'theme_accent_light' => ALSP_Config::get('light_accent_color', ''),
            'theme_accent_dark' => ALSP_Config::get('dark_accent_color', ''),
            'auto_focus_select' => $autoFocusSelect,
            'templates' => $templates,
            'lang' => [
                'new_ticket' => __('New Ticket', 'awedesk'),
                'new_ticket_submitted' => __('Done!', 'awedesk'),
                'ticket_takeover' => __('Are you sure?', 'awedesk'),
                'ticket_closed' => __('Are you sure?', 'awedesk'),
                'ticket_deleted' => __('Are you sure?', 'awedesk'),
                'empty_trash' => __('Are you sure?', 'awedesk'),
                'emptied_trash' => __('Done!', 'awedesk'),
                'session_expired_message' => __('Session expired. Please reload the page to continue.', 'awedesk'),
            ],
        ];

        $alspData['page_id_client'] = ALSP_Config::get('support_page_client_select', -1);
        $alspData['page_id_agent'] = ALSP_Config::get('support_page_agent_select', $alspData['page_id_client'], TRUE);
        $alspData['page_' . $alspData['page_id_client']] = get_permalink($alspData['page_id_client']);
        $alspData['page_' . $alspData['page_id_agent']] = get_permalink($alspData['page_id_agent']);

        $alspData = apply_filters('alsp_frontend_data', $alspData);
        ksort($alspData);

        wp_localize_script(
            $this->is_dev() ? 'alsp-base' : 'alsp',
            'alspData',
            $alspData
        );
    }

    public function get_assets_url($path)
    {
        $url = $this->assets_url . $path;

        if ($this->is_dev()) {
            $mtime = filemtime($this->assets_dir . $path);
            $url = add_query_arg('mtime', $mtime, $url);
        }

        return $url;
    }

    /**
     * Get template string.
     *
     * @param string $name Template name.
     * @param array  $data Optional data to be passed to the template.
     * @param array  $opts Optional settings to be applied to the template.
     * @return string
     */
    public function get_template($name, $data = [], $opts = [])
    {
        $file = $this->templates_dir . $name . '.php';
        if (!file_exists($file) && is_dir($this->templates_dir . $name)) {
            $file = $this->templates_dir . $name . '/_index.php';
        }

        $template = '';

        if (file_exists($file)) {
            try {
                ob_start();
                if (NULL !== $data) {
                    extract($data);
                }
                include $file;
                $template = ob_get_clean();

                if (isset($opts['trim']) && $opts['trim']) {
                    $template = preg_replace('/\s+/', ' ', $template);
                    $template = preg_replace('/> </', '><', $template);
                }

                if (isset($opts['nested']) && $opts['nested']) {
                    $template = preg_replace_callback('/\{\{(.*?)\}\}/', function ($matches) {
                        $content = str_replace('\'', '\\\'', $matches[1]);
                        return '{{= \'{{' . $content .  '}\}\' }}';
                    }, $template);
                }
            } catch (Exception $e) {
                // Do nothing.
            }
        } else {
            //new ALSP_Debug("Missing template $file");
        }

        return apply_filters('alsp_template', trim($template), $name);
    }

    /*
     * execute a template with provided data
     * @param string $section application section to load the template from
     * @param string $template name of template file within section to load
     * @param array $data array of data variables to pass to template
     * @param boolean if TRUE will return the output of the template
     */
    public static function exec_template($section, $template, $data = NULL, $return_output = FALSE)
    {
        /**
         * @since WordPress 5.8 legacy-widget-preview GET was set
         * @since WordPress 5.9 it is no longer passed, so we force set it when called from the WP preview URL
         */
        if (stristr($_SERVER['REQUEST_URI'], 'wp-json') && stristr($_SERVER['REQUEST_URI'], 'widget-types')) {
            $_GET['legacy-widget-preview'] = TRUE;
        }

        // If the Block Editor preview is loading a widget, pass args through Gecko filters to modify the args
        if (is_array($data) && isset($data['args']) && isset($_GET['legacy-widget-preview'])) {

            $data['args'] = apply_filters('alsp_fix_block_legacy_widget', $data['args'], $data['instance']);

            //echo "<span style='opacity:0.5;font-size:10px;'>".htmlspecialchars($data['args']['before_widget'])."</span>";
        }

        $templ = ALSP_Frontend_Assets::get_instance()->get_template($section, $template);

        if ($return_output) {
            ob_start();
        }

        do_action('alsp_action_before_exec_template', $section, $template, $data, $return_output);

        if (NULL !== $data)
            extract($data);
        include($templ);

        do_action('alsp_action_after_exec_template', $section, $template, $data, $return_output);
        if ($return_output) {
            $ret = ob_get_clean();
            return ($ret);
        }
    }

    public static function get_url($key)
    {
        $keys = [
            'forgot_password' => wp_lostpassword_url(),
            'register' => wp_registration_url(),
        ];

        if (isset($keys[$key])) {
            return apply_filters('alsp_get_url', $keys[$key], $key);
        }

        return FALSE;
    }
}

// EOF
