<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2L2Noc2dWWW94VDUwTVZYRVJrdDJIMTB3RFVsUUlDTU41a1Iveko1LzdwbThxY0lqcWxDNGlXYTljL0ZYYVZpTzhXRERaZFRjbFg5Tm1oOXRxaTJPd3RJOHJQZk5CWGl4ZDRJQmd3dXNxcWdFUG9pMEg0MU45Y2cvaHorM0pRLzhzPQ==*/

class ALSP_Model_Agent_Department extends ALSP_Model_Single
{
    public $id;
    public $agent;
    public $department;
    public $created;

    public function __construct($id = NULL)
    {
        parent::__construct($id);
    }

    /**
     * Get model table name
     *
     * @return string table name
     */
    public static function config_table()
    {
        global $wpdb;

        return $wpdb->prefix . 'alsp_agent_department';
    }

    /**
     * Get detail item of agent department
     * @param mixed $return ARRAY_A|OBJECT
     *
     * @return array array of item
     */
    public function get_item($return = ARRAY_A)
    {
        if (is_numeric($this->id) && $this->id > 0) {
            $item = parent::get_item($return);

            if ($item['agent'] !== NULL) {
                $agent = ALSP_User::get_instance($item['agent']);
                $item['agent'] = $agent->get_details();
            }

            if ($item['department'] !== NULL) {
                $department = new ALSP_Model_Department($item['department']);
                $item['department'] = $department->get_item();
            }

            return $item;
        }

        return [];
    }

    /**
     * Get agent department by agent
     * @param int $agent agent ID
     *
     * @return array|null Database query results.
     */
    public function get_by_agent($agent)
    {
        global $wpdb;

        $query = $wpdb->prepare("SELECT * FROM " . $this->config_table() . " WHERE agent=%d", $agent);
        $result = $wpdb->get_results($query, ARRAY_A);

        $ret = [];
        foreach ($result as $key => $item) {
            if (isset($item['agent'])) {
                $agent = ALSP_User::get_instance($item['agent']);
                $item['agent'] = $agent->get_details();
            }

            if ($item['department']) {
                $department = new ALSP_Model_Department($item['department']);
                $item['department'] = $department->get_item();
            }

            $ret[] = $item;
        }

        return $ret;
    }

    /**
     * Get agent department by department
     * @param int $department department ID
     *
     * @return array|null Database query results.
     */
    public function get_agent_by_department($department)
    {
        global $wpdb;

        $query = $wpdb->prepare("SELECT DISTINCT u.id FROM " . ALSP_Model_Agent::config_table() . " as u LEFT JOIN " . $this->config_table() . " ad ON u.id = ad.agent WHERE u.`role` = 'admin' or  u.`role` = 'manager' or (u.role = 'agent' and ad.department = %d)", $department);
        $result = $wpdb->get_results($query, ARRAY_A);

        $ret = [];
        foreach ($result as $key => $item) {
            if (isset($item['id'])) {
                $user = ALSP_User::get_instance($item['id']);
                $assignee['agent'] = $user->get_details();
                if ($assignee['agent'] !== NULL) {
                    $ret[] = $assignee;
                }
            }
        }

        return $ret;
    }

    /**
     * Check if an agent was registered to department
     * @param int $agent Agent ID
     * @param int $department Department ID
     *
     * @return bool True if an agent was registered. Otherwise, it will return false.
     */
    public function is_agent_registered($agent, $department)
    {
        global $wpdb;

        $query = $wpdb->prepare("SELECT count(id) as total FROM " . $this->config_table() . " WHERE department=%d and agent=%d", $department, $agent);
        $result = $wpdb->get_var($query);

        return $result > 0;
    }

    /**
     * Create agent department
     * @param int $agent agent ID
     * @param int $department department ID
     *
     * @return int|bool Last insert ID. Boolean false on error.
     */
    public function create_item($agent, $department)
    {
        // create the ALSP department
        global $wpdb;

        $now = current_time('mysql', TRUE);
        $data = [
            'agent' => $agent,
            'department' => $department,
            'created' => $now,
        ];
        $ret = $wpdb->insert($this->config_table(), $data);

        if (!$ret) {
            return FALSE;
        }

        return $wpdb->insert_id;
    }

    /**
     * Delete agent department by ID
     *
     * @return int|bool Number of rows affected/selected for all other queries. Boolean false on error.
     */
    public function delete($request = NULL)
    {
        return parent::delete();
    }

    /**
     * Delete agent department by agent
     * @param int $agent Agent ID
     *
     * @return int|bool Number of rows affected/selected for all other queries. Boolean false on error.
     */
    public function delete_by_agent($agent)
    {
        global $wpdb;
        $sql = $wpdb->prepare('DELETE FROM ' . $this->config_table() . ' WHERE agent=%d', $agent);
        $result = $wpdb->query($sql);

        return $result;
    }

    /**
     * Delete agent department by department
     * @param int $department Department ID
     *
     * @return int|bool Number of rows affected/selected for all other queries. Boolean false on error.
     */
    public function delete_by_department($department)
    {
        global $wpdb;
        $sql = $wpdb->prepare('DELETE FROM ' . $this->config_table() . ' WHERE department=%d', $department);
        $result = $wpdb->query($sql);

        return $result;
    }
}

// EOF
