<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2K1FKQXFxVGVWQWhGb1JKWnZxWG9pSU5kWkdYeU11N0ZDRmR5UU9MZUJ6bWovZ3lzQ2Z6OWF6QmxDQ2M4bER2eUdLMDNOeXpCL2pTZWFjQ3hSSzdRdmJadnpwNnFRSXRkQ0lDcm92K3Yrd3A5cDlPbTRYbkxrQ0RoUlZIUVVYRlYwPQ==*/

class ALSP_Model_Department extends ALSP_Model_Single
{
    public $id;
    public $name;
    public $description;
    public $author;
    public $default_assignee;
    public $agent_only;
    public $order;
    public $created;
    public $updated;
    public $published;

    public function __construct($id = NULL)
    {
        $this->editable_properties = ['name', 'description', 'default_assignee', 'agent_only', 'published', 'order'];

        parent::__construct($id);
    }

    public static function config_table()
    {
        global $wpdb;

        return $wpdb->prefix . 'alsp_departments';
    }

    public function get_item($return = ARRAY_A)
    {
        if (is_numeric($this->id) && $this->id > 0) {
            $item = parent::get_item($return);
            if ($item['author'] > 0) {
                $author = ALSP_User::get_instance($item['author']);
                $item['author'] = $author->get_details();
            } else {
                $item['author'] = NULL;
                $item['default_assignee'] = NULL;
            }

            if ($item['default_assignee'] > 0) {
                $default_assignee = ALSP_User::get_instance($item['default_assignee']);
                $item['default_assignee'] = $default_assignee->get_details();
            } else {
                $item['default_assignee'] = NULL;
            }

            // for admin preferences
            $open = get_user_meta(get_current_user_id(), 'alsp_admin_departments_open_' . $item['id'], TRUE);

            // force open
            if (!$open && !strlen($open)) {
                $open = 1;
            }

            $item['open'] = (strlen($open) && 1 == $open) ? 1 : 0;

            $agent_department = new ALSP_Model_Agent_Department(NULL);
            $item['list_assignee'] = $agent_department->get_agent_by_department($item['id']);

            return $item;
        }

        return [];
    }

    /**
     * Create agent department
     * @param int $name name
     * @param int $description description
     * @param int $default_assignee default assignee agent ID
     * @param bool $published status publish
     * @param bool $agent_only status publish
     * 
     * @return int|bool Last insert ID. Boolean false on error.
     */
    public function create_item($name, $description, $default_assignee, $published, $agent_only, $order = 0)
    {
        // create the ALSP department
        global $wpdb;

        $now = current_time('mysql', TRUE);
        $data = [
            'name' => $name,
            'description' => $description,
            'default_assignee' => $default_assignee,
            'published' => $published,
            'agent_only' => $agent_only,
            'author' => get_current_user_id(),
            'created' => $now,
        ];
        $ret = $wpdb->insert($this->config_table(), $data);

        $lastid = FALSE;
        if ($ret) {
            $lastid = $wpdb->insert_id;
        }

        // default order same with ID
        if(!$order) {
            $dept = new ALSP_Model_Department($lastid);
            $dept->set('order', $lastid);
        }

        return $lastid;
    }

    public function update_item($id, $name, $description)
    {
        $ret = FALSE;
        // update the ALSP department
        global $wpdb;
        $sql = "SELECT * FROM `" . $this->config_table() . "` " .
            " WHERE `id`=%d ";
        $res = $wpdb->get_results($wpdb->prepare($sql, $id), ARRAY_A);
        if (count($res) > 0) {
            // if alsp_departments record exist, update it
            $now = current_time('mysql', TRUE);
            $data = [
                'name' => $name,
                'description' => $description,
                'updated' => $now,
            ];
            $ret = $wpdb->update($this->config_table(), $data, ['id' => $id]);
        }

        return $ret;
    }

    /**
     * Remove default assignee based on agent_id.
     * This happen when agent was demote/removed in list agent.
     * @param int $agent_id agent ID
     * 
     * @return int|false The number of rows updated, or false on error.
     */
    public function remove_default_assignee($agent_id)
    {
        global $wpdb;

        // if alsp_departments record doesn't exist, create it
        $now = current_time('mysql', TRUE);
        $data = [
            'default_assignee' => 0,
            'updated' => $now,
        ];
        $ret = $wpdb->update($this->config_table(), $data, ['default_assignee' => $agent_id]);

        return $ret;
    }

    public function delete($request = NULL)
    {
        parent::delete();

        $agent_department = new ALSP_Model_Agent_Department(NULL);
        $agent_department->delete_by_agent($this->id);
    }
}

// EOF