<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2L0g5d0VNcXEyNHVQNy82anBRSTJtNXh5c25rZjlLVkdZRGtWUUJsL21RbnlnWXpnWkdLcGhVVGNxSDFWaDhVYnFJRGNjV3BiOWg1VnBqdHF2UW4rWDU2QlJscy9meG1KdnhNMFBISUQrbWR3PT0=*/

class ALSP_Model_Ticket extends ALSP_Model_Single
{
	public $author;
	public $agent;
	public $client;
	public $department;
	public $subject;
	public $secret;
	public $status;
	public $auth_code;
	public $created;
	public $updated;
	public $closed;
	public $last_reply_by;
	public $last_reply_time_agent;
	public $last_reply_time_client;
	public $last_reply_time;
	public $needs_attention;
	public $prevent_autoclose;
	public $prevent_clientclose;
	public $prevent_reopen;

	public $agent_reply_count;
	public $client_reply_count;

	public $watched = false;

	public function __construct($id = NULL)
	{
		$this->editable_properties = ['department', 'status', 'needs_attention', 'prevent_autoclose', 'prevent_clientclose', 'prevent_reopen'];
		parent::__construct($id);
	}

	/**
	 * Get table name
	 *
	 * @return string Table name.
	 */
	public static function config_table()
	{
		global $wpdb;

		return $wpdb->prefix . 'alsp_tickets';
	}

	public function get_item($return = ARRAY_A)
	{
		if (is_numeric($this->id) && $this->id > 0) {
			$item = parent::get_item($return);

			$item['last_reply_by'] = $item['last_reply_by'] ? ALSP_User::get_instance($item['last_reply_by'])->get_details() : ALSP_User::get_instance($item['client'])->get_details();

			$item['author'] = $item['author'] ? ALSP_User::get_instance($item['author'])->get_details() : $item['author'];
			$item['client'] = $item['client'] ? ALSP_User::get_instance($item['client'])->get_details() : $item['client'];
			$item['agent'] = $item['agent'] ? ALSP_User::get_instance($item['agent'])->get_details() : $item['agent'];

			$item['created_absolute'] = ALSP_Utility_String::date_absolute($item['created']);
			$item['created_relative'] = ALSP_Utility_String::date_relative($item['created']);

			$item['updated_absolute'] = ALSP_Utility_String::date_absolute($item['updated']);
			$item['updated_relative'] = ALSP_Utility_String::date_relative($item['updated']);

			$item['closed_relative'] = "-";
			if ($item['closed'] !== NULL) {
				$item['closed_absolute'] = ALSP_Utility_String::date_absolute($item['closed']);
				$item['closed_relative'] = ALSP_Utility_String::date_relative($item['closed']);
			}

			$item['last_reply_time_agent_relative'] = "-";
			if ($item['last_reply_time_agent'] !== NULL) {
				$item['last_reply_time_agent_absolute'] = ALSP_Utility_String::date_absolute($item['last_reply_time_agent']);
				$item['last_reply_time_agent_relative'] = ALSP_Utility_String::date_relative($item['last_reply_time_agent']);
			}

			$item['last_reply_time_client_absolute'] = ALSP_Utility_String::date_absolute($item['last_reply_time_client']);
			$item['last_reply_time_client_relative'] = ALSP_Utility_String::date_relative($item['last_reply_time_client']);

			$item['last_reply_time_absolute'] = ALSP_Utility_String::date_absolute($item['last_reply_time']);
			$item['last_reply_time_relative'] = ALSP_Utility_String::date_relative($item['last_reply_time']);
			$item['last_reply_time_short'] = ALSP_Utility_String::date_relative_short($item['last_reply_time']);

			if ($item['department']) {
				$department = new ALSP_Model_Department($item['department']);
				$item['department'] = $department->get_item();
			} else {
				$item['department'] = null;
			}

			$user = ALSP_User::get_instance();
			if (current_user_can('manage_options') || $user->is_manager() || $user->is_agent()) {
				$lock_data = ALSP_Model_Ticket_Lock::read($this->id);
				$item['lock'] = $lock_data['lock'];
				$item['lock_agent'] = $lock_data['lock_agent'];

				if (apply_filters('alsp_is_pro', FALSE)) {
					$watcher = new ALSP_Model_Ticket_Watcher($this->id, get_current_user_id());
					$watched = $watcher->read();
					$item['watched'] = intval(is_array($watched));
				}
			} else {
				$item['lock'] = null;
				$item['lock_agent'] = null;
			}

			$item['secret'] = ALSP_Utility_Crypto::decrypt($item['secret']);

			return $item;
		}
	}

	/**
	 * Create ticket.
	 * @param int $client client ID
	 * @param int $author author ID
	 * @param null|int $agent agent ID
	 * @param null|int $department department ID
	 * @param string $subject subject
	 * @param string $status Status
	 * @param int $agent_reply_count agent reply count
	 * @param int $client_reply_count client reply count
	 *
	 * @return int|bool Last insert ID. Boolean false on error.
	 */
	public function create_item($client, $author, $agent, $department, $subject, $status, $secret, $agent_reply_count, $client_reply_count)
	{
		// create the ALSP department
		global $wpdb;

		// Set ticket to needing attention,
		// if the ticket is not in any department. or assigned to nobody.
		$needs_attention = 0;
		if (!$agent || !$department) {
			$needs_attention = 1;
		}

		$now = current_time('mysql', TRUE);
		$auth_code = ALSP_Utility_String::get_random_string();

		$secret = (is_string($secret) && strlen($secret)) ? $secret : "";
		$secret = ALSP_Utility_Crypto::encrypt($secret);

		$data = [
			'subject' => $subject,
			'secret' => $secret,
			'client' => $client,
			'author' => $author,
			'agent' => $agent,
			'department' => $department,
			'status' => $status,
			'created' => $now,
			'updated' => $now,
			'last_reply_time_client' => $now,
			'last_reply_time' => $now,
			'agent_reply_count' => $agent_reply_count,
			'client_reply_count' => $client_reply_count,
			'auth_code' => $auth_code,
			'needs_attention' => $needs_attention,
		];
		$res = $wpdb->insert($this->config_table(), $data);

		$lastid = FALSE;
		if ($res) {
			$lastid = $wpdb->insert_id;
		}

		if ($lastid) {
			do_action('alsp_ticket_create_succeded', $lastid);
		}

		return $lastid;
	}

	/**
	 * Assign ticket.
	 *
	 * @param $author - ticket author.
	 * @param $ticket - ticket ID.
	 * @param $agent - Agent ID.
	 * @param $status - ticket Status.
	 * @return int|false The number of rows updated, or false on error.
	 */
	public function assign_ticket($author, $ticket, $agent, $status = '')
	{
		global $wpdb;

		$now = current_time('mysql', TRUE);
		$data = [
			'agent' => $agent,
			'updated' => $now,
		];

		if ($status !== '') {
			$data['status'] = $status;
		}

		$result = $wpdb->update($this->config_table(), $data, ['id' => $ticket]);

		return $result;
	}

	/**
	 * Closed ticket.
	 *
	 * @param $ticket - ticket ID.
	 * @return int|false The number of rows updated, or false on error.
	 */
	public function close_ticket($ticket)
	{
		$now = current_time('mysql', TRUE);
		$data = [
			'status' => 'closed',
			'updated' => $now,
			'closed' => $now,
		];
		$result = $this->wpdb->update($this->config_table(), $data, ['id' => $ticket]);

		return $result;
	}

	/**
	 * Reopen ticket with specific status
	 *
	 * @param $ticket - ticket ID.
	 * @param $status - status.
	 * @return int|false The number of rows updated, or false on error.
	 */
	public function reopen_ticket($ticket, $status)
	{
		$now = current_time('mysql', TRUE);
		$data = [
			'status' => $status,
			'updated' => $now,
		];
		$result = $this->wpdb->update($this->config_table(), $data, ['id' => $ticket]);

		return $result;
	}

	/**
	 * Update private notes ticket
	 *
	 * @param $ticket - ticket ID.
	 * @param $secret - secret.
	 * @return int|false The number of rows updated, or false on error.
	 */
	public function update_private_notes($ticket, $secret)
	{
		$now = current_time('mysql', TRUE);
		$data = [
			'secret' => $secret,
			'updated' => $now,
		];
		$result = $this->wpdb->update($this->config_table(), $data, ['id' => $ticket]);

		return $result;
	}

	/**
	 * Anonymize ticket data
	 *
	 * @param $ticket - ticket ID.
	 * @return int|false The number of rows updated, or false on error.
	 */
	public function anonymize_ticket($ticket)
	{
		$now = current_time('mysql', TRUE);
		$data = [
			'client' => 0,
			'author' => 0,
			'updated' => $now,
		];
		$result = $this->wpdb->update($this->config_table(), $data, ['id' => $ticket]);

		return $result;
	}

	/**
	 * Delete the ticket
	 */
	public function delete($request = NULL)
	{
		parent::delete();

		$ticket_message = new ALSP_Model_Ticket_Message(NULL);
		$ticket_message->delete_by_ticket($this->id);

		if (class_exists('ALSP_Model_Ticket_Attachment')) {
			$ticket_attachment = new ALSP_Model_Ticket_Attachment(NULL);
			$list_attachment = $ticket_attachment->get_by_ticket($this->id);
			foreach ($list_attachment as $key => $attachment) {
				$file = WP_CONTENT_DIR . $attachment['file_path'];
				if (file_exists($file)) {
					unlink($file);
				}
			}

			// Delete the attachment record
			$ticket_attachment->delete_by_ticket($this->id);
		}

		if (class_exists('ALSP_Model_Staff_Notes')) {
			$staff_notes = new ALSP_Model_Staff_Notes(NULL);
			$staff_notes->delete_by_ticket($this->id);
		}

		if (class_exists('ALSP_Model_Ticket_Lock')) {
			ALSP_Model_Ticket_Lock::delete_by_ticket($this->id);
		}

		// Delete mailqueue
		if (class_exists('ALSP_Email_Queue')) {
			ALSP_Email_Queue::delete_by_ticket($this->id);
		}
	}


	public static function get_followers($ticket_id, $exclude = []) {

		if(!is_array($exclude)) {
			$exclude = [$exclude];
		}

		$exclude = array_map('intval', $exclude);

		$ticket_data = (new ALSP_Model_Ticket($ticket_id))->get_item();

		$recipients = [];
		$assignee_id = (int) $ticket_data['agent']['id'];
		if(!in_array($assignee_id, $exclude)) {
			$recipients[] = $assignee_id;
		}

		global $wpdb;
		$sql = "SELECT * FROM {$wpdb->prefix}alsp_ticket_watchers WHERE ticket = $ticket_id";

		$watchers = $wpdb->get_results( $sql, ARRAY_A );
		if ( count( $watchers ) ) {
			foreach ( $watchers as $watcher ) {
				$watcher_id = (int) $watcher['user'];
				if(!in_array($watcher_id, $exclude)) {
					$recipients[] = $watcher_id;
				}
			}
		}

		$recipients = array_map('intval', $recipients);
		$recipients = array_unique($recipients);


		return $recipients;
	}
}

// EOF
