<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2L2tob2N3cjVONCtVOCtSU2JzYWVtN1hpTGFwVlRib2wreUVqd3MwUUJaTXA4bXEyN09JTmIwWkZZS2phaE9qd2l4S0RGOEpVaW9CMitkQTBVYVBiOWQ2dkRYWEVTaDlqWlhxL3VEK2lkSUlHUlMzU2pjdmVMeTVRaVVZQStWSXU4PQ==*/

class ALSP_Model_Ticket_Lock
{
	public static function read($id)
	{
		$prefs = ALSP_Ticket_Preferences::get_instance($id);

		$lock = NULL;
		$lock_agent = NULL;

		$timestamp = (int) $prefs->get(sanitize_key('lock'));
		$timestamp_expiration = 4 * 60; // Timestamp is expired in 4 minutes.

		if ($timestamp) {
			if (time() - $timestamp < $timestamp_expiration) {
				$lock = 1;

				$agent = $prefs->get(sanitize_key('lock_agent'));
				if (!empty($agent)) {
					$agent_model = ALSP_User::get_instance($agent);
					if ($agent_model->wp_user->ID) {
						$lock_agent = $agent_model->get_details();
					}
				}
			} else {
				// Delete lock state if expired.
				$prefs->set(sanitize_key('lock'), NULL);
				$prefs->set(sanitize_key('lock_agent'), NULL);
			}
		}

		return [
			'lock' => $lock,
			'lock_agent' => $lock_agent,
		];
	}

	public static function set($id, $agent)
	{
		$prefs = ALSP_Ticket_Preferences::get_instance($id);
		$prefs->set(sanitize_key('lock'), time());
		$prefs->set(sanitize_key('lock_agent'), $agent);

		$agent_model = ALSP_User::get_instance($agent);
		if ($agent_model->wp_user->ID) {
			$lock_agent = $agent_model->get_details();
		} else {
			$lock_agent = NULL;
		}

		return [
			'lock' => 1,
			'lock_agent' => $lock_agent,
		];
	}

	public static function unset($id)
	{
		$prefs = ALSP_Ticket_Preferences::get_instance($id);
		$prefs->set(sanitize_key('lock'), NULL);
		$prefs->set(sanitize_key('lock_agent'), NULL);

		return [
			'lock' => NULL,
			'lock_agent' => NULL,
		];
	}

	/**
	 * Delete ticket preference by ticket_id
	 * @param int $ticket_id Ticket ID
	 * 
	 * @return int|bool Number of rows affected/selected for all other queries. Boolean false on error.
	 */
	public static function delete_by_ticket($ticket_id)
	{
		global $wpdb;

		$sql = $wpdb->prepare('DELETE FROM ' . $wpdb->prefix . 'alsp_ticket_preferences' . ' WHERE ticket=%d', $ticket_id);
		$result = $wpdb->query($sql);

		return $result;
	}
}

// EOF
