<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OHQ4ZWx5cllPNGdMNUcrZjFmYmR1c25kTTdwN2d3Y3B6cG9EbUtqM2d2RXZ3VzVMYmdoajVXOTMxYS9mUlJmNTFvY0pPMWZwQit6T3RuR2E5NGZtbTA1QXhUUGZOZDZtaG83bTZQZTlDQlc1MHh5YzFoL1ZZa3JPdUdHYkRJRVVZPQ==*/

class ALSP_Model_Ticket_Message extends ALSP_Model_Single
{
    public $author;
    public $ticket;
    public $type;
    public $message;
    public $created;
    public $is_deleted;

    public function __construct($id = NULL)
    {
        $this->editable_properties = ['messages'];

        parent::__construct($id);
    }

    /**
     * Get model table name
     *
     * @return string table name
     */
    public static function config_table()
    {
        global $wpdb;

        return $wpdb->prefix . 'alsp_ticket_messages';
    }

    /**
     * Get detail item of ticket message
     *
     * @param mixed $return ARRAY_A|OBJECT
     * @return array array of item
     */
    public function get_item($return = ARRAY_A)
    {
        if (is_numeric($this->id) && $this->id > 0) {
            $item = parent::get_item($return);
            //new ALSP_Debug(print_r($item,true));
            if ($item['author']) {
                $author = ALSP_User::get_instance($item['author']);
                $author = $author->get_details();
            } else {
                $author = [
                    'id' => $item['author'],
                    'display_name' => 'System',
                    'user_avatar' => get_avatar_url($item['author']),
                ];
            }
            $item['author'] = $author;
            $item['created_absolute'] = ALSP_Utility_String::date_absolute($item['created']);
            $item['created_relative'] = ALSP_Utility_String::date_relative($item['created']);
            $item['created_short'] = ALSP_Utility_String::date_relative_short($item['created']);
            $item['message'] = ALSP_Utility_String::sanitize_message($item['message']);

            $req = new WP_REST_Request;
            $req['message_id'] = $this->id;

            if (apply_filters('alsp_is_pro', FALSE)) {
                $attachments_model = new ALSP_Model_Ticket_Attachments($req);

                $attachments = $attachments_model->read();
                $item['attachments'] = $attachments;

                if (1 == ALSP_Config::get('attachment_enabled', 1)) {
                    $failed_attachments = ALSP_Config::get('failed_attachments_' . $this->id, FALSE, TRUE);
                    if ($failed_attachments) {
                        $item['failed_attachments'] = maybe_unserialize($failed_attachments);
                    }
                }
            }

            return $item;
        }

        return [];
    }

    /**
     * Create ticket message.
     * @param int $ticket ticket ID
     * @param int $author ticket author
     * @param string $type type of ticket message
     * @param string $message content of ticket message
     *
     * @return int|bool Last insert ID. Boolean false on error.
     */
    public function create_item($ticket, $author_id, $type, $message, $do_action = FALSE)
    {
        global $wpdb;


        $now = current_time('mysql', TRUE);
        $data = [
            'ticket' => $ticket,
            'author' => $author_id,
            'type' => $type,
            'message' => ALSP_Utility_String::sanitize_message($message),
            'created' => $now,
        ];

        $res = $wpdb->insert($this->config_table(), $data);

        $lastid = FALSE;
        if ($res) {
            $lastid = $wpdb->insert_id;
        }

        if ($lastid && $do_action) {
            do_action('alsp_message_create_succeded', $type);
        }

        return $lastid;
    }

    /**
     * Get first ticket message.
     * @param $ticket ticket ID
     *
     * @return int|false The number of rows updated, or false on error.
     */
    public function get_first_message($ticket)
    {
        global $wpdb;

        $query = $this->wpdb->prepare("SELECT * FROM " . $this->config_table() . " WHERE ticket=%d order by id asc limit 1", $ticket);
        $result = $this->wpdb->get_row($query, ARRAY_A);

        $ret = "";
        if (is_array($result)) {
            $ret = $result['message'];
        }

        return $ret;
    }

    /**
     * Anonymize ticket message data
     * @param $message_id message ID.
     *
     * @return int|false The number of rows updated, or false on error.
     */
    public function anonymize_ticket_message($message_id)
    {
        $data = [
            'author' => 0,
        ];
        $result = $this->wpdb->update($this->config_table(), $data, ['id' => $message_id]);

        return $result;
    }

    /**
     * Delete ticket message by ticket_id
     * @param int $ticket_id Ticket ID
     *
     * @return int|bool Number of rows affected/selected for all other queries. Boolean false on error.
     */
    public function delete_by_ticket($ticket_id)
    {
        global $wpdb;

        $sql = $wpdb->prepare('DELETE FROM ' . $this->config_table() . ' WHERE ticket=%d', $ticket_id);
        $result = $wpdb->query($sql);

        return $result;
    }
}

// EOF
