<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OFYzdnBPQjJBdnA3K1RlWThkS3kzSWdYVWhrY0R0MUZQdkJxVXJkRUM4VjNmWWZaOW9xWGNTTndNSncvaCt5bUV0bkpCQmhKN2dCaWYyeUxNeng5bVpSRk1NNTQvVXNINEI1U3N6S29vazVpWEtPVEhCZCs0cWNJK3R1UktQbzV3PQ==*/

/**
 * ALSP_Model_Ticket_Status represents model for tickets filter.
 */
class ALSP_Model_Ticket_Status
{

	public static function name($collection)
	{
		$defaults = [
			'open' => self::default_name('open'),
            'mine' => self::default_name('mine'),
            'watched' => self::default_name('watched'),
            'recent' => __('History', 'awedesk'),
			'new' => self::default_name('new'),
			'pending' => self::default_name('pending'),
			'answered' => self::default_name('answered'),
			'closed' => self::default_name('closed'),
			'deleted' => self::default_name('deleted'),
		];

		$defaults = apply_filters('alsp_ticket_filter_name', $defaults);

		return isset($defaults[$collection]) ? $defaults[$collection] : "Unknown";
	}

	public static function default_name($collection)
	{
		$defaults = [
			'open' => __('All Open', 'awedesk'),
			'mine' => __('Mine', 'awedesk'),
            'watched' => __('Saved', 'awedesk'),
            'recent' => __('History', 'awedesk'),
			'new' => __('New', 'awedesk'),
			'pending' => __('Pending', 'awedesk'),
			'answered' => __('Answered', 'awedesk'),
			'closed' => __('Closed', 'awedesk'),
			'deleted' => __('Trashed', 'awedesk'),
		];

		$defaults = apply_filters('alsp_ticket_filter_default_name', $defaults);

		return isset($defaults[$collection]) ? $defaults[$collection] : $collection;
	}

	public static function default_desc($collection)
	{
		$defaults = [
			'open' => __('Tickets that are not Closed or Trashed', 'awedesk'),
			'mine' => __('All tickets assigned to me', 'awedesk'),
            'watched' => __('Tickets bookmarked for later', 'awedesk'),
            'recent' => __('Recenlty viewed Tickets', 'awedesk'),
			'new' => __('Pending tickets with no agent replies', 'awedesk'),
			'pending' => __('Tickets with an action required by staff', 'awedesk'),
			'answered' => __('Answered but not solved, waiting for client reply', 'awedesk'),
			'closed' => __('All solved tickets', 'awedesk'),
			'deleted' => __('All tickets in Trash', 'awedesk'),
		];

		$defaults = apply_filters('alsp_ticket_filter_default_desc', $defaults);

		return isset($defaults[$collection]) ? $defaults[$collection] : $collection;
	}

	public static function icon($collection)
	{
		$defaults = [
			'open' => ALSP_Config::get('tickets_open_icon', self::default_icon('open'), 1),
			'mine' => ALSP_Config::get('tickets_mine_icon', self::default_icon('mine'), 1),
            'watched' => ALSP_Config::get('tickets_watched_icon', self::default_icon('watched'), 1),
            'recent' => ALSP_Config::get('tickets_recent_icon', self::default_icon('recent'), 1),
			'new' => ALSP_Config::get('tickets_new_icon', self::default_icon('new'), 1),
			'answered' => ALSP_Config::get('tickets_answered_icon', self::default_icon('answered'), 1),
			'pending' => ALSP_Config::get('tickets_pending_icon', self::default_icon('pending'), 1),
			'closed' => ALSP_Config::get('tickets_closed_icon', self::default_icon('closed'), 1),
			'deleted' => ALSP_Config::get('tickets_deleted_icon', self::default_icon('deleted'), 1),
		];

		$defaults = apply_filters('alsp_ticket_filter_icon', $defaults);

		return isset($defaults[$collection]) ? $defaults[$collection] : 'alsp-i-list';
	}

	public static function color($collection)
	{
		return ALSP_Config::get("tickets_{$collection}_color", '', 1);
	}

	public static function default_icon($collection)
	{
		$defaults = [
			'open' => 'alsp-i-list-timeline',
			'mine' => 'alsp-i-user-check',
            'watched' => 'alsp-i-bookmark',
            'recent' => 'alsp-i-clock',
			'new' => 'alsp-i-inbox-in',
			'pending' => 'alsp-i-hourglass-start',
			'answered' => 'alsp-i-comment-alt-check',
			'closed' => 'alsp-i-memo-circle-check',
			'deleted' => 'alsp-i-trash',
		];

		$defaults = apply_filters('alsp_ticket_filter_default_icon', $defaults);

		return isset($defaults[$collection]) ? $defaults[$collection] : 'alsp-i-list';
	}

	/**
	 * Get list ticket statuses
	 *
	 * @return array list of ticket statuses
	 */
	public static function list()
	{
		$statuses = [
            [
                'status' => 'new',
                'label' => self::name('new'),
            ],
            [
                'status' => 'agent',
                'label' => self::name('pending'),
            ],
			[
				'status' => 'agent_escalated',
				'label' => sprintf(__('%s - Escalated', 'awedesk'), self::name('pending')),
			],
			[
				'status' => 'agent_hold',
				'label' => sprintf(__('%s - On Hold', 'awedesk'), self::name('pending')),
			],
			[
				'status' => 'client',
				'label' => self::name('answered'),
			],

			[
				'status' => 'closed',
				'label' => self::name('closed'),
			],
        ];

		# Agent can't delete ticket
		$current_user = ALSP_User::get_instance();
		if ($current_user->is_admin() || $current_user->is_manager()) {
			array_push($statuses, [
                'status' => 'deleted',
                'label' => self::name('deleted'),
            ]);
		}

        $statuses = apply_filters('alsp_ticket_status_list', $statuses);

		return $statuses;
	}

	public function read()
	{
		$result = [];

		$current_user = ALSP_User::get_instance();
		if ($current_user->is_admin() || $current_user->is_manager()) {
			$statuses = ['open', 'mine', 'watched', 'recent','new', 'pending', 'answered', 'closed', 'deleted'];
		} else if ($current_user->is_agent()) {
			$statuses = ['open', 'mine', 'watched', 'new', 'pending', 'answered', 'closed'];
		} else {
			$statuses = ['mine'];
		}

        if (!apply_filters('alsp_is_pro', FALSE) && array_key_exists('watched', $statuses)) {
            unset($statuses['watched']);
        }

		$statuses = apply_filters('alsp_ticket_filter_list', $statuses);
		foreach ($statuses as $key => $value) {
			$result[$value] = [
				'id' => $value,
				'label' => self::name($value),
				'icon' => self::icon($value),
				'color' => self::color($value),
			];
		}
		return $result;
	}
}

// EOF
