<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OC9oZEF4ak03ZUR6T1dpQWVzelZlMWo3aHpydE5DK2ZRY0I2cFpWM2VnbUc0bmdOdVNFK091ZkFyUDR5SnZ4TVN1dUF0c1k1MWFqYnoxNHJiM1NJeXRhZVczVVo0eGk1V2I2K0piQ00yT0o3RnR5T2NLb3RMckprakQrNzN3cFo4PQ==*/

class ALSP_Model_Tickets extends ALSP_Model_List
{

    public function __construct(WP_REST_Request $request)
    {
        parent::__construct($request);
    }

    protected function query_start()
    {
        $this->query = "SELECT DISTINCT `tickets`.`id` FROM {$this->table} tickets";
    }

    protected function query_where($where = [])
    {
        global $wpdb;

        if ($this->request['id']) {
            $where['id'] = $wpdb->prepare('`tickets`.`id`=%d', $this->request['id']);
        }


        if ($this->request['client']) {
            $where['client'] = $wpdb->prepare('`tickets`.`client`=%d', $this->request['client']);
        }

        if ($this->request['agent']) {
            $where['agent'] = $wpdb->prepare('`tickets`.`agent`=%d', $this->request['agent']);
        }

        if ($this->request['department']) {
            $where['department'] = $wpdb->prepare('`tickets`.`department`=%d', $this->request['department']);
        }

        if ($this->request['last_reply_by']) {
            $where['repliers'] = $wpdb->prepare('`repliers`.`id`=%d', $this->request['last_reply_by']);
        }

        if (count($where)) {
            $where = " WHERE " . implode(' AND ', $where);
        }

        $this->query .= is_string($where) ? $where : FALSE;
        $this->query_count .= is_string($where) ? $where : FALSE;
    }

    protected function query_order($order = [])
    {
        if(isset($this->request['list']) && $this->request['list']=='recent') {
            $this->query .= " ORDER BY `recent_tickets`.`updated` DESC";
        } elseif (count($order) > 0) {
            $order_by = [];
            foreach ($order as $key => $value) {

                $check_key = str_replace( ['.','`'],'', $key);
                if($check_key=='ticketsstatus') {

                    // Custom ordering for the status field, as A-Z won't work
                    $status_order = [
                        0 => 'new',
                        10 => 'agent_escalated',
                        20 => 'agent',
                        30 => 'agent_hold',
                        40 => 'client',
                        100 => 'closed',
                        200 => 'deleted',
                    ];

                    // Wrap values in quotes for SQL
                    foreach($status_order as $k=>$v) {
                        $status_order[$k]="'$v'";
                    }

                    // The order of keys must be reversed if we sort desc
                    if($value=='desc') {
                        $status_order = array_reverse($status_order, TRUE);
                    }

                    // Implode custom sort field and apply the FIELD() method
                    $status_order_string = implode(',', $status_order);
                    $order_by[] = " FIELD(status, $status_order_string)";
                    $order_by[] = " `tickets`.`last_reply_time` ASC";
                } else {
                    $order_by[] = " $key $value";
                }
            }
            $this->query .= " ORDER BY " . implode(',', $order_by);
        } else {
            $this->query .= " ORDER BY `tickets`.`last_reply_time` DESC";
        }

        //new ALSP_Debug($this->query);
    }

    protected function query_count()
    {
        $this->query_count = "SELECT count(DISTINCT `tickets`.`id`) FROM {$this->table} tickets";
    }

    private function build_query()
    {
        $join = [];
        $join['users'] = "LEFT JOIN {$this->wpdb->users} users ON tickets.author = users.id";
        $join['departments'] = "LEFT JOIN {$this->wpdb->prefix}alsp_departments departments ON tickets.department = departments.id";
        $join['agents'] = "LEFT JOIN {$this->wpdb->users} agents ON tickets.agent = agents.id";
        $join['repliers'] = "LEFT JOIN {$this->wpdb->users} repliers ON IF(tickets.last_reply_by > 0, tickets.last_reply_by, tickets.client) = repliers.id";

        /**
         * agent can see own tickets, assigned tickets, tickets from department they have access to,
         * but for example can't delete tickets or see them
         */

        $where = [];

        // Needs attention flag
        if (intval($this->request['needs_attention']) == 1) {
            $where['needs_attention'] = '`tickets`.`needs_attention`= 1';
        }

        // Ticket Listing
        if ($this->request['list']) {
            $current_user = ALSP_User::get_instance();
            if ($current_user->is_agent()) {
                $agent_department = new ALSP_Model_Agent_Department(NULL);
                $list_department = $agent_department->get_by_agent($current_user->get('user_id'));
                $department = [];
                foreach ($list_department as $key => $dept) {
                    $department[] = $dept['department']['id'];
                }

                $where['agent_department'] = "(`tickets`.`agent`=" . $current_user->get('user_id') . " OR `tickets`.`department`= 0)";
                if (count($department) > 0) {
                    $where['agent_department'] = "(`tickets`.`agent`=" . $current_user->get('user_id') . "  OR `tickets`.`author`=" . $current_user->get('user_id') . " OR (`tickets`.`department`= 0 OR `tickets`.`department`IN (" . implode(",", $department) . ")))";
                }
            }

            //
            if($current_user->is_client() || !$current_user->is_staff()) {
                $where['client'] = '`tickets`.`client`=' . $current_user->get('user_id');
            }


            if ($this->request['list'] == 'mine') {
                if ($current_user->is_client()) {
                    $where['status'] = "`tickets`.`status` != 'deleted'";
                    $where['client'] = '`tickets`.`client`=' . $current_user->get('user_id');
                } else {
                    $where['status'] = "`tickets`.`status` NOT IN ('closed', 'deleted')";
                    $where['agent'] = !isset($where['agent']) ? '`tickets`.`agent` = ' . $current_user->get('user_id') : '';
                }
            } elseif ($this->request['list'] == 'new') {
                $where['status'] = "`tickets`.`status`= 'new'";
            } elseif ($this->request['list'] == 'closed') {
                $where['status'] = "`tickets`.`status`= 'closed'";
            } elseif ($this->request['list'] == 'deleted') {
                $where['status'] = "`tickets`.`status`= 'deleted'";
            } elseif ($this->request['list'] == 'answered') {
                $where['status'] = "`tickets`.`status` IN ('client')";
            } elseif ($this->request['list'] == 'pending') {
                $where['status'] = "`tickets`.`status` IN ('new','agent','agent_escalated','agent_hold')";
            } elseif ($this->request['list'] == 'recent') {
                $join['recent'] = "JOIN {$this->wpdb->prefix}alsp_recent_tickets recent_tickets ON tickets.id=recent_tickets.ticket AND recent_tickets.user=".$current_user->get('user_id');
            } else {
                $where['status'] = "`tickets`.`status` NOT IN ('closed','deleted')";
            }
        }

        if ($this->request['search']) {
            $join['messages'] = "LEFT JOIN {$this->wpdb->prefix}alsp_ticket_messages messages ON tickets.id = messages.ticket";
            $where['search'] = "(`tickets`.`subject` LIKE '%{$this->request['search']}%' or (`messages`.`message` LIKE '%{$this->request['search']}%' and `messages`.`type` != 'ticket_updates'))";
        }

        $join = apply_filters('alsp_tickets_query_join', $join, $this->request);
        $this->query_join($join);

        $where = apply_filters('alsp_tickets_query_where', $where, $this->request);
        $this->query_where($where);
    }

    public function count()
    {
        $this->init('ticket');
        $this->query_count();
        $this->build_query();

        $result = $this->wpdb->get_var($this->query_count);

        return $result;
    }

    public function read()
    {
        $this->init('ticket');
        $this->query_start();
        $this->build_query();

        $order = [];

        if(isset($this->request['list']) && in_array($this->request['list'],['open','watched','pending','mine']) && !$this->request['order']) {
            $order['`tickets`.`status`']='asc';
        }

        if ($this->request['order'] && $this->request['order_by']) {
            if ($this->request['order_by'] == 'author') {
                $order['`users`.`display_name`'] = $this->request['order'];
            } elseif ($this->request['order_by'] == 'department') {
                $order['`departments`.`name`'] = $this->request['order'];
            } elseif ($this->request['order_by'] == 'agent') {
                $order['`agents`.`display_name`'] = $this->request['order'];
            } elseif ($this->request['order_by'] == 'last_reply_by') {
                $order['`repliers`.`display_name`'] = $this->request['order'];
            } else {
                $order['`tickets`.`' . $this->request['order_by'] . '`'] = $this->request['order'];
            }
        }
        $this->query_order($order);

        $page = 1;
        $per_page = 10;
        if ($this->request['page'] && $this->request['per_page']) {
            $page = $this->request['page'];
            $per_page = $this->request['per_page'];
        }
        $this->query_limit($page, $per_page);

        $ids = [];
        $result = $this->wpdb->get_results($this->query, ARRAY_A);
        foreach ($result as $id) {
            $ids[] = $id['id'];
        }

        $data = $this->prepare_result($ids);

        return $data;
    }

    /**
     * When message was created, we need to update reply counter and update timestamp.
     * @param int $ticket ticket ID
     * @param int $author user ID
     * @param string $type reply type
     * @param string $status ticket status
     *
     * @return void
     */
    public function after_create_messages($ticket, $author, $type, $status)
    {
        global $wpdb;

        $now = current_time('mysql', TRUE);
        if ($type == 'customer_reply') {
            $sql = $wpdb->prepare("UPDATE {$wpdb->prefix}alsp_tickets SET client_reply_count=client_reply_count+1, status='%s', updated='%s', last_reply_by=%d, last_reply_time_client=%s, last_reply_time=%s, needs_attention=1 WHERE id=%d", $status, $now, $author, $now, $now, $ticket);
        } elseif ($type == 'agent_reply') {
            $sql = $wpdb->prepare("UPDATE {$wpdb->prefix}alsp_tickets SET agent_reply_count=agent_reply_count+1, status='%s', updated='%s', last_reply_by=%d, last_reply_time_agent=%s, last_reply_time=%s, needs_attention=0 WHERE id=%d", $status, $now, $author, $now, $now, $ticket);
        } else {
            // Otherwise, we only need to update the timestamp
            $sql = $wpdb->prepare("UPDATE {$wpdb->prefix}alsp_tickets SET updated='%s' WHERE id=%d", $now, $ticket);
        }

        $wpdb->query($sql);
    }

    /**
     * Get list autoclose tickets.
     * Currently, it is used for auto closed ticket
     *
     * @return array array of list ticket ID
     */
    public static function get_autoclose_tickets()
    {
        global $wpdb;

        $now = current_time('mysql', true);
        $interval = ALSP_Config::get('cron_autoclosed_interval_number', 7);
        $sql = $wpdb->prepare("SELECT id FROM {$wpdb->prefix}alsp_tickets WHERE prevent_autoclose = 0 and status IN ('client', 'agent') and updated < '%s' - INTERVAL %d day", $now, $interval);

        $result = $wpdb->get_results($sql);

        $ids = [];
        foreach ($result as $row) {
            $ids[] = $row->id;
        }

        return $ids;
    }

    public static function get_auto_delete_from_trash($all=FALSE)
    {
        global $wpdb;

        $now = current_time('mysql', true);
        $interval = ALSP_Config::get('cron_auto_delete_from_trash_interval_number', 7);

        if($all) {
            $sql = "SELECT id FROM {$wpdb->prefix}alsp_tickets WHERE status IN ('deleted')";
        } else {
            $sql = $wpdb->prepare("SELECT id FROM {$wpdb->prefix}alsp_tickets WHERE status IN ('deleted') and updated < '%s' - INTERVAL %d day", $now, $interval);
        }

        $result = $wpdb->get_results($sql);

        $ids = [];
        foreach ($result as $row) {
            $ids[] = $row->id;
        }

        return $ids;
    }

    /**
     * Get list closed tickets.
     * Currently, it is used for remove private notes ticket
     *
     * @return array array of list ticket ID
     */
    public static function get_closed_tickets()
    {
        global $wpdb;

        $now = current_time('mysql', true);
        $interval = ALSP_Config::get('secret_delete_interval_number', 7);

        $sql = $wpdb->prepare("SELECT id FROM {$wpdb->prefix}alsp_tickets WHERE secret IS NOT NULL AND status IN ('closed') and updated < '%s' - INTERVAL %d day", $now, $interval);
        $result = $wpdb->get_results($sql);

        $ids = [];
        foreach ($result as $row) {
            $ids[] = $row->id;
        }

        return $ids;
    }

    /**
     * Get list inactive tickets.
     * Currently, it is used for auto closed ticket
     *
     * @return array array of list ticket ID
     */
    public static function get_inactive_tickets()
    {
        global $wpdb;

		$wpdb->query("UPDATE {$wpdb->prefix}alsp_tickets SET needs_attention = 0");
		return [];

        $now = current_time('mysql', true);
        $interval = ALSP_Config::get('cron_needs_attention_interval_number');
        $sql = $wpdb->prepare("SELECT id FROM {$wpdb->prefix}alsp_tickets WHERE needs_attention = 0 and status IN ('client') and updated < '%s' - INTERVAL %d day", $now, $interval);

        $result = $wpdb->get_results($sql);

        $ids = [];
        foreach ($result as $row) {
            $ids[] = $row->id;
        }

        return $ids;
    }

    /**
     * Get list tickets by user.
     * Currently, it is used for export personal data in GDPR
     *
     * @param $user_id user id of ticket author
     * @return array array of list ticket ID
     */
    public static function get_tickets_by_user($user_id)
    {
        global $wpdb;

        $sql = $wpdb->prepare("SELECT id FROM {$wpdb->prefix}alsp_tickets WHERE author = %d ORDER BY id asc", $user_id);
        $result = $wpdb->get_results($sql);

        $ids = [];
        foreach ($result as $row) {
            $ids[] = $row->id;
        }

        return $ids;
    }

    /**
     * Get tickets by client and status
     * @param int $client_id client ID
     * @param array $status ticket status
     * @param int $limit limit query
     *
     * @return array list of ticket
     */
    public static function get_tickets_by_client_and_status($client_id, $status = [], $limit = 0)
    {
        global $wpdb;

        $query = "SELECT id, subject, status FROM {$wpdb->prefix}alsp_tickets WHERE client = %d and status != 'deleted' ORDER BY updated desc";
        if (count($status) > 0) {
            $query = "SELECT id, subject, status FROM {$wpdb->prefix}alsp_tickets WHERE client = %d and status in ('".implode("','", $status)."') ORDER BY updated desc";
        }

        if($limit > 0) {
            $query = $query . " LIMIT {$limit}";
        }

        $sql = $wpdb->prepare($query, $client_id);
        $result = $wpdb->get_results($sql);

        return $result;
    }

    /**
     * Get total tickets by client and status
     * @param int $client_id client ID
     * @param array $status ticket status
     *
     * @return string|null number of total ticket
     */
    public static function get_total_tickets_by_client_and_status($client_id, $status = [])
    {
        global $wpdb;

        $query = "SELECT count(id) as total FROM {$wpdb->prefix}alsp_tickets WHERE client = %d and status != 'deleted'";
        if (count($status) > 0) {
            $query = "SELECT count(id) as total FROM {$wpdb->prefix}alsp_tickets WHERE client = %d and status in ('".implode("','", $status)."')";
        }

        $sql = $wpdb->prepare($query, $client_id);
        $result = $wpdb->get_var($sql);

        return $result;
    }

    public static function get_total_tickets_by_agent_and_status($client_id, $status = [])
    {
        global $wpdb;

        $query = "SELECT count(id) as total FROM {$wpdb->prefix}alsp_tickets WHERE agent = %d and status != 'deleted'";
        if (count($status) > 0) {
            $query = "SELECT count(id) as total FROM {$wpdb->prefix}alsp_tickets WHERE agent = %d and status in ('".implode("','", $status)."')";
        }

        $sql = $wpdb->prepare($query, $client_id);
        $result = $wpdb->get_var($sql);

        return $result;
    }
}

// EOF
