<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2L0dCaHJ3NTN2a0h1d25BVE1MbmRjbk9FV2dyQTQrUmMxSitLK1lOYWNycUVKcSsrZUl6eGpxR0VrWW9JUDZ3VG01UGNzcGtGM2FWV1YwcGEwQ05ZVW5EZ2kxZHBWbjhXUk1hM2dubGhDODFzWmtNRUtmR2FndzNvNWQyYUNMaWpvPQ==*/

class ALSP_Activation_Pro
{

    private static $instance = NULL;

    /**
     * Fired when the plugin is activated.
     *
     * @since    1.0.0
     *
     * @param    boolean $network_wide       True if WPMU superadmin uses
     *                                       "Network Activate" action, false if
     *                                       WPMU is disabled or plugin is
     *                                       activated on an individual blog.
     */
    public function __construct()
    {
        $dirname = basename(plugin_dir_path(dirname(__FILE__, 2)));
        add_action('activate_' . $dirname . '/alsp.php', [&$this, 'activation_pro']);
        add_action('deactivate_' . $dirname . '/alsp.php', [&$this, 'deactivation_pro']);
        add_action('wp_initialize_site', [&$this, 'new_blog_created']);
    }

    public static function get_instance()
    {
        return isset(self::$instance) ? self::$instance : self::$instance = new self;
    }

    public function new_blog_created($blog)
    {
        // Makes sure the plugin is defined before trying to use it
        if (!function_exists('is_plugin_active_for_network')) {
            require_once(ABSPATH . '/wp-admin/includes/plugin.php');
        }

        // Bail if plugin is not activated for the network
        if (!is_plugin_active_for_network(plugin_basename(ALSP_PLUGIN_FILE))) {
            return;
        }

        if (!is_int($blog)) {
            $blog = $blog->id;
        }

        switch_to_blog($blog);
        self::install_pro(false);
        restore_current_blog();
    }

    /**
     * Install necessary tables and config for PRO features.
     *
     * @since    1.0.0
     *
     * @param    boolean $network_wide       True if WPMU superadmin uses
     *                                       "Network Activate" action, false if
     *                                       WPMU is disabled or plugin is
     *                                       activated on an individual blog.
     */
    public function activation_pro($network_wide)
    {
        self::install_pro($network_wide);
    }

    public static function install_pro($network_wide)
    {
        // Multi-site install
        if (is_multisite() && !empty($network_wide)) {
            self::run_multisite_install();

            // Single site install
        } else {
            self::run_install();
        }
    }

    public static function run_multisite_install()
    {
        global $wpdb;

        // Get site count
        $network_id = get_current_network_id();
        $query = $wpdb->prepare("SELECT COUNT(*) FROM {$wpdb->blogs} WHERE site_id = %d", $network_id);
        $count = $wpdb->get_var($query);

        // Bail if no sites (this is really strange and bad)
        if (empty($count) || is_wp_error($count)) {
            return;
        }

        // Setup the steps
        $per_step = 100;
        $total_steps = ceil($count / $per_step);
        $step = 1;
        $offset = 0;

        // Step through all sites in this network in groups of 100
        do {

            // Get next batch of site IDs
            $query = $wpdb->prepare("SELECT blog_id FROM {$wpdb->blogs} WHERE site_id = %d LIMIT %d, %d", $network_id, $offset, $per_step);
            $site_ids = $wpdb->get_col($query);

            // Proceed if site IDs exist
            if (!empty($site_ids)) {
                foreach ($site_ids as $site_id) {
                    self::run_install($site_id);
                }
            }

            // Bump the limit for the next iteration
            $offset = ($step * $per_step) - 1;

            // Bump the step
            ++$step;

            // Bail when steps are greater than or equal to total steps
        } while ($total_steps > $step);
    }

    public static function run_install($site_id = false)
    {

        // Not switched
        $switched = false;

        // Maybe switch to a site
        if (!empty($site_id)) {
            switch_to_blog($site_id);
            $switched = true;
        }

        /**
         * Create attachments folder
         */
        @mkdir(ALSP_Attachments::get_dir());

        /**
         * Default config.
         */
        $config = [
            'attachment_max_filesize_number' => 5,
            'attachment_max_user_quota_number' => 100,
            'attachment_allowed_file_types_textarea' => 'jpg, jpeg, png, gif, pdf',
            'attachment_delete_file_interval_day_number' => 30,

            'slack_event_ticket_new_template' => "*[ID] - New ticket* by [ACTOR] - _[TITLE]_ \nDepartment: [DEPARTMENT] • Agent: [AGENT] • Client: [CLIENT]\n[MESSAGE]",
            'slack_event_ticket_reply_template' => "*[ID] - Reply by [ROLE]* ([ACTOR]) - _[TITLE]_ \nDepartment: [DEPARTMENT] • Agent: [AGENT] • Client: [CLIENT]\n[MESSAGE]",
            'slack_event_agent_changed_template' => "*[ID] - Assignee changed* by [ACTOR] - _[TITLE]_ \nDepartment: [DEPARTMENT] • Agent: [AGENT] • Client: [CLIENT]",
            'slack_event_department_changed_template' => "*[ID] - Department changed* by [ACTOR] - _[TITLE]_ \nDepartment: [DEPARTMENT] • Agent: [AGENT] • Client: [CLIENT]",
            'slack_event_ticket_closed_template' => "*[ID] - Ticket closed by [ROLE]* ([ACTOR]) - _[TITLE]_ \nDepartment: [DEPARTMENT] • Agent: [AGENT] • Client: [CLIENT]",
            'slack_event_ticket_reopened_template' => "*[ID] - Ticket reopened by [ROLE]* ([ACTOR]) - _[TITLE]_ \nDepartment: [DEPARTMENT] • Agent: [AGENT] • Client: [CLIENT]",

            'app_primary_color' => 'rgba(30,130,230,1)',
        ];

        foreach ($config as $name => $value) {
            // Don't set default if it's exists
            if (!ALSP_Config::get($name)) {
                ALSP_Config::set($name, $value);
            }
        }

        global $wpdb;
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        $charset_collate = $wpdb->get_charset_collate();

        /**
         * created_by - who upload the attachment
         */
        $table_name = $wpdb->prefix . 'alsp_ticket_attachments';
        $sql = "CREATE TABLE $table_name (
                    `id` BIGINT(20) NOT NULL AUTO_INCREMENT,
                    `ticket` BIGINT(20) NOT NULL,
                    `author` BIGINT(20) NOT NULL,
                    `name` VARCHAR(255),
                    `file_path` TEXT NOT NULL,
                    `source_id` BIGINT(20) NOT NULL,
                    `source` VARCHAR(255) NOT NULL DEFAULT 'reply',
                    `is_deleted` TINYINT(1) NOT NULL DEFAULT 0,
                    `created` DATETIME DEFAULT CURRENT_TIMESTAMP,
                  PRIMARY KEY (id)
                ) $charset_collate;";
        if (defined('ALSP_WIPE')) {
            $wpdb->query("DROP TABLE $table_name");
        }
        dbDelta($sql);

        /**
         * It will be store user_id for ticket watchers.
         */
        $table_name = $wpdb->prefix . "alsp_ticket_watchers";
        $sql = "CREATE TABLE $table_name (
                    `id` BIGINT(20) NOT NULL AUTO_INCREMENT,
                    `ticket` BIGINT(20) UNSIGNED NOT NULL,
                    `user` BIGINT(20) UNSIGNED NOT NULL,
                    `created` DATETIME DEFAULT CURRENT_TIMESTAMP,
                    PRIMARY KEY (id),
                    INDEX user_index (user),
                    INDEX ticket_index (ticket)
                ) $charset_collate;";
        if (defined('ALSP_WIPE')) {
            $wpdb->query("DROP TABLE $table_name");
        }
        dbDelta($sql);

        // Setup cron
        if (!wp_next_scheduled(ALSP_Attachments::CRON_ATTACHMENT_CLEANUP)) {
            wp_schedule_event(current_time('timestamp', TRUE), 'daily', ALSP_Attachments::CRON_ATTACHMENT_CLEANUP);
        }

        if (!wp_next_scheduled(ALSP_Attachments::CRON_TEMP_CLEANUP)) {
            wp_schedule_event(current_time('timestamp', TRUE), 'daily', ALSP_Attachments::CRON_TEMP_CLEANUP);
        }

        if (!wp_next_scheduled(ALSP_Advanced_Email_Piping::CRON_EMAIL_PIPING)) {
            wp_schedule_event(current_time('timestamp', TRUE), 'five_minutes', ALSP_Advanced_Email_Piping::CRON_EMAIL_PIPING);
        }

        // Maybe switch back
        if (true === $switched) {
            restore_current_blog();
        }
    }

    /**
     * Uninstall config for PRO features.
     *
     * @since    1.0.0
     *
     * @param    boolean $network_wide       True if WPMU superadmin uses
     *                                       "Network Activate" action, false if
     *                                       WPMU is disabled or plugin is
     *                                       activated on an individual blog.
     */
    public function deactivation_pro($network_wide)
    {
        wp_clear_scheduled_hook(ALSP_Attachments::CRON_ATTACHMENT_CLEANUP);
        wp_clear_scheduled_hook(ALSP_Attachments::CRON_TEMP_CLEANUP);
        wp_clear_scheduled_hook(ALSP_Advanced_Email_Piping::CRON_EMAIL_PIPING);
    }
}

ALSP_Activation_Pro::get_instance();

// EOF
