<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2L2hqSE9ncDBnUHFYZ3BvNG5nTitiWmtwMlAwVHNQNHNtMWp2d0NCbGdSdVdNTUZJLzRSQmE0VllVT3RoRzk4SU43Wi9LREhqSUpvSXp5ekxNQjkzQ2FKZzk2NUd4a211Z1ZOU3dHdWN3S2tCMkJBekorN3VMWDJRVFo2eWd6TmtjQmx6UTBNb2wxbkZXZkN5ZWQ2Rk8v*/

class ALSP_EP_Imap_Importer_Attachments
{
    /**
     * Parse attachments
     *
     * @param object        $mail - email object.
     * @return void|array
     */
    public static function parse_attachments($mail)
    {
        $result = [];
        if (!$mail->hasAttachments()) {
            return;
        }

        $attachments = $mail->getAttachments();
        foreach ($attachments as $attachment) {
            // @TODO: embed attachment
            // if ( $general['body-reference'] == 'html' && $attachment->contentId && preg_match( '/cid:' . $attachment->contentId . '/', $email->html_body ) ) { //phpcs:ignore
            // 	self::upload_embeded_img( $attachment, $mail, $email );
            // } else {
            $result[] = self::upload_thread_attachment($attachment);
            // }
        }

        return $result;
    }

    /**
     * Upload embeded image in the description
     *
     * @param object        $attachment - attachment array received from IMAP.
     * @param ALSP_EP_Email $email - email object.
     * @return void
     */
    private static function upload_embeded_img($attachment, $email)
    {

        // Database insert array init.
        $data = ['name' => $attachment->name];

        // saperate file name and extension.
        $filename = explode('.', $attachment->name);
        $extension = strtolower($filename[count($filename) - 1]);
        unset($filename[count($filename) - 1]);
        $filename = implode('.', $filename);
        $filename = str_replace(' ', '_', $filename);
        $filename = str_replace(',', '_', $filename);
        $filename = uniqid() . '_' . preg_replace('/[^A-Za-z0-9\-]/', '', $filename);

        // allowed file extenstions.
        $img_extensions = ['png', 'jpeg', 'jpg', 'gif'];
        if (!(in_array($extension, $img_extensions))) {
            self::upload_thread_attachment($attachment);
        }

        $data['is_image'] = 1;
        $data['source'] = 'img_editor';

        // Create file path.
        $today = new DateTime();
        $upload_dir = wp_upload_dir();
        $filepath = $upload_dir['basedir'] . '/wpsc/' . $today->format('Y');
        if (!file_exists($filepath)) {
            mkdir($filepath, 0755, true);
        }
        $filepath .= '/' . $today->format('m');
        if (!file_exists($filepath)) {
            mkdir($filepath, 0755, true);
        }
        $filepath .= '/' . $filename . '.' . $extension;

        $filepath_short = '/wpsc/' . $today->format('Y') . '/' . $today->format('m') . '/' . $filename . '.' . $extension;
        $data['file_path'] = $filepath_short;

        // Create time.
        $data['date_created'] = $today->format('Y-m-d H:i:s');

        // write file.
        file_put_contents($filepath, $attachment->getContents()); //phpcs:ignore

        // insert db record.
        $attach = WPSC_Attachment::insert($data);

        if ($attach->id) {
            $img_url = home_url('/') . '?wpsc_attachment=' . $attach->id;
            $email->html_body = str_replace('cid:' . $attachment->contentId, $img_url, $email->html_body); //phpcs:ignore
        }
    }

    /**
     * Upload regular thread attachments
     *
     * @param object         $attachment - attachment array received from microsoft exchange server.
     * @return array
     */
    private static function upload_thread_attachment($attachment)
    {
        // saperate file name and extension.
        $filename = explode('.', $attachment->name);
        $extension = strtolower($filename[count($filename) - 1]);
        unset($filename[count($filename) - 1]);
        $filename = implode('.', $filename);
        $filename = str_replace(' ', '_', $filename);
        $filename = str_replace(',', '_', $filename);
        $filename = uniqid() . '_' . preg_replace('/[^A-Za-z0-9\-]/', '', $filename);

        // allowed file extenstions.
        $allowed_file_extensions = ALSP_Config::get('attachment_allowed_file_types_textarea');
        $allowed_file_extensions = explode(',', $allowed_file_extensions);
        $allowed_file_extensions = array_map('trim', $allowed_file_extensions);
        $allowed_file_extensions = array_map('strtolower', $allowed_file_extensions);
        if (!(in_array($extension, $allowed_file_extensions))) {
            return ['file_name' => $attachment->name, 'status' => FALSE, 'error' => __('File extension not allowed!', 'awedesk')];
        }

        // check for allowed file size.
        $max_upload_size = ALSP_Config::get('attachment_max_filesize_number');

        // use WP max upload size if it is smaller than PeepSo max upload size
        $wp_max_size = max(wp_max_upload_size(), 0);
        $wp_max_size /= pow(1024, 2);
        if ($wp_max_size < $max_upload_size) {
            $max_upload_size = $wp_max_size;
        }

        $allowed_file_size = intval($max_upload_size) * 1048576;
        if ($attachment->sizeInBytes >= $allowed_file_size) { //phpcs:ignore
            return ['file_name' => $attachment->name, 'status' => FALSE, 'error' => __('File size exceeds allowed limit!', 'awedesk')];
        }

        // Check for image type.
        $img_extensions = ['png', 'jpeg', 'jpg', 'bmp', 'pdf', 'gif'];
        if (!in_array($extension, $img_extensions)) {
            $extension .= '.txt';
        } else {
            $data['is_image'] = 1;
        }

        // Write to temp folder
        $tmp_dir = ALSP_Attachments::get_tmp_dir();
        if (!file_exists($tmp_dir)) {
            mkdir($tmp_dir, 0755, true);
        }
        $destination = $tmp_dir . $filename . '.' . $extension;

        // write file.
        file_put_contents($destination, $attachment->getContents()); //phpcs:ignore

        return ['file_name' => $filename . '.' . $extension, 'status' => TRUE];
    }

}

// EOF