<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2K2Z2UnpsYVNsK1hWQ3FYNlB5MWtzalFDNWpURjBpeEpPQTBGMXRMVThyUjBkTUs3c0M4WWFyQTFDYUMzeHVERCtkVldYN2ZrSHVJOWpaOUZBSndWNG5PejltWmhSOFBGeFdaTXZoNGJiR1YwOFRwMDNHbDNZVXdkS2RJQWwxUjRSUW5rNEhDbWdNM3FhbEZTbjVwaXZx*/

class ALSP_EP_Microsoft_Exchange_Importer_Attachments
{
    /**
     * Parse attachments
     *
     * @param array         $message - message array.
     * @return void|array
     */
    public static function parse_attachments($message)
    {
        $attachments = [];
        $access_token = ALSP_Config_EP_Microsoft_Exchange::get_access_token();
        if (false === $access_token) {
            return [];
        }

        // get message attachments.
        $response = wp_remote_get(
            'https://graph.microsoft.com/v1.0/me/messages/' . $message['id'] . '/attachments',
            [
                'headers' => [
                    'Authorization' => 'Bearer ' . $access_token,
                ],
            ]
        );

        if (!ALSP_Config_EP_Microsoft_Exchange::is_valid_response($response)) {
            return [];
        }

        $response = json_decode($response['body'], true);
        foreach ($response['value'] as $attachment) {
            // if ($attachment['contentId'] && preg_match('/cid:' . $attachment['contentId'] . '/', $email->html_body)) {
            //     $attachments[] = self::upload_embeded_img($attachment);
            // } else {
            $attachments[] = self::upload_thread_attachment($attachment);
            // }
        }

        return $attachments;
    }

    /**
     * Upload embeded image in the description
     *
     * @param array         $attachment - attachment array received from microsoft exchange server.
     * @return void|string
     */
    private static function upload_embeded_img($attachment)
    {
        $filename = uniqid() . '_' . sanitize_file_name($attachment['name']);
        $extension = strtolower(pathinfo($filename, PATHINFO_EXTENSION));

        // Check file extension.
        $img_extensions = ['jpg', 'jpeg', 'png', 'gif'];
        if (!in_array($extension, $img_extensions)) {
            return self::upload_thread_attachment($attachment);
        }

        // Write to temp folder
        $tmp_dir = ALSP_Attachments::get_tmp_dir();
        if (!file_exists($tmp_dir)) {
            mkdir($tmp_dir, 0755, true);
        }
        $destination = $tmp_dir . $filename . '.' . $extension;

        // write file.
        $file_contents = strtr(
            $attachment['contentBytes'],
            [
                '-' => '+',
                '_' => '/',
            ]
        );
        file_put_contents($destination, base64_decode($file_contents)); //phpcs:ignore

        // if ($attach->id) {
        //     $img_url = home_url('/') . '?wpsc_attachment=' . $attach->id;
        //     $email->html_body = str_replace('cid:' . $attachment['contentId'], $img_url, $email->html_body);
        // }

        return $filename . '.' . $extension;
    }

    /**
     * Upload regular thread attachments
     *
     * @param array         $attachment - attachment array received from microsoft exchange server.
     * @return array
     */
    private static function upload_thread_attachment($attachment)
    {
        $filename = uniqid() . '_' . sanitize_file_name($attachment['name']);
        $extension = strtolower(pathinfo($filename, PATHINFO_EXTENSION));

        // Allowed file extension.
        $allowed_file_extensions = ALSP_Config::get('attachment_allowed_file_types_textarea');
        $allowed_file_extensions = explode(',', $allowed_file_extensions);
        $allowed_file_extensions = array_map('trim', $allowed_file_extensions);
        $allowed_file_extensions = array_map('strtolower', $allowed_file_extensions);
        if (!(in_array($extension, $allowed_file_extensions))) {
            return ['file_name' => $attachment['name'], 'status' => FALSE, 'error' => __('File extension not allowed!', 'awedesk')];
        }

        // Write to temp folder
        $tmp_dir = ALSP_Attachments::get_tmp_dir();
        if (!file_exists($tmp_dir)) {
            mkdir($tmp_dir, 0755, true);
        }
        $destination = $tmp_dir . $filename . '.' . $extension;

        // write file.
        $file_contents = strtr(
            $attachment['contentBytes'],
            [
                '-' => '+',
                '_' => '/',
            ]
        );

        $filesize = (int) (strlen(rtrim($file_contents, '=')) * 3 / 4);

        // check for allowed file size.
        $max_upload_size = ALSP_Config::get('attachment_max_filesize_number');

        // use WP max upload size if it is smaller than PeepSo max upload size
        $wp_max_size = max(wp_max_upload_size(), 0);
        $wp_max_size /= pow(1024, 2);
        if ($wp_max_size < $max_upload_size) {
            $max_upload_size = $wp_max_size;
        }

        $allowed_file_size = intval($max_upload_size) * 1048576;
        if ($filesize >= $allowed_file_size) { //phpcs:ignore
            return ['file_name' => $attachment['name'], 'status' => FALSE, 'error' => __('File size exceeds allowed limit!', 'awedesk')];
        }

        file_put_contents($destination, base64_decode($file_contents)); //phpcs:ignore

        return ['file_name' => $filename . '.' . $extension, 'status' => TRUE];
    }

}

// EOF