<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OFFRdG14Q2gxdER3aEZYNUdsNHl4b2g1bnVmS3NUS0wxT3VneklOdURNRncxN0NjcDhKWkwvVVYzV05kN0ZVbW1UaU83bFllUEY3SXcyc1ZCcnoxK3pEVDg3aFhKUE8ySlM2aXVpQ3MzYysrMlVnZlRBZ2xEZENmQ0pLR3VSUmpTUUxWSkhDSE9lNHhNUUl6WGQ3Wngx*/

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly!
}

class ALSP_Config_Department_EP_Imap
{
    private static $instance = NULL;

    public static function get_instance()
    {
        return (NULL == self::$instance) ? self::$instance = new self() : self::$instance;
    }

    private function __construct()
    {
        add_filter('alsp_config_description', function ($desc) {
            $request = new WP_REST_Request;
            $model_department = new ALSP_Model_Departments($request);
            foreach ($model_department->read() as $key => $dept) {
                if (!isset($dept['id']) && !$dept['id']) {
                    continue;
                }

                $prefix = 'dept' . $dept['id'] . '_';
                $desc[$prefix . 'ep_imap_email_label'] = FALSE;
                $desc[$prefix . 'ep_imap_incoming_mail_server_label'] = FALSE;
                $desc[$prefix . 'ep_imap_is_active_label'] = FALSE;
                $desc[$prefix . 'ep_imap_message'] = '<div class="alsp-opt__form-actions">
                    <button class="alsp-opt__action alsp-btn alsp-btn--action" data-id="ep_imap_check" data-btn="check" data-tooltip="Check" style="">
                        <i class="alsp-i-plug"></i> Check
                        <i class="alsp-i-spinner alsp-i-spin" data-loading style="display:none"></i>
                    </button>
                </div>';
            }

            return $desc;
        });

        add_filter('alsp_config', function ($config, $position) {
            if (strpos($position, 'department-config-' === FALSE)) {
                return $config;
            }

            $imap_encryption = [
                'ssl' => 'SSL',
                'tls' => 'TLS',
                'none' => __('Disabled', 'awedesk'),
            ];

            $request = new WP_REST_Request;
            $model_department = new ALSP_Model_Departments($request);
            foreach ($model_department->read() as $key => $dept) {
                if (!isset($dept['id']) && !$dept['id']) {
                    continue;
                }

                $prefix = 'dept' . $dept['id'] . '_';
                $dep = $prefix . 'ep_connection_select=imap';
                $is_active = ALSP_Config::get($prefix . 'ep_imap_is_active', 0);
                $status = $is_active ? __('Connected', 'awedesk') : __('Not Connected', 'awedesk');

                $config['department-config-imap-conn-' . $dept['id']] = [
                    'id' => 'department_config_imap_conn-' . $dept['id'],
                    'title' => 'IMAP',
                    'icon' => 'alsp-i-clip',
                    'description' => sprintf(__('Configure connection: %s', 'awedesk'), 'IMAP'),
                    'position' => 'department-config-' . $dept['id'],
                    'items' => [
                        $prefix . 'ep_imap_email_label' => ALSP_Config_Panel::admin_item($prefix . 'ep_imap_email_label', $dep, __('Email', 'awedesk')),
                        $prefix . 'ep_imap_email_password' => ALSP_Config_Panel::admin_item($prefix . 'ep_imap_email_password', $dep, __('Password', 'awedesk')),
                        $prefix . 'ep_imap_encryption_select' => ALSP_Config_Panel::admin_item($prefix . 'ep_imap_encryption_select', $dep, __('Encryption', 'awedesk'), ['options' => $imap_encryption]),
                        $prefix . 'ep_imap_incoming_mail_server_label' => ALSP_Config_Panel::admin_item($prefix . 'ep_imap_incoming_mail_server_label', $dep, __('Incoming Mail Server', 'awedesk')),
                        $prefix . 'ep_imap_port_number' => ALSP_Config_Panel::admin_item($prefix . 'ep_imap_port_number', $dep, __('Port', 'awedesk')),
                        $prefix . 'ep_imap_is_active_label' => ALSP_Config_Panel::admin_item($prefix . 'ep_imap_is_active_label', $dep, __('Status', 'awedesk'), ['default_value' => $status, 'disabled' => TRUE]),
                        $prefix . 'ep_imap_message' => ALSP_Config_Panel::admin_item($prefix . 'ep_imap_message', $dep, FALSE),
                    ],
                ];
            }

            return $config;
        }, 31, 2);

        add_action('admin_notices',[&$this, 'connection_error_admin_notice']);
    }

    /**
     * IMAP Connection error admin notice
     *
     * @return void
     */
    public static function connection_error_admin_notice()
    {
        if (!is_admin()) {
            return;
        }

        $request = new WP_REST_Request;
        $model_department = new ALSP_Model_Departments($request);
        foreach ($model_department->read() as $key => $dept) {
            if (!isset($dept['id']) && !$dept['id']) {
                continue;
            }

            $prefix = 'dept' . $dept['id'] . '_';
            $connection = ALSP_Config::get($prefix . 'ep_connection_select', 'imap');
            $connection_notice = ALSP_Config::get($prefix . 'ep_imap_connection_notice');
            $imap_is_active = ALSP_Config::get($prefix . 'ep_imap_is_active', 0);
            // return if either connection is active or notice dismissed.
            if (!($connection == 'imap' && !$imap_is_active && $connection_notice)) {
                continue;
            }
            ?>
            <div class="alsp imap connection-error notice notice-error">
                <p>
                    Awedesk: Email Piping encountered a connection error
                </p>
                <p>
                    <a href="<?php echo esc_url_raw(admin_url('admin.php?page=alsp-department')); ?>">
                        <?php echo esc_attr__('View Details', 'awedesk'); ?>
                    </a>
                </p>
            </div>
            <?php
        }
    }
}

// ALSP_Config_Department_EP_Imap::get_instance();

// EOF
