<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OFFRdG14Q2gxdER3aEZYNUdsNHl4bytwNDZQSVFoNStkbi9YRWgxcFAvUEJ2Rnd0Rjk4a3FBR2ErdGt4MjZRKzF0Z1N2Ylk1OHRpNi9XUnMyUFZtUHRQb2dHTHVUVm9hWGpOa2x0cDBWWnJTOVBjQWkrWjh2ZXdaZ3VRejRlL3hmOExuRU1sY1ZRT2dlZm02bm1ZMHM1TEFyMTdwVHMxdm5oVVhsK2c2WWtyUT09*/

class ALSP_Config_Department_EP_Microsoft_Exchange
{
    private static $instance = NULL;

    public static function get_instance()
    {
        return (NULL == self::$instance) ? self::$instance = new self() : self::$instance;
    }

    private function __construct()
    {
        add_filter('alsp_config_description', function ($desc) {
            $request = new WP_REST_Request;
            $model_department = new ALSP_Model_Departments($request);
            foreach ($model_department->read() as $key => $dept) {
                if (!isset($dept['id']) && !$dept['id']) {
                    continue;
                }
                $prefix = 'dept' . $dept['id'] . '_';
                $desc[$prefix . 'ep_me_tenant_id_label'] = FALSE;
                $desc[$prefix . 'ep_me_client_id_label'] = FALSE;
                $desc[$prefix . 'ep_me_client_secret_label'] = FALSE;
                $desc[$prefix . 'ep_me_is_active_label'] = FALSE;
                $desc[$prefix . 'ep_me_redirect_uri_label'] = FALSE;
                $desc[$prefix . 'ep_me_piping_address_label'] = FALSE;
                $desc[$prefix . 'ep_me_last_error_label'] = FALSE;
                $desc[$prefix . 'ep_me_message'] = '<div class="alsp-opt__form-actions">
                    <button class="alsp-opt__action alsp-btn alsp-btn--action" data-id="ep_me_check" data-btn="check" data-tooltip="Check" style="">
                        <i class="alsp-i-plug"></i> Check
                        <i class="alsp-i-spinner alsp-i-spin" data-loading style="display:none"></i>
                    </button>
                </div>';
            }

            return $desc;
        });

        add_filter('alsp_config', function ($config, $position) {
            if (strpos($position, 'department-config-' === FALSE)) {
                return $config;
            }

            $request = new WP_REST_Request;
            $model_department = new ALSP_Model_Departments($request);
            foreach ($model_department->read() as $key => $dept) {
                if (!isset($dept['id']) && !$dept['id']) {
                    continue;
                }

                $prefix = 'dept' . $dept['id'] . '_';
                $dep = $prefix . 'ep_connection_select=microsoft-exchange';
                $is_active = ALSP_Config::get($prefix . 'ep_me_is_active', 0);
                $status = $is_active ? __('Connected', 'awedesk') : __('Not Connected', 'awedesk');
                $last_error = ALSP_Config::get($prefix . 'ep_me_last_error', '');

                $config['department-config-me-conn-' . $dept['id']] = [
                    'id' => 'department_config_me_conn-' . $dept['id'],
                    'title' => 'Microsoft Exchange',
                    'icon' => 'alsp-i-clip',
                    'description' => sprintf(__('Configure connection: %s', 'awedesk'), 'Microsoft Exchange'),
                    'position' => 'department-config-' . $dept['id'],
                    'items' => [
                        $prefix . 'ep_me_tenant_id_label' => ALSP_Config_Panel::admin_item($prefix . 'ep_me_tenant_id_label', $dep, __('Tenant ID', 'awedesk')),
                        $prefix . 'ep_me_client_id_label' => ALSP_Config_Panel::admin_item($prefix . 'ep_me_client_id_label', $dep, __('Client ID', 'awedesk')),
                        $prefix . 'ep_me_client_secret_label' => ALSP_Config_Panel::admin_item($prefix . 'ep_me_client_secret_label', $dep, __('Client Secret', 'awedesk')),
                        $prefix . 'ep_me_is_active_label' => ALSP_Config_Panel::admin_item($prefix . 'ep_me_is_active_label', $dep, __('Status', 'awedesk'), ['default_value' => $status, 'disabled' => TRUE]),
                        $prefix . 'ep_me_redirect_uri_label' => ALSP_Config_Panel::admin_item($prefix . 'ep_me_redirect_uri_label', $dep, __('Redirect URI', 'awedesk'), ['default_value' => esc_url(admin_url('admin.php')), 'disabled' => TRUE]),
                        $prefix . 'ep_me_piping_address_label' => ALSP_Config_Panel::admin_item($prefix . 'ep_me_piping_address_label', $dep, __('Piping Address', 'awedesk'), ['default_value' => '', 'disabled' => TRUE]),
                        $prefix . 'ep_me_last_error_label' => ALSP_Config_Panel::admin_item($prefix . 'ep_me_last_error_label', $dep, __('Last Error', 'awedesk'), ['default_value' => $last_error, 'disabled' => TRUE]),
                        $prefix . 'ep_me_message' => ALSP_Config_Panel::admin_item($prefix . 'ep_me_message', $dep, FALSE),
                    ],
                ];
            }

            return $config;
        }, 33, 2);

        // microsoft exchange process connection response.
        add_action('admin_init', ['ALSP_Config_EP_Config_Connection_Checker', 'process_response_microsoft_exchange']);
    }

    /**
     * Get new access token using refresh token
     *
     * @return mixed
     */
    public static function get_access_token()
    {
        $access_token = get_transient('alsp_ep_me_access_token');
        if ($access_token !== false) {
            return $access_token;
        }

        $refresh_token = ALSP_Config::get('ep_me_refresh_token', '');
        if ($refresh_token === '') {
            return false;
        }

        $client_id = ALSP_Config::get('ep_me_client_id_label');
        $client_secret = ALSP_Config::get('ep_me_client_secret_label');
        $tenant_id = ALSP_Config::get('ep_me_tenant_id_label');

        // get access token using refresh token.
        $response = wp_remote_post(
            'https://login.microsoftonline.com/' . $tenant_id . '/oauth2/v2.0/token',
            [
                'body' => [
                    'client_id' => $client_id,
                    'client_secret' => $client_secret,
                    'grant_type' => 'refresh_token',
                    'refresh_token' => $refresh_token,
                ],
            ]
        );

        if (!self::is_valid_response($response)) {
            return false;
        }

        $response = json_decode($response['body']);
        set_transient('alsp_ep_me_access_token', $response->access_token, $response->expires_in - 5);
        return $response->access_token;
    }

    /**
     * Check whether API response is valid
     *
     * @param array $response - REST API response array.
     * @return boolean
     */
    public static function is_valid_response($response)
    {
        if (is_wp_error($response)) {
            $last_error = $response->get_error_message();
            ALSP_Config::set('ep_me_last_error', $last_error);
            return FALSE;
        }

        if ($response['response']['code'] !== 200) {
            $last_error = json_decode($response['body']);
            $last_error = $last_error->error->message;
            ALSP_Config::set('ep_me_last_error', $last_error);
            return FALSE;
        }

        ALSP_Config::set('ep_me_last_error', '');
        return TRUE;
    }
}

// ALSP_Config_Department_EP_Microsoft_Exchange::get_instance();

// EOF
