<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OW1UR0xoa1FnL0VvWXFGd3RqK2xJem5ad3NwMGE4UEpUNWJqMW9NeENpRXZTWWtoZ2xhV0szQlVSZnE1L3NyeG5yVmVoay9kbmQyamgvNjNBNDQ2a3I2NmsyMmxSWFJtL3Ixa3FPeU9BT3VnPT0=*/

class ALSP_Push {

	private static $instances=[];
	private $user_id = NULL;
	private $tokens = [];

	public static function get_instance($user_id) {
		return self::$instances[$user_id] ?? self::$instances[$user_id] = new self($user_id);
	}

	private function __construct($user_id) {

		$this->user_id = intval($user_id);
		if($this->user_id) {

			global $wpdb;


			$user_tokens = $wpdb->get_results("SELECT token FROM {$wpdb->prefix}alsp_app_push WHERE user = {$this->user_id}", ARRAY_A);
			if (!count($user_tokens)) {
				new ALSP_Debug('No tokens found for user '.$this->user_id);
			} else {
				$user_tokens = array_column($user_tokens, 'token');
				new ALSP_Debug(count($user_tokens).' Tokens found for user '.$this->user_id);
				new ALSP_Debug(print_r($user_tokens, true));
				$this->tokens = $user_tokens;
			}
		}
	}

	/**
	 * Sends a push notifications to all agents (assigned and watchers)
	 * @param $ticket_id - which ticket
	 * @param $actor_id - who is taking the action (so we can exclude them from the notification and put it in the notification who did something) 0 for system notifications
	 * @param $action - what action was performed, send() decides based on action and actor_id what strings to use
	 * @param $args - message (for replies), assignee (for assignee changes) etc
	 *
	 * @return void
	 */
	public static function send_to_agents($ticket_id, $actor_id, $action, $args=[]) {
		try {
			$recipients = ALSP_Model_Ticket::get_followers($ticket_id, $actor_id);

			if(!count($recipients)) { return; }

			$actor_name = "";
			$actor_staff = "";
			if($actor_id > 0) {
				$actor = ALSP_User::get_instance( $actor_id );
				$actor_details = $actor->get_details();
				$actor_name  = is_array($actor_details) ? $actor_details['display_name'] : "";
				$actor_staff = $actor->is_staff();
			}

			$ticket_data = (new ALSP_Model_Ticket($ticket_id))->get_item();

			foreach ( $recipients as $recipient ) {
				ALSP_Push::get_instance( $recipient )->send( $action, $args['message'] ?? "", [
					'actor_name'          => $actor_name,
					'actor_staff'         => $actor_staff,
					'action_id'       => intval($args['action_id']) ?? NULL,
					'action_label' => $args['action_label'] ?? NULL,
					'ticket_number'  => $ticket_data['id'],
					'ticket_subject' => $ticket_data['subject']
				]);
			}
		} catch ( Exception $e ) {
			new ALSP_Debug( $e->getMessage() );
		}
	}

	public function send($action, $message, $args = []) {
		$server = get_bloginfo('name');
		$ticket = $args['ticket_number'];
		$subject =  $args['ticket_subject'];
		$actor_staff = $args['actor_staff'] ?? false;
		$actor_role = $actor_staff ? __('Agent', 'awedesk') : __('Client', 'awedesk');
		$actor_name =  strlen($args['actor_name']) ?? $actor_role;
		$actor_full_name = strlen($args['actor_name']) ?"$actor_name ($actor_role)" : $actor_role;

		$server = strlen($server) ? "$server " : "";
		$ticket = strlen($ticket) ? " [#$ticket] " : "";
		$subject = strlen($subject) ? "$subject" : "";


		$title = htmlspecialchars_decode($server.$ticket.$subject);
		$message = htmlspecialchars_decode($message);

		$action_id = $args['action_id'] ?? 0;
		$action_label = $args['action_label'] ?? $message;

		$hasActor = strlen($actor_name) > 0;

		if (!count($this->tokens)) {
			return FALSE;
		}

		if($action == 'agent') {
			if(!$action_id) {
				// Unassigned
				$subtitle =  sprintf(__('%s removed the assignee', 'awedesk'), $actor_name);
			} elseif ( $action_id == $this->user_id ) {
				// Assigned to me
				$subtitle = $hasActor
					? sprintf(__('%s assigned to you', 'awedesk'), $actor_name)
					:__('Assigned to you', 'awedesk');
			} else {
				// Reassigned
				$assignee = ALSP_User::get_instance($action_id)->get_details()['display_name'];
				$subtitle = $hasActor
					? sprintf(__('%s assigned to %s', 'awedesk'), $actor_name, $assignee)
					:sprintf(__('Assigned to %s', 'awedesk'), $assignee);
			}
		} else {
			$messages = [
				// Actions that do not depend on actor presence
				'autoclosed'      => __('Closed automatically', 'awedesk'),

				// @todo unassigned tickets should be sent to old assignee even if they are not a watcher

				// Actions where actor presence changes formatting
				'closed'    => sprintf(__('%s closed the ticket', 'awedesk'), $actor_full_name),
				'reopened'  => sprintf(__('%s reopened the ticket', 'awedesk'), $actor_full_name),
				'private_note'    => sprintf(__('%s updated the private note', 'awedesk'), $actor_full_name),
				'reply'    => sprintf(__('%s replied').": %s", $actor_full_name, $action_label),
				'department'    => sprintf(__('%s moved to %s'), $actor_name, $action_label),
				'status'    => sprintf(__('%s set status %s'), $actor_name, $action_label),

			];

			// @todo we can build strings for the new notifications table here - bold the actor and the action_label, with exception of message and store it to db
			// or to maintain translatability, only store metadata (action, actor, etc), and have the notifications endpoint render them
			// actor/agent label would be floating in the app

			if (isset($messages[$action])) {
				$subtitle = $messages[$action];
			} else {
				$subtitle = "$actor_full_name / $action / $action_label";
			}
		}

		$subtitle = htmlspecialchars_decode($subtitle);

		// Prepare the request parameters
		$data = [
			'to'            => $this->tokens,
			'sound'         => 'default',
			'title'         =>  $this->trim($this->sanitize($title), 55),
			'body'          =>  $this->trim($this->sanitize($subtitle), 100),
			'data'          => [
				'server'        => get_bloginfo('url'),
				'ticket_number' => $args['ticket_number'] ?? NULL,
			]
		];

		$queryParams = [
			'action'   => 'forward',
			'data'     => json_encode($data),
		];

		// Build the GET request URL
		$url = "https://awedesk.com/pushapi/v1/?" . http_build_query($queryParams);

		new ALSP_Debug("PUSH {$this->user_id} " . $url);

		// Send the GET request to the new API
		$response = wp_remote_get($url);

		$body = wp_remote_retrieve_body($response);

		try {
			$responseData = json_decode($body, true);
			if (isset($responseData['data'][0]['status']) && $responseData['data'][0]['status'] === 'ok') {
				return TRUE;
			} else {
				new ALSP_Debug(print_r($responseData, true));
				return FALSE;
			}
		} catch (Exception $e) {
			return FALSE;
		}
	}

	private function sanitize($text) {

		// Remove all line breaks, tabs and double spaces
		$text = preg_replace(array('/\s{2,}/', '/[\t\n]/'), ' ', $text);

		// Remove HTML
		$text = strip_tags($text);

		return $text;
	}

	private function trim($text, $maxLength = 55, $more='...') {

		$maxLength = $maxLength - strlen($more);
		// Check if the text is already within the limit
		if (ALSP_Utility_String::maybe_mb_strlen($text) <= $maxLength) {
			return $text;
		}

		// Trim the text to the maximum length
		$trimmed = ALSP_Utility_String::maybe_mb_substr($text, 0, $maxLength);

		// Find the position of the last space within the trimmed text
		$lastSpace = strrpos($trimmed, ' ');

		if ($lastSpace !== false) {
			// Trim the text at the last space to avoid breaking a word
			$trimmed = ALSP_Utility_String::maybe_mb_substr($trimmed, 0, $lastSpace);
		}

		return $trimmed.$more;
	}
}

