<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2ODRYVUc5Q2F5bDM0UWFMWHRUOGdyaWN3NXRZMWU2T0lTOTZlNTlPNjRQZHJtN2NvNnFBcWhONFk4VFphRjEzWDY5NEMrdzk5ejkrNm90SUVxUE4rdkJVaDZucUUvVWFpd2g3dGxXcmVFbFRrVFhRdFJyZXBwWm01cEhSdXJKaHVFPQ==*/

class ALSP_Attachments
{

    const CRON_ATTACHMENT_CLEANUP = 'alsp_attachment_cleanup_event';
    const CRON_TEMP_CLEANUP = 'alsp_temp_cleanup_event';
    private static $instance = NULL;

    /**
     * Connections
     *
     * @var array
     */
    public static $piping_connections = [];

    public static function get_instance()
    {
        return (NULL == self::$instance) ? self::$instance = new self() : self::$instance;
    }

    private function __construct()
    {
        if (is_admin()) {
            add_action('admin_init', function () {
                if (0 == ALSP_Config::get('cron_attachment_cleanup_enabled')) {
                    if (!stristr(json_encode(_get_cron_array()), ALSP_Attachments::CRON_ATTACHMENT_CLEANUP)) {
                        add_action('admin_notices',[$this, 'cron_attachment_cleanup_notice']);
                    }
                }

                if (0 == ALSP_Config::get('cron_temp_cleanup_enabled')) {
                    if (!stristr(json_encode(_get_cron_array()), ALSP_Attachments::CRON_TEMP_CLEANUP)) {
                        add_action('admin_notices', [$this, 'cron_temp_cleanup_notice']);
                    }
                }
            });
        }

        add_action('wp', function () {
            if (isset($_GET['alsp_process_attachment_cleanup'])) {
                ALSP_Attachments::attachment_cleanup();
                die();
            }

            if (isset($_GET['alsp_process_temp_cleanup'])) {
                ALSP_Attachments::temp_cleanup();
                die();
            }

            if (isset($_REQUEST['alsp_attachment'])) {
                $attachment_id = intval($_REQUEST['alsp_attachment']);
                $attachment = new ALSP_Model_Ticket_Attachment($attachment_id);
                if (!$attachment->id) {
                    return;
                }

                $auth_code = isset($_REQUEST['auth_code']) ? sanitize_text_field($_REQUEST['auth_code']) : '';
                $ticket = new ALSP_Model_Ticket($attachment->ticket);
                if (!$ticket->id) {
                    wp_send_json_error('Bad request!', 400);
                }

                $auth_code = isset($_REQUEST['auth_code']) ? sanitize_text_field($_REQUEST['auth_code']) : '';
                if (!$auth_code || $auth_code != $ticket->auth_code) {
                    wp_send_json_error('Unauthorized!', 401);
                }

                self::file_download($attachment);
            }
        }, -1);

        add_action('alsp_create_ticket_message_after', [&$this, 'insert_message_attachment'], 10, 3);

        if (1 == ALSP_Config::get('attachment_enabled', 1)) {
            add_filter('alsp_template', [&$this, 'override_template'], 10, 2);
            add_action('alsp_template_insert', [&$this, 'insert_template']);
        }
    }

    /**
     * Download attachment
     *
     * @param ALSP_Model_Ticket_Attachment $attachment - attachment object.
     * @return void
     */
    public static function file_download($attachment)
    {

        // Turn off output buffering.
        if (ob_get_level()) {
            ob_end_clean();
        }

        $file_path = WP_CONTENT_DIR . $attachment->file_path;
        if (!file_exists($file_path)) {
            return;
        }

        // Check whether attachment is of image type.
        $mime_type = mime_content_type($file_path);
        $arr_mime_type = explode('/', $mime_type);
        $type = $arr_mime_type[0] ?? '';

        if ($mime_type == 'image/vnd.dwg') {
            // Set the appropriate MIME type for DWG files
            header('Content-Type: application/octet-stream');
            header('Content-Disposition: attachment; filename="' . basename($file_path) . '"');
            header('Content-Length: ' . filesize($file_path));

            // Read the file and output it to the browser
            readfile($file_path);
            exit;
        }

        if ($type == 'image') {
            if (ob_get_length() > 0) {
                ob_clean();
            }
            header('Content-Type: ' . $mime_type);
            readfile($file_path); // phpcs:ignore
            exit(0);
        }

        header('Content-Description: File Transfer');
        header('Cache-Control: public');
        header('Content-Type: application/force-download');
        header('Content-Disposition: attachment;filename="' . $attachment->name . '"');
        header('Content-Length: ' . filesize($file_path));
        flush();
        readfile($file_path); // phpcs:ignore
        exit(0);
    }


    public static function cron_attachment_cleanup_notice()
    {
        $result = FALSE;
        if (!wp_next_scheduled(ALSP_Attachments::CRON_ATTACHMENT_CLEANUP)) {
            $result = wp_schedule_event(current_time('timestamp', TRUE), 'daily', ALSP_Attachments::CRON_ATTACHMENT_CLEANUP);
        }

        if (!$result || $result instanceof WP_Error) {
            echo '<div class="error alsp">' .
                'Awedesk: "Attachment Cleanup" cron encountered some problems and was re-initialized.<br><small>If you see this message repeatedly, there might be something wrong with your WordPress Cron. Consider deactivating and re-activating Awedesk or contacting Support.</small>'
                . '</strong></div>';
        }
    }

    public static function cron_temp_cleanup_notice()
    {
        $result = FALSE;
        if (!wp_next_scheduled(ALSP_Attachments::CRON_TEMP_CLEANUP)) {
            $result = wp_schedule_event(current_time('timestamp', TRUE), 'daily', ALSP_Attachments::CRON_TEMP_CLEANUP);
        }

        if (!$result || $result instanceof WP_Error) {
            echo '<div class="error alsp">' .
                'Awedesk: "Temporary Folder Cleanup" cron encountered some problems and was re-initialized.<br><small>If you see this message repeatedly, there might be something wrong with your WordPress Cron. Consider deactivating and re-activating Awedesk or contacting Support.</small>'
                . '</strong></div>';
        }
    }

    /**
     * Insert attachment
     * @param WP_REST_Request $request
     * @param int $ticket ticket ID
     * @param int $message_id message ID
     *
     * @return void
     */
    public function insert_message_attachment($request, $ticket, $message_id)
    {
        $files = $request['files'];

        if (!is_array($files)) {
            return;
        }

        foreach ($files as $key => $file) {
            // Move file from tmp folder
            $source = ALSP_Attachments::get_tmp_dir() . $file;
            if (!file_exists($source)) {
                continue;
            }

            $destination = ALSP_Attachments::get_dir() . $file;
            rename($source, $destination);

            // Store file path without wp-content.
            $file_path = str_replace(WP_CONTENT_DIR, '', $destination);

            $ticket_attachment = new ALSP_Model_Ticket_Attachment();
            $result = $ticket_attachment->create_item($ticket, get_current_user_id(), $file, $file_path, $message_id, 'reply');
            if (!$result) {
                continue;
            }
        }
    }

    /**
     * Return the Awedesk attachment working directory, adjusted for MultiSite installs
     * @return string Awedesk working directory
     */
    public static function get_dir()
    {
        $alsp_dir = ALSP::get_alsp_dir();
        $attachment_dir = $alsp_dir . 'attachment' . DIRECTORY_SEPARATOR;

        return ($attachment_dir);
    }

    /**
     * Get attachment temporary directory
     * @return string Temporary attachment directory
     */
    public static function get_tmp_dir()
    {
        return (self::get_dir() . 'tmp' . DIRECTORY_SEPARATOR);
    }

    /**
     * Return the Awedesk attachment working directory as a URL
     * @return string Awedesk working directory URL
     */
    public static function get_uri()
    {
        $alsp_uri = ALSP::get_alsp_uri();
        $attachment_uri = $alsp_uri . 'attachment/';

        return ($attachment_uri);
    }

    public static function attachment_cleanup()
    {
        $list_id = ALSP_Model_Ticket_Attachments::get_inactive_tickets_attachments();
        foreach ($list_id as $key => $id) {
            $attachment_model = new ALSP_Model_Ticket_Attachment($id);
            if (!$attachment_model->id) {
                error_log(json_encode([
                    'success' => FALSE,
                    'error' => 'attachment_not_found',
                    'message' => 'Something went wrong'
                ]));
            }

            $detail = $attachment_model->get_item();
            $path = WP_CONTENT_DIR . $detail['file_path'];
            if (file_exists($path)) {
                unlink($path);
            }

            if (FALSE === $attachment_model->soft_delete()) {
                error_log(json_encode([
                    'success' => FALSE,
                    'error' => 'failed_to_delete_attachment',
                    'message' => 'Something went wrong'
                ]));
            }
        }
    }

    public static function temp_cleanup()
    {
        $folder = ALSP_Attachments::get_tmp_dir(); // Specify the path to your temporary folder

        // Check if the folder exists
        if (is_dir($folder)) {
            // Get the current timestamp
            $current_time = time();

            // Get a list of files in the folder
            $files = scandir($folder);

            // Iterate through each file
            foreach ($files as $file) {
                // Exclude current directory and parent directory entries
                if ($file != '.' && $file != '..') {
                    // Get the last modification time of the file
                    $file_modified_time = filemtime($folder . '/' . $file);

                    // Get the attachment creation in file
                    $arr_file = explode('_', $file);
                    if (count($arr_file) > 0 && intval($arr_file[0])) {
                        // Attempt to parse the string as a timestamp
                        $timestamp = ((string) (int) $arr_file[0] === $arr_file[0])
                        && ($arr_file[0] <= PHP_INT_MAX)
                        && ($arr_file[0] >= ~PHP_INT_MAX);
                        $file_modified_time = $timestamp ? $arr_file[0] : $file_modified_time;
                    }

                    // Calculate the difference between current time and last modification time in seconds
                    $difference_in_seconds = $current_time - $file_modified_time;

                    // If the difference is greater than 3 day (86400 * 3 seconds), remove the file
                    $interval = ALSP_Config::get('cron_temp_cleanup_interval_number', 3);
                    if ($difference_in_seconds > (86400 * intval($interval))) {
                        unlink($folder . '/' . $file);
                        error_log("Removed: $file\n");
                    }
                }
            }
        }
    }

    public function override_template($template, $name)
    {
        if ($name === 'new_ticket' || $name === 'guest') {
            $assets = ALSP_Frontend_Assets::get_instance();
            $attachments = $assets->get_template('pro/ticket/new_ticket_attachments');
            $template = str_replace('<!-- data-alsp:attachments -->', $attachments, $template);
        } else if ($name === 'ticket/message') {
            $assets = ALSP_Frontend_Assets::get_instance();
            $message_attachment = $assets->get_template('pro/ticket/message_attachments');
            $template = preg_replace('#(</div>\s*</div>\s*)$#', "{$message_attachment}$1", $template);
        } else if ($name === 'ticket/reply') {
            $assets = ALSP_Frontend_Assets::get_instance();
            $attachments = $assets->get_template('pro/ticket/reply_attachments');
            $attachment_button = $assets->get_template('pro/ticket/reply_attachment_button');
            $template = str_replace('<!-- data-alsp:attachments -->', $attachments, $template);
            $template = str_replace('<!-- data-alsp:attachment_button -->', $attachment_button, $template);
        }

        return $template;
    }

    public function insert_template($template)
    {
        if ('ticket/message/1' === $template) {
            $assets = ALSP_Frontend_Assets::get_instance();
            echo $assets->get_template('pro/ticket/message_attachments');
        }
    }
}

ALSP_Attachments::get_instance();

// EOF
