<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OEdoUE4vK0p3cG5RWmtXOFJnWU1RVUMvSWM5bEIrbEx3ZVprc1JkdGM0dTlaWUdwT1g4L3NDdUthbEZ0RGREZ2h0TmpQWHVUU2tyUjl0NmhJbk9QUWszTE11aDA2ZDhUMWR5QmRzMXVpYThkK1NiUk56N1B3ck9RTjRUSUx3cEswPQ==*/

class ALSP_Model_Ticket_Attachment extends ALSP_Model_Single
{
    public $author;
    public $ticket;
    public $name;
    public $file_path;
    public $source_id;
    public $source;
    public $created;

    public function __construct($id = NULL)
    {
        parent::__construct($id);
    }

    public static function config_table()
    {
        global $wpdb;

        return $wpdb->prefix . 'alsp_ticket_attachments';
    }

    public function get_item($return = ARRAY_A)
    {
        if (is_numeric($this->id) && $this->id > 0) {
            $item = parent::get_item($return);
            if ($item['author']) {
                $author = ALSP_User::get_instance($item['author']);
                $author = $author->get_details();
            } else {
                $author = [
                    'id' => $item['author'],
                    'display_name' => 'System',
                    'user_avatar' => get_avatar_url($item['author']),
                ];
            }
            $item['author'] = $author;
            $item['created_absolute'] = ALSP_Utility_String::date_absolute($item['created']);
            $item['created_relative'] = ALSP_Utility_String::date_relative($item['created']);

            return $item;
        }
    }


    public function create_item($ticket, $author, $name, $file_path, $source_id, $source = 'reply')
    {
        // create the ALSP department
        global $wpdb;

        $now = current_time('mysql', TRUE);
        $data = [
            'ticket' => $ticket,
            'author' => $author,
            'name' => $name,
            'file_path' => $file_path,
            'source_id' => $source_id,
            'source' => $source,
            'created' => $now,
        ];
        $res = $wpdb->insert($this->config_table(), $data);

        $lastid = FALSE;
        if ($res) {
            $lastid = $wpdb->insert_id;
        }

        return $lastid;
    }

    public function soft_delete()
    {
        global $wpdb;

        $data = [
            'is_deleted' => 1,
        ];
        $result = $wpdb->update($this->config_table(), $data, ['id' => $this->id]);

        return $result;
    }

    /**
     * Get all attachment by ticket_id
     *
     * @param int $ticket_id Ticket ID
     *
     * @return array|bool
     */
    public function get_by_ticket($ticket_id)
    {
        global $wpdb;

        $sql = $wpdb->prepare('SELECT * FROM ' . $this->config_table() . ' WHERE ticket=%d', $ticket_id);
        $result = $this->wpdb->get_results($sql, ARRAY_A);

        return $result;
    }

    /**
     * Delete ticket attachment by ticket_id
     * @param int $ticket_id Ticket ID
     *
     * @return int|bool Number of rows affected/selected for all other queries. Boolean false on error.
     */
    public function delete_by_ticket($ticket_id)
    {
        global $wpdb;

        $sql = $wpdb->prepare('DELETE FROM ' . $this->config_table() . ' WHERE ticket=%d', $ticket_id);
        $result = $wpdb->query($sql);

        return $result;
    }
}

// EOF
