<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OEdoUE4vK0p3cG5RWmtXOFJnWU1RVVB1UnBZVVFNdXRUS0VaZ2FJbUc3NFRoeTY3VDFEZTMrRWRlRkF3QzFzV2ptZHhlZnRyS3RpcVZzSExNNkhjV0p1UGlUcCtoNDZGVTFuU1pLTUdjbGlGcUZHYkdlM3pLOE1PYWgyV2p6RzRrPQ==*/

class ALSP_Model_Ticket_Attachments extends ALSP_Model_List
{

    public function __construct(WP_REST_Request $request)
    {
        parent::__construct($request);
    }

    public function read()
    {
        $this->init('ticket_attachment');
        $this->query_start();

        $where = [];
        // filter by ticket number
        if ($this->request['message_id']) {
            if (is_numeric($this->request['message_id']) && $this->request['message_id'] > 0) {
                // Specific collection
                $where['message_id'] = $this->wpdb->prepare('source_id=%d', $this->request['message_id']);
            }
        }

        $where = apply_filters('alsp_ticket_attachment_query_where', $where, $this->request);

        $this->query_where($where);

        $order = [
            'created' => 'ASC',
        ];
        $this->query_order($order);

        $ids = [];
        $result = $this->wpdb->get_results($this->query, ARRAY_A);
        foreach ($result as $id) {
            $ids[] = $id['id'];
        }

        $data = $this->prepare_result($ids);

        return $data;
    }

    /**
     * Get list ticket attachment by user.
     * Currently, it is used for export personal data in GDPR
     * 
     * @param $user_id user id of ticket author
     * @return array array of list ticket ID
     */
    public static function get_ticket_attachments_by_user($user_id)
    {
        global $wpdb;

        $sql = $wpdb->prepare("SELECT id FROM {$wpdb->prefix}alsp_ticket_attachments WHERE author = %d and is_deleted = false ORDER BY id asc", $user_id);
        $result = $wpdb->get_results($sql);

        $ids = [];
        foreach ($result as $row) {
            $ids[] = $row->id;
        }

        return $ids;
    }

    /**
     * Get list inactive tickets attachment.
     * Currently, it is used for auto closed ticket
     * 
     * @return array array of list ticket ID
     */
    public static function get_inactive_tickets_attachments()
    {
        global $wpdb;

        $interval = ALSP_Config::get('attachment_delete_file_interval_day_number', 30);
        $sql = $wpdb->prepare("SELECT ta.id FROM wp_alsp_tickets t JOIN wp_alsp_ticket_messages tm ON t.id = tm.ticket JOIN wp_alsp_ticket_attachments ta ON tm.id = ta.source_id WHERE ta.is_deleted = 0 and t.status ='closed' and t.updated < now() - INTERVAL %s day", $interval);

        $result = $wpdb->get_results($sql);

        $ids = [];
        foreach ($result as $row) {
            $ids[] = $row->id;
        }

        return $ids;
    }
}

// EOF