<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OGdVcDdxdFhBUjZTSnRaSS81cGl0bUxmL1E3UEdXTmxDQVM5UTFXWDdVY1pkdHEvWU5xenAvUDA2RUNWbFZwUWYxck14OFh4M09Gazh2eFNWcnFaYlhGalFZNU9jWC9nWTFVb0hXMG5VZDlmSkl2OW54SHJscVN2cWQ4VTl6ME9rPQ==*/

class ALSP_CPT_Integration
{
    private static $instance = NULL;

    public static function get_instance()
    {
        return (NULL == self::$instance) ? self::$instance = new self() : self::$instance;
    }

    private function __construct()
    {
        if(apply_filters('awecloud_is_client', FALSE)) { return; }

        // Reset cache when config is updated
        add_action('alsp_config_set', function ($name, $value) {
            if (stristr($name, 'docs_cpt_')) {
                ALSP_Mayfly::del('all_enabled_cpts');
                ALSP_Mayfly::del_like('alsp_suggestions%');
            }
        }, 10, 2);

        // Generic integrations menu
        add_filter('alsp_config_menu', function ($menu) {
            $menu['integrations'] = [
                'id' => 'integrations',
                'label' => __('Integrations', 'awedesk'),
                'icon' => 'alsp-i-code-compare',
            ];

            return $menu;
        }, 30);

        // Options descriptions
        add_filter('alsp_config_description', function ($desc) {

            $desc = array_merge($desc, [
                'docs_enabled' => __('Post types selected below will be automatically searched to suggest solutions to tickets.', 'awedesk'),
            ]);
            return $desc;
        }, 30);

        // Build configuration based on registered CPTs
        add_filter('alsp_config', function ($config, $position) {

            if ($position !== 'integrations-right') {
                return $config;
            }

            $config['docs'] = [
                'id' => 'docs',
                'title' => __('Suggested content', 'awedesk'),
                'icon' => 'alsp-i-folder-tree',
                'position' => 'integrations-right',
                'items' => [
                    'docs_enabled' => ALSP_Config_Panel::admin_item('docs_enabled'),
                    'docs_cpt_results_number' => ALSP_Config_Panel::admin_item('docs_cpt_results_number', 'docs_enabled', __('Results', 'awedesk'), ['after' => __('per post type', 'awedesk')]),
                    'docs_cpt_words_number' => ALSP_Config_Panel::admin_item('docs_cpt_words_number', 'docs_enabled', __('Preview', 'awedesk'), ['after' => __('words', 'awedesk')]),
                ],
            ];

            // Get all WP post types
            $post_types = get_post_types(['public' => TRUE], 'objects');

            foreach ($post_types as $post_type) {

                // Skip PeepSo and Attachment
                if (stristr($post_type->name, 'peepso')) {
                    continue;
                }
                if ('attachment' == $post_type->name) {
                    continue;
                }

                $separator = 'docs_cpt_' . $post_type->name . '_separator';
                $enabled = 'docs_cpt_' . $post_type->name . '_enabled';
                $label = 'docs_cpt_' . $post_type->name . '_label';
                $position = 'docs_cpt_' . $post_type->name . '_order_number';

                $config['docs']['items'][$separator] = ALSP_Config_Panel::admin_item($separator, 'docs_enabled', $post_type->label);
                $config['docs']['items'][$enabled] = ALSP_Config_Panel::admin_item($enabled, 'docs_enabled');
                $config['docs']['items'][$label] = ALSP_Config_Panel::admin_item($label, $enabled);
                $config['docs']['items'][$position] = ALSP_Config_Panel::admin_item($position, $enabled, __('Order', 'awedesk'), ['default_value' => 0]);
            }

            return $config;
        }, 30, 2);
    }

    // Get all CPTS enabled in ALSP config, ordered by position
    public function all_enabled($ensure = FALSE)
    {
        $enabled_post_types = ALSP_Mayfly::get('all_enabled_cpts');

        // Refetch all enabled cpts if mayfly is null or is empty
        if (NULL === $enabled_post_types || empty($enabled_post_types)) {

            $post_types = get_post_types(['public' => TRUE], 'objects');

            $enabled_post_types = [];

            foreach ($post_types as $post_type) {
                $cfg_enabled = 'docs_cpt_' . $post_type->name . '_enabled';
                $cfg_position = 'docs_cpt_' . $post_type->name . '_order_number';
                $cfg_label = 'docs_cpt_' . $post_type->name . '_label';
                // Check if this CPT is enabled and still registered
                if (ALSP_Config::get($cfg_enabled) && post_type_exists($post_type->name)) {
                    $enabled_post_types[$post_type->name] = [
                        'order' => (int)ALSP_Config::get($cfg_position, 0, TRUE),
                        'label' => ALSP_Config::get($cfg_label, $post_type->label, TRUE),
                    ];
                }
            }

            uasort($enabled_post_types, function ($a, $b) {
                return $a['order'] - $b['order'];
            });

            ALSP_Mayfly::set('all_enabled_cpts', $enabled_post_types, 60);
        }

        if ($ensure) {
            foreach ($enabled_post_types as $cpt => $data) {
                if (!post_type_exists($cpt)) {
                    unset($enabled_post_types[$cpt]);
                }
            }
        }

        return $enabled_post_types;
    }
}

add_action('init', function() {
    if (apply_filters('alsp_is_pro', FALSE)) {
        ALSP_CPT_Integration::get_instance();
    }
});

// EOF
