<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OWswakhqUVdCeHN0czFGRkZRaWRMUjJ3Y3piZVpUalNEWS95ZkRjR09lbGFTVVE1bWlnTUMxQ0hiOU5GdXNobTdrUUd1Y2E4TzRVOFFCRXZlK0hpblJjdDBNWW9ZZlBtRUpWalFlWjVsKzNBPT0=*/

class ALSP_Slack
{
    private static $instance;

    private $message_max_lines = 5;

    private $ticket_model = NULL;
    private $message_model = NULL;
    private $templates = [];

    public static function instance() {
        return self::$instance ?? self::$instance = new self();
    }

    private function __construct()
    {
        add_action('init', function(){
            if(!ALSP_Config::get('slack_enabled') || !strlen(ALSP_Config::get('slack_webhook_label'))) {
                return;
            }

            // New Ticket
            add_action('alsp_ticket_new', function ($ticket_id, $message_id) {
                $this->process($ticket_id, $message_id, 'ticket_new');
            }, 1, 2);

            // Ticket Reply
            add_action('alsp_ticket_reply', function ($ticket_id, $message_id) {
                $this->process($ticket_id, $message_id, 'ticket_reply');
            }, 1, 2);

            // Ticket reassigned
            add_action('alsp_ticket_assignee_changed', function ($ticket_id, $agent_id) {
                $this->process($ticket_id, NULL, 'agent_changed');
            },1,2);

            // Ticket closed
            add_action('alsp_ticket_closed', function ($ticket_id, $actor = NULL) {
                $this->process($ticket_id, NULL, 'ticket_closed', $actor);
            },1,2);

            // Ticket reopened
            add_action('alsp_ticket_reopened', function ($ticket_id) {
                $this->process($ticket_id, NULL, 'ticket_reopened');
            });

            // Department changed
            add_action('alsp_ticket_department_changed', function ($ticket_id, $department_id) {
                $this->process($ticket_id, NULL, 'department_changed');
            },1,2);
        });



    }

    private function get_template($template)
    {
        $result = ALSP_Config::get('slack_event_'.$template.'_template');

        return strlen($result) ? $result : "Template not found: *".str_replace('_', ' ', $template).'*';
    }

    private function process($ticket_id, $message_id, $template, $actor_id = NULL) {

        if(!ALSP_Config::get('slack_event_'.$template.'_enabled')) {
            return;
        }

        $template = $this->get_template($template);

        $actor = NULL;
        $text = '';

        $ticket = new ALSP_Model_Ticket($ticket_id);

        if(NULL !== $message_id) {
            $message = new ALSP_Model_Ticket_Message($message_id);
            // In case of new tickets and replies the actor is the person who wrote the message
            $actor = ALSP_User::get_instance($message->author);
            $text = strip_tags($message->message);
            $text = $this->blockquote($text);
        }

        if(is_numeric($actor_id)) {
            // If actor ID was manually passed
            $actor = ALSP_User::get_instance($actor_id);
        } elseif(NULL == $actor && get_current_user_id()) {
            // Default to current user
            $actor = ALSP_User::get_instance();
        }


        $department = $ticket->department ? (new ALSP_Model_Department($ticket->department)) : NULL;
        $client = ALSP_User::get_instance($ticket->client);
        $agent = $ticket->agent ? ALSP_User::get_instance($ticket->agent) : NULL;

        $actor_name = __('Unknown','awedesk');
        $actor_role = ($actor instanceof ALSP_User && !$actor->is_client()) ? __('Agent', 'awedesk') : __('Client', 'awedesk');

        if($actor instanceof ALSP_User && $actor->wp_user instanceof WP_User) {
            if($actor->is_staff()) {
                $actor_name = "<".ALSP::get_alsp_agent_uri()."#tickets/open,agent={$actor->wp_user->ID}|{$actor->wp_user->display_name}>";
            } else {
                $actor_name = "<".ALSP::get_alsp_agent_uri()."#tickets/open,client={$actor->wp_user->ID}|{$actor->wp_user->display_name}>";
            }
        }

        // If action taken by system (for example autoclose)
        if(FALSE === $actor_id) {
            $actor_role = __('System', 'awedesk');
            $actor_name = __('System', 'awedesk');
        }

        $statuses = ALSP_Model_Ticket_Status::list();
        $status = __('Unknown', 'awedesk');
        foreach($statuses as $s) {
            if($s['status'] == $ticket->status) {
                $status = $s['label'];
                break;
            }
        }

        $tokens = [
            'ID'            => "<".ALSP::get_alsp_ticket_uri($ticket->client, $ticket_id, $message_id)."|#{$ticket_id}>",
            'TITLE'          => "<".ALSP::get_alsp_ticket_uri($ticket->client, $ticket_id, $message_id)."|{$ticket->subject}>",
            'MESSAGE'       => $text,

            'ACTOR'         => $actor_name,
            'ROLE'    => $actor_role,
            'CLIENT'        => ($client instanceof ALSP_User && $client->wp_user instanceof WP_User)
                ? "<".ALSP::get_alsp_agent_uri()."#tickets/open,client={$client->wp_user->ID}|{$client->wp_user->display_name}>"
                : __('Unknown','awedesk'),
            'AGENT'         => ($agent instanceof ALSP_User && $agent->wp_user instanceof WP_User)
                ? "<".ALSP::get_alsp_agent_uri()."#tickets/open,agent={$agent->wp_user->ID}|{$agent->wp_user->display_name}>"
                : __('Unassigned', 'awedesk'),

            'DEPARTMENT'    => ($department instanceof ALSP_Model_Department)
                ? "<".ALSP::get_alsp_agent_uri()."#tickets/open,dept={$department->id}|{$department->name}>"
                : __('No department', 'awedesk'),
            'STATUS'        => $status,
        ];

        foreach($tokens as $key => $value) {
            $template = str_replace("[$key]", $value, $template);
        }

        $icons = [
            'department' => 'open_file_folder',
            'agent' => 'ballot_box_with_check',
            'client' => 'bust_in_silhouette',
            'status' => 'information_source',
        ];

        foreach($icons as $key => $value) {
            $template = str_replace(":$key:", ":$value:", $template);
        }

        $payload = $template;

        $data = [
            'text' => $payload
        ];

        $data = json_encode($data);

        // CURL start

        $curl_fields = [
            CURLOPT_POST => TRUE,
            CURLOPT_POSTFIELDS => $data,
            CURLOPT_RETURNTRANSFER => TRUE,
        ];

        $curl = curl_init(ALSP_Config::get('slack_webhook_label'));

        foreach ($curl_fields as $k => $v) {
            curl_setopt($curl, $k, $v);
        }

        $result = curl_exec($curl);
        $error = curl_error($curl);

        curl_close($curl);

        if ('ok' !== $result) {
            $error = strlen($error) ? $error : $result;
            //die("Error $error");
        }
    }



    private function blockquote($text) {

        if(stristr($text,PHP_EOL)) {
            $text = explode(PHP_EOL, $text);
            $result = [];
            $i=0;
            foreach ($text as $line) {
                if($i++ >= $this->message_max_lines) {
                    break;
                }

                $result[] = '>'.$line;
            }
            $text = implode(PHP_EOL, $result);
        } else {
            $text = '>'.$text;
        }

        return $text;
    }
}

ALSP_Slack::instance();
