<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2KzdOZWw1bXZKalgxMm4wTG1oaEFpc0JOT1ZkenNXR2pETVhkVnNXUElzUVdtKzdCRnRnWFIrVDRFemE0UlIraWl1WE1tYUZsaGtoWVMrejBnZWN5VmNPVVlHOHh6QmYzV29SbWlic0hHMkRXUDRqc1Q2UlZ1VnJnZGMyTVpXOXVjPQ==*/

class ALSP_WooCommerce_Integrations
{
    private static $instance = NULL;

    private static $slug;

    public static function get_instance()
    {
        return (NULL == self::$instance) ? self::$instance = new self() : self::$instance;
    }

    private function slug()
    {
        return self::$slug ?? self::$slug = ALSP_Config::get('woo_profile_slug', 'support', 1);
    }

    private function maybe_flush($force = FALSE)
    {
        $slug = $this->slug();
        if ($force || $slug != ALSP_Config::get('woo_slug_cache')) {
            flush_rewrite_rules(TRUE);
            //new ALSP_Debug("ALSP flush_rewrite_rules $slug => " . ALSP_Config::get('woo_slug_cache'));
            ALSP_Config::set('woo_slug_cache', $slug);
        }
    }

    private function __construct()
    {
        add_action('init', array(&$this, 'init'));

        if (ALSP_Config::get('woo_profile_enabled')) {
            add_action('init', function () {
                add_rewrite_endpoint($this->slug(), EP_PAGES);
                $this->maybe_flush();
            });

            add_action('alsp_config_set', function ($name, $value) {
                // Always make sure to flush whenever any changes in our slug or page client
                if (
                    $name == 'woo_profile_slug' ||
                    ($name == 'support_page_client_select' && $value == 'woo')
                ) {
                    $this->maybe_flush(TRUE);
                }
            }, 10, 2);

            add_action('woocommerce_account_' . $this->slug() . '_endpoint', function () {
                echo do_shortcode('[awedesk]');
            });
        }
    }

    public function init()
    {
        add_filter('alsp_config_menu', function ($menu) {
            if (apply_filters('awecloud_is_client', FALSE)) {
                return $menu;
            }
            $menu['integrations'] = [
                'id' => 'integrations',
                'label' => __('Integrations', 'awedesk'),
                'icon' => 'alsp-i-code-compare',
            ];

            return $menu;
        }, 30);

        add_filter('alsp_config', function ($config, $position) {
            if (apply_filters('awecloud_is_client', FALSE)) {
                return $config;
            }

            if ($position !== 'integrations-left') {
                return $config;
            }

            $config['integrations_woo'] = [
                'id' => 'integrations_woo',
                'title' => 'WooCommerce',
                'icon' => 'alsp-i-shopping-cart',
                'position' => 'integrations-left',
                'items' => [
                    'woo_enabled_separator' => ALSP_Config_Panel::admin_item('woo_enabled_separator', FALSE, 'Tickets'),
                    'woo_integration_enabled' => ALSP_Config_Panel::admin_item('woo_integration_enabled'),

                    'woo_profile_separator' => ALSP_Config_Panel::admin_item('woo_profile_separator', FALSE, 'Account'),
                    'woo_profile_enabled' => ALSP_Config_Panel::admin_item('woo_profile_enabled'),
                    'woo_profile_label' => ALSP_Config_Panel::admin_item('woo_profile_label', 'woo_profile_enabled'),
                    'woo_profile_slug' => ALSP_Config_Panel::admin_item('woo_profile_slug', 'woo_profile_enabled'),
                    'woo_profile_position' => ALSP_Config_Panel::admin_item('woo_profile_position', 'woo_profile_enabled'),
                ],
            ];
            return $config;
        }, 30, 2);


        if (!class_exists('WooCommerce')) {
            return;
        }


        if (!apply_filters('alsp_is_pro', FALSE)) {
            return;
        }

        if (class_exists('ALSP_Pages')) {
            if (ALSP_Config::get('woo_profile_enabled', 0)) {
                add_filter('alsp_config_pages', function ($pages) {
                    $pages['woo'] = ['id' => 'woo', 'label' => 'WooCommerce Account'];
                    return $pages;
                });
            } else {
                ALSP_Pages::get_instance()->find_pages();
            }
        }

        if (ALSP_Config::get('woo_profile_enabled', 0)) {
            add_filter('woocommerce_account_menu_items', function ($items) {
                $additional_config = [$this->slug() => ALSP_Config::get('woo_profile_label', __('Support', 'awedesk'), 1)];

                // Two notes here on the -2
                // Slicing function was designed to put new stuff AFTER certain position, thats -1
                // Array are numbered from 0, so we need to do another -1
                $slice = ALSP_Config::get('woo_profile_position', 10, 1) - 2;

                // If user goes beyond -1, we will loop over to the end of the array, so just assume this means "top"
                if ($slice < -1) {
                    $slice = -1;
                }

                if (count($items) > $slice) {
                    $items = alsp_config_slice($items, $additional_config, $slice, TRUE);
                } else {
                    $items = $items + $additional_config;
                }

                $this->maybe_flush();

                return $items;
            });
        }

        if (ALSP_Config::get('woo_integration_enabled', 0)) {
            add_action('alsp_template_insert', [&$this, 'insert_template']);
        }

        $slug = $this->slug();
        add_filter('alsp_frontend_data', function ($data) use ($slug) {
            $data['page_woo'] = get_permalink(wc_get_page_id('myaccount')) . $slug . '/';
            $data['is_wc_page'] = home_url($_SERVER['REQUEST_URI']) === $data['page_woo'];

            return $data;
        });

        add_filter('alsp_ticket_uri', function ($url, $id, $root_page_id, $user_id) use ($slug) {
            if ($root_page_id == 'woo') {
                $url = get_permalink(wc_get_page_id('myaccount')) . $slug . "/#ticket/$id";
            }

            return $url;
        }, 10, 4);

        add_filter('query_vars', function ($vars) {
            $vars[] = $this->slug();
            $this->maybe_flush();
            return $vars;
        });

        add_filter('woocommerce_rest_prepare_customer', function ($response, $user_data, $request) {
            $customer = new WC_Customer($user_data->ID);
            $response->data['alsp'] = [
                'total_spent_formatted' => wc_price($customer->get_total_spent()),

                'currency' => get_woocommerce_currency_symbol(),
                'total_spent' => $customer->get_total_spent(),
                'total_purchases' => $customer->get_order_count(),
            ];

            return $response;
        }, 10, 3);

        add_filter('woocommerce_rest_prepare_shop_order_object', function ($response, $post, $request) {
            $date_format = ALSP_Utility_WP::get_option('date_format');

            $response->data['alsp'] = [
                'date_created_formatted' => wp_date($date_format, strtotime($response->data['date_created']))
            ];

            return $response;
        }, 10, 3);

        add_filter('woocommerce_rest_check_permissions', function ($permission, $context, $object_id, $post_type) {
            // Staffs (agents, managers, and admins) should be able to read purchase history.
            if ($context === 'read' && in_array($post_type, ['user', 'shop_order'])) {
                $user = ALSP_User::get_instance();
                if ($user->is_staff()) {
                    $permission = true;
                }
            }

            return $permission;
        }, 10, 4);
    }

    public function insert_template($template)
    {
        if ('ticket/sidebar/1' === $template) {
            $assets = ALSP_Frontend_Assets::get_instance();
            echo $assets->get_template('pro/ticket/sidebar_woo_summary');
        } else if ('ticket/sidebar/2' === $template) {
            $assets = ALSP_Frontend_Assets::get_instance();
            echo $assets->get_template('pro/ticket/sidebar_woo_detail');
        }
    }
}

ALSP_WooCommerce_Integrations::get_instance();
// EOF
