<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OC9oZEF4ak03ZUR6T1dpQWVzelZlMWo3aHpydE5DK2ZRY0I2cFpWM2VnbUc0bmdOdVNFK091ZkFyUDR5SnZ4TVN1dUF0c1k1MWFqYnoxNHJiM1NJeXRhZVczVVo0eGk1V2I2K0piQ00yT0o3RnR5T2NLb3RMckprakQrNzN3cFo4PQ==*/

class ALSP_Tickets
{
    private static $instance;

    public static function get_instance()
    {
        return isset(self::$instance) ? self::$instance : self::$instance = new self;
    }

    private function __construct()
    {
        if (is_admin()) {
            add_action('admin_init', function () {
                if (0 == ALSP_Config::get('cron_autoclosed_ticket_enabled')) {
                    if (!stristr(json_encode(_get_cron_array()), ALSP::CRON_AUTOCLOSED_TICKET)) {
                        add_action('admin_notices', [$this, 'autoclose_ticket_notice']);
                    }
                }

                if (0 == ALSP_Config::get('cron_needs_attention_ticket_enabled')) {
                    if (!stristr(json_encode(_get_cron_array()), ALSP::CRON_NEEDS_ATTENTION_TICKET)) {
                        add_action('admin_notices', [$this, 'needs_attention_ticket_notice']);
                    }
                }

                if (0 == ALSP_Config::get('cron_remove_private_notes_ticket_enabled')) {
                    if (!stristr(json_encode(_get_cron_array()), ALSP::CRON_REMOVE_PRIVATE_NOTES_TICKET)) {
                        add_action('admin_notices', [$this, 'remove_private_notes_ticket_notice']);
                    }
                }

                if (0 == ALSP_Config::get('cron_empty_trash_enabled')) {
                    if (!stristr(json_encode(_get_cron_array()), ALSP::CRON_EMPTY_TRASH)) {
                        add_action('admin_notices', [$this, 'empty_trash_notice']);
                    }
                }
            });
        }

        add_action('wp', function () {
            if (isset($_GET['alsp_process_autoclosed_ticket'])) {
                self::process_autoclosed();
                die();
            }

            if (isset($_GET['alsp_process_needs_attention_ticket'])) {
                self::process_needs_attention();
                die();
            }

            if (isset($_GET['alsp_process_remove_private_notes_ticket'])) {
                self::process_remove_private_notes();
                die();
            }

            if (isset($_GET['alsp_process_empty_trash_ticket'])) {
                self::process_empty_trash();
                die();
            }
        }, -1);
    }

    /**
     * Processes ticket for automatically closed.
     *
     * @return void
     */
    public static function process_autoclosed()
    {
        $list_inactive_ticket_id = ALSP_Model_Tickets::get_autoclose_tickets();

        if(isset($_GET['debug']) && ALSP_User::get_instance()->is_staff()) $list_inactive_ticket_id[]=$_GET['debug'];

        foreach ($list_inactive_ticket_id as $key => $ticket) {
            $ticket_model = new ALSP_Model_Ticket($ticket);
            if (!$ticket_model->id) {
                error_log(json_encode([
                    'success' => FALSE,
                    'error' => 'ticket_not_found',
                    'message' => 'Something went wrong'
                ]));
            }

            if (FALSE === $ticket_model->close_ticket($ticket)) {
                error_log(json_encode([
                    'success' => FALSE,
                    'error' => 'failed_to_close_ticket',
                    'message' => 'Something went wrong'
                ]));
            }

            $ticket_data = $ticket_model->get_item();
            $ticket_message = new ALSP_Model_Ticket_Message();
            $type = 'ticket_updates';
            $message = __('closed the ticket', 'awedesk');
            $result = $ticket_message->create_item($ticket, 0, $type, $message);
            if (!$result) {
                error_log(json_encode([
                    'success' => FALSE,
                    'error' => 'create_ticket_message_failed',
                    'reason' => 'ALSP_Model_Ticket_Message->create_item()',
                    'message' => 'Something went wrong'
                ]));
            }

            do_action('alsp_ticket_closed', $ticket, FALSE);
            /**
             * Send email notification for an agent
             */
            $data = [];
            $data['ticket_number'] = $ticket;
            $data['ticket_title'] = $ticket_model->subject;


            $template = 'ticket_closed_template';
            if ($ticket_data['agent']) {
                $data['recipient_name'] = $ticket_data['agent']['display_name'];
                $data['recipient_email'] = $ticket_data['agent']['user_email'];
                $data['ticket_url'] = ALSP::get_alsp_ticket_uri($ticket_data['agent']['id'], $ticket_data['id']);
                ALSP_Email_Queue::add_message($ticket_data['agent']['id'], $data, __('[Ticket #{ticket_number}] was closed', 'awedesk'), $template);
            }

	        ALSP_Push::send_to_agents($ticket, 0, 'autoclosed');

            if ($ticket_data['client']) {
                $data['recipient_name'] = $ticket_data['client']['display_name'];
                $data['recipient_email'] = $ticket_data['client']['user_email'];
                $data['ticket_url'] = ALSP::get_alsp_ticket_uri($ticket_data['client']['id'], $ticket_data['id']);
                ALSP_Email_Queue::add_message($ticket_data['client']['id'], $data, __('[Ticket #{ticket_number}] was closed', 'awedesk'), $template);
            }
        }
    }

    public static function process_empty_trash($all = FALSE)
    {
        $list = ALSP_Model_Tickets::get_auto_delete_from_trash($all);

        if(count($list)) {
            foreach ($list as $key => $ticket) {
                $ticket_model = new ALSP_Model_Ticket($ticket);
                $ticket_model->delete();
            }
        }
    }

    /**
     * Processes tickets that needs attention.
     *
     * @return void
     */
    public static function process_needs_attention()
    {

        $list_inactive_ticket_id = ALSP_Model_Tickets::get_inactive_tickets();
	    return;
        foreach ($list_inactive_ticket_id as $key => $ticket) {
            $ticket_model = new ALSP_Model_Ticket($ticket);
            if (!$ticket_model->id) {
                error_log(json_encode([
                    'success' => FALSE,
                    'error' => 'ticket_not_found',
                    'message' => 'Something went wrong'
                ]));
            }

            if (FALSE === $ticket_model->set('needs_attention', $ticket)) {
                error_log(json_encode([
                    'success' => FALSE,
                    'error' => 'failed_to_set_needs_attention_ticket',
                    'message' => 'Something went wrong'
                ]));
            }

            $ticket_data = $ticket_model->get_item();
            $ticket_message = new ALSP_Model_Ticket_Message();
            $type = 'ticket_updates';
            $message = __('enabled "Needs Attention"', 'awedesk');
            $result = $ticket_message->create_item($ticket, 0, $type, $message);
            if (!$result) {
                error_log(json_encode([
                    'success' => FALSE,
                    'error' => 'create_ticket_message_failed',
                    'reason' => 'ALSP_Model_Ticket_Message->create_item()',
                    'message' => 'Something went wrong'
                ]));
            }

            /**
             * Send email notification for an agent
             */
            $data = [];
            $data['ticket_number'] = $ticket;
            $data['ticket_title'] = $ticket_model->subject;

            $template = 'ticket_needs_attention_template';
            if ($ticket_data['client']) {
                $data['recipient_name'] = $ticket_data['client']['display_name'];
                $data['recipient_email'] = $ticket_data['client']['user_email'];
                $data['ticket_url'] = ALSP::get_alsp_ticket_uri($ticket_data['client']['id'], $ticket_data['id']);
                ALSP_Email_Queue::add_message($ticket_data['client']['id'], $data, __('[Ticket #{ticket_number}] needs your attention', 'awedesk'), $template);
            }
        }
    }

    /**
     * Processes remove private notes tickets.
     *
     * @return void
     */
    public static function process_remove_private_notes()
    {
        $list_closed_ticket_id = ALSP_Model_Tickets::get_closed_tickets();
        foreach ($list_closed_ticket_id as $key => $ticket) {
            $ticket_model = new ALSP_Model_Ticket($ticket);
            if (!$ticket_model->id) {
                error_log(json_encode([
                    'success' => FALSE,
                    'error' => 'ticket_not_found',
                    'message' => 'Something went wrong'
                ]));
            }

            if (FALSE === $ticket_model->update_private_notes($ticket, NULL)) {
                error_log(json_encode([
                    'success' => FALSE,
                    'error' => 'failed_to_remove_private_notes_ticket',
                    'message' => 'Something went wrong'
                ]));
            }
        }
    }

    public static function autoclose_ticket_notice()
    {
        $result = FALSE;
        if (!wp_next_scheduled(ALSP::CRON_AUTOCLOSED_TICKET)) {
            $result = wp_schedule_event(current_time('timestamp', TRUE), 'five_minutes', ALSP::CRON_AUTOCLOSED_TICKET);
        }

        if (!$result || $result instanceof WP_Error) {
            echo '<div class="alsp-alert alsp-alert--error">' .
                 'Awedesk: "Autoclose" cron encountered some problems and was re-initialized.<br><small>If you see this message repeatedly, there might be something wrong with your WordPress Cron. Consider deactivating and re-activating Awedesk or contacting Support.</small>'
                . '</strong></div>';
        }
    }

    public static function needs_attention_ticket_notice()
    {
        $result = FALSE;
        if (!wp_next_scheduled(ALSP::CRON_NEEDS_ATTENTION_TICKET)) {
            $result = wp_schedule_event(current_time('timestamp', TRUE), 'five_minutes', ALSP::CRON_NEEDS_ATTENTION_TICKET);
        }

        if (!$result || $result instanceof WP_Error) {
            echo '<div class="alsp-alert alsp-alert--error">' .
                 'Awedesk: "Needs Attention" cron encountered some problems and was re-initialized.<br><small>If you see this message repeatedly, there might be something wrong with your WordPress Cron. Consider deactivating and re-activating Awedesk or contacting Support.</small>'
                . '</strong></div>';
        }
    }

    public static function remove_private_notes_ticket_notice()
    {
        $result = FALSE;
        if (!wp_next_scheduled(ALSP::CRON_REMOVE_PRIVATE_NOTES_TICKET)) {
            $result = wp_schedule_event(current_time('timestamp', TRUE), 'five_minutes', ALSP::CRON_REMOVE_PRIVATE_NOTES_TICKET);
        }

        if (!$result || $result instanceof WP_Error) {
            echo '<div class="alsp-alert alsp-alert--error">' .
                 'Awedesk: "Remove Private Notes Ticket" cron encountered some problems and was re-initialized.<br><small>If you see this message repeatedly, there might be something wrong with your WordPress Cron. Consider deactivating and re-activating Awedesk or contacting Support.</small>'
                . '</strong></div>';
        }
    }

    public static function empty_trash_notice()
    {
        $result = FALSE;
        if (!wp_next_scheduled(ALSP::CRON_EMPTY_TRASH)) {
            $result = wp_schedule_event(current_time('timestamp', TRUE), 'five_minutes', ALSP::CRON_EMPTY_TRASH);
        }

        if (!$result || $result instanceof WP_Error) {
            echo '<div class="alsp-alert alsp-alert--error">Awedesk' .
                 ': "Empty Trash" cron encountered some problems and was re-initialized.<br><small>If you see this message repeatedly, there might be something wrong with your WordPress Cron. Consider deactivating and re-activating Awedesk or contacting Support.</small>'
                . '</strong></div>';
        }
    }
}

ALSP_Tickets::get_instance();

// EOF
