<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OHZzb3gwQ1Jwb1NXZ2U4bVo3OGtDaGhKZXpMdjMrRHgyK05sVkcxVXRjVjB3dC80NFMvRXpPSllwWlM3THN1UkdjUDhKZzA3ZGFkUVNJd1dtRklwVzlWVG9UaHFBekxHTU1PazBpY2ROTE5nPT0=*/

class ALSP_Admin
{

    private static $instance;

    public static function get_instance()
    {
        return isset(self::$instance) ? self::$instance : self::$instance = new self;
    }

    private function __construct()
    {
        add_action('init', [&$this, 'init'], PHP_INT_MAX);
    }

    public function init()
    {
        // check for wp-admin/user.php page and include hooks/classes for user list
        add_action('set_user_role', [&$this, 'after_role_change'], 10, 3);
        add_action('user_register', [&$this, 'after_user_register'], 10, 2);
        add_action('delete_user', [&$this, 'after_delete_user']);

        // Multisite
        if (is_multisite()) {
            add_action('remove_user_from_blog', [&$this, 'after_delete_user'], 10, 1);
            add_action('user_register', [&$this, 'add_user_to_all_sites'], 10, 2);
            add_action('added_existing_user', [&$this, 'added_existing_user'], 10, 2);

            //When a new blog is created, auto-register all unblocked users for it.
            add_action('wpmu_new_blog', [&$this, 'register_existing_user'], 100, 2);
        }
    }

    /**
     * We should change role in our tables based on role changes
     * @param int $id User ID
     * @param string $role The new role.
     * @param array $old_role An array of the user's previous roles.
     *
     * @return void
     */
    public function after_role_change($id, $role, $old_role)
    {
        if (in_array($role, $old_role)) {
            return;
        }

        // set role
        $user = ALSP_User::get_instance($id);
        if (in_array($role, [ALSP_User::ROLE_AGENT, ALSP_User::ROLE_MANAGER])) {
            $user->set_user_role($role);
        } else if (in_array($role, ['alsp_' . ALSP_User::ROLE_AGENT, 'alsp_' . ALSP_User::ROLE_MANAGER])) {
            $user->set_user_role(str_replace('alsp_', '', $role));
        } else if ($role === 'administrator') {
            $user->set_user_role(ALSP_User::ROLE_ADMIN);
        } else {
            $user->set_user_role(ALSP_User::ROLE_CLIENT);
        }
    }

    /**
     * Setup ALSP users when new user was registered.
     * It will set the user role accordingly based on provided role.
     * If new user have role admin, then we need to register to our tables with predefined admin constant.
     * @param int $user_id User ID
     * @param array $userdata The raw array of data passed to wp_insert_user() .
     *
     * @return void
     */
    public function after_user_register($user_id, $userdata)
    {
        $role = isset($userdata['role']) ? $userdata['role'] : 'subscriber';

        // set role
        $user = ALSP_User::get_instance($user_id);
        if (in_array($role, [ALSP_User::ROLE_AGENT, ALSP_User::ROLE_MANAGER])) {
            $user->set_user_role($role);
        } else if (in_array($role, ['alsp_' . ALSP_User::ROLE_AGENT, 'alsp_' . ALSP_User::ROLE_MANAGER])) {
            $user->set_user_role(str_replace('alsp_', '', $role));
        } else if ($role === 'administrator') {
            $user->set_user_role(ALSP_User::ROLE_ADMIN);
        }
    }

    /**
     * Cleanup user data when user deleted.
     *
     * @return void
     */
    public function after_delete_user($user_id)
    {
        global $wpdb;

        $wpdb->query("DELETE FROM {$wpdb->prefix}alsp_tickets where author={$user_id}");
        $wpdb->query("DELETE FROM {$wpdb->prefix}alsp_ticket_messages where author={$user_id}");
        $wpdb->query("DELETE FROM {$wpdb->prefix}alsp_user_preferences where author={$user_id}");
        $wpdb->query("DELETE FROM {$wpdb->prefix}alsp_users where id={$user_id}");
        $wpdb->query("DELETE FROM {$wpdb->prefix}alsp_agent_department where agent={$user_id}");
    }

    public function alsp_user_profile_fields($user)
    {
        if (!current_user_can('edit_user', $user->ID)) {
            return;
        }

        $alsp_user = ALSP_User::get_instance($user->ID);
        if ($alsp_user->get('role') != ALSP_User::ROLE_AGENT) {
            // Do not show if it's not an agent.
            return;
        }

        ?>
        <h2>
            <?php echo __('Agent', 'awedesk'); ?>
        </h2>

        <?php if (apply_filters('alsp_is_pro', FALSE)) { ?>
            <?php

            if ((new ALSP_Signature($user->ID))->can_use_custom()) {
                if (isset($_REQUEST['alsp_signature'])) { ?>
                    <style>
                        #alsp-signature {
                            border: solid 5px #3e56f1;
                            padding: 10px;
                        }
                    </style>
                <?php } ?>
                <div <?php if (isset($_REQUEST['alsp_signature'])) echo 'id="alsp-signature"'; ?>>
                    <a id="alsp_signature"></a>
                    <table class="form-table">
                        <tr>
                            <th><label for="alsp_signature"><?php _e('Signature', 'awedesk'); ?></label></th>
                            <td>
                                <textarea name="alsp_signature" id="alsp_signature" rows="5" cols="30"><?php echo wp_kses_post(get_the_author_meta('alsp_signature', $user->ID)); ?></textarea>
                            </td>
                        </tr>
                    </table>
                </div>
            <?php } ?>
        <?php } ?>

        <table class="form-table" role="presentation">
            <tr class="user-alsp-department-wrap">
                <th scope="row">
                    <?php echo __('Department', 'awedesk'); ?>
                </th>
                <td>
                    <fieldset id="vip-icons" class="scheme-list">
                        <legend class="screen-reader-text"><span>
                                <?php echo __('Department', 'awedesk'); ?>
                            </span></legend>
                        <?php
                        $selectedDept = [];
                        $agent_department = new ALSP_Model_Agent_Department(NULL);
                        $departments = $agent_department->get_by_agent($user->ID);
                        foreach ($departments as $key => $department) {
                            $selectedDept[] = $department['department']['id'];
                        }

                        $request = new WP_REST_Request;
                        $ModelDepartment = new ALSP_Model_Departments($request);
                        foreach ($ModelDepartment->read() as $key => $dept) {
                        ?>
                            <div class="color-option">
                                <input name="alsp-department[]" id="alsp-department_<?php echo $key; ?>" type="checkbox" value="<?php echo $dept['id']; ?>" class="tog" <?php echo (in_array($dept['id'], $selectedDept)) ? 'checked="checked"' : ''; ?>>
                                <label for="vip_icon_<?php echo $key; ?>">
                                    <?php echo $dept['name']; ?>
                                </label>
                            </div>
                        <?php
                        }
                        ?>
                    </fieldset>
                </td>
            </tr>
            </tbody>
        </table>
<?php

    }

    /**
     * Save ALSP user when admin / user do update in edit users page
     * @param int $user_id
     * @param WP_User $old_user_data
     * @param array $userdata
     *
     * @return void
     */
    public function save_alsp_user_profile_fields($user_id, $old_user_data, $userdata)
    {
        if (!current_user_can('edit_user', $user_id)) {
            return;
        }

        $wprole_alsp_agent = 'alsp_' . ALSP_User::ROLE_AGENT;
        $wprole_alsp_manager = 'alsp_' . ALSP_User::ROLE_MANAGER;

        // Cleanup role
        $wpuser = get_user_by('id', $user_id);
        $wpuser->remove_role($wprole_alsp_agent);
        $wpuser->remove_role($wprole_alsp_manager);

        // Add additional role
        $user = ALSP_User::get_instance($user_id);
        $role = $user->get('role');
        if (in_array($role, [ALSP_User::ROLE_AGENT, ALSP_User::ROLE_MANAGER])) {
            $wprole = 'alsp_' . $role;
            $wpuser->add_role($wprole);
        } else if (in_array($role, ['alsp_' . ALSP_User::ROLE_AGENT, 'alsp_' . ALSP_User::ROLE_MANAGER])) {
            $wpuser->add_role($role);
        }

        if (isset($_POST['alsp-department'])) {
            $department = $_POST['alsp-department'] ?? [];
            $agent_department = new ALSP_Model_Agent_Department(NULL);

            $agent_department->delete_by_agent($user_id);
            foreach ($department as $key => $dept) {
                $agent_department->create_item($user_id, $dept);
            }
        }

        return;
    }

    /**
     * Add existing user to site.
     */
    public function added_existing_user($user_id, $result)
    {
        if ($result instanceof WP_Error) {
            return;
        }

        $wpuser = get_user_by('id', $user_id);
        $user = ALSP_User::get_instance($user_id);
        if (in_array('alsp_' . ALSP_User::ROLE_AGENT, $wpuser->roles)) {
            $user->set_user_role(ALSP_User::ROLE_AGENT);
        } else if (in_array('alsp_' . ALSP_User::ROLE_MANAGER, $wpuser->roles)) {
            $user->set_user_role(ALSP_User::ROLE_MANAGER);
        } else if (in_array('administrator', $wpuser->roles)) {
            $user->set_user_role(ALSP_User::ROLE_ADMIN);
        } else {
            $user->set_user_role(ALSP_User::ROLE_CLIENT);
        }
    }

    /**
     * Add user to all site
     */
    public function add_user_to_all_sites($user_id, $userdata)
    {
        global $wpdb;

        // Get list of all site IDs
        $site_ids = $wpdb->get_col("SELECT blog_id FROM $wpdb->blogs");

        // Add user to each site
        foreach ($site_ids as $site_id) {
            // Check if user is already a member of the site
            if (!is_user_member_of_blog($user_id, $site_id)) {
                // Add user to the site
                add_user_to_blog($site_id, $user_id, 'subscriber');
            }
        }
    }

    public function register_existing_user($blog_id, $user_id)
    {
        global $wpdb;

        $users = $wpdb->get_col($wpdb->prepare("SELECT $wpdb->users.ID FROM $wpdb->users ORDER BY %s ASC", 'ID'));
        // register each user to the new website.
        foreach ($users as $user => $uid) {
            if ($user_id == 1) {
                add_user_to_blog($blog_id, $uid, "administrator");
            }
            //auto register a new user for all previous blogs.But will this create duplicates for existing users? Nope. It doesn't!
            else if ($user_id == $uid) {
                //add_user_to_blog( $blogid, $user_id, "administrator" );
                $blogs = $wpdb->get_col($wpdb->prepare("SELECT blog_id FROM $wpdb->blogs WHERE site_id = %d  AND archived = '0'  AND spam = '0' AND deleted = '0' ORDER BY path", $wpdb->siteid));
                foreach ($blogs as $blog => $bid) {
                    //if this is the blog the user just created, then let them be an admin.
                    if ($bid == $blog_id)
                        add_user_to_blog($bid, $user_id, "administrator");
                    else
                        add_user_to_blog($bid, $user_id, "subscriber");
                }
            } else
                add_user_to_blog($blog_id, $uid, "subscriber"); //if user is not an admin and not a new user, then just make them a subscriber.
        }
    }
}


ALSP_Admin::get_instance();

// EOF
