<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2ODh5dE00UkMrbHIyVng2U3hIaHhxTWNwVnJsb1Z1SXVlTjhXdHFMdWEwRVJsVEwvdE1jR3dkRnVaeWJVc3ZEMi81QnI5Zm02V1Y2MkxTV0g2VFZBYklFaE14Zjh6QnBGUHg0dnY2OFFxVEh3PT0=*/

class ALSP_User
{
    private static $instances = NULL;

    const TABLE = 'alsp_users';

    const ROLE_ADMIN = 'admin';
    const ROLE_MANAGER = 'manager';
    const ROLE_AGENT = 'agent';
    const ROLE_CLIENT = 'client';

    public $wp_user = FALSE;

    public $user_id = NULL;
    public $role = NULL;

    static $is_admin = NULL;
    static $is_manager = NULL;
    static $is_agent = NULL;
    static $is_client = NULL;

    /**
     * @param mixed $id - FALSE default to current user, INT load a specific ID, 0 to load a barebones instance with fields
     * @return ALSP_User
     */
    public static function get_instance($id = FALSE)
    {
        // default to current user ID
        if (FALSE === $id) {
            $id = get_current_user_id();
        }

        // if user ID is 0 we might just want access to some methods and profile fields
        if (!isset(self::$instances[$id])) {
            self::$instances[$id] = new self($id);
        }

        return self::$instances[$id];
    }

    /**
     * Constructor
     * @param int $id The user id to create this instance for
     */
    private function __construct($id = 0)
    {
        if ($id > 0) {
            $this->user_id = $id;
            $this->wp_user = get_user_by('id', $id);

            if (FALSE === $this->wp_user)
                return;

            global $wpdb;
            $sql = "SELECT * FROM `{$wpdb->prefix}" . self::TABLE . "` " .
                " WHERE `id`=%d ";
            $res = $wpdb->get_row($wpdb->prepare($sql, $id), ARRAY_A);
            if ($res == NULL) {
                // if alsp_users record doesn't exist, create it
                $data = [
                    'id' => $id,
                    'author' => get_current_user_id(),
                    'role' => self::ROLE_CLIENT,
                ];
                $wpdb->insert($wpdb->prefix . self::TABLE, $data);
                $res = $wpdb->get_row($wpdb->prepare($sql, $id), ARRAY_A);
            }

            if ($res != NULL) {
                $this->role = $res['role'];
            }
        }
    }

    public function get($prop = '')
    {
        if (empty($prop)) {
            return NULL;
        }

        return $this->$prop ? $this->$prop : NULL;
    }

    public static function get_id()
    {
        if ($id = get_current_user_id()) {
            return $id;
        }

        if ($id = (ALSP_Demo::get_instance())->get_id()) {
            return $id;
        }

        return FALSE;
    }

    public function is_admin()
    {
        if (user_can($this->user_id,'administrator')) {
            return TRUE;
        }

        if ($this->role == self::ROLE_ADMIN) {
            return TRUE;
        }

        return FALSE;
    }

    public function is_manager()
    {
        if ($this->role == self::ROLE_MANAGER) {
            return TRUE;
        }

        return FALSE;
    }

    public function is_agent()
    {
        if ($this->role == self::ROLE_AGENT) {
            return TRUE;
        }

        return FALSE;
    }

    public function is_staff()
    {
        return ($this->is_agent() || $this->is_manager() || $this->is_admin());
    }

    public function is_client()
    {
        if ($this->role == self::ROLE_CLIENT) {
            return TRUE;
        }

        return FALSE;
    }

    public function can_open_on_behalf()
    {
        if (!$this->is_staff())
            return FALSE;
        if (!ALSP_Config::get('on_behalf_enabled'))
            return FALSE;
        if ($this->is_admin())
            return TRUE;

        $roles = ALSP_Config::get('on_behalf_roles_select');

        if ($roles >= 2 && $this->is_manager())
            return TRUE;
        if ($roles == 3 && $this->is_agent())
            return TRUE;

        return FALSE;
    }

    public function can_register_on_behalf()
    {
        if (!$this->is_staff())
            return FALSE;
        if (!ALSP_Config::get('on_behalf_enabled'))
            return FALSE;
        if ($this->is_admin())
            return TRUE;

        $roles = ALSP_Config::get('on_behalf_register_role_select');

        if ($roles >= 2 && $this->is_manager())
            return TRUE;
        if ($roles == 3 && $this->is_agent())
            return TRUE;

        return FALSE;
    }

    public function get_details()
    {
        if ($this->wp_user === FALSE) {
            return NULL;
        }

        if (isset($this->wp_user->data)) {

            if(!strlen($this->wp_user->data->display_name)) {
                $f = $this->wp_user->data->first_name;
                $l =$this->wp_user->data->last_name;

                $name = trim("$f $l");
                if(!strlen($name)) {
                    $name = $this->wp_user->data->user_login;
                }

                wp_update_user([
                    'ID' => $this->wp_user->data->ID,
                    'display_name' => $name,
                ]);
            }

            return [
                'id' => $this->wp_user->data->ID,
                'user_role' => $this->role,
                'user_login' => $this->wp_user->data->user_login,
                'user_nicename' => $this->wp_user->data->user_nicename,
                'user_email' => $this->wp_user->data->user_email,
                'user_url' => $this->wp_user->data->user_url,
                'display_name' => $this->wp_user->data->display_name,
                'user_registered' => sprintf(__('%s ago', 'awedesk'), human_time_diff(strtotime($this->wp_user->data->user_registered))),
                'user_avatar' => get_avatar_url($this->wp_user->data->ID),
            ];
        }

        return NULL;
    }

    /**
     * Get list user id by role
     */
    public static function get_users_by_role($role)
    {
        global $wpdb;

        $ids = [];
        $result = $wpdb->get_results($wpdb->prepare("SELECT id FROM {$wpdb->prefix}" . self::TABLE . " WHERE role=%s", $role), ARRAY_A);
        if ($result) {
            foreach ($result as $id) {
                $ids[] = $id['id'];
            }
        }

        return $ids;
    }

    /*
     * creates a WordPress user and "marks" it as a ALSP user
     * @param string $fname The new user's first name
     * @param string $lname The new user's last name
     * @param string $uname The new user's username
     * @param string $email The new user's email address
     * @param string $passw The new user's password
     * @param string $role The new user's role
     * @return multi The new user's id on success or FALSE on error
     */
    public static function create_user($fname, $lname, $uname, $email, $passw, $role = ALSP_User::ROLE_CLIENT)
    {
        $role = apply_filters('alsp_user_default_role', $role);

        $wp_role = $role;
        if ($role == ALSP_User::ROLE_CLIENT) {
            $wp_role = 'subscriber';
        }

        // create the alsp_users table record
        $data = [
            'user_login' => $uname,
            'user_pass' => $passw,
            'user_nicename' => $uname,
            'user_email' => $email,
            'first_name' => $fname,
            'last_name' => $lname,
            'user_registered' => current_time('mysql'),
            'role' => $wp_role,
        ];

        $id = wp_insert_user($data);

        if (is_wp_error($id)) {
            return (FALSE);
        }

        // create the ALSP user
        global $wpdb;
        $sql = "SELECT * FROM `{$wpdb->prefix}" . self::TABLE . "` " .
            " WHERE `id`=%d ";
        $res = $wpdb->get_results($wpdb->prepare($sql, $id), ARRAY_A);
        if (count($res) == 0) {
            // if alsp_users record doesn't exist, create it
            $now = current_time('mysql', TRUE);
            $data = [
                'id' => $id,
                'author' => get_current_user_id(),
                'role' => $role,
                'created' => $now,
                'updated' => $now,
            ];
            $wpdb->insert($wpdb->prefix . self::TABLE, $data);
            $res = $wpdb->get_results($wpdb->prepare($sql, $id), ARRAY_A);
        }

        return ($id);
    }

    /*
     * updates a WordPress user and "marks" it as a ALSP user
     * @param string $role The new user's role
     * @return multi The new user's id on success or FALSE on error
     */
    public static function update_user($id, $role = ALSP_User::ROLE_CLIENT)
    {

        $user = get_user_by('ID', $id);

        $wprole_alsp_agent = 'alsp_' . ALSP_User::ROLE_AGENT;
        $wprole_alsp_manager = 'alsp_' . ALSP_User::ROLE_MANAGER;

        // Clean up roles alsp
        $user->remove_role($wprole_alsp_agent);
        $user->remove_role($wprole_alsp_manager);

        // Add additional role
        if (in_array($role, [ALSP_User::ROLE_AGENT, ALSP_User::ROLE_MANAGER])) {
            $wprole = 'alsp_' . $role;
            $user->add_role($wprole);
        }

        $now = current_time('mysql', TRUE);

        // update the ALSP user
        global $wpdb;
        $sql = "SELECT * FROM `{$wpdb->prefix}" . self::TABLE . "` " .
            " WHERE `id`=%d ";
        $res = $wpdb->get_results($wpdb->prepare($sql, $id), ARRAY_A);
        if (count($res) == 0) {
            // if alsp_users record doesn't exist, create it
            $data = [
                'id' => $id,
                'author' => get_current_user_id(),
                'role' => $role,
                'created' => $now,
                'updated' => $now,
            ];
            $wpdb->insert($wpdb->prefix . self::TABLE, $data);
        } else {
            // if alsp_users record exist, update it
            $data = [
                'author' => get_current_user_id(),
                'role' => $role,
                'updated' => $now,
            ];
            $wpdb->update($wpdb->prefix . self::TABLE, $data, ['id' => $id]);
        }

        return ($id);
    }

    public static function check_unique_username($username, $suffix = 0)
    {
        $new_username = $username . ($suffix > 0 ? $suffix : '');
        $user = get_user_by('login', $new_username);
        if ($user) {
            return self::check_unique_username($new_username, $suffix + 1);
        }

        return $new_username;
    }

    public static function create_user_from_email($email, $name = '')
    {
        $user_login = ALSP_Utility_String::strip_domain_from_email(sanitize_user($email));
        $user_login = self::check_unique_username($user_login);
        $user_email = sanitize_email($email);

        // Since loginpress is not compatible with function `register_new_user`,
        // We need to surpress error message, to prevent error.
        if (ALSP_Third_Party::has_loginpress_custom_password()) {
            add_filter('registration_errors', function ($errors, $sanitized_user_login, $user_email) {
                $user_pass = wp_generate_password( 12, false );
                $_POST['loginpress-reg-pass'] = isset($_POST['loginpress-reg-pass']) ? $_POST['loginpress-reg-pass'] : $user_pass;
                $_POST['loginpress-reg-pass-2'] = isset($_POST['loginpress-reg-pass-2']) ? $_POST['loginpress-reg-pass-2'] : $user_pass;

                return $errors;
            }, 9, 3);
        }

        $user_id = register_new_user($user_login, $user_email);

        if (is_wp_error($user_id)) {
            new ALSP_Debug("Error: " . $user_id->get_error_message());
            return FALSE;
        }

        if (strlen($name) > 0) {
            // Split the full name into an array of words
            $name_array = explode(' ', $name);

            // Get the combined first name
            $first_name = implode(' ', array_slice($name_array, 0, -1));

            // Check if the array has at least two elements
            if (count($name_array) >= 2) {

                // Get the last name
                $last_name = end($name_array);

                wp_update_user([
                    'ID' => $user_id, // this is the ID of the user you want to update.
                    'first_name' => $first_name,
                    'last_name' => $last_name,
                    'display_name' => $name,
                ]);
            }
        }

        add_user_meta($user_id, 'alsp_source', 'email_piping');
        return $user_id;
    }

    /**
     * Changes the user's role to the specified value
     * @param string $role The new role to change the user to
     */
    public function set_user_role($role)
    {
        global $wpdb;

        if ($wpdb->update($wpdb->prefix . self::TABLE, ['role' => $role], ['id' => $this->user_id])) {
            do_action('alsp_action_user_role_change', $this->user_id, $role);
        }
    }

}

// EOF
