<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OUl2cW1KT0MzalNRK3lhR2l1OC9XTXVsd3YvWnRWcEV6eEQrblpidHdreDUycFFXZWtzaHc3NThBdWZkVVdKZ2tqRmRYRjJiUndScTMzZHM3UFJMMVBIaFdoc0R0YWx2WHE0ME9JSzNucWd3PT0=*/

/**
 * This class provides superficial encryption
 * We use it only to ensure secret ticket notes aren't easily accessible in case of a database leak
 */
class ALSP_Utility_Crypto {
    public static function encrypt($text) {

        if (!strlen($text)) {
            return '';
        }

        // Generate a random IV & pad to 16 bytes if necessary
        $iv = openssl_random_pseudo_bytes(16);
        if (strlen($iv) !== 16) {
            $iv = str_pad($iv, 16, "\0");
        }

        $ciphertext_raw = openssl_encrypt($text, "AES-128-CBC", self::key(), OPENSSL_RAW_DATA, $iv);
        $hmac = hash_hmac('sha256', $ciphertext_raw, self::key(), TRUE);
        $ciphertext = base64_encode($iv . $hmac . $ciphertext_raw);

        return $ciphertext;
    }

    public static function decrypt($ciphertext) {
        if (!strlen($ciphertext)) {
            return '';
        }

        $c = base64_decode($ciphertext);

        $ivlen = 16;
        $sha2len = 32;

        $iv = substr($c, 0, $ivlen);

        // Pad the IV to 16 bytes if necessary
        if (strlen($iv) !== 16) {
            $iv = str_pad($iv, 16, "\0");
        }

        $hmac = substr($c, $ivlen, $sha2len);
        $ciphertext_raw = substr($c, $ivlen + $sha2len);

        $original_plaintext = openssl_decrypt($ciphertext_raw, "AES-128-CBC", self::key(), OPENSSL_RAW_DATA, $iv);
        $calcmac = hash_hmac('sha256', $ciphertext_raw, self::key(), TRUE);

        if (hash_equals($hmac, $calcmac)) { //PHP 5.6+ Timing attack safe string comparison
            return $original_plaintext;
        } else {
            return 'Decryption failed!';
        }
    }

    private static function key() {
        return defined('AUTH_KEY') ? AUTH_KEY : NULL;
    }
}
