<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OWR5VUlSU3F1RG15M1dJVWNTbnkxOTFlZGIvNUVqRGpzbi9Ed2JER01WQ1V2SXBIL0NYc0gvTWVzYll6VGpmR1lxeTgvT1pXZEt5bEVnSHp2aUlQeURjL1pDVXNXdFlEOUZhMityZlRBUzVnPT0=*/

class ALSP_Utility_String
{

    /** NUMBERS **/
    public static function intval($text)
    {

        if (filter_var($text, FILTER_VALIDATE_INT)) {
            return intval($text);
        }

        if (is_numeric($text)) {
            return $text;
        }

        return 0;
    }

    public static function shorten_big_number($n, $decimals = 1, $threshold = 1000)
    {

        if ($n < $threshold) {
            return $n;
        }

        if ($n < 1000000) {
            return number_format($n / 1000, $decimals) . _x('k', 'Short for thousands', 'awedesk');
        }

        return number_format($n / 1000000, $decimals) . _x('m', 'Short for millions', 'awedesk');
    }

    /** STRINGS **/

    public static function sanitize_message($text, $check_cache = TRUE)
    {

        $text = trim($text);
        $text = trim($text, "\n");
        $text = trim($text, "\t");
        $libraries = dirname(dirname(dirname(__FILE__))) . '/libraries/';

        $allowed_tags_required = [
            'p', 'br', 'hr',                                    // Linebreaks & spacers
            'a',                                                // Links
        ];

        $allowed_tags_optional = [
            'b', 'strong', 'u', 'i', 'em',                        // Formatting
            'h1', 'h2', 'h3', 'h4', 'h5', 'h6',                      // Headers
            'ul', 'ol', 'li',                                     // Lists
        ];

        $allowed_tags = implode(',', array_merge($allowed_tags_required, apply_filters('alsp_allowed_html', $allowed_tags_optional)));

        $autolink_max_length = intval(apply_filters('autolink_max_length', 100));
        if ($autolink_max_length < 10)
            $autolink_max_length = 10;

        // Check result cache
        $hash = md5($text . strlen($text) . $allowed_tags . $autolink_max_length);
        if ($check_cache && $cache = ALSP_Mayfly::get($hash)) {
            if (strlen($cache)) {
                return "$cache\n<!--cached-->";
            }
        }

        /*** BEFORE HTMLPURIFIER ***/

        // Detect links
        try {
            // Detect URLs with lib_autolink
            if (!function_exists('alsp_autolink')) {
                require_once($libraries . 'autolink/lib_autolink.php');
            }
            $text_autolink = alsp_autolink($text, $autolink_max_length);
            $text_autolink = alsp_autolink_email($text_autolink);
        } catch (Exception $e) {
            // If lib_autolink explodes for some reason, fall back to WP make_clickable
            $text_autolink = make_clickable($text);
        }

        $text = $text_autolink;

        // Normalize whitespace
//        $text = self::normalize_whitespace($text);

        // Normalize linebreaks & formatting
        $text = self::wpautop($text);

        // Apply length limit (rough, but HTMLPurifier will attempt to close missing tags so nothing should explode)
        $limit = max(ALSP_Config::get('chars_limit_content_number', 10000), 100);
        $text = substr($text, 0, $limit);

        /*** HTMLPURIFIER ***/

        if (!class_exists('HTMLPurifier')) {
            require_once($libraries . 'htmlpurifier/HTMLPurifier.standalone.php');
        }

        // Make sure only allowed tags with no onclick, class, id etc are passed to the result
        $config = HTMLPurifier_Config::createDefault();
        $config->set('HTML.AllowedAttributes', 'href,title');
        $config->set('HTML.AllowedElements', $allowed_tags);

        $purifier = new HTMLPurifier($config);
        $text = $purifier->purify($text);

        /*** AFTER HTMLPURIFIER ***/

        // All links should be target="_blank" - HTMLPurifier deletes that
        $text = str_replace('<a ', '<a target="_blank" ', $text);

        // Store cache
        ALSP_Mayfly::set($hash, $text, 3600);

        return $text;
    }

    private static function wpautop($text, $br = true)
    {

        $pre_tags = [];

        if (trim($text) === '') {
            return '';
        }

        // Just to make things a little easier, pad the end.
        $text = $text . "\n";

        /*
         * Pre tags shouldn't be touched by autop.
         * Replace pre tags with placeholders and bring them back after autop.
         */
        if (str_contains($text, '<pre')) {
            $text_parts = explode('</pre>', $text);
            $last_part = array_pop($text_parts);
            $text = '';
            $i = 0;

            foreach ($text_parts as $text_part) {
                $start = strpos($text_part, '<pre');

                // Malformed HTML?
                if (false === $start) {
                    $text .= $text_part;
                    continue;
                }

                $name = "<pre wp-pre-tag-$i></pre>";
                $pre_tags[$name] = substr($text_part, $start) . '</pre>';

                $text .= substr($text_part, 0, $start) . $name;
                $i++;
            }

            $text .= $last_part;
        }
        // Change multiple <br>'s into two line breaks, which will turn into paragraphs.
        $text = preg_replace('|<br\s*/?>\s*<br\s*/?>|', "\n\n", $text);

        $allblocks = '(?:table|thead|tfoot|caption|col|colgroup|tbody|tr|td|th|div|dl|dd|dt|ul|ol|li|pre|form|map|area|blockquote|address|style|p|h[1-6]|hr|fieldset|legend|section|article|aside|hgroup|header|footer|nav|figure|figcaption|details|menu|summary)';

        // Add a double line break above block-level opening tags.
        $text = preg_replace('!(<' . $allblocks . '[\s/>])!', "\n\n$1", $text);

        // Add a double line break below block-level closing tags.
        $text = preg_replace('!(</' . $allblocks . '>)!', "$1\n\n", $text);

        // Add a double line break after hr tags, which are self closing.
        $text = preg_replace('!(<hr\s*?/?>)!', "$1\n\n", $text);

        // Standardize newline characters to "\n".
        $text = str_replace(array("\r\n", "\r"), "\n", $text);

        // Find newlines in all elements and add placeholders.
        $text = wp_replace_in_html_tags($text, ["\n" => ' <!-- wpnl --> ']);

        // Collapse line breaks before and after <option> elements so they don't get autop'd.
        if (str_contains($text, '<option')) {
            $text = preg_replace('|\s*<option|', '<option', $text);
            $text = preg_replace('|</option>\s*|', '</option>', $text);
        }

        /*
         * Collapse line breaks inside <object> elements, before <param> and <embed> elements
         * so they don't get autop'd.
         */
        if (str_contains($text, '</object>')) {
            $text = preg_replace('|(<object[^>]*>)\s*|', '$1', $text);
            $text = preg_replace('|\s*</object>|', '</object>', $text);
            $text = preg_replace('%\s*(</?(?:param|embed)[^>]*>)\s*%', '$1', $text);
        }

        /*
         * Collapse line breaks inside <audio> and <video> elements,
         * before and after <source> and <track> elements.
         */
        if (str_contains($text, '<source') || str_contains($text, '<track')) {
            $text = preg_replace('%([<\[](?:audio|video)[^>\]]*[>\]])\s*%', '$1', $text);
            $text = preg_replace('%\s*([<\[]/(?:audio|video)[>\]])%', '$1', $text);
            $text = preg_replace('%\s*(<(?:source|track)[^>]*>)\s*%', '$1', $text);
        }

        // Collapse line breaks before and after <figcaption> elements.
        if (str_contains($text, '<figcaption')) {
            $text = preg_replace('|\s*(<figcaption[^>]*>)|', '$1', $text);
            $text = preg_replace('|</figcaption>\s*|', '</figcaption>', $text);
        }

        // Remove more than two contiguous line breaks.
        $text = preg_replace("/\n\n+/", "\n\n", $text);

        // Split up the contents into an array of strings, separated by double line breaks.
        $paragraphs = preg_split('/\n\s*\n/', $text, -1, PREG_SPLIT_NO_EMPTY);

        // Reset $text prior to rebuilding.
        $text = '';

        // Rebuild the content as a string, wrapping every bit with a <p>.
        foreach ($paragraphs as $paragraph) {
            $text .= '<p>' . trim($paragraph, "\n") . "</p>\n";
        }

        // Under certain strange conditions it could create a P of entirely whitespace.
        $text = preg_replace('|<p>\s*</p>|', '', $text);

        // Add a closing <p> inside <div>, <address>, or <form> tag if missing.
        $text = preg_replace('!<p>([^<]+)</(div|address|form)>!', '<p>$1</p></$2>', $text);

        // If an opening or closing block element tag is wrapped in a <p>, unwrap it.
        $text = preg_replace('!<p>\s*(</?' . $allblocks . '[^>]*>)\s*</p>!', '$1', $text);

        // In some cases <li> may get wrapped in <p>, fix them.
        $text = preg_replace('|<p>(<li.+?)</p>|', '$1', $text);

        // If a <blockquote> is wrapped with a <p>, move it inside the <blockquote>.
        $text = preg_replace('|<p><blockquote([^>]*)>|i', '<blockquote$1><p>', $text);
        $text = str_replace('</blockquote></p>', '</p></blockquote>', $text);

        // If an opening or closing block element tag is preceded by an opening <p> tag, remove it.
        $text = preg_replace('!<p>\s*(</?' . $allblocks . '[^>]*>)!', '$1', $text);

        // If an opening or closing block element tag is followed by a closing <p> tag, remove it.
        $text = preg_replace('!(</?' . $allblocks . '[^>]*>)\s*</p>!', '$1', $text);

        // Optionally insert line breaks.
        if ($br) {
            // Replace newlines that shouldn't be touched with a placeholder.
            $text = preg_replace_callback('/<(script|style|svg|math).*?<\/\\1>/s', '_autop_newline_preservation_helper', $text);

            // Normalize <br>
            $text = str_replace(array('<br>', '<br/>'), '<br />', $text);

            // Replace any new line characters that aren't preceded by a <br /> with a <br />.
            $text = preg_replace('|(?<!<br />)\s*\n|', "<br />\n", $text);

            // Replace newline placeholders with newlines.
            $text = str_replace('<WPPreserveNewline />', "\n", $text);
        }

        // If a <br /> tag is after an opening or closing block tag, remove it.
        $text = preg_replace('!(</?' . $allblocks . '[^>]*>)\s*<br />!', '$1', $text);

        // If a <br /> tag is before a subset of opening or closing block tags, remove it.
        $text = preg_replace('!<br />(\s*</?(?:p|li|div|dl|dd|dt|th|pre|td|ul|ol)[^>]*>)!', '$1', $text);
        $text = preg_replace("|\n</p>$|", '</p>', $text);

        // Replace placeholder <pre> tags with their original content.
        if (!empty($pre_tags)) {
            $text = str_replace(array_keys($pre_tags), array_values($pre_tags), $text);
        }

        // Restore newlines in all elements.
        if (str_contains($text, '<!-- wpnl -->')) {
            $text = str_replace(array(' <!-- wpnl --> ', '<!-- wpnl -->'), "\n", $text);
        }

        return $text;
    }

    private static function normalize_whitespace($str)
    {
        $str = trim($str);
        $str = str_replace("\r", "\n", $str);
        $str = preg_replace(array('/\n+/', '/[ \t]+/'), ["\n\n ", ' '], $str);
        return $str;
    }

    public static function maybe_mb_substr(string $string, int $start, int $end)
    {

        if (function_exists('mb_substr')) {
            return mb_substr($string, $start, $end);
        }

        return substr($string, $start, $end);
    }

    public static function maybe_mb_strlen(string $string)
    {

        if (function_exists('mb_strlen')) {
            return mb_strlen($string);
        }

        return strlen($string);
    }

    public static function maybe_mb_ucwords(string $string, $mode = MB_CASE_TITLE)
    {
        if (function_exists('mb_convert_case')) {
            return mb_convert_case($string, $mode);
        }

        return ucwords($string);
    }

    public static function strip_domain_from_email(string $email)
    {
        // split on @ and return last value of array (the domain)
        $email = explode('@', $email);

        if (count($email)) {
            return $email[0];
        }

        // Otherwise
        return $email;
    }

    /**
     * Create a random string
     *
     * @param integer $length -  random string length.
     * @return string
     */
    public static function get_random_string($length = 8)
    {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $string = '';
        for ($i = 0; $i < $length; $i++) {
            $string .= $characters[wp_rand(0, strlen($characters) - 1)];
        }
        return $string;
    }

    public static function date_relative($date)
    {
        return sprintf(__('%s ago', 'awedesk'), self::human_time_diff(strtotime($date)));
    }



    public static function date_absolute($date)
    {
        return get_date_from_gmt($date, ALSP_Utility_WP::get_option('date_format') . ' ' . ALSP_Utility_WP::get_option('time_format'));
    }

    public static function date_relative_short($date)
    {
        return self::human_time_diff_short(strtotime($date));
    }

    public static function human_time_diff_short($from, $to = 0) {
        if (empty($to)) {
            $to = time();
        }

        $diff = (int) abs($to - $from);

        if ($diff < HOUR_IN_SECONDS) {
            return sprintf(__('%sh', 'awedesk'), '<1');
        } elseif ($diff < DAY_IN_SECONDS && $diff >= HOUR_IN_SECONDS) {
            $hours = floor($diff / HOUR_IN_SECONDS);
            return sprintf(__('%sh', 'awedesk'), $hours);
        } elseif ($diff < WEEK_IN_SECONDS && $diff >= DAY_IN_SECONDS) {
            $days = floor($diff / DAY_IN_SECONDS);
            return sprintf(__('%sd', 'awedesk'), $days);
        } elseif ($diff < MONTH_IN_SECONDS && $diff >= WEEK_IN_SECONDS) {
            $weeks = floor($diff / WEEK_IN_SECONDS);
            return sprintf(__('%sw', 'awedesk'), $weeks);
        } else if ($diff < YEAR_IN_SECONDS && $diff >= MONTH_IN_SECONDS) {
            $months = floor($diff / MONTH_IN_SECONDS);
            return sprintf(__('%sm', 'awedesk'), $months);
        } else {
            $years = floor($diff / YEAR_IN_SECONDS);
            return sprintf(__('%sy', 'awedesk'), $years);
        }
    }


    public static function human_time_diff($from, $to = 0)
    {

        if (empty($to)) {
            $to = time();
        }

        $diff = (int) abs($to - $from);

        if ($diff < MINUTE_IN_SECONDS) {
            return sprintf(__('%s mins', 'awedesk'), '< 5');
        } elseif ($diff < HOUR_IN_SECONDS && $diff >= MINUTE_IN_SECONDS) {

            $mins = round($diff / MINUTE_IN_SECONDS);

            // Less than 5 minutes = "Just now"
            if ($mins < 5) {
                return sprintf(__('%s mins', 'awedesk'), '< 5');
            }

            // More than 5 but less than an hour
            $mins = (round($mins) % 5 === 0) ? round($mins) : round(($mins + 5 / 2) / 5) * 5;

            if ($mins <= 55) {
                return sprintf(__('%s mins', 'awedesk'), $mins);
            }
        }

        return human_time_diff($from, $to);


        //        elseif ( $diff < DAY_IN_SECONDS && $diff >= HOUR_IN_SECONDS ) {
//            $hours = round( $diff / HOUR_IN_SECONDS );
//            if ( $hours <= 1 ) {
//                $hours = 1;
//            }
//            /* translators: Time difference between two dates, in hours. %s: Number of hours. */
//            $since = sprintf( _n( '%s hour', '%s hours', $hours ), $hours );
//        } elseif ( $diff < WEEK_IN_SECONDS && $diff >= DAY_IN_SECONDS ) {
//            $days = round( $diff / DAY_IN_SECONDS );
//            if ( $days <= 1 ) {
//                $days = 1;
//            }
//            /* translators: Time difference between two dates, in days. %s: Number of days. */
//            $since = sprintf( _n( '%s day', '%s days', $days ), $days );
//        } elseif ( $diff < MONTH_IN_SECONDS && $diff >= WEEK_IN_SECONDS ) {
//            $weeks = round( $diff / WEEK_IN_SECONDS );
//            if ( $weeks <= 1 ) {
//                $weeks = 1;
//            }
//            /* translators: Time difference between two dates, in weeks. %s: Number of weeks. */
//            $since = sprintf( _n( '%s week', '%s weeks', $weeks ), $weeks );
//        } elseif ( $diff < YEAR_IN_SECONDS && $diff >= MONTH_IN_SECONDS ) {
//            $months = round( $diff / MONTH_IN_SECONDS );
//            if ( $months <= 1 ) {
//                $months = 1;
//            }
//            /* translators: Time difference between two dates, in months. %s: Number of months. */
//            $since = sprintf( _n( '%s month', '%s months', $months ), $months );
//        } elseif ( $diff >= YEAR_IN_SECONDS ) {
//            $years = round( $diff / YEAR_IN_SECONDS );
//            if ( $years <= 1 ) {
//                $years = 1;
//            }
//            /* translators: Time difference between two dates, in years. %s: Number of years. */
//            $since = sprintf( _n( '%s year', '%s years', $years ), $years );
//        }
//
//        return $since;
    }
}

// EOF
