<?php /*QW9Weng5ayt0UStQZ2dZRWxzTXVFakd2c21tZ093UnM1WWJLK1grblg2OFlCNzBhcjRhR0t2bkhzejRtZUkzZzdISjg0cDlxYkw2SmQ4WDZFWjRLYjcyTFdHc2xFWmtSb1kxVm1mL09EQXBLVFdQMm9BYzVtd3BxbE4vM25ZbVNBZzhUYnBZbmlPc3d6YVJwUTd1bWZIRlI4VlNCejh3ZUVQcjJySUpmanBZPQ==*/

class ALSP_Template
{
    private static $instance;

    public $assets_url;
    public $assets_dir;
    public $templates_dir;
    public static function get_instance($component = '')
    {
        return isset(self::$instance) ? self::$instance : self::$instance = new self($component);
    }

    private function __construct($component = '')
    {
        $dir = dirname(dirname(__FILE__));
        $component_dir = '';
        if (!empty($component)) {
            $component_dir = 'classes/' . $component . '/';
        }

        $this->assets_url = plugin_dir_url($dir) . $component_dir . 'assets/';
        $this->assets_dir = plugin_dir_path($dir) . $component_dir . 'assets/';
        $this->templates_dir = plugin_dir_path($dir) . $component_dir . 'templates/';

    }
    public function is_dev()
    {
        // @TODO: Remove short-circuit before release.
        // return defined('ALSP_DEV') && ALSP_DEV;
        return true;
    }


    public function get_assets_url($path)
    {
        $url = $this->assets_url . $path;

        if ($this->is_dev()) {
            $mtime = filemtime($this->assets_dir . $path);
            $url = add_query_arg('mtime', $mtime, $url);
        }

        return $url;
    }

    /**
     * Get template string.
     *
     * @param string $name Template name.
     * @param array  $data Optional data to be passed to the template.
     * @param array  $opts Optional settings to be applied to the template.
     * @return string
     */
    public function get_template($name, $data = [], $opts = [])
    {
        $file = $this->templates_dir . $name . '.php';
        $template = '';

        if (file_exists($file)) {
            try {
                ob_start();
                if (NULL !== $data) {
                    extract($data);
                }
                include $file;
                $template = ob_get_clean();

                if (isset($opts['trim']) && $opts['trim']) {
                    $template = preg_replace('/\s+/', ' ', $template);
                    $template = preg_replace('/> </', '><', $template);
                }
            } catch (Exception $e) {
                // Do nothing.
            }
        } else {
            //new ALSP_Debug("Missing template $file");
        }

        return trim($template);
    }

    /*
     * execute a template with provided data
     * @param string $component name of component folder
     * @param string $section application section to load the template from
     * @param string $template name of template file within section to load
     * @param array $data array of data variables to pass to template
     * @param boolean if TRUE will return the output of the template
     */
    public static function exec_template($component, $section, $template, $data = NULL, $return_output = FALSE)
    {
        /**
         * @since WordPress 5.8 legacy-widget-preview GET was set
         * @since WordPress 5.9 it is no longer passed, so we force set it when called from the WP preview URL
         */
        if (stristr($_SERVER['REQUEST_URI'], 'wp-json') && stristr($_SERVER['REQUEST_URI'], 'widget-types')) {
            $_GET['legacy-widget-preview'] = TRUE;
        }

        // If the Block Editor preview is loading a widget, pass args through Gecko filters to modify the args
        if (is_array($data) && isset($data['args']) && isset($_GET['legacy-widget-preview'])) {

            $data['args'] = apply_filters('alsp_fix_block_legacy_widget', $data['args'], $data['instance']);

            //echo "<span style='opacity:0.5;font-size:10px;'>".htmlspecialchars($data['args']['before_widget'])."</span>";
        }

        $templ = ALSP_Frontend_Assets::get_instance($component)->get_template($section, $template);

        if ($return_output) {
            ob_start();
        }

        do_action('alsp_action_before_exec_template', $section, $template, $data, $return_output);

        if (NULL !== $data)
            extract($data);
        include($templ);

        do_action('alsp_action_after_exec_template', $section, $template, $data, $return_output);
        if ($return_output) {
            $ret = ob_get_clean();
            return ($ret);
        }
    }
}

// EOF